! O.K. 08/10/2008 
! this file contains first caller to the set of amplitudes^2. O'Mega generates
! function spin_sum_sqme_1 for any process requested. To be able to implement
! more than just one ME from O'Mega, first the caller_omega_amplitude_...
! modules are defined that use an appropriate: use omega_amplitude_...
! Universal module which is being called by fpmc is the last module of the file

!##############################################################################
! TGC caller
!##############################################################################
module caller_omega_amplitude_tgc
   use omega_kinds
   use omega_parameters
   use omega95
   use omega_amplitude_aaww_ac

   public :: call_omega_amplitude_tgc
contains   
   function call_omega_amplitude_tgc(p) result(amp2)

   real(kind=omega_prec), dimension(0:,:), intent(in) :: p     
   real(kind=omega_prec)  :: amp2
   amp2 = spin_sum_sqme_1(p, 1)
   end function

end module caller_omega_amplitude_tgc
!##############################################################################
! new caller
!##############################################################################
module caller_omega_amplitude_new
   use omega_kinds
   use omega_parameters
   use omega95
!   use omega_amplitude_new

   public :: call_omega_amplitude_new
contains   
   function call_omega_amplitude_new(p) result(amp2)

   real(kind=omega_prec), dimension(0:,:), intent(in) :: p     
   real(kind=omega_prec)  :: amp2
   amp2=0
   print *, 'omeg_interface.f90: aawwanom = 3, example interface to OMega'
   print *, 'omeg_interface.f90: Define which ME^2 to call first!'
   stop
!   amp2 = spin_sum_sqme_1(p, 1)
   end function

end module caller_omega_amplitude_new


!##############################################################################
! Universal module called by fpmc, select to call O'Mega ME there
!##############################################################################

! calculate t u  from variables below and call the matrix element      
module omega_interface
   use omega_kinds
   use caller_omega_amplitude_tgc
   use caller_omega_amplitude_new
   use omega_parameters
   use omega95
   private 

   public :: test_
   public :: testf
   public :: get_spin_sum_sqme_1
contains

   ! test rutine
   function testf(input, input2) result(res)
      real(kind=omega_prec) :: res
      real(kind=omega_prec),intent(in)  :: input, input2

      res = input
      print *, 'Input 1 ', res
      print *, 'Input 2 ', input2
   end function

   ! test rutine
   subroutine test(input, res)
      real(kind=omega_prec) :: res
      real(kind=omega_prec) :: input

      res = 5
      res = input
      print *, 'Set test func ', res
   end subroutine   
      

   ! wraper for O'Mega: amp2 = result, the rest imput parameters
   function get_spin_sum_sqme_1(aawwanom, s, t, wmass, d_kappa, lambda, cutoff_scale) result(amp2)
      integer :: i
      real(kind=omega_prec), dimension(0:3) :: totmom, p13, p14, p12
      real(kind=omega_prec) :: u, test_t, test_u
      integer, parameter :: debug =0    ! 10, 100

      real(kind=omega_prec)  :: amp2
      integer, intent(in) :: aawwanom
      real(kind=omega_prec), intent(in) :: s, t, wmass
      real(kind=omega_prec), intent(in) :: d_kappa, lambda, cutoff_scale
      real(kind=omega_prec) :: mw, mwsq, beta, costheta, sintheta
      real(kind=omega_prec), dimension(0:3,4):: p

      ! temp variables energy of gamma and W
      real(kind=omega_prec) egamma, ew, k_mod, m1, m2, fac
      if(debug.gt.9) print *,'Entering fpmc_to_omega interface...'

      if(debug.gt.99) &
         write (6, fmt='(A30, F20.2, A10, F20.2, A10, F10.2, &
         A10, F10.2, A10, F10.2)') 'Entered params: E_cms ', s, 't ', &
         t, 'Mw' , wmass, 'd_kappa', d_kappa, 'lambda', lambda 

      ! initial set     
      call setup_parameters()


      ! shortcut
      if(wmass.lt.0) then
         mw = mass(24)
      else 
         mass(24) = wmass
         mw = mass(24)
      endif   
!      width(24)=0

      if(debug.gt.99) &
         write (6, fmt='(A10, F15.2)') 'W mass: ', mw

      ! calculate cos and sin
      if(debug.gt.99)then
         write (6, fmt='(A10, F15.2)') 'W mass: ', mw
         write (6, fmt='(A10, F15.2)') '2(t-mw**2)/s ', 2*(t-mw**2)/s
      endif   
      beta=sqrt(1-4*mw**2/s)
      u = -t - s + 2*mw**2
!      costheta=(1+2*(t-mw**2)/s)/beta
      ! I will make the mw->0 approximation
      costheta = (t-u)/beta/s

      if((costheta.gt.1).or.(costheta.lt.-1))then
          print *, 'fpmc2omega interface: costheta out of range!'
          write (6, fmt='(A10, F15.2)') 'costheta ', costheta
          write (6, fmt='(A10, F15.2)') '1-4*mw**2/s ', 1-4*mw**2/s
         stop
      endif   
      sintheta=sqrt(1-costheta**2)

!      print *, 'wr costheta', costheta
!      print *, 'wr sintehta', sintheta

      !     u,t gamma gamma -> w w  
      test_t = -s/2.*(1-sqrt(1.-4.*mw**2/s)*costheta)+mw**2
      test_u = -s-t+2*mw**2

      if(debug.gt.99) then
         print *, ''
         write (6, fmt='(A5, F25.2)') 's', s
         write ( unit = *, fmt='(A5, F25.2)') 't', t
         write (6, fmt='(A8, F25.10)') 'test_t', test_t
         write (6, fmt='(A8, F25.10)') 'test_u', test_u
      end if   
      
      
   !   4-momenta explicit specification
      
      ! gammas
      egamma = sqrt(s)/2

      ! gamma1
      p(0,1) = egamma
      p(1,1) = 0
      p(2,1) = 0
      p(3,1) = egamma
      ! gamma2
      p(0,2) = egamma
      p(1,2) = 0
      p(2,2) = 0
      p(3,2) = -egamma
      
      ! W+W-
      em = sqrt(s)/2
      k_mod = sqrt(s/4 - mw**2)
      
      if(debug.gt.99) then
         write (6, fmt='(A8, F25.10)') 'em', em
         write (6, fmt='(A8, F25.10)') 'kmod', k_mod
      end if   
      
      p(0,3) = em
      p(1,3) = 0
      p(2,3) = -k_mod*sintheta 
      p(3,3) = -k_mod*costheta

      p(0,4) = em
      p(1,4) = 0
      p(2,4) = k_mod*sintheta
      p(3,4) = k_mod*costheta

      p12 = p(:,1) + p(:,2)
      p13 = p(:,1) - p(:,3)
      p14 = p(:,1) - p(:,4)

!    
      if(debug.gt.99) then
         print *, ''
         write( 6, fmt='(A8, A12, A12, A12, A12)') 'Momentum', 'E', 'px', 'py', 'pz'
         write( 6, fmt='(A8, 4F12.4)') ' 1: ',  p(0,1), p(1,1), p(2,1), p(3,1)
         write( 6, fmt='(A8, 4F12.4)') ' 2: ',  p(0,2), p(1,2), p(2,2), p(3,2) 
         write( 6, fmt='(A8, 4F12.4)') ' 3: ',  p(0,3), p(1,3), p(2,3), p(3,3)
         write( 6, fmt='(A8, 4F12.4)') ' 4: ',  p(0,4), p(1,4), p(2,4), p(3,4)
      end if   

      ! test of momenta
      m1 =0; m2 =0
      do i=0,3 
         if(i.eq.0) then
            fac = 1d0
         else 
            fac = -1d0
         end if

         m1 = m1 + fac* p(i,3)*p(i,3)
         m2 = m2 + fac* p(i,4)*p(i,4)
      end do   

      m1 = sqrt(m1); m2 = sqrt(m2);
      totmom = p(:,3) + p(:,4)

      if(debug.gt.99) then
         write ( 6, fmt='(A28, F10.2, A8, F10.2)') 'Mass of outgoing W+ W- check m1', m1, 'm2', m2
         write ( 6, fmt='(A15, 5F10.2)' ) 'Total s check, should be', sqrt(s), totmom(0), totmom(1), totmom(2), totmom(3)
      end if

      amp2 = 0


      ! set the anomalous WWA coupling

      ! triple gauge coupling
      if(aawwanom.eq.2) then
         ! if cutoff_scale < 0 (0GeV) simply turn it off
         if(cutoff_scale.lt.0) then
            kappaa = 1. + d_kappa
            lambdaa = lambda
         else   
            kappaa = 1. + d_kappa/(1+s/cutoff_scale**2)**2
            lambdaa = lambda/(1+s/cutoff_scale**2)**2
         endif   

         amp2=call_omega_amplitude_tgc(p)
         ! magic factor independent of S, T and MW which brings O'Mega
         ! matrix element equal to the analytic formula 
         ! with rel difference 1E-8
         ! Pehaps the difference between the alphem
         amp2 = amp2*(1+0.000533625)  

      ! quartic coupling
      elseif(aawwanom.eq.3) then
         amp2=call_omega_amplitude_new(p)
       else 
         write ( 6, fmt='(A28, I2)') 'omega_interface.f90: Nonstandard choice of aawwanom = ', aawwanom
         stop
      endif   


! complete formula, for testing
!      mwsq = mw*mw
!      amp2=alpha**2/s &
!                 *(1-2*s*(2*s+3*mwsq)/(3*(mwsq-t)*(mwsq-test_u)) &
!                 +2*s**2*(s**2+3*mwsq*mwsq)/(3*(mwsq-t)**2*(mwsq-test_u)**2))

      if(debug.gt.99) write ( 6, fmt='(A8, F20.10)'), 'amp2: ', amp2
   end function

end module omega_interface
!##############################################################################


