*-- 17/11/06: KEPKA - implementation of new tabels, see below      
*###########################################################
* Main routine for parameterisations of different qcd fits
* QCD fits to F2D(3)
*
*   Input:   ZT = x_{i/IP}
*            Q2T = photon virtuality
*       --- current  hep-ex/0606004
*           IFIT = 100 fit A
*           IFIT = 101 fit B

*       --- old (hep-ph/0609291) ---      
*           IFIT = 10, new H1 QCD fit 
*           IFIT = 20, new ZEUS QCD fit 
*           IFIT = 30, combned H1 and ZEUS QCD fit
*       --- old versions ---       
*           IFIT = 2 (NLO fit as in H1 1994 hep-ex/9708016)
*           IFIT = 5 (LO fits as in H1 1994 hep-ex/9708016)
*           IFIT = 8 (extended version of H1 1994 hep-ph/0602228)
*       --- obsolete ---       
*           IFIT = 1, 3, 4, 6
*           IFIT = 102, 105, 108, 110, 120, 130
*
*           Call with IFIT>0 to initialise the parameterisation,
*           then call with ifit=0 for subsequent access. 
*                 
*
* Output: XPQ(-6:6): PDG style array of partons, 0=gluon.
*         Note that these parton distributions are normalised
*         to represent xpom times the 'true' parton distributions 
*         multiplied by the flux factor at xpom integrated 
*         over t in the interval -1.0<t<t_min. 
*         
*###########################################################

      SUBROUTINE QCDFIT(ZT,Q2T,XPQ,IFIT)

c      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      INTEGER INDICE,IFIT
      data indice/0/
      real*8 zt,q2t
      ! general array for  uds g anti-sdu      
      DOUBLE PRECISION XPQ
      DIMENSION XPQ(-6:6)

      ! for QCD_2006, structure functions - not used
      DOUBLE PRECISION F2(2),FL(2),C2(2),CL(2) !



C Indice is an internal variable: indice=1 old H1 fits
C                                 indice=2 new fits from Laurent
c This is necessary since the tables do not have the same sizes...


      IF(IFIT.GT.0) THEN !ifit turns to zero after first cal1
        indice = 0
        IF(IFIT.EQ.2)      THEN 
                           indice = 1
        ELSEIF(IFIT.EQ.5)  THEN 
                           indice = 1
        ELSEIF(IFIT.EQ.8)  THEN 
                           indice = 2
        ELSEIF(IFIT.EQ.10) THEN 
                           indice = 2
        ELSEIF(IFIT.EQ.20) THEN 
                           indice = 2
        ELSEIF(IFIT.EQ.30) THEN 
                           indice = 2
        ELSEIF(IFIT.EQ.100) THEN 
                           indice = 3
                           ifit = 1
        ELSEIF(IFIT.EQ.101) THEN 
                           indice = 4
                           ifit =2
        ELSE 
           print *, 'Obsolete choice of IFIT=', IFIT, '. See h1qcd.f'
           STOP
        ENDIF
      ENDIF

      if(indice.eq.1) call QCD_1994(ZT,Q2T,XPQ,IFIT)
      if(indice.eq.2) call QCD_LS(ZT,Q2T,XPQ,IFIT)
      if(indice.eq.3) call QCD_2006(ZT,Q2T,IFIT,XPQ,F2,FL,C2,CL)
      if(indice.eq.4) call QCD_2006(ZT,Q2T,IFIT,XPQ,F2,FL,C2,CL)
      
      
      return
      end
      
      
      SUBROUTINE QCD_LS(ZT,Q2T,XPQ,IFIT)
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      INTEGER NQ2,NZ,IFIT
      real*8 zt,q2t
      real*8 uds11,uds12,uds21,uds22
      real*8 c11,c12,c21,c22
      real*8 g11,g12,g21,g22
      real*8 dlz,dlq2
      integer iz,jq2
      PARAMETER (NQ2MAX=100,NZMAX=100)
      dimension beta_grid(nzmax),q2_grid(nq2max)
      DIMENSION IPAR(3), XPQ(-6:6)
      DATA IPAR/1,4,0/
      DIMENSION UDS_GRID(NZMAX,NQ2MAX),C_GRID(NZMAX,NQ2MAX),
     &          G_GRID(NZMAX,NQ2MAX)
      COMMON /CPARAMB/ BETA_GRID, Q2_GRID,UDS_GRID,C_GRID,G_GRID 
      
      ! gluon variation
      real*8 GLU_NU

      IF(IFIT.EQ.8 ) CALL I_NLOH194EXT_LS
      IF(IFIT.EQ.10) CALL I_NLOH1_LS 
      IF(IFIT.EQ.20) CALL I_NLOZEUS_LS 
      IF(IFIT.EQ.30) CALL I_NLOH1ZEUS_LS 

C Initialise xpq
      DO I=-6,6
        XPQ(I)=0.D0
      ENDDO

      Q2L = 3.3D0   ! Q2min
      Q2U = 35000.D0 ! Q2max   for LHC
c      Q2U = 150000.D0 ! Q2max
      ZL  = 1.0d-3 ! beta_min
      ZU  = 1.D0   ! beta_max

c  Protect if variables outside limits
      IF (Q2T.GT.Q2U) Q2T = Q2U*0.999999
      IF (Q2T.LT.Q2L) Q2T = Q2L*1.000001
      IF (ZT.GT.ZU) ZT = ZU*0.999999
      IF (ZT.LT.ZL) ZT = ZL*1.000001      


c find indices for beta, q2
      iz=0
      do 1511 i=1,nzmax-1
         if(zt.ge.beta_grid(i).and.zt.le.beta_grid(i+1)) 
     +	      iz=i
1511  continue      
c      print *,'iz : ',iz,beta_grid(iz)
      jq2=0
      do 1512 j=1,nq2max-1
         if(q2t.ge.q2_grid(j).and.q2t.le.q2_grid(j+1)) 
     +	      jq2=j
1512  continue      

c      print *,'jq2 : ',jq2,q2_grid(jq2)

C Light Flavour Singlet
      UDS11 = UDS_GRID(iz,jq2)
      UDS12 = UDS_GRID(iz,jq2+1)
      UDS21 = UDS_GRID(iz+1,jq2)
      UDS22 = UDS_GRID(iz+1,jq2+1)
  
c      print *,'uds:',uds11,uds12,uds21,uds22

C Charm
      C11 = C_GRID(iz,jq2)
      C12 = C_GRID(iz,jq2+1)
      C21 = C_GRID(iz+1,jq2)
      C22 = C_GRID(iz+1,jq2+1)

C Gluon
      G11 = G_GRID(iz,jq2)
      G12 = G_GRID(iz,jq2+1)
      G21 = G_GRID(iz+1,jq2)
      G22 = G_GRID(iz+1,jq2+1)
 
c      print *,'g:',g11,g12,g21,g22

      dlz1 = beta_grid(iz)
      dlz2 = beta_grid(iz+1)
      dlq21 = dlog(q2_grid(jq2)*1.D0)
      dlq22 = dlog(q2_grid(jq2+1)*1.D0)
      dlz = zt
      dlq2 = dlog(q2t*1.D0)
 
c      print *,'kin:',dlz1,dlz2,dlq21,dlq22
c      print *,'kin:',dlz,dlq2


C Interpolate to parton distributions to ZT,Q2T
      UDS_T = XYINTER(DLZ1,DLZ2,DLQ21,DLQ22,DLZ,DLQ2,
     &     UDS11,UDS12,UDS21,UDS22)
      C_T = XYINTER(DLZ1,DLZ2,DLQ21,DLQ22,DLZ,DLQ2,
     &     C11,C12,C21,C22)
      G_T = XYINTER(DLZ1,DLZ2,DLQ21,DLQ22,DLZ,DLQ2,
     &     G11,G12,G21,G22)

C Fill XPQ array
c put charm as light quark for now...
      XPQ(-4) = C_T
c      XPQ(-4) = UDS_T/100.D0
      XPQ(-3) = UDS_T
      XPQ(-2) = UDS_T
      XPQ(-1) = UDS_T
      XPQ( 0) = G_T
      XPQ( 1) = UDS_T
      XPQ( 2) = UDS_T
      XPQ( 3) = UDS_T
      XPQ( 4) = C_T
c      XPQ( 4) = UDS_T/100.D0


c      print *,'xpq:',xpq(-1),xpq(0)

c CHR modify high x gluon
c      GLU_NU=0.0d0
c      IF(IFIT.NE.0) print *,' Gluon variation, normally should be =0 ',
c     . GLU_NU
c      XPQ( 0) = xpq(0) * (1.D0-zt)**(GLU_NU)


C Check output
      DO I=-6,6
        IF(XPQ(I).LT.-0.1D-10) THEN
          WRITE(6,*)'Error in H1 LS fit parameterisation',XPQ(I)
          STOP
        ENDIF
      ENDDO

      RETURN
      END
      
c------------------------------------------------------------------ 

      SUBROUTINE I_NLOH194EXT_LS

      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      PARAMETER (NQ2MAX=100,NZMAX=100)
      DIMENSION UDS_GRID(NZMAX,NQ2MAX),C_GRID(NZMAX,NQ2MAX),
     &          G_GRID(NZMAX,NQ2MAX)
      dimension beta_grid(nzmax),q2_grid(nq2max)
      DIMENSION UDS_GRDL(NZMAX,NQ2MAX),C_GRDL(NZMAX,NQ2MAX),
     &          G_GRDL(NZMAX,NQ2MAX)
      COMMON /CPARAMB/ BETA_GRID, Q2_GRID,UDS_GRID,C_GRID,G_GRID 
      INTEGER LUNDAT             ! input channel for reading CMSJET param
      Character *80 CMSJDAT      ! Name of the file
      DATA    LUNDAT /85/, CMSJDAT/'./Tables/pdftable_h194ext.txt'/


      OPEN(LUNDAT,FILE=CMSJDAT,STATUS='OLD')
     

c read input tables and put numbers in grid
      do iq2_test = 1,nq2max
      do iz_test  = 1,nzmax

      read(LUNDAT,*) beta_grid(iz_test),q2_grid(iq2_test),
     + uds_grid(iz_test,iq2_test), 
     + g_grid(iz_test,iq2_test),
     + c_grid(iz_test,iq2_test)  

       uds_grid(iz_test,iq2_test)=uds_grid(iz_test,iq2_test)/6.

c      write(91,1004) beta_grid(iz_test),q2_grid(iq2_test),
c     + uds_grid(iz_test,iq2_test),
c     + g_grid(iz_test,iq2_test),
c     + c_grid(iz_test,iq2_test)

      enddo
      enddo

 1004 format(2(F14.7,1X),3(F10.5,1X))

      RETURN
      END

c------------------------------------------------------------------ 

      SUBROUTINE I_NLOH1_LS

      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      PARAMETER (NQ2MAX=100,NZMAX=100)
      DIMENSION UDS_GRID(NZMAX,NQ2MAX),C_GRID(NZMAX,NQ2MAX),
     &          G_GRID(NZMAX,NQ2MAX)
      dimension beta_grid(nzmax),q2_grid(nq2max)
      DIMENSION UDS_GRDL(NZMAX,NQ2MAX),C_GRDL(NZMAX,NQ2MAX),
     &          G_GRDL(NZMAX,NQ2MAX)
      COMMON /CPARAMB/ BETA_GRID, Q2_GRID,UDS_GRID,C_GRID,G_GRID 
      INTEGER LUNDAT             ! input channel for reading CMSJET param
      Character *80 CMSJDAT      ! Name of the file
      DATA    LUNDAT /85/, CMSJDAT/'./Tables/pdftable_h197.txt'/
c      DATA    LUNDAT /85/, CMSJDAT/'table_h194.txt'/


      OPEN(LUNDAT,FILE=CMSJDAT,STATUS='OLD')
     

c read input tables and put numbers in grid
      do iq2_test = 1,nq2max
      do iz_test  = 1,nzmax

      read(LUNDAT,*) beta_grid(iz_test),q2_grid(iq2_test),
     + uds_grid(iz_test,iq2_test), 
     + g_grid(iz_test,iq2_test),
     + c_grid(iz_test,iq2_test)  

       uds_grid(iz_test,iq2_test)=uds_grid(iz_test,iq2_test)/6.

c      write(91,1004) beta_grid(iz_test),q2_grid(iq2_test),
c     + uds_grid(iz_test,iq2_test),
c     + g_grid(iz_test,iq2_test),
c     + c_grid(iz_test,iq2_test)

      enddo
      enddo

 1004 format(2(F14.7,1X),3(F10.5,1X))

      RETURN
      END


c___________________________________________________________________ 
      SUBROUTINE I_NLOH1ZEUS_LS

      
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      PARAMETER (NQ2MAX=100,NZMAX=100)
      DIMENSION UDS_GRID(NZMAX,NQ2MAX),C_GRID(NZMAX,NQ2MAX),
     &          G_GRID(NZMAX,NQ2MAX)
      dimension beta_grid(nzmax),q2_grid(nq2max)
      DIMENSION UDS_GRDL(NZMAX,NQ2MAX),C_GRDL(NZMAX,NQ2MAX),
     &          G_GRDL(NZMAX,NQ2MAX)
      COMMON /CPARAMB/ BETA_GRID, Q2_GRID,UDS_GRID,C_GRID,G_GRID 
      INTEGER LUNDAT             ! input channel for reading CMSJET param
      Character *80 CMSJDAT      ! Name of the file
      DATA    LUNDAT /85/, CMSJDAT/'./Tables/pdftable_h1zeus.txt'/
c      DATA    LUNDAT /85/, CMSJDAT/'table_h194.txt'/


      OPEN(LUNDAT,FILE=CMSJDAT,STATUS='OLD')
     

c read input tables and put numbers in grid
      do iq2_test = 1,nq2max
      do iz_test  = 1,nzmax

      read(LUNDAT,*) beta_grid(iz_test),q2_grid(iq2_test),
     + uds_grid(iz_test,iq2_test), 
     + g_grid(iz_test,iq2_test),
     + c_grid(iz_test,iq2_test)  

       uds_grid(iz_test,iq2_test)=uds_grid(iz_test,iq2_test)/6.

c      write(91,1004) beta_grid(iz_test),q2_grid(iq2_test),
c     + uds_grid(iz_test,iq2_test),
c     + g_grid(iz_test,iq2_test),
c     + c_grid(iz_test,iq2_test)

      enddo
      enddo

 1004 format(2(F14.7,1X),3(F10.5,1X))

      RETURN
      END

c___________________________________________________________________ 
      SUBROUTINE I_NLOZEUS_LS


      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      PARAMETER (NQ2MAX=100,NZMAX=100)
      DIMENSION UDS_GRID(NZMAX,NQ2MAX),C_GRID(NZMAX,NQ2MAX),
     &          G_GRID(NZMAX,NQ2MAX)
      dimension beta_grid(nzmax),q2_grid(nq2max)
      DIMENSION UDS_GRDL(NZMAX,NQ2MAX),C_GRDL(NZMAX,NQ2MAX),
     &          G_GRDL(NZMAX,NQ2MAX)
      COMMON /CPARAMB/ BETA_GRID, Q2_GRID,UDS_GRID,C_GRID,G_GRID
      INTEGER LUNDAT             ! input channel for reading CMSJET param
      Character *80 CMSJDAT      ! Name of the file
      DATA    LUNDAT /85/, CMSJDAT/'./Tables/pdftable_zeus.txt'/

      OPEN(LUNDAT,FILE=CMSJDAT,STATUS='OLD')


c read input tables and put numbers in grid
      do iq2_test = 1,nq2max
      do iz_test  = 1,nzmax

      read(LUNDAT,*) beta_grid(iz_test),q2_grid(iq2_test),
     + uds_grid(iz_test,iq2_test),
     + g_grid(iz_test,iq2_test),
     + c_grid(iz_test,iq2_test)

       uds_grid(iz_test,iq2_test)=uds_grid(iz_test,iq2_test)/6.

c      write(91,1004) beta_grid(iz_test),q2_grid(iq2_test),
c     + uds_grid(iz_test,iq2_test),
c     + g_grid(iz_test,iq2_test),
c     + c_grid(iz_test,iq2_test)

      enddo
      enddo

 1004 format(2(F14.7,1X),3(F10.5,1X))

      RETURN
      END

c------------------------------------------------------------------- 

      SUBROUTINE QCD_1994(ZT,Q2T,XPQ,IFIT)
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      INTEGER IQ2,IZ,NQ2,NZ,IFIT
      PARAMETER (NQ2MAX=30,NZMAX=50)
      DIMENSION IPAR(3), XPQ(-6:6)
      DATA IPAR/1,4,0/
      DIMENSION UDS_GRD(NZMAX,NQ2MAX),C_GRD(NZMAX,NQ2MAX),
     &          G_GRD(NZMAX,NQ2MAX)
      COMMON /CPARAM/ UDS_GRD,C_GRD,G_GRD

      ! gluon variation
      real*8 GLU_NU


C Initialise Parameterisation from data statements
C Indice is an internal variable: indice=1 old H1 fits
C                                 indice=2  new fits from Laurent

      IF(IFIT.GT.0) THEN
        IF(IFIT.EQ.1) CALL I_NLO_Q3G0
        IF(IFIT.EQ.2) CALL I_NLO_Q3G1
        IF(IFIT.EQ.3) CALL I_NLO_Q3G3
        IF(IFIT.EQ.4) CALL I_LO_Q3G0
        IF(IFIT.EQ.5) CALL I_LO_Q3G1
        IF(IFIT.EQ.6) CALL I_LO_Q3G3
      ENDIF

C Initialise xpq
      DO I=-6,6
        XPQ(I)=0.D0
      ENDDO
      
C Define grid
      Q2L = 3.D0   ! Q2min
      Q2U = 100.D0 ! Q2max
      ZL  = 0.01D0 ! beta_min
      ZM  = 0.50D0 ! beta_mid (end of log binning)
      ZU  = 1.D0   ! beta_max
      NQ2 = 30     ! Points in Q2 grid
      NZa = 25     ! Points in log beta grid
      NZ  = 50     ! Total points in grid
      DQ2 = (DLOG(Q2U)-DLOG(Q2L))/DFLOAT(NQ2-1)
      DZa = DLOG(ZM/ZL)/DFLOAT(NZa-1)
      DZb = (ZU-ZM)/DFLOAT(NZ-NZa)

C Check input parameters
C      write(*,*) Q2L,Q2T,ZT,ZU
c CHR 
      IF (Q2T.GT.75.D0) Q2T = 75.D0
cc      IF (Q2T.GT.100.D0) Q2T = 100.D0
      IF (Q2T.LT.Q2L) Q2T = Q2L
C      Q2T=MIN(75.0d0,MAX(Q2L,Q2T))

      IF (ZT.GT.ZU) ZT = ZU
c CHR 
      IF (ZT.LT.0.04D0) ZT = 0.04D0      
cc      IF (ZT.LT.0.01D0) ZT = 0.01D0      
C      ZT =MIN(ZU,MAX(0.04D0,ZT))

C Lower Grid Point
      IQ2 = INT(DLOG(Q2T/Q2L)/DQ2)+1
      IZ  = INT(DLOG(MIN(ZT,ZM)/ZL)/DZa)+1
     &      + INT(MAX(0.D0,ZT-ZM)/DZb)

C Central and boundary z,Q2 points for interpolation
      DLQ2  = DLOG(Q2T)
      DLQ21 = DLOG(Q2L)+DFLOAT(IQ2-1)*DQ2
      DLQ22 = DLOG(Q2L)+DFLOAT(IQ2)*DQ2
      IF(IZ.LT.NZa) THEN
        DLZ  = DLOG(ZT) ! Logarithmic interpolation
        DLZ1 = DLOG(DEXP(DLOG(ZL)+DFLOAT(MIN(IZ,NZa)-1)*DZa)
     &              + DFLOAT(MAX(IZ-NZa,0))*DZb)
        DLZ2 = DLOG(DEXP(DLOG(ZL)+DFLOAT(MIN(IZ+1,NZa)-1)*DZa)
     &              + DFLOAT(MAX(IZ+1-NZa,0))*DZb)
      ELSE
        DLZ  = ZT ! Linear interpolation
        DLZ1 = DEXP(DLOG(ZL)+DFLOAT(MIN(IZ,NZa)-1)*DZa)
     &              + DFLOAT(MAX(IZ-NZa,0))*DZb
        DLZ2 = DEXP(DLOG(ZL)+DFLOAT(MIN(IZ+1,NZa)-1)*DZa)
     &              + DFLOAT(MAX(IZ+1-NZa,0))*DZb
      ENDIF

C Light Flavour Singlet
      UDS11 = UDS_GRD(IZ,IQ2)
      UDS12 = UDS_GRD(IZ,IQ2+1)
      UDS21 = UDS_GRD(IZ+1,IQ2)
      UDS22 = UDS_GRD(IZ+1,IQ2+1)

C Charm
      C11 = C_GRD(IZ,IQ2)
      C12 = C_GRD(IZ,IQ2+1)
      C21 = C_GRD(IZ+1,IQ2)
      C22 = C_GRD(IZ+1,IQ2+1)

C Gluon
      G11 = G_GRD(IZ,IQ2)
      G12 = G_GRD(IZ,IQ2+1)
      G21 = G_GRD(IZ+1,IQ2)
      G22 = G_GRD(IZ+1,IQ2+1)

C Interpolate to parton distributions to ZT,Q2T
      UDS_T = XYINTER(DLZ1,DLZ2,DLQ21,DLQ22,DLZ,DLQ2,
     &     UDS11,UDS12,UDS21,UDS22)
      C_T = XYINTER(DLZ1,DLZ2,DLQ21,DLQ22,DLZ,DLQ2,
     &     C11,C12,C21,C22)
      G_T = XYINTER(DLZ1,DLZ2,DLQ21,DLQ22,DLZ,DLQ2,
     &     G11,G12,G21,G22)

C Fill XPQ array
C OK: 13/01/07 to be compatible with version of DPEMC 2.7,
C we set the c-quark to 0 by default. It used to be set to 0 at
C fpmc.f
      
c      XPQ(-4) = C_T
      XPQ(-4) = 0d0
      XPQ(-3) = UDS_T
      XPQ(-2) = UDS_T
      XPQ(-1) = UDS_T
      XPQ( 0) = G_T
      XPQ( 1) = UDS_T
      XPQ( 2) = UDS_T
      XPQ( 3) = UDS_T
      XPQ( 4) = 0d0
c      XPQ( 4) = C_T

c      print '(A, 3F8.2)', 'h1qcd', XPQ(-1), XPQ( 0), XPQ( 1)

c CHR modify high x gluon

      GLU_NU=0.0d0
      IF(IFIT.NE.0) print *,' Gluon variation, normally should be =0 ', 
     . GLU_NU
      XPQ( 0) = xpq(0) * (1.D0-zt)**(GLU_NU)


C Check output
      DO I=-6,6
        IF(XPQ(I).LT.-0.1D-10) THEN
          WRITE(6,*)'Error in H1 fit parameterisation',XPQ(I)
          STOP
        ENDIF
      ENDDO

      RETURN
      END

C========================================================================
C 2D Interpolation to point within square (path dependent but simple)
C========================================================================
      DOUBLE PRECISION FUNCTION
     &  XYINTER(X1,X2,Y1,Y2,XT,YT,F11,F12,F21,F22)
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)

      XEXP = (XT-X1)/(X2-X1)
      YEXP = (YT-Y1)/(Y2-Y1)

      XYINTER = F11 + XEXP*(F21-F11)
     &          + YEXP*(F12 + XEXP*(F22-F12) - (F11 + XEXP*(F21-F11)))
      RETURN
      END

      SUBROUTINE I_NLO_Q3G0
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      PARAMETER (NQ2MAX=30,NZMAX=50)
      DIMENSION UDS_GRD(NZMAX,NQ2MAX),C_GRD(NZMAX,NQ2MAX),
     &          G_GRD(NZMAX,NQ2MAX)
      DIMENSION UDS_GRDL(NZMAX,NQ2MAX),C_GRDL(NZMAX,NQ2MAX),
     &          G_GRDL(NZMAX,NQ2MAX)
      COMMON /CPARAM/ UDS_GRD,C_GRD,G_GRD 

      data ((uds_grdl(i,j),j=1,30),i=1,10)/
     & 0.11592D-01, 0.11667D-01, 0.11748D-01, 0.11835D-01, 0.11926D-01,
     & 0.12021D-01, 0.12120D-01, 0.12223D-01, 0.12330D-01, 0.12439D-01,
     & 0.12552D-01, 0.12668D-01, 0.12786D-01, 0.12907D-01, 0.13030D-01,
     & 0.13155D-01, 0.13282D-01, 0.13411D-01, 0.13541D-01, 0.13674D-01,
     & 0.13807D-01, 0.13942D-01, 0.14079D-01, 0.14217D-01, 0.14356D-01,
     & 0.14495D-01, 0.14636D-01, 0.14778D-01, 0.14920D-01, 0.15063D-01,
     & 0.11649D-01, 0.11722D-01, 0.11801D-01, 0.11884D-01, 0.11972D-01,
     & 0.12064D-01, 0.12159D-01, 0.12258D-01, 0.12360D-01, 0.12465D-01,
     & 0.12573D-01, 0.12683D-01, 0.12795D-01, 0.12910D-01, 0.13027D-01,
     & 0.13146D-01, 0.13266D-01, 0.13388D-01, 0.13512D-01, 0.13637D-01,
     & 0.13763D-01, 0.13891D-01, 0.14019D-01, 0.14149D-01, 0.14280D-01,
     & 0.14411D-01, 0.14543D-01, 0.14676D-01, 0.14810D-01, 0.14944D-01,
     & 0.11717D-01, 0.11788D-01, 0.11864D-01, 0.11945D-01, 0.12030D-01,
     & 0.12118D-01, 0.12209D-01, 0.12304D-01, 0.12402D-01, 0.12502D-01,
     & 0.12605D-01, 0.12710D-01, 0.12817D-01, 0.12926D-01, 0.13037D-01,
     & 0.13149D-01, 0.13264D-01, 0.13379D-01, 0.13496D-01, 0.13614D-01,
     & 0.13733D-01, 0.13853D-01, 0.13975D-01, 0.14097D-01, 0.14219D-01,
     & 0.14343D-01, 0.14467D-01, 0.14592D-01, 0.14717D-01, 0.14843D-01,
     & 0.11798D-01, 0.11868D-01, 0.11942D-01, 0.12020D-01, 0.12101D-01,
     & 0.12186D-01, 0.12274D-01, 0.12365D-01, 0.12459D-01, 0.12555D-01,
     & 0.12653D-01, 0.12753D-01, 0.12855D-01, 0.12959D-01, 0.13064D-01,
     & 0.13171D-01, 0.13279D-01, 0.13388D-01, 0.13499D-01, 0.13610D-01,
     & 0.13722D-01, 0.13836D-01, 0.13950D-01, 0.14064D-01, 0.14180D-01,
     & 0.14296D-01, 0.14412D-01, 0.14529D-01, 0.14646D-01, 0.14764D-01,
     & 0.11892D-01, 0.11960D-01, 0.12032D-01, 0.12108D-01, 0.12187D-01,
     & 0.12269D-01, 0.12353D-01, 0.12441D-01, 0.12530D-01, 0.12622D-01,
     & 0.12716D-01, 0.12811D-01, 0.12908D-01, 0.13007D-01, 0.13107D-01,
     & 0.13208D-01, 0.13310D-01, 0.13413D-01, 0.13518D-01, 0.13623D-01,
     & 0.13729D-01, 0.13836D-01, 0.13943D-01, 0.14051D-01, 0.14159D-01,
     & 0.14268D-01, 0.14377D-01, 0.14486D-01, 0.14596D-01, 0.14706D-01,
     & 0.12003D-01, 0.12069D-01, 0.12139D-01, 0.12212D-01, 0.12289D-01,
     & 0.12368D-01, 0.12449D-01, 0.12533D-01, 0.12619D-01, 0.12706D-01,
     & 0.12796D-01, 0.12887D-01, 0.12979D-01, 0.13073D-01, 0.13168D-01,
     & 0.13264D-01, 0.13360D-01, 0.13458D-01, 0.13557D-01, 0.13656D-01,
     & 0.13755D-01, 0.13856D-01, 0.13956D-01, 0.14058D-01, 0.14159D-01,
     & 0.14261D-01, 0.14363D-01, 0.14465D-01, 0.14568D-01, 0.14670D-01,
     & 0.12132D-01, 0.12197D-01, 0.12265D-01, 0.12336D-01, 0.12410D-01,
     & 0.12487D-01, 0.12565D-01, 0.12646D-01, 0.12728D-01, 0.12812D-01,
     & 0.12897D-01, 0.12984D-01, 0.13072D-01, 0.13161D-01, 0.13251D-01,
     & 0.13342D-01, 0.13433D-01, 0.13525D-01, 0.13618D-01, 0.13712D-01,
     & 0.13806D-01, 0.13900D-01, 0.13994D-01, 0.14089D-01, 0.14184D-01,
     & 0.14280D-01, 0.14375D-01, 0.14471D-01, 0.14566D-01, 0.14662D-01,
     & 0.12285D-01, 0.12348D-01, 0.12415D-01, 0.12484D-01, 0.12556D-01,
     & 0.12630D-01, 0.12706D-01, 0.12783D-01, 0.12862D-01, 0.12942D-01,
     & 0.13024D-01, 0.13106D-01, 0.13190D-01, 0.13275D-01, 0.13360D-01,
     & 0.13446D-01, 0.13532D-01, 0.13619D-01, 0.13707D-01, 0.13795D-01,
     & 0.13883D-01, 0.13972D-01, 0.14060D-01, 0.14149D-01, 0.14238D-01,
     & 0.14327D-01, 0.14416D-01, 0.14505D-01, 0.14594D-01, 0.14683D-01,
     & 0.12466D-01, 0.12529D-01, 0.12594D-01, 0.12661D-01, 0.12731D-01,
     & 0.12802D-01, 0.12876D-01, 0.12950D-01, 0.13026D-01, 0.13103D-01,
     & 0.13181D-01, 0.13260D-01, 0.13339D-01, 0.13420D-01, 0.13500D-01,
     & 0.13582D-01, 0.13663D-01, 0.13746D-01, 0.13828D-01, 0.13911D-01,
     & 0.13994D-01, 0.14077D-01, 0.14160D-01, 0.14243D-01, 0.14326D-01,
     & 0.14409D-01, 0.14492D-01, 0.14575D-01, 0.14658D-01, 0.14741D-01,
     & 0.12677D-01, 0.12739D-01, 0.12803D-01, 0.12869D-01, 0.12937D-01,
     & 0.13006D-01, 0.13077D-01, 0.13148D-01, 0.13221D-01, 0.13295D-01,
     & 0.13369D-01, 0.13444D-01, 0.13520D-01, 0.13597D-01, 0.13673D-01,
     & 0.13750D-01, 0.13827D-01, 0.13905D-01, 0.13982D-01, 0.14060D-01,
     & 0.14138D-01, 0.14216D-01, 0.14293D-01, 0.14371D-01, 0.14448D-01,
     & 0.14526D-01, 0.14603D-01, 0.14680D-01, 0.14757D-01, 0.14834D-01/

      data ((uds_grdl(i,j),j=1,30),i=11,20)/
     & 0.12924D-01, 0.12985D-01, 0.13048D-01, 0.13113D-01, 0.13179D-01,
     & 0.13246D-01, 0.13314D-01, 0.13383D-01, 0.13453D-01, 0.13524D-01,
     & 0.13594D-01, 0.13666D-01, 0.13738D-01, 0.13811D-01, 0.13883D-01,
     & 0.13956D-01, 0.14028D-01, 0.14101D-01, 0.14174D-01, 0.14247D-01,
     & 0.14320D-01, 0.14393D-01, 0.14465D-01, 0.14537D-01, 0.14609D-01,
     & 0.14681D-01, 0.14753D-01, 0.14824D-01, 0.14896D-01, 0.14966D-01,
     & 0.13216D-01, 0.13277D-01, 0.13339D-01, 0.13402D-01, 0.13466D-01,
     & 0.13531D-01, 0.13597D-01, 0.13663D-01, 0.13731D-01, 0.13798D-01,
     & 0.13866D-01, 0.13934D-01, 0.14002D-01, 0.14071D-01, 0.14139D-01,
     & 0.14208D-01, 0.14276D-01, 0.14344D-01, 0.14413D-01, 0.14481D-01,
     & 0.14549D-01, 0.14617D-01, 0.14684D-01, 0.14751D-01, 0.14818D-01,
     & 0.14885D-01, 0.14951D-01, 0.15017D-01, 0.15083D-01, 0.15148D-01,
     & 0.13558D-01, 0.13618D-01, 0.13678D-01, 0.13740D-01, 0.13802D-01,
     & 0.13865D-01, 0.13929D-01, 0.13993D-01, 0.14057D-01, 0.14121D-01,
     & 0.14186D-01, 0.14250D-01, 0.14315D-01, 0.14380D-01, 0.14444D-01,
     & 0.14508D-01, 0.14572D-01, 0.14636D-01, 0.14700D-01, 0.14764D-01,
     & 0.14827D-01, 0.14889D-01, 0.14952D-01, 0.15014D-01, 0.15075D-01,
     & 0.15137D-01, 0.15198D-01, 0.15258D-01, 0.15318D-01, 0.15378D-01,
     & 0.13962D-01, 0.14020D-01, 0.14080D-01, 0.14140D-01, 0.14200D-01,
     & 0.14261D-01, 0.14322D-01, 0.14383D-01, 0.14444D-01, 0.14505D-01,
     & 0.14566D-01, 0.14627D-01, 0.14688D-01, 0.14749D-01, 0.14809D-01,
     & 0.14869D-01, 0.14929D-01, 0.14988D-01, 0.15047D-01, 0.15106D-01,
     & 0.15164D-01, 0.15222D-01, 0.15279D-01, 0.15336D-01, 0.15392D-01,
     & 0.15448D-01, 0.15504D-01, 0.15559D-01, 0.15614D-01, 0.15668D-01,
     & 0.14437D-01, 0.14495D-01, 0.14553D-01, 0.14611D-01, 0.14669D-01,
     & 0.14727D-01, 0.14785D-01, 0.14843D-01, 0.14901D-01, 0.14958D-01,
     & 0.15016D-01, 0.15073D-01, 0.15130D-01, 0.15186D-01, 0.15242D-01,
     & 0.15297D-01, 0.15352D-01, 0.15407D-01, 0.15461D-01, 0.15515D-01,
     & 0.15568D-01, 0.15621D-01, 0.15673D-01, 0.15724D-01, 0.15775D-01,
     & 0.15826D-01, 0.15876D-01, 0.15925D-01, 0.15974D-01, 0.16022D-01,
     & 0.14985D-01, 0.15041D-01, 0.15096D-01, 0.15152D-01, 0.15207D-01,
     & 0.15262D-01, 0.15317D-01, 0.15371D-01, 0.15425D-01, 0.15478D-01,
     & 0.15531D-01, 0.15584D-01, 0.15636D-01, 0.15688D-01, 0.15739D-01,
     & 0.15789D-01, 0.15839D-01, 0.15888D-01, 0.15937D-01, 0.15985D-01,
     & 0.16033D-01, 0.16080D-01, 0.16126D-01, 0.16171D-01, 0.16217D-01,
     & 0.16261D-01, 0.16305D-01, 0.16349D-01, 0.16392D-01, 0.16434D-01,
     & 0.15625D-01, 0.15678D-01, 0.15730D-01, 0.15782D-01, 0.15833D-01,
     & 0.15884D-01, 0.15934D-01, 0.15984D-01, 0.16033D-01, 0.16082D-01,
     & 0.16129D-01, 0.16177D-01, 0.16223D-01, 0.16269D-01, 0.16315D-01,
     & 0.16359D-01, 0.16403D-01, 0.16446D-01, 0.16489D-01, 0.16531D-01,
     & 0.16572D-01, 0.16613D-01, 0.16653D-01, 0.16692D-01, 0.16731D-01,
     & 0.16769D-01, 0.16806D-01, 0.16843D-01, 0.16880D-01, 0.16915D-01,
     & 0.16366D-01, 0.16414D-01, 0.16462D-01, 0.16508D-01, 0.16554D-01,
     & 0.16599D-01, 0.16644D-01, 0.16688D-01, 0.16731D-01, 0.16773D-01,
     & 0.16814D-01, 0.16855D-01, 0.16895D-01, 0.16934D-01, 0.16973D-01,
     & 0.17010D-01, 0.17047D-01, 0.17083D-01, 0.17119D-01, 0.17154D-01,
     & 0.17188D-01, 0.17221D-01, 0.17253D-01, 0.17285D-01, 0.17316D-01,
     & 0.17347D-01, 0.17377D-01, 0.17407D-01, 0.17435D-01, 0.17463D-01,
     & 0.17218D-01, 0.17260D-01, 0.17300D-01, 0.17340D-01, 0.17379D-01,
     & 0.17416D-01, 0.17453D-01, 0.17489D-01, 0.17524D-01, 0.17558D-01,
     & 0.17591D-01, 0.17624D-01, 0.17655D-01, 0.17686D-01, 0.17716D-01,
     & 0.17744D-01, 0.17773D-01, 0.17800D-01, 0.17827D-01, 0.17853D-01,
     & 0.17878D-01, 0.17902D-01, 0.17926D-01, 0.17949D-01, 0.17971D-01,
     & 0.17993D-01, 0.18015D-01, 0.18035D-01, 0.18055D-01, 0.18075D-01,
     & 0.18189D-01, 0.18221D-01, 0.18252D-01, 0.18282D-01, 0.18310D-01,
     & 0.18337D-01, 0.18364D-01, 0.18389D-01, 0.18413D-01, 0.18436D-01,
     & 0.18459D-01, 0.18480D-01, 0.18501D-01, 0.18521D-01, 0.18539D-01,
     & 0.18557D-01, 0.18575D-01, 0.18591D-01, 0.18607D-01, 0.18622D-01,
     & 0.18636D-01, 0.18650D-01, 0.18663D-01, 0.18675D-01, 0.18687D-01,
     & 0.18698D-01, 0.18709D-01, 0.18719D-01, 0.18728D-01, 0.18737D-01/

      data ((uds_grdl(i,j),j=1,30),i=21,30)/
     & 0.19281D-01, 0.19299D-01, 0.19316D-01, 0.19331D-01, 0.19345D-01,
     & 0.19358D-01, 0.19370D-01, 0.19381D-01, 0.19391D-01, 0.19400D-01,
     & 0.19408D-01, 0.19415D-01, 0.19421D-01, 0.19427D-01, 0.19432D-01,
     & 0.19436D-01, 0.19439D-01, 0.19442D-01, 0.19444D-01, 0.19445D-01,
     & 0.19446D-01, 0.19446D-01, 0.19446D-01, 0.19445D-01, 0.19443D-01,
     & 0.19441D-01, 0.19439D-01, 0.19436D-01, 0.19433D-01, 0.19429D-01,
     & 0.20475D-01, 0.20473D-01, 0.20469D-01, 0.20464D-01, 0.20458D-01,
     & 0.20452D-01, 0.20444D-01, 0.20435D-01, 0.20426D-01, 0.20416D-01,
     & 0.20405D-01, 0.20393D-01, 0.20381D-01, 0.20368D-01, 0.20354D-01,
     & 0.20340D-01, 0.20325D-01, 0.20310D-01, 0.20295D-01, 0.20278D-01,
     & 0.20262D-01, 0.20245D-01, 0.20228D-01, 0.20210D-01, 0.20192D-01,
     & 0.20173D-01, 0.20155D-01, 0.20136D-01, 0.20117D-01, 0.20097D-01,
     & 0.21760D-01, 0.21729D-01, 0.21697D-01, 0.21665D-01, 0.21632D-01,
     & 0.21598D-01, 0.21564D-01, 0.21529D-01, 0.21494D-01, 0.21458D-01,
     & 0.21422D-01, 0.21385D-01, 0.21348D-01, 0.21311D-01, 0.21274D-01,
     & 0.21236D-01, 0.21199D-01, 0.21161D-01, 0.21122D-01, 0.21084D-01,
     & 0.21046D-01, 0.21007D-01, 0.20969D-01, 0.20930D-01, 0.20891D-01,
     & 0.20852D-01, 0.20814D-01, 0.20775D-01, 0.20736D-01, 0.20697D-01,
     & 0.23077D-01, 0.23007D-01, 0.22936D-01, 0.22866D-01, 0.22796D-01,
     & 0.22726D-01, 0.22657D-01, 0.22587D-01, 0.22517D-01, 0.22448D-01,
     & 0.22379D-01, 0.22310D-01, 0.22242D-01, 0.22173D-01, 0.22105D-01,
     & 0.22038D-01, 0.21970D-01, 0.21903D-01, 0.21837D-01, 0.21770D-01,
     & 0.21704D-01, 0.21639D-01, 0.21574D-01, 0.21509D-01, 0.21445D-01,
     & 0.21380D-01, 0.21317D-01, 0.21253D-01, 0.21190D-01, 0.21128D-01,
     & 0.24302D-01, 0.24179D-01, 0.24056D-01, 0.23935D-01, 0.23815D-01,
     & 0.23697D-01, 0.23580D-01, 0.23464D-01, 0.23349D-01, 0.23236D-01,
     & 0.23124D-01, 0.23013D-01, 0.22904D-01, 0.22795D-01, 0.22688D-01,
     & 0.22582D-01, 0.22477D-01, 0.22374D-01, 0.22271D-01, 0.22169D-01,
     & 0.22069D-01, 0.21970D-01, 0.21871D-01, 0.21774D-01, 0.21678D-01,
     & 0.21583D-01, 0.21489D-01, 0.21396D-01, 0.21303D-01, 0.21212D-01,
     & 0.24567D-01, 0.24427D-01, 0.24290D-01, 0.24154D-01, 0.24020D-01,
     & 0.23888D-01, 0.23757D-01, 0.23628D-01, 0.23501D-01, 0.23375D-01,
     & 0.23251D-01, 0.23129D-01, 0.23007D-01, 0.22888D-01, 0.22770D-01,
     & 0.22653D-01, 0.22538D-01, 0.22424D-01, 0.22311D-01, 0.22200D-01,
     & 0.22090D-01, 0.21982D-01, 0.21875D-01, 0.21769D-01, 0.21664D-01,
     & 0.21560D-01, 0.21458D-01, 0.21357D-01, 0.21257D-01, 0.21158D-01,
     & 0.24804D-01, 0.24648D-01, 0.24495D-01, 0.24344D-01, 0.24196D-01,
     & 0.24049D-01, 0.23905D-01, 0.23762D-01, 0.23622D-01, 0.23483D-01,
     & 0.23347D-01, 0.23212D-01, 0.23079D-01, 0.22948D-01, 0.22819D-01,
     & 0.22692D-01, 0.22566D-01, 0.22442D-01, 0.22319D-01, 0.22198D-01,
     & 0.22079D-01, 0.21961D-01, 0.21845D-01, 0.21730D-01, 0.21617D-01,
     & 0.21505D-01, 0.21394D-01, 0.21285D-01, 0.21177D-01, 0.21070D-01,
     & 0.25013D-01, 0.24841D-01, 0.24672D-01, 0.24505D-01, 0.24342D-01,
     & 0.24181D-01, 0.24022D-01, 0.23865D-01, 0.23712D-01, 0.23560D-01,
     & 0.23411D-01, 0.23264D-01, 0.23119D-01, 0.22977D-01, 0.22836D-01,
     & 0.22698D-01, 0.22561D-01, 0.22427D-01, 0.22294D-01, 0.22163D-01,
     & 0.22034D-01, 0.21907D-01, 0.21782D-01, 0.21658D-01, 0.21536D-01,
     & 0.21415D-01, 0.21296D-01, 0.21179D-01, 0.21063D-01, 0.20949D-01,
     & 0.25194D-01, 0.25004D-01, 0.24819D-01, 0.24636D-01, 0.24457D-01,
     & 0.24281D-01, 0.24107D-01, 0.23937D-01, 0.23769D-01, 0.23605D-01,
     & 0.23443D-01, 0.23283D-01, 0.23126D-01, 0.22972D-01, 0.22820D-01,
     & 0.22670D-01, 0.22523D-01, 0.22378D-01, 0.22235D-01, 0.22094D-01,
     & 0.21956D-01, 0.21819D-01, 0.21685D-01, 0.21552D-01, 0.21421D-01,
     & 0.21292D-01, 0.21165D-01, 0.21040D-01, 0.20916D-01, 0.20794D-01,
     & 0.25345D-01, 0.25138D-01, 0.24935D-01, 0.24736D-01, 0.24541D-01,
     & 0.24349D-01, 0.24161D-01, 0.23976D-01, 0.23794D-01, 0.23616D-01,
     & 0.23441D-01, 0.23269D-01, 0.23100D-01, 0.22933D-01, 0.22770D-01,
     & 0.22609D-01, 0.22451D-01, 0.22295D-01, 0.22142D-01, 0.21991D-01,
     & 0.21843D-01, 0.21697D-01, 0.21553D-01, 0.21411D-01, 0.21272D-01,
     & 0.21134D-01, 0.20999D-01, 0.20865D-01, 0.20734D-01, 0.20605D-01/

      data ((uds_grdl(i,j),j=1,30),i=31,40)/
     & 0.25467D-01, 0.25242D-01, 0.25021D-01, 0.24805D-01, 0.24593D-01,
     & 0.24385D-01, 0.24182D-01, 0.23982D-01, 0.23786D-01, 0.23594D-01,
     & 0.23406D-01, 0.23220D-01, 0.23038D-01, 0.22860D-01, 0.22685D-01,
     & 0.22512D-01, 0.22343D-01, 0.22177D-01, 0.22013D-01, 0.21852D-01,
     & 0.21694D-01, 0.21539D-01, 0.21386D-01, 0.21235D-01, 0.21087D-01,
     & 0.20941D-01, 0.20797D-01, 0.20656D-01, 0.20517D-01, 0.20380D-01,
     & 0.25558D-01, 0.25314D-01, 0.25074D-01, 0.24841D-01, 0.24612D-01,
     & 0.24388D-01, 0.24169D-01, 0.23954D-01, 0.23743D-01, 0.23537D-01,
     & 0.23335D-01, 0.23137D-01, 0.22942D-01, 0.22751D-01, 0.22564D-01,
     & 0.22380D-01, 0.22200D-01, 0.22023D-01, 0.21848D-01, 0.21677D-01,
     & 0.21509D-01, 0.21344D-01, 0.21182D-01, 0.21023D-01, 0.20866D-01,
     & 0.20712D-01, 0.20560D-01, 0.20410D-01, 0.20263D-01, 0.20119D-01,
     & 0.25618D-01, 0.25354D-01, 0.25096D-01, 0.24844D-01, 0.24598D-01,
     & 0.24357D-01, 0.24121D-01, 0.23891D-01, 0.23665D-01, 0.23445D-01,
     & 0.23229D-01, 0.23017D-01, 0.22809D-01, 0.22606D-01, 0.22406D-01,
     & 0.22211D-01, 0.22019D-01, 0.21831D-01, 0.21646D-01, 0.21465D-01,
     & 0.21287D-01, 0.21113D-01, 0.20941D-01, 0.20773D-01, 0.20608D-01,
     & 0.20445D-01, 0.20285D-01, 0.20127D-01, 0.19973D-01, 0.19821D-01,
     & 0.25646D-01, 0.25361D-01, 0.25083D-01, 0.24813D-01, 0.24548D-01,
     & 0.24290D-01, 0.24038D-01, 0.23791D-01, 0.23551D-01, 0.23315D-01,
     & 0.23085D-01, 0.22860D-01, 0.22639D-01, 0.22423D-01, 0.22211D-01,
     & 0.22004D-01, 0.21801D-01, 0.21602D-01, 0.21406D-01, 0.21215D-01,
     & 0.21027D-01, 0.20843D-01, 0.20662D-01, 0.20485D-01, 0.20311D-01,
     & 0.20140D-01, 0.19972D-01, 0.19807D-01, 0.19645D-01, 0.19486D-01,
     & 0.25642D-01, 0.25336D-01, 0.25038D-01, 0.24748D-01, 0.24465D-01,
     & 0.24189D-01, 0.23919D-01, 0.23656D-01, 0.23400D-01, 0.23149D-01,
     & 0.22905D-01, 0.22665D-01, 0.22431D-01, 0.22202D-01, 0.21978D-01,
     & 0.21759D-01, 0.21544D-01, 0.21334D-01, 0.21128D-01, 0.20926D-01,
     & 0.20728D-01, 0.20535D-01, 0.20345D-01, 0.20159D-01, 0.19976D-01,
     & 0.19796D-01, 0.19620D-01, 0.19448D-01, 0.19278D-01, 0.19112D-01,
     & 0.25599D-01, 0.25270D-01, 0.24950D-01, 0.24640D-01, 0.24338D-01,
     & 0.24043D-01, 0.23756D-01, 0.23476D-01, 0.23203D-01, 0.22937D-01,
     & 0.22677D-01, 0.22423D-01, 0.22175D-01, 0.21933D-01, 0.21696D-01,
     & 0.21465D-01, 0.21239D-01, 0.21018D-01, 0.20801D-01, 0.20589D-01,
     & 0.20381D-01, 0.20178D-01, 0.19979D-01, 0.19784D-01, 0.19593D-01,
     & 0.19405D-01, 0.19221D-01, 0.19041D-01, 0.18864D-01, 0.18690D-01,
     & 0.25522D-01, 0.25170D-01, 0.24828D-01, 0.24496D-01, 0.24174D-01,
     & 0.23860D-01, 0.23555D-01, 0.23257D-01, 0.22967D-01, 0.22685D-01,
     & 0.22410D-01, 0.22142D-01, 0.21880D-01, 0.21624D-01, 0.21375D-01,
     & 0.21131D-01, 0.20893D-01, 0.20660D-01, 0.20433D-01, 0.20210D-01,
     & 0.19992D-01, 0.19780D-01, 0.19571D-01, 0.19368D-01, 0.19168D-01,
     & 0.18972D-01, 0.18780D-01, 0.18592D-01, 0.18408D-01, 0.18228D-01,
     & 0.25408D-01, 0.25031D-01, 0.24666D-01, 0.24312D-01, 0.23969D-01,
     & 0.23635D-01, 0.23310D-01, 0.22994D-01, 0.22687D-01, 0.22389D-01,
     & 0.22098D-01, 0.21814D-01, 0.21538D-01, 0.21269D-01, 0.21006D-01,
     & 0.20750D-01, 0.20500D-01, 0.20256D-01, 0.20018D-01, 0.19785D-01,
     & 0.19557D-01, 0.19335D-01, 0.19117D-01, 0.18905D-01, 0.18697D-01,
     & 0.18493D-01, 0.18293D-01, 0.18098D-01, 0.17907D-01, 0.17720D-01,
     & 0.25253D-01, 0.24850D-01, 0.24460D-01, 0.24083D-01, 0.23717D-01,
     & 0.23362D-01, 0.23017D-01, 0.22683D-01, 0.22358D-01, 0.22042D-01,
     & 0.21735D-01, 0.21436D-01, 0.21145D-01, 0.20861D-01, 0.20586D-01,
     & 0.20317D-01, 0.20055D-01, 0.19799D-01, 0.19550D-01, 0.19306D-01,
     & 0.19069D-01, 0.18837D-01, 0.18610D-01, 0.18389D-01, 0.18173D-01,
     & 0.17961D-01, 0.17754D-01, 0.17552D-01, 0.17354D-01, 0.17161D-01,
     & 0.25053D-01, 0.24623D-01, 0.24206D-01, 0.23804D-01, 0.23415D-01,
     & 0.23038D-01, 0.22672D-01, 0.22317D-01, 0.21974D-01, 0.21640D-01,
     & 0.21316D-01, 0.21001D-01, 0.20695D-01, 0.20397D-01, 0.20107D-01,
     & 0.19826D-01, 0.19551D-01, 0.19284D-01, 0.19024D-01, 0.18770D-01,
     & 0.18522D-01, 0.18281D-01, 0.18045D-01, 0.17815D-01, 0.17591D-01,
     & 0.17372D-01, 0.17158D-01, 0.16948D-01, 0.16744D-01, 0.16544D-01/

      data ((uds_grdl(i,j),j=1,30),i=41,50)/
     & 0.24804D-01, 0.24343D-01, 0.23899D-01, 0.23470D-01, 0.23055D-01,
     & 0.22655D-01, 0.22267D-01, 0.21891D-01, 0.21528D-01, 0.21175D-01,
     & 0.20834D-01, 0.20502D-01, 0.20181D-01, 0.19868D-01, 0.19565D-01,
     & 0.19270D-01, 0.18983D-01, 0.18704D-01, 0.18433D-01, 0.18168D-01,
     & 0.17911D-01, 0.17660D-01, 0.17416D-01, 0.17178D-01, 0.16946D-01,
     & 0.16719D-01, 0.16498D-01, 0.16282D-01, 0.16072D-01, 0.15866D-01,
     & 0.24498D-01, 0.24005D-01, 0.23530D-01, 0.23072D-01, 0.22631D-01,
     & 0.22205D-01, 0.21793D-01, 0.21395D-01, 0.21011D-01, 0.20639D-01,
     & 0.20279D-01, 0.19931D-01, 0.19593D-01, 0.19265D-01, 0.18947D-01,
     & 0.18639D-01, 0.18339D-01, 0.18049D-01, 0.17766D-01, 0.17491D-01,
     & 0.17224D-01, 0.16964D-01, 0.16711D-01, 0.16465D-01, 0.16225D-01,
     & 0.15992D-01, 0.15764D-01, 0.15542D-01, 0.15326D-01, 0.15114D-01,
     & 0.24123D-01, 0.23593D-01, 0.23085D-01, 0.22595D-01, 0.22125D-01,
     & 0.21671D-01, 0.21235D-01, 0.20813D-01, 0.20407D-01, 0.20015D-01,
     & 0.19635D-01, 0.19269D-01, 0.18914D-01, 0.18571D-01, 0.18239D-01,
     & 0.17917D-01, 0.17605D-01, 0.17302D-01, 0.17009D-01, 0.16724D-01,
     & 0.16447D-01, 0.16179D-01, 0.15917D-01, 0.15664D-01, 0.15417D-01,
     & 0.15177D-01, 0.14943D-01, 0.14715D-01, 0.14494D-01, 0.14278D-01,
     & 0.23659D-01, 0.23089D-01, 0.22543D-01, 0.22020D-01, 0.21517D-01,
     & 0.21034D-01, 0.20570D-01, 0.20123D-01, 0.19693D-01, 0.19279D-01,
     & 0.18879D-01, 0.18494D-01, 0.18122D-01, 0.17763D-01, 0.17415D-01,
     & 0.17080D-01, 0.16755D-01, 0.16440D-01, 0.16136D-01, 0.15841D-01,
     & 0.15556D-01, 0.15279D-01, 0.15010D-01, 0.14749D-01, 0.14495D-01,
     & 0.14250D-01, 0.14011D-01, 0.13779D-01, 0.13553D-01, 0.13333D-01,
     & 0.23061D-01, 0.22446D-01, 0.21858D-01, 0.21296D-01, 0.20757D-01,
     & 0.20242D-01, 0.19748D-01, 0.19274D-01, 0.18819D-01, 0.18382D-01,
     & 0.17961D-01, 0.17557D-01, 0.17168D-01, 0.16793D-01, 0.16431D-01,
     & 0.16082D-01, 0.15745D-01, 0.15420D-01, 0.15106D-01, 0.14803D-01,
     & 0.14509D-01, 0.14225D-01, 0.13950D-01, 0.13683D-01, 0.13425D-01,
     & 0.13176D-01, 0.12934D-01, 0.12698D-01, 0.12470D-01, 0.12249D-01,
     & 0.22282D-01, 0.21611D-01, 0.20972D-01, 0.20364D-01, 0.19785D-01,
     & 0.19233D-01, 0.18708D-01, 0.18206D-01, 0.17726D-01, 0.17265D-01,
     & 0.16824D-01, 0.16402D-01, 0.15998D-01, 0.15609D-01, 0.15235D-01,
     & 0.14875D-01, 0.14529D-01, 0.14196D-01, 0.13875D-01, 0.13566D-01,
     & 0.13268D-01, 0.12980D-01, 0.12701D-01, 0.12432D-01, 0.12172D-01,
     & 0.11922D-01, 0.11679D-01, 0.11444D-01, 0.11216D-01, 0.10995D-01,
     & 0.21251D-01, 0.20541D-01, 0.19866D-01, 0.19224D-01, 0.18612D-01,
     & 0.18030D-01, 0.17474D-01, 0.16944D-01, 0.16438D-01, 0.15953D-01,
     & 0.15490D-01, 0.15048D-01, 0.14624D-01, 0.14218D-01, 0.13829D-01,
     & 0.13455D-01, 0.13096D-01, 0.12752D-01, 0.12422D-01, 0.12105D-01,
     & 0.11799D-01, 0.11504D-01, 0.11221D-01, 0.10948D-01, 0.10684D-01,
     & 0.10432D-01, 0.10188D-01, 0.99523D-02, 0.97246D-02, 0.95045D-02,
     & 0.20194D-01, 0.19290D-01, 0.18444D-01, 0.17649D-01, 0.16904D-01,
     & 0.16206D-01, 0.15559D-01, 0.14948D-01, 0.14371D-01, 0.13826D-01,
     & 0.13310D-01, 0.12827D-01, 0.12372D-01, 0.11940D-01, 0.11529D-01,
     & 0.11137D-01, 0.10765D-01, 0.10413D-01, 0.10081D-01, 0.97639D-02,
     & 0.94605D-02, 0.91703D-02, 0.88927D-02, 0.86271D-02, 0.83733D-02,
     & 0.81342D-02, 0.79044D-02, 0.76835D-02, 0.74712D-02, 0.72671D-02,
     & 0.14067D-01, 0.13304D-01, 0.12595D-01, 0.11936D-01, 0.11323D-01,
     & 0.10756D-01, 0.10239D-01, 0.97547D-02, 0.93004D-02, 0.88745D-02,
     & 0.84753D-02, 0.81069D-02, 0.77637D-02, 0.74394D-02, 0.71329D-02,
     & 0.68433D-02, 0.65697D-02, 0.63131D-02, 0.60750D-02, 0.58484D-02,
     & 0.56328D-02, 0.54277D-02, 0.52325D-02, 0.50469D-02, 0.48708D-02,
     & 0.47073D-02, 0.45509D-02, 0.44011D-02, 0.42578D-02, 0.41207D-02,
     & 0.89783D-03, 0.84445D-03, 0.79500D-03, 0.74918D-03, 0.70677D-03,
     & 0.66768D-03, 0.63239D-03, 0.59940D-03, 0.56857D-03, 0.53977D-03,
     & 0.51287D-03, 0.48823D-03, 0.46540D-03, 0.44389D-03, 0.42362D-03,
     & 0.40454D-03, 0.38657D-03, 0.36980D-03, 0.35435D-03, 0.33969D-03,
     & 0.32578D-03, 0.31259D-03, 0.30007D-03, 0.28821D-03, 0.27699D-03,
     & 0.26666D-03, 0.25681D-03, 0.24739D-03, 0.23841D-03, 0.22983D-03/

      data ((g_grdl(i,j),j=1,30),i=1,10)/
     & 0.10003D-07, 0.50248D-02, 0.10108D-01, 0.15241D-01, 0.20417D-01,
     & 0.25630D-01, 0.30879D-01, 0.36151D-01, 0.41441D-01, 0.46746D-01,
     & 0.52060D-01, 0.57390D-01, 0.62726D-01, 0.68059D-01, 0.73387D-01,
     & 0.78708D-01, 0.84021D-01, 0.89329D-01, 0.94635D-01, 0.99924D-01,
     & 0.10519D+00, 0.11044D+00, 0.11567D+00, 0.12088D+00, 0.12607D+00,
     & 0.13126D+00, 0.13641D+00, 0.14154D+00, 0.14664D+00, 0.15172D+00,
     & 0.10002D-07, 0.48864D-02, 0.98142D-02, 0.14777D-01, 0.19768D-01,
     & 0.24785D-01, 0.29828D-01, 0.34882D-01, 0.39942D-01, 0.45006D-01,
     & 0.50070D-01, 0.55141D-01, 0.60211D-01, 0.65270D-01, 0.70315D-01,
     & 0.75345D-01, 0.80360D-01, 0.85364D-01, 0.90362D-01, 0.95335D-01,
     & 0.10028D+00, 0.10521D+00, 0.11011D+00, 0.11498D+00, 0.11983D+00,
     & 0.12467D+00, 0.12948D+00, 0.13426D+00, 0.13900D+00, 0.14372D+00,
     & 0.10002D-07, 0.47464D-02, 0.95185D-02, 0.14311D-01, 0.19119D-01,
     & 0.23940D-01, 0.28780D-01, 0.33618D-01, 0.38451D-01, 0.43279D-01,
     & 0.48097D-01, 0.52916D-01, 0.57726D-01, 0.62516D-01, 0.67286D-01,
     & 0.72034D-01, 0.76761D-01, 0.81470D-01, 0.86169D-01, 0.90838D-01,
     & 0.95477D-01, 0.10009D+00, 0.10467D+00, 0.10922D+00, 0.11374D+00,
     & 0.11825D+00, 0.12273D+00, 0.12717D+00, 0.13158D+00, 0.13595D+00,
     & 0.10003D-07, 0.46054D-02, 0.92216D-02, 0.13845D-01, 0.18471D-01,
     & 0.23099D-01, 0.27738D-01, 0.32364D-01, 0.36976D-01, 0.41572D-01,
     & 0.46152D-01, 0.50725D-01, 0.55282D-01, 0.59812D-01, 0.64315D-01,
     & 0.68791D-01, 0.73239D-01, 0.77665D-01, 0.82076D-01, 0.86452D-01,
     & 0.90794D-01, 0.95103D-01, 0.99377D-01, 0.10362D+00, 0.10783D+00,
     & 0.11203D+00, 0.11619D+00, 0.12031D+00, 0.12440D+00, 0.12844D+00,
     & 0.10003D-07, 0.44615D-02, 0.89200D-02, 0.13373D-01, 0.17817D-01,
     & 0.22253D-01, 0.26691D-01, 0.31106D-01, 0.35499D-01, 0.39867D-01,
     & 0.44212D-01, 0.48543D-01, 0.52852D-01, 0.57127D-01, 0.61370D-01,
     & 0.65580D-01, 0.69757D-01, 0.73907D-01, 0.78039D-01, 0.82132D-01,
     & 0.86187D-01, 0.90204D-01, 0.94185D-01, 0.98130D-01, 0.10204D+00,
     & 0.10594D+00, 0.10979D+00, 0.11361D+00, 0.11738D+00, 0.12112D+00,
     & 0.10003D-07, 0.43148D-02, 0.86136D-02, 0.12895D-01, 0.17157D-01,
     & 0.21401D-01, 0.25638D-01, 0.29845D-01, 0.34020D-01, 0.38164D-01,
     & 0.42276D-01, 0.46370D-01, 0.50435D-01, 0.54462D-01, 0.58450D-01,
     & 0.62401D-01, 0.66315D-01, 0.70198D-01, 0.74058D-01, 0.77877D-01,
     & 0.81654D-01, 0.85391D-01, 0.89088D-01, 0.92748D-01, 0.96371D-01,
     & 0.99976D-01, 0.10354D+00, 0.10706D+00, 0.11054D+00, 0.11399D+00,
     & 0.10002D-07, 0.41649D-02, 0.83020D-02, 0.12410D-01, 0.16490D-01,
     & 0.20542D-01, 0.24580D-01, 0.28579D-01, 0.32539D-01, 0.36462D-01,
     & 0.40346D-01, 0.44207D-01, 0.48034D-01, 0.51817D-01, 0.55557D-01,
     & 0.59256D-01, 0.62914D-01, 0.66537D-01, 0.70135D-01, 0.73688D-01,
     & 0.77197D-01, 0.80663D-01, 0.84088D-01, 0.87473D-01, 0.90820D-01,
     & 0.94146D-01, 0.97429D-01, 0.10067D+00, 0.10387D+00, 0.10703D+00,
     & 0.10001D-07, 0.40125D-02, 0.79861D-02, 0.11921D-01, 0.15818D-01,
     & 0.19679D-01, 0.23519D-01, 0.27312D-01, 0.31061D-01, 0.34765D-01,
     & 0.38426D-01, 0.42057D-01, 0.45651D-01, 0.49197D-01, 0.52696D-01,
     & 0.56150D-01, 0.59560D-01, 0.62933D-01, 0.66276D-01, 0.69573D-01,
     & 0.72824D-01, 0.76030D-01, 0.79194D-01, 0.82316D-01, 0.85398D-01,
     & 0.88458D-01, 0.91474D-01, 0.94448D-01, 0.97380D-01, 0.10027D+00,
     & 0.10002D-07, 0.38576D-02, 0.76662D-02, 0.11427D-01, 0.15142D-01,
     & 0.18813D-01, 0.22456D-01, 0.26046D-01, 0.29586D-01, 0.33076D-01,
     & 0.36519D-01, 0.39927D-01, 0.43293D-01, 0.46608D-01, 0.49873D-01,
     & 0.53090D-01, 0.56261D-01, 0.59391D-01, 0.62490D-01, 0.65540D-01,
     & 0.68543D-01, 0.71501D-01, 0.74414D-01, 0.77285D-01, 0.80116D-01,
     & 0.82922D-01, 0.85684D-01, 0.88403D-01, 0.91081D-01, 0.93718D-01,
     & 0.10001D-07, 0.36983D-02, 0.73387D-02, 0.10923D-01, 0.14454D-01,
     & 0.17935D-01, 0.21380D-01, 0.24768D-01, 0.28100D-01, 0.31378D-01,
     & 0.34604D-01, 0.37792D-01, 0.40934D-01, 0.44022D-01, 0.47058D-01,
     & 0.50044D-01, 0.52981D-01, 0.55876D-01, 0.58737D-01, 0.61548D-01,
     & 0.64312D-01, 0.67029D-01, 0.69701D-01, 0.72330D-01, 0.74918D-01,
     & 0.77480D-01, 0.79998D-01, 0.82474D-01, 0.84909D-01, 0.87304D-01/

      data ((g_grdl(i,j),j=1,30),i=11,20)/
     & 0.99996D-08, 0.35346D-02, 0.70033D-02, 0.10409D-01, 0.13754D-01,
     & 0.17043D-01, 0.20291D-01, 0.23476D-01, 0.26602D-01, 0.29670D-01,
     & 0.32683D-01, 0.35653D-01, 0.38575D-01, 0.41440D-01, 0.44252D-01,
     & 0.47012D-01, 0.49723D-01, 0.52389D-01, 0.55019D-01, 0.57599D-01,
     & 0.60131D-01, 0.62616D-01, 0.65056D-01, 0.67453D-01, 0.69809D-01,
     & 0.72137D-01, 0.74422D-01, 0.76665D-01, 0.78868D-01, 0.81031D-01,
     & 0.99991D-08, 0.33677D-02, 0.66627D-02, 0.98883D-02, 0.13048D-01,
     & 0.16146D-01, 0.19198D-01, 0.22183D-01, 0.25105D-01, 0.27967D-01,
     & 0.30771D-01, 0.33529D-01, 0.36236D-01, 0.38886D-01, 0.41481D-01,
     & 0.44023D-01, 0.46514D-01, 0.48960D-01, 0.51368D-01, 0.53727D-01,
     & 0.56037D-01, 0.58300D-01, 0.60519D-01, 0.62695D-01, 0.64830D-01,
     & 0.66937D-01, 0.69001D-01, 0.71024D-01, 0.73007D-01, 0.74953D-01,
     & 0.99973D-08, 0.31956D-02, 0.63127D-02, 0.93552D-02, 0.12327D-01,
     & 0.15233D-01, 0.18087D-01, 0.20872D-01, 0.23591D-01, 0.26248D-01,
     & 0.28845D-01, 0.31394D-01, 0.33890D-01, 0.36328D-01, 0.38710D-01,
     & 0.41039D-01, 0.43317D-01, 0.45549D-01, 0.47742D-01, 0.49885D-01,
     & 0.51981D-01, 0.54031D-01, 0.56037D-01, 0.58000D-01, 0.59924D-01,
     & 0.61817D-01, 0.63670D-01, 0.65483D-01, 0.67258D-01, 0.68996D-01,
     & 0.99985D-08, 0.30204D-02, 0.59576D-02, 0.88160D-02, 0.11600D-01,
     & 0.14314D-01, 0.16972D-01, 0.19559D-01, 0.22079D-01, 0.24535D-01,
     & 0.26930D-01, 0.29274D-01, 0.31564D-01, 0.33796D-01, 0.35973D-01,
     & 0.38096D-01, 0.40169D-01, 0.42195D-01, 0.44181D-01, 0.46119D-01,
     & 0.48010D-01, 0.49856D-01, 0.51660D-01, 0.53422D-01, 0.55145D-01,
     & 0.56837D-01, 0.58490D-01, 0.60105D-01, 0.61683D-01, 0.63226D-01,
     & 0.10003D-07, 0.28414D-02, 0.55959D-02, 0.82683D-02, 0.10863D-01,
     & 0.13386D-01, 0.15849D-01, 0.18240D-01, 0.20562D-01, 0.22820D-01,
     & 0.25016D-01, 0.27160D-01, 0.29249D-01, 0.31281D-01, 0.33257D-01,
     & 0.35181D-01, 0.37055D-01, 0.38883D-01, 0.40671D-01, 0.42411D-01,
     & 0.44106D-01, 0.45757D-01, 0.47367D-01, 0.48938D-01, 0.50470D-01,
     & 0.51971D-01, 0.53435D-01, 0.54862D-01, 0.56255D-01, 0.57614D-01,
     & 0.10001D-07, 0.26554D-02, 0.52214D-02, 0.77033D-02, 0.10106D-01,
     & 0.12434D-01, 0.14700D-01, 0.16893D-01, 0.19019D-01, 0.21079D-01,
     & 0.23078D-01, 0.25023D-01, 0.26914D-01, 0.28749D-01, 0.30530D-01,
     & 0.32259D-01, 0.33939D-01, 0.35574D-01, 0.37169D-01, 0.38718D-01,
     & 0.40224D-01, 0.41688D-01, 0.43112D-01, 0.44498D-01, 0.45848D-01,
     & 0.47167D-01, 0.48450D-01, 0.49699D-01, 0.50916D-01, 0.52101D-01,
     & 0.10001D-07, 0.24649D-02, 0.48393D-02, 0.71284D-02, 0.93373D-02,
     & 0.11471D-01, 0.13540D-01, 0.15538D-01, 0.17468D-01, 0.19334D-01,
     & 0.21139D-01, 0.22890D-01, 0.24588D-01, 0.26230D-01, 0.27821D-01,
     & 0.29362D-01, 0.30855D-01, 0.32305D-01, 0.33714D-01, 0.35080D-01,
     & 0.36405D-01, 0.37691D-01, 0.38939D-01, 0.40150D-01, 0.41327D-01,
     & 0.42474D-01, 0.43587D-01, 0.44669D-01, 0.45720D-01, 0.46741D-01,
     & 0.10001D-07, 0.22695D-02, 0.44484D-02, 0.65421D-02, 0.85558D-02,
     & 0.10495D-01, 0.12367D-01, 0.14170D-01, 0.15906D-01, 0.17580D-01,
     & 0.19195D-01, 0.20756D-01, 0.22265D-01, 0.23721D-01, 0.25127D-01,
     & 0.26486D-01, 0.27799D-01, 0.29070D-01, 0.30302D-01, 0.31493D-01,
     & 0.32645D-01, 0.33761D-01, 0.34841D-01, 0.35887D-01, 0.36900D-01,
     & 0.37885D-01, 0.38838D-01, 0.39762D-01, 0.40658D-01, 0.41527D-01,
     & 0.10002D-07, 0.20694D-02, 0.40494D-02, 0.59455D-02, 0.77628D-02,
     & 0.95064D-02, 0.11183D-01, 0.12792D-01, 0.14337D-01, 0.15823D-01,
     & 0.17251D-01, 0.18626D-01, 0.19951D-01, 0.21226D-01, 0.22454D-01,
     & 0.23637D-01, 0.24778D-01, 0.25878D-01, 0.26940D-01, 0.27964D-01,
     & 0.28953D-01, 0.29907D-01, 0.30828D-01, 0.31719D-01, 0.32579D-01,
     & 0.33410D-01, 0.34214D-01, 0.34991D-01, 0.35743D-01, 0.36470D-01,
     & 0.10003D-07, 0.18648D-02, 0.36427D-02, 0.53391D-02, 0.69591D-02,
     & 0.85075D-02, 0.99892D-02, 0.11407D-01, 0.12764D-01, 0.14063D-01,
     & 0.15309D-01, 0.16504D-01, 0.17650D-01, 0.18750D-01, 0.19806D-01,
     & 0.20821D-01, 0.21795D-01, 0.22732D-01, 0.23632D-01, 0.24499D-01,
     & 0.25332D-01, 0.26135D-01, 0.26907D-01, 0.27651D-01, 0.28367D-01,
     & 0.29057D-01, 0.29721D-01, 0.30362D-01, 0.30980D-01, 0.31575D-01/

      data ((g_grdl(i,j),j=1,30),i=21,30)/
     & 0.10004D-07, 0.16559D-02, 0.32286D-02, 0.47235D-02, 0.61454D-02,
     & 0.74988D-02, 0.87870D-02, 0.10015D-01, 0.11187D-01, 0.12305D-01,
     & 0.13372D-01, 0.14391D-01, 0.15365D-01, 0.16296D-01, 0.17187D-01,
     & 0.18039D-01, 0.18856D-01, 0.19637D-01, 0.20385D-01, 0.21102D-01,
     & 0.21789D-01, 0.22448D-01, 0.23081D-01, 0.23688D-01, 0.24271D-01,
     & 0.24828D-01, 0.25364D-01, 0.25879D-01, 0.26373D-01, 0.26849D-01,
     & 0.99978D-08, 0.14417D-02, 0.28053D-02, 0.40960D-02, 0.53184D-02,
     & 0.64764D-02, 0.75720D-02, 0.86125D-02, 0.96011D-02, 0.10541D-01,
     & 0.11434D-01, 0.12282D-01, 0.13088D-01, 0.13857D-01, 0.14589D-01,
     & 0.15287D-01, 0.15953D-01, 0.16587D-01, 0.17190D-01, 0.17766D-01,
     & 0.18316D-01, 0.18842D-01, 0.19345D-01, 0.19825D-01, 0.20284D-01,
     & 0.20720D-01, 0.21137D-01, 0.21536D-01, 0.21918D-01, 0.22284D-01,
     & 0.99975D-08, 0.12264D-02, 0.23812D-02, 0.34692D-02, 0.44948D-02,
     & 0.54612D-02, 0.63691D-02, 0.72276D-02, 0.80396D-02, 0.88079D-02,
     & 0.95350D-02, 0.10221D-01, 0.10869D-01, 0.11483D-01, 0.12067D-01,
     & 0.12621D-01, 0.13146D-01, 0.13644D-01, 0.14114D-01, 0.14560D-01,
     & 0.14985D-01, 0.15390D-01, 0.15774D-01, 0.16140D-01, 0.16487D-01,
     & 0.16813D-01, 0.17125D-01, 0.17421D-01, 0.17703D-01, 0.17972D-01,
     & 0.10002D-07, 0.10109D-02, 0.19580D-02, 0.28457D-02, 0.36779D-02,
     & 0.44573D-02, 0.51831D-02, 0.58661D-02, 0.65088D-02, 0.71137D-02,
     & 0.76828D-02, 0.82150D-02, 0.87146D-02, 0.91862D-02, 0.96315D-02,
     & 0.10052D-01, 0.10448D-01, 0.10821D-01, 0.11169D-01, 0.11498D-01,
     & 0.11810D-01, 0.12105D-01, 0.12384D-01, 0.12647D-01, 0.12895D-01,
     & 0.13125D-01, 0.13343D-01, 0.13550D-01, 0.13745D-01, 0.13930D-01,
     & 0.10001D-07, 0.79450D-03, 0.15346D-02, 0.22239D-02, 0.28659D-02,
     & 0.34626D-02, 0.40121D-02, 0.45262D-02, 0.50069D-02, 0.54564D-02,
     & 0.58763D-02, 0.62647D-02, 0.66259D-02, 0.69648D-02, 0.72826D-02,
     & 0.75804D-02, 0.78593D-02, 0.81187D-02, 0.83578D-02, 0.85825D-02,
     & 0.87935D-02, 0.89915D-02, 0.91770D-02, 0.93507D-02, 0.95128D-02,
     & 0.96602D-02, 0.97986D-02, 0.99284D-02, 0.10050D-01, 0.10164D-01,
     & 0.10001D-07, 0.74287D-03, 0.14337D-02, 0.20762D-02, 0.26734D-02,
     & 0.32273D-02, 0.37358D-02, 0.42106D-02, 0.46540D-02, 0.50677D-02,
     & 0.54534D-02, 0.58091D-02, 0.61390D-02, 0.64479D-02, 0.67370D-02,
     & 0.70074D-02, 0.72601D-02, 0.74944D-02, 0.77095D-02, 0.79111D-02,
     & 0.81001D-02, 0.82770D-02, 0.84424D-02, 0.85967D-02, 0.87403D-02,
     & 0.88700D-02, 0.89914D-02, 0.91050D-02, 0.92111D-02, 0.93099D-02,
     & 0.10000D-07, 0.69333D-03, 0.13371D-02, 0.19347D-02, 0.24892D-02,
     & 0.30025D-02, 0.34720D-02, 0.39098D-02, 0.43178D-02, 0.46978D-02,
     & 0.50515D-02, 0.53764D-02, 0.56769D-02, 0.59579D-02, 0.62203D-02,
     & 0.64651D-02, 0.66934D-02, 0.69045D-02, 0.70973D-02, 0.72777D-02,
     & 0.74464D-02, 0.76038D-02, 0.77506D-02, 0.78873D-02, 0.80139D-02,
     & 0.81275D-02, 0.82335D-02, 0.83323D-02, 0.84242D-02, 0.85095D-02,
     & 0.10000D-07, 0.64593D-03, 0.12447D-02, 0.17995D-02, 0.23134D-02,
     & 0.27880D-02, 0.32206D-02, 0.36233D-02, 0.39979D-02, 0.43461D-02,
     & 0.46695D-02, 0.49655D-02, 0.52385D-02, 0.54932D-02, 0.57306D-02,
     & 0.59517D-02, 0.61572D-02, 0.63466D-02, 0.65188D-02, 0.66796D-02,
     & 0.68295D-02, 0.69691D-02, 0.70988D-02, 0.72192D-02, 0.73303D-02,
     & 0.74291D-02, 0.75211D-02, 0.76065D-02, 0.76856D-02, 0.77587D-02,
     & 0.10000D-07, 0.60039D-03, 0.11560D-02, 0.16700D-02, 0.21451D-02,
     & 0.25828D-02, 0.29804D-02, 0.33499D-02, 0.36929D-02, 0.40111D-02,
     & 0.43059D-02, 0.45749D-02, 0.48221D-02, 0.50523D-02, 0.52663D-02,
     & 0.54652D-02, 0.56497D-02, 0.58190D-02, 0.59721D-02, 0.61148D-02,
     & 0.62474D-02, 0.63705D-02, 0.64846D-02, 0.65901D-02, 0.66870D-02,
     & 0.67724D-02, 0.68516D-02, 0.69248D-02, 0.69923D-02, 0.70544D-02,
     & 0.10000D-07, 0.55674D-03, 0.10711D-02, 0.15460D-02, 0.19841D-02,
     & 0.23868D-02, 0.27512D-02, 0.30891D-02, 0.34022D-02, 0.36921D-02,
     & 0.39601D-02, 0.42035D-02, 0.44265D-02, 0.46338D-02, 0.48261D-02,
     & 0.50043D-02, 0.51691D-02, 0.53198D-02, 0.54553D-02, 0.55811D-02,
     & 0.56978D-02, 0.58058D-02, 0.59055D-02, 0.59973D-02, 0.60813D-02,
     & 0.61545D-02, 0.62221D-02, 0.62843D-02, 0.63414D-02, 0.63935D-02/

      data ((g_grdl(i,j),j=1,30),i=31,40)/
     & 0.10000D-07, 0.51481D-03, 0.98959D-03, 0.14271D-02, 0.18300D-02,
     & 0.21993D-02, 0.25320D-02, 0.28401D-02, 0.31250D-02, 0.33881D-02,
     & 0.36308D-02, 0.38504D-02, 0.40507D-02, 0.42365D-02, 0.44085D-02,
     & 0.45674D-02, 0.47140D-02, 0.48475D-02, 0.49667D-02, 0.50771D-02,
     & 0.51791D-02, 0.52732D-02, 0.53598D-02, 0.54391D-02, 0.55114D-02,
     & 0.55735D-02, 0.56306D-02, 0.56828D-02, 0.57305D-02, 0.57737D-02,
     & 0.10000D-07, 0.47455D-03, 0.91142D-03, 0.13132D-02, 0.16824D-02,
     & 0.20199D-02, 0.23227D-02, 0.26025D-02, 0.28607D-02, 0.30986D-02,
     & 0.33175D-02, 0.35145D-02, 0.36937D-02, 0.38595D-02, 0.40125D-02,
     & 0.41535D-02, 0.42831D-02, 0.44006D-02, 0.45048D-02, 0.46010D-02,
     & 0.46896D-02, 0.47710D-02, 0.48456D-02, 0.49135D-02, 0.49750D-02,
     & 0.50272D-02, 0.50748D-02, 0.51181D-02, 0.51573D-02, 0.51926D-02,
     & 0.10000D-07, 0.43592D-03, 0.83647D-03, 0.12041D-02, 0.15411D-02,
     & 0.18484D-02, 0.21228D-02, 0.23758D-02, 0.26087D-02, 0.28229D-02,
     & 0.30193D-02, 0.31953D-02, 0.33547D-02, 0.35017D-02, 0.36371D-02,
     & 0.37614D-02, 0.38754D-02, 0.39781D-02, 0.40685D-02, 0.41516D-02,
     & 0.42279D-02, 0.42977D-02, 0.43612D-02, 0.44189D-02, 0.44707D-02,
     & 0.45138D-02, 0.45529D-02, 0.45882D-02, 0.46199D-02, 0.46480D-02,
     & 0.10000D-07, 0.39881D-03, 0.76455D-03, 0.10995D-02, 0.14059D-02,
     & 0.16844D-02, 0.19319D-02, 0.21595D-02, 0.23687D-02, 0.25604D-02,
     & 0.27358D-02, 0.28921D-02, 0.30330D-02, 0.31626D-02, 0.32816D-02,
     & 0.33905D-02, 0.34899D-02, 0.35791D-02, 0.36568D-02, 0.37280D-02,
     & 0.37930D-02, 0.38522D-02, 0.39058D-02, 0.39542D-02, 0.39972D-02,
     & 0.40323D-02, 0.40638D-02, 0.40920D-02, 0.41169D-02, 0.41388D-02,
     & 0.10001D-07, 0.36331D-03, 0.69582D-03, 0.99968D-03, 0.12769D-02,
     & 0.15281D-02, 0.17501D-02, 0.19539D-02, 0.21406D-02, 0.23113D-02,
     & 0.24670D-02, 0.26049D-02, 0.27286D-02, 0.28420D-02, 0.29458D-02,
     & 0.30405D-02, 0.31266D-02, 0.32033D-02, 0.32694D-02, 0.33297D-02,
     & 0.33846D-02, 0.34342D-02, 0.34788D-02, 0.35188D-02, 0.35540D-02,
     & 0.35819D-02, 0.36067D-02, 0.36286D-02, 0.36477D-02, 0.36641D-02,
     & 0.99998D-08, 0.32896D-03, 0.62939D-03, 0.90329D-03, 0.11525D-02,
     & 0.13777D-02, 0.15754D-02, 0.17564D-02, 0.19219D-02, 0.20727D-02,
     & 0.22098D-02, 0.23304D-02, 0.24380D-02, 0.25364D-02, 0.26261D-02,
     & 0.27075D-02, 0.27812D-02, 0.28464D-02, 0.29020D-02, 0.29524D-02,
     & 0.29979D-02, 0.30389D-02, 0.30754D-02, 0.31078D-02, 0.31359D-02,
     & 0.31575D-02, 0.31764D-02, 0.31928D-02, 0.32067D-02, 0.32183D-02,
     & 0.99996D-08, 0.29617D-03, 0.56604D-03, 0.81147D-03, 0.10341D-02,
     & 0.12346D-02, 0.14094D-02, 0.15691D-02, 0.17147D-02, 0.18469D-02,
     & 0.19666D-02, 0.20713D-02, 0.21639D-02, 0.22484D-02, 0.23251D-02,
     & 0.23944D-02, 0.24569D-02, 0.25116D-02, 0.25575D-02, 0.25990D-02,
     & 0.26362D-02, 0.26694D-02, 0.26987D-02, 0.27243D-02, 0.27463D-02,
     & 0.27623D-02, 0.27761D-02, 0.27876D-02, 0.27971D-02, 0.28047D-02,
     & 0.99998D-08, 0.26474D-03, 0.50538D-03, 0.72366D-03, 0.92110D-03,
     & 0.10982D-02, 0.12515D-02, 0.13911D-02, 0.15180D-02, 0.16328D-02,
     & 0.17364D-02, 0.18262D-02, 0.19051D-02, 0.19768D-02, 0.20416D-02,
     & 0.20998D-02, 0.21520D-02, 0.21972D-02, 0.22346D-02, 0.22680D-02,
     & 0.22978D-02, 0.23240D-02, 0.23470D-02, 0.23667D-02, 0.23832D-02,
     & 0.23945D-02, 0.24038D-02, 0.24113D-02, 0.24171D-02, 0.24211D-02,
     & 0.99999D-08, 0.23459D-03, 0.44730D-03, 0.63967D-03, 0.81313D-03,
     & 0.96801D-03, 0.11010D-02, 0.12218D-02, 0.13312D-02, 0.14298D-02,
     & 0.15183D-02, 0.15944D-02, 0.16607D-02, 0.17206D-02, 0.17745D-02,
     & 0.18227D-02, 0.18655D-02, 0.19022D-02, 0.19318D-02, 0.19582D-02,
     & 0.19813D-02, 0.20015D-02, 0.20188D-02, 0.20334D-02, 0.20452D-02,
     & 0.20524D-02, 0.20580D-02, 0.20620D-02, 0.20647D-02, 0.20659D-02,
     & 0.99999D-08, 0.20577D-03, 0.39183D-03, 0.55957D-03, 0.71028D-03,
     & 0.84421D-03, 0.95814D-03, 0.10613D-02, 0.11543D-02, 0.12378D-02,
     & 0.13124D-02, 0.13758D-02, 0.14305D-02, 0.14797D-02, 0.15237D-02,
     & 0.15627D-02, 0.15971D-02, 0.16262D-02, 0.16490D-02, 0.16690D-02,
     & 0.16864D-02, 0.17012D-02, 0.17137D-02, 0.17238D-02, 0.17317D-02,
     & 0.17355D-02, 0.17380D-02, 0.17392D-02, 0.17393D-02, 0.17383D-02/

      data ((g_grdl(i,j),j=1,30),i=41,50)/
     & 0.10000D-07, 0.17816D-03, 0.33876D-03, 0.48307D-03, 0.61222D-03,
     & 0.72641D-03, 0.82251D-03, 0.90917D-03, 0.98701D-03, 0.10566D-02,
     & 0.11184D-02, 0.11701D-02, 0.12144D-02, 0.12539D-02, 0.12889D-02,
     & 0.13198D-02, 0.13467D-02, 0.13691D-02, 0.13859D-02, 0.14005D-02,
     & 0.14129D-02, 0.14232D-02, 0.14316D-02, 0.14380D-02, 0.14426D-02,
     & 0.14436D-02, 0.14436D-02, 0.14427D-02, 0.14409D-02, 0.14382D-02,
     & 0.10000D-07, 0.15185D-03, 0.28829D-03, 0.41042D-03, 0.51924D-03,
     & 0.61487D-03, 0.69432D-03, 0.76568D-03, 0.82947D-03, 0.88614D-03,
     & 0.93616D-03, 0.97742D-03, 0.10122D-02, 0.10430D-02, 0.10701D-02,
     & 0.10937D-02, 0.11140D-02, 0.11304D-02, 0.11422D-02, 0.11521D-02,
     & 0.11603D-02, 0.11668D-02, 0.11717D-02, 0.11751D-02, 0.11770D-02,
     & 0.11759D-02, 0.11741D-02, 0.11715D-02, 0.11683D-02, 0.11644D-02,
     & 0.10000D-07, 0.12674D-03, 0.24019D-03, 0.34132D-03, 0.43098D-03,
     & 0.50924D-03, 0.57328D-03, 0.63052D-03, 0.68140D-03, 0.72632D-03,
     & 0.76564D-03, 0.79746D-03, 0.82376D-03, 0.84687D-03, 0.86697D-03,
     & 0.88424D-03, 0.89884D-03, 0.91024D-03, 0.91774D-03, 0.92381D-03,
     & 0.92851D-03, 0.93193D-03, 0.93413D-03, 0.93516D-03, 0.93499D-03,
     & 0.93240D-03, 0.92928D-03, 0.92564D-03, 0.92152D-03, 0.91694D-03,
     & 0.10000D-07, 0.10289D-03, 0.19460D-03, 0.27595D-03, 0.34766D-03,
     & 0.40974D-03, 0.45959D-03, 0.50390D-03, 0.54302D-03, 0.57728D-03,
     & 0.60698D-03, 0.63042D-03, 0.64932D-03, 0.66572D-03, 0.67978D-03,
     & 0.69162D-03, 0.70137D-03, 0.70858D-03, 0.71262D-03, 0.71561D-03,
     & 0.71759D-03, 0.71863D-03, 0.71877D-03, 0.71805D-03, 0.71643D-03,
     & 0.71292D-03, 0.70905D-03, 0.70485D-03, 0.70033D-03, 0.69551D-03,
     & 0.10000D-07, 0.80279D-04, 0.15148D-03, 0.21428D-03, 0.26926D-03,
     & 0.31637D-03, 0.35330D-03, 0.38589D-03, 0.41444D-03, 0.43918D-03,
     & 0.46037D-03, 0.47652D-03, 0.48909D-03, 0.49981D-03, 0.50879D-03,
     & 0.51613D-03, 0.52191D-03, 0.52577D-03, 0.52719D-03, 0.52787D-03,
     & 0.52788D-03, 0.52723D-03, 0.52597D-03, 0.52412D-03, 0.52165D-03,
     & 0.51777D-03, 0.51368D-03, 0.50941D-03, 0.50496D-03, 0.50033D-03,
     & 0.10000D-07, 0.58794D-04, 0.11064D-03, 0.15606D-03, 0.19550D-03,
     & 0.22888D-03, 0.25424D-03, 0.27643D-03, 0.29566D-03, 0.31212D-03,
     & 0.32597D-03, 0.33601D-03, 0.34340D-03, 0.34953D-03, 0.35447D-03,
     & 0.35829D-03, 0.36104D-03, 0.36245D-03, 0.36208D-03, 0.36128D-03,
     & 0.36006D-03, 0.35844D-03, 0.35645D-03, 0.35411D-03, 0.35138D-03,
     & 0.34768D-03, 0.34390D-03, 0.34004D-03, 0.33610D-03, 0.33210D-03,
     & 0.10000D-07, 0.39068D-04, 0.73247D-04, 0.10290D-03, 0.12835D-03,
     & 0.14950D-03, 0.16474D-03, 0.17791D-03, 0.18915D-03, 0.19858D-03,
     & 0.20632D-03, 0.21145D-03, 0.21483D-03, 0.21747D-03, 0.21941D-03,
     & 0.22070D-03, 0.22136D-03, 0.22121D-03, 0.21994D-03, 0.21846D-03,
     & 0.21678D-03, 0.21490D-03, 0.21285D-03, 0.21063D-03, 0.20822D-03,
     & 0.20525D-03, 0.20228D-03, 0.19930D-03, 0.19631D-03, 0.19331D-03,
     & 0.10000D-07, 0.20280D-04, 0.37909D-04, 0.53088D-04, 0.65994D-04,
     & 0.76556D-04, 0.83810D-04, 0.90012D-04, 0.95230D-04, 0.99528D-04,
     & 0.10296D-03, 0.10502D-03, 0.10617D-03, 0.10698D-03, 0.10747D-03,
     & 0.10765D-03, 0.10755D-03, 0.10705D-03, 0.10601D-03, 0.10488D-03,
     & 0.10369D-03, 0.10242D-03, 0.10109D-03, 0.99696D-04, 0.98229D-04,
     & 0.96510D-04, 0.94805D-04, 0.93115D-04, 0.91438D-04, 0.89774D-04,
     & 0.10000D-07, 0.64613D-05, 0.12030D-04, 0.16779D-04, 0.20769D-04,
     & 0.23968D-04, 0.26020D-04, 0.27746D-04, 0.29167D-04, 0.30303D-04,
     & 0.31172D-04, 0.31592D-04, 0.31726D-04, 0.31768D-04, 0.31724D-04,
     & 0.31599D-04, 0.31397D-04, 0.31082D-04, 0.30606D-04, 0.30116D-04,
     & 0.29616D-04, 0.29104D-04, 0.28582D-04, 0.28051D-04, 0.27506D-04,
     & 0.26896D-04, 0.26297D-04, 0.25709D-04, 0.25132D-04, 0.24566D-04,
     & 0.10000D-07, 0.15171D-06, 0.27078D-06, 0.36887D-06, 0.44749D-06,
     & 0.50537D-06, 0.53079D-06, 0.54971D-06, 0.56257D-06, 0.56976D-06,
     & 0.57167D-06, 0.56301D-06, 0.54825D-06, 0.53278D-06, 0.51664D-06,
     & 0.49988D-06, 0.48253D-06, 0.46374D-06, 0.44235D-06, 0.42163D-06,
     & 0.40158D-06, 0.38214D-06, 0.36331D-06, 0.34505D-06, 0.32727D-06,
     & 0.30920D-06, 0.29192D-06, 0.27538D-06, 0.25957D-06, 0.24445D-06/

      data ((c_grdl(i,j),j=1,30),i=1,10)/
     & 0.23432D-05, 0.23506D-05, 0.23571D-05, 0.23629D-05, 0.23680D-05,
     & 0.23725D-05, 0.23765D-05, 0.23801D-05, 0.23833D-05, 0.23860D-05,
     & 0.23885D-05, 0.23907D-05, 0.23926D-05, 0.23944D-05, 0.23959D-05,
     & 0.23972D-05, 0.23984D-05, 0.23995D-05, 0.24004D-05, 0.24013D-05,
     & 0.24020D-05, 0.24027D-05, 0.24032D-05, 0.24037D-05, 0.24042D-05,
     & 0.24046D-05, 0.24050D-05, 0.24053D-05, 0.24056D-05, 0.24058D-05,
     & 0.23232D-05, 0.23319D-05, 0.23396D-05, 0.23464D-05, 0.23524D-05,
     & 0.23577D-05, 0.23625D-05, 0.23666D-05, 0.23704D-05, 0.23736D-05,
     & 0.23766D-05, 0.23791D-05, 0.23814D-05, 0.23834D-05, 0.23852D-05,
     & 0.23868D-05, 0.23882D-05, 0.23895D-05, 0.23906D-05, 0.23916D-05,
     & 0.23924D-05, 0.23932D-05, 0.23939D-05, 0.23945D-05, 0.23950D-05,
     & 0.23955D-05, 0.23959D-05, 0.23963D-05, 0.23966D-05, 0.23969D-05,
     & 0.22996D-05, 0.23098D-05, 0.23189D-05, 0.23269D-05, 0.23340D-05,
     & 0.23403D-05, 0.23458D-05, 0.23508D-05, 0.23552D-05, 0.23590D-05,
     & 0.23625D-05, 0.23655D-05, 0.23682D-05, 0.23706D-05, 0.23727D-05,
     & 0.23746D-05, 0.23762D-05, 0.23777D-05, 0.23790D-05, 0.23801D-05,
     & 0.23812D-05, 0.23821D-05, 0.23829D-05, 0.23836D-05, 0.23842D-05,
     & 0.23848D-05, 0.23853D-05, 0.23857D-05, 0.23861D-05, 0.23864D-05,
     & 0.22719D-05, 0.22839D-05, 0.22945D-05, 0.23040D-05, 0.23123D-05,
     & 0.23197D-05, 0.23263D-05, 0.23321D-05, 0.23372D-05, 0.23418D-05,
     & 0.23458D-05, 0.23494D-05, 0.23526D-05, 0.23554D-05, 0.23579D-05,
     & 0.23601D-05, 0.23621D-05, 0.23638D-05, 0.23653D-05, 0.23667D-05,
     & 0.23679D-05, 0.23690D-05, 0.23699D-05, 0.23707D-05, 0.23715D-05,
     & 0.23721D-05, 0.23727D-05, 0.23732D-05, 0.23737D-05, 0.23741D-05,
     & 0.22391D-05, 0.22533D-05, 0.22658D-05, 0.22770D-05, 0.22868D-05,
     & 0.22955D-05, 0.23032D-05, 0.23101D-05, 0.23162D-05, 0.23215D-05,
     & 0.23263D-05, 0.23305D-05, 0.23342D-05, 0.23376D-05, 0.23405D-05,
     & 0.23431D-05, 0.23454D-05, 0.23474D-05, 0.23492D-05, 0.23508D-05,
     & 0.23523D-05, 0.23535D-05, 0.23546D-05, 0.23556D-05, 0.23565D-05,
     & 0.23573D-05, 0.23580D-05, 0.23586D-05, 0.23591D-05, 0.23596D-05,
     & 0.22007D-05, 0.22173D-05, 0.22321D-05, 0.22451D-05, 0.22567D-05,
     & 0.22670D-05, 0.22761D-05, 0.22842D-05, 0.22913D-05, 0.22976D-05,
     & 0.23032D-05, 0.23082D-05, 0.23126D-05, 0.23165D-05, 0.23200D-05,
     & 0.23230D-05, 0.23257D-05, 0.23281D-05, 0.23303D-05, 0.23322D-05,
     & 0.23338D-05, 0.23353D-05, 0.23366D-05, 0.23378D-05, 0.23388D-05,
     & 0.23397D-05, 0.23405D-05, 0.23413D-05, 0.23419D-05, 0.23425D-05,
     & 0.31169D-05, 0.28891D-05, 0.25513D-05, 0.22077D-05, 0.22214D-05,
     & 0.22334D-05, 0.22442D-05, 0.22536D-05, 0.22621D-05, 0.22695D-05,
     & 0.22761D-05, 0.22820D-05, 0.22871D-05, 0.22917D-05, 0.22958D-05,
     & 0.22994D-05, 0.23026D-05, 0.23054D-05, 0.23079D-05, 0.23102D-05,
     & 0.23121D-05, 0.23139D-05, 0.23154D-05, 0.23168D-05, 0.23180D-05,
     & 0.23191D-05, 0.23200D-05, 0.23209D-05, 0.23216D-05, 0.23223D-05,
     & 0.35047D-05, 0.33626D-05, 0.32065D-05, 0.30302D-05, 0.28176D-05,
     & 0.24983D-05, 0.22065D-05, 0.22177D-05, 0.22276D-05, 0.22364D-05,
     & 0.22442D-05, 0.22510D-05, 0.22571D-05, 0.22626D-05, 0.22673D-05,
     & 0.22716D-05, 0.22754D-05, 0.22787D-05, 0.22816D-05, 0.22843D-05,
     & 0.22866D-05, 0.22886D-05, 0.22905D-05, 0.22921D-05, 0.22935D-05,
     & 0.22948D-05, 0.22959D-05, 0.22969D-05, 0.22978D-05, 0.22985D-05,
     & 0.37825D-05, 0.36775D-05, 0.35659D-05, 0.34480D-05, 0.33232D-05,
     & 0.31900D-05, 0.30454D-05, 0.28824D-05, 0.26824D-05, 0.23192D-05,
     & 0.22066D-05, 0.22147D-05, 0.22218D-05, 0.22282D-05, 0.22338D-05,
     & 0.22388D-05, 0.22433D-05, 0.22472D-05, 0.22507D-05, 0.22538D-05,
     & 0.22565D-05, 0.22589D-05, 0.22610D-05, 0.22629D-05, 0.22646D-05,
     & 0.22661D-05, 0.22674D-05, 0.22686D-05, 0.22696D-05, 0.22706D-05,
     & 0.39884D-05, 0.39114D-05, 0.38282D-05, 0.37404D-05, 0.36489D-05,
     & 0.35545D-05, 0.34577D-05, 0.33587D-05, 0.32574D-05, 0.31533D-05,
     & 0.30454D-05, 0.29317D-05, 0.28084D-05, 0.26674D-05, 0.24810D-05,
     & 0.22003D-05, 0.22055D-05, 0.22101D-05, 0.22142D-05, 0.22178D-05,
     & 0.22211D-05, 0.22239D-05, 0.22264D-05, 0.22287D-05, 0.22306D-05,
     & 0.22324D-05, 0.22340D-05, 0.22353D-05, 0.22366D-05, 0.22376D-05/

      data ((c_grdl(i,j),j=1,30),i=11,20)/
     & 0.41292D-05, 0.40791D-05, 0.40213D-05, 0.39577D-05, 0.38901D-05,
     & 0.38197D-05, 0.37476D-05, 0.36747D-05, 0.36018D-05, 0.35293D-05,
     & 0.34577D-05, 0.33873D-05, 0.33183D-05, 0.32509D-05, 0.31851D-05,
     & 0.31211D-05, 0.30586D-05, 0.29978D-05, 0.29384D-05, 0.28803D-05,
     & 0.28233D-05, 0.27670D-05, 0.27112D-05, 0.26552D-05, 0.25984D-05,
     & 0.25393D-05, 0.24759D-05, 0.24028D-05, 0.22992D-05, 0.21989D-05,
     & 0.42015D-05, 0.41804D-05, 0.41490D-05, 0.41098D-05, 0.40646D-05,
     & 0.40152D-05, 0.39630D-05, 0.39093D-05, 0.38550D-05, 0.38008D-05,
     & 0.37476D-05, 0.36956D-05, 0.36454D-05, 0.35971D-05, 0.35511D-05,
     & 0.35075D-05, 0.34663D-05, 0.34275D-05, 0.33913D-05, 0.33575D-05,
     & 0.33261D-05, 0.32971D-05, 0.32704D-05, 0.32458D-05, 0.32232D-05,
     & 0.32026D-05, 0.31838D-05, 0.31666D-05, 0.31511D-05, 0.31370D-05,
     & 0.41971D-05, 0.42089D-05, 0.42070D-05, 0.41942D-05, 0.41730D-05,
     & 0.41454D-05, 0.41132D-05, 0.40777D-05, 0.40402D-05, 0.40017D-05,
     & 0.39630D-05, 0.39248D-05, 0.38875D-05, 0.38515D-05, 0.38171D-05,
     & 0.37846D-05, 0.37539D-05, 0.37252D-05, 0.36985D-05, 0.36738D-05,
     & 0.36510D-05, 0.36301D-05, 0.36110D-05, 0.35935D-05, 0.35776D-05,
     & 0.35632D-05, 0.35502D-05, 0.35384D-05, 0.35278D-05, 0.35183D-05,
     & 0.41048D-05, 0.41543D-05, 0.41863D-05, 0.42038D-05, 0.42096D-05,
     & 0.42062D-05, 0.41956D-05, 0.41797D-05, 0.41599D-05, 0.41374D-05,
     & 0.41132D-05, 0.40881D-05, 0.40628D-05, 0.40378D-05, 0.40134D-05,
     & 0.39899D-05, 0.39676D-05, 0.39466D-05, 0.39269D-05, 0.39086D-05,
     & 0.38917D-05, 0.38761D-05, 0.38618D-05, 0.38488D-05, 0.38369D-05,
     & 0.38262D-05, 0.38164D-05, 0.38076D-05, 0.37997D-05, 0.37926D-05,
     & 0.39140D-05, 0.40052D-05, 0.40754D-05, 0.41274D-05, 0.41642D-05,
     & 0.41886D-05, 0.42029D-05, 0.42090D-05, 0.42089D-05, 0.42040D-05,
     & 0.41956D-05, 0.41847D-05, 0.41721D-05, 0.41585D-05, 0.41444D-05,
     & 0.41302D-05, 0.41161D-05, 0.41026D-05, 0.40896D-05, 0.40773D-05,
     & 0.40657D-05, 0.40550D-05, 0.40450D-05, 0.40359D-05, 0.40275D-05,
     & 0.40198D-05, 0.40129D-05, 0.40066D-05, 0.40009D-05, 0.39958D-05,
     & 0.36173D-05, 0.37531D-05, 0.38643D-05, 0.39540D-05, 0.40256D-05,
     & 0.40813D-05, 0.41238D-05, 0.41553D-05, 0.41779D-05, 0.41933D-05,
     & 0.42029D-05, 0.42080D-05, 0.42097D-05, 0.42088D-05, 0.42059D-05,
     & 0.42018D-05, 0.41968D-05, 0.41912D-05, 0.41854D-05, 0.41795D-05,
     & 0.41736D-05, 0.41679D-05, 0.41625D-05, 0.41574D-05, 0.41526D-05,
     & 0.41482D-05, 0.41441D-05, 0.41403D-05, 0.41369D-05, 0.41337D-05,
     & 0.32119D-05, 0.33922D-05, 0.35461D-05, 0.36761D-05, 0.37845D-05,
     & 0.38742D-05, 0.39475D-05, 0.40073D-05, 0.40554D-05, 0.40937D-05,
     & 0.41238D-05, 0.41473D-05, 0.41654D-05, 0.41791D-05, 0.41893D-05,
     & 0.41967D-05, 0.42020D-05, 0.42055D-05, 0.42078D-05, 0.42091D-05,
     & 0.42096D-05, 0.42096D-05, 0.42092D-05, 0.42086D-05, 0.42078D-05,
     & 0.42068D-05, 0.42059D-05, 0.42049D-05, 0.42039D-05, 0.42029D-05,
     & 0.27056D-05, 0.29259D-05, 0.31200D-05, 0.32889D-05, 0.34350D-05,
     & 0.35601D-05, 0.36664D-05, 0.37562D-05, 0.38318D-05, 0.38949D-05,
     & 0.39476D-05, 0.39915D-05, 0.40279D-05, 0.40581D-05, 0.40830D-05,
     & 0.41036D-05, 0.41206D-05, 0.41346D-05, 0.41461D-05, 0.41556D-05,
     & 0.41635D-05, 0.41701D-05, 0.41755D-05, 0.41800D-05, 0.41837D-05,
     & 0.41868D-05, 0.41895D-05, 0.41917D-05, 0.41935D-05, 0.41951D-05,
     & 0.21218D-05, 0.23701D-05, 0.25959D-05, 0.27989D-05, 0.29793D-05,
     & 0.31379D-05, 0.32762D-05, 0.33965D-05, 0.35004D-05, 0.35898D-05,
     & 0.36665D-05, 0.37321D-05, 0.37882D-05, 0.38361D-05, 0.38770D-05,
     & 0.39119D-05, 0.39417D-05, 0.39673D-05, 0.39892D-05, 0.40080D-05,
     & 0.40241D-05, 0.40379D-05, 0.40499D-05, 0.40602D-05, 0.40691D-05,
     & 0.40768D-05, 0.40835D-05, 0.40893D-05, 0.40944D-05, 0.40988D-05,
     & 0.38054D-05, 0.35981D-05, 0.20010D-05, 0.22261D-05, 0.24315D-05,
     & 0.26171D-05, 0.27834D-05, 0.29312D-05, 0.30617D-05, 0.31761D-05,
     & 0.32763D-05, 0.33638D-05, 0.34401D-05, 0.35065D-05, 0.35642D-05,
     & 0.36143D-05, 0.36578D-05, 0.36957D-05, 0.37286D-05, 0.37572D-05,
     & 0.37822D-05, 0.38039D-05, 0.38229D-05, 0.38395D-05, 0.38540D-05,
     & 0.38667D-05, 0.38778D-05, 0.38876D-05, 0.38961D-05, 0.39036D-05/

      data ((c_grdl(i,j),j=1,30),i=21,30)/
     & 0.28922D-05, 0.34104D-05, 0.37371D-05, 0.37900D-05, 0.34198D-05,
     & 0.20242D-05, 0.22087D-05, 0.23762D-05, 0.25272D-05, 0.26627D-05,
     & 0.27835D-05, 0.28908D-05, 0.29857D-05, 0.30694D-05, 0.31431D-05,
     & 0.32079D-05, 0.32649D-05, 0.33150D-05, 0.33591D-05, 0.33978D-05,
     & 0.34318D-05, 0.34618D-05, 0.34881D-05, 0.35112D-05, 0.35316D-05,
     & 0.35495D-05, 0.35653D-05, 0.35792D-05, 0.35915D-05, 0.36023D-05,
     & 0.32094D-05, 0.32875D-05, 0.32156D-05, 0.31226D-05, 0.35177D-05,
     & 0.37562D-05, 0.37984D-05, 0.35837D-05, 0.28863D-05, 0.20743D-05,
     & 0.22087D-05, 0.23300D-05, 0.24389D-05, 0.25362D-05, 0.26233D-05,
     & 0.27008D-05, 0.27697D-05, 0.28308D-05, 0.28849D-05, 0.29329D-05,
     & 0.29753D-05, 0.30129D-05, 0.30461D-05, 0.30754D-05, 0.31014D-05,
     & 0.31243D-05, 0.31446D-05, 0.31625D-05, 0.31784D-05, 0.31924D-05,
     & 0.11029D-05, 0.21836D-05, 0.31245D-05, 0.32213D-05, 0.31830D-05,
     & 0.31402D-05, 0.30852D-05, 0.34216D-05, 0.36595D-05, 0.37881D-05,
     & 0.37983D-05, 0.36776D-05, 0.33926D-05, 0.28004D-05, 0.20310D-05,
     & 0.21165D-05, 0.21932D-05, 0.22620D-05, 0.23234D-05, 0.23781D-05,
     & 0.24269D-05, 0.24704D-05, 0.25090D-05, 0.25433D-05, 0.25738D-05,
     & 0.26010D-05, 0.26250D-05, 0.26464D-05, 0.26653D-05, 0.26821D-05,
     & 0.63210D-09, 0.15563D-06, 0.68930D-06, 0.15281D-05, 0.24694D-05,
     & 0.31786D-05, 0.32704D-05, 0.32881D-05, 0.32823D-05, 0.27832D-05,
     & 0.30853D-05, 0.33349D-05, 0.35297D-05, 0.36710D-05, 0.37613D-05,
     & 0.38041D-05, 0.38037D-05, 0.37642D-05, 0.36885D-05, 0.35790D-05,
     & 0.34360D-05, 0.32564D-05, 0.30301D-05, 0.27236D-05, 0.19769D-05,
     & 0.20065D-05, 0.20329D-05, 0.20564D-05, 0.20772D-05, 0.20958D-05,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.22839D-07, 0.25890D-06,
     & 0.76150D-06, 0.14535D-05, 0.22123D-05, 0.28856D-05, 0.32301D-05,
     & 0.32703D-05, 0.32545D-05, 0.31953D-05, 0.32827D-05, 0.31075D-05,
     & 0.28801D-05, 0.30516D-05, 0.31983D-05, 0.33220D-05, 0.34251D-05,
     & 0.35101D-05, 0.35794D-05, 0.36353D-05, 0.36797D-05, 0.37147D-05,
     & 0.37419D-05, 0.37626D-05, 0.37782D-05, 0.37895D-05, 0.37976D-05,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.63401D-07,
     & 0.36247D-06, 0.88613D-06, 0.15568D-05, 0.22647D-05, 0.28834D-05,
     & 0.32302D-05, 0.32854D-05, 0.32010D-05, 0.32752D-05, 0.32578D-05,
     & 0.32674D-05, 0.29979D-05, 0.28792D-05, 0.30197D-05, 0.31409D-05,
     & 0.32448D-05, 0.33331D-05, 0.34079D-05, 0.34709D-05, 0.35239D-05,
     & 0.35683D-05, 0.36055D-05, 0.36365D-05, 0.36623D-05, 0.36838D-05,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.27590D-08,
     & 0.12074D-06, 0.46345D-06, 0.99194D-06, 0.16311D-05, 0.22869D-05,
     & 0.28578D-05, 0.32126D-05, 0.32668D-05, 0.31212D-05, 0.32769D-05,
     & 0.31847D-05, 0.32735D-05, 0.32621D-05, 0.30719D-05, 0.28283D-05,
     & 0.29439D-05, 0.30446D-05, 0.31321D-05, 0.32078D-05, 0.32730D-05,
     & 0.33293D-05, 0.33777D-05, 0.34193D-05, 0.34552D-05, 0.34860D-05,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.16572D-07, 0.18662D-06, 0.55532D-06, 0.10743D-05, 0.16751D-05,
     & 0.22796D-05, 0.28085D-05, 0.31705D-05, 0.32136D-05, 0.32530D-05,
     & 0.32083D-05, 0.32881D-05, 0.32009D-05, 0.32711D-05, 0.32765D-05,
     & 0.31981D-05, 0.28946D-05, 0.28305D-05, 0.29140D-05, 0.29872D-05,
     & 0.30514D-05, 0.31075D-05, 0.31564D-05, 0.31992D-05, 0.32365D-05,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.43045D-07, 0.25354D-06, 0.63231D-06, 0.11308D-05,
     & 0.16890D-05, 0.22436D-05, 0.27344D-05, 0.30990D-05, 0.32112D-05,
     & 0.32797D-05, 0.30928D-05, 0.32334D-05, 0.32880D-05, 0.31815D-05,
     & 0.32504D-05, 0.32811D-05, 0.32727D-05, 0.32199D-05, 0.31009D-05,
     & 0.27538D-05, 0.28146D-05, 0.28682D-05, 0.29153D-05, 0.29568D-05,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.15371D-08, 0.78592D-07, 0.31502D-06, 0.69029D-06,
     & 0.11602D-05, 0.16735D-05, 0.21800D-05, 0.26347D-05, 0.29956D-05,
     & 0.32125D-05, 0.32080D-05, 0.32851D-05, 0.31055D-05, 0.32209D-05,
     & 0.32818D-05, 0.32677D-05, 0.31977D-05, 0.32461D-05, 0.32738D-05,
     & 0.32828D-05, 0.32744D-05, 0.32495D-05, 0.32075D-05, 0.31458D-05/

      data ((c_grdl(i,j),j=1,30),i=31,40)/
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.89117D-08, 0.11654D-06, 0.36563D-06,
     & 0.72687D-06, 0.11626D-05, 0.16301D-05, 0.20901D-05, 0.25093D-05,
     & 0.28597D-05, 0.31151D-05, 0.32318D-05, 0.32264D-05, 0.32855D-05,
     & 0.31199D-05, 0.31716D-05, 0.32432D-05, 0.32819D-05, 0.32833D-05,
     & 0.31958D-05, 0.32004D-05, 0.32326D-05, 0.32555D-05, 0.32707D-05,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.21982D-07, 0.15158D-06,
     & 0.40123D-06, 0.74118D-06, 0.11393D-05, 0.15610D-05, 0.19759D-05,
     & 0.23594D-05, 0.26928D-05, 0.29605D-05, 0.31492D-05, 0.32318D-05,
     & 0.32033D-05, 0.32743D-05, 0.32755D-05, 0.30775D-05, 0.31532D-05,
     & 0.32104D-05, 0.32510D-05, 0.32763D-05, 0.32875D-05, 0.32841D-05,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.21346D-09, 0.37999D-07,
     & 0.17930D-06, 0.42049D-06, 0.73368D-06, 0.10923D-05, 0.14690D-05,
     & 0.18397D-05, 0.21871D-05, 0.24977D-05, 0.27621D-05, 0.29742D-05,
     & 0.31292D-05, 0.32190D-05, 0.31940D-05, 0.32188D-05, 0.32691D-05,
     & 0.32854D-05, 0.32601D-05, 0.31186D-05, 0.31177D-05, 0.31585D-05,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.21677D-08,
     & 0.53519D-07, 0.19726D-06, 0.42305D-06, 0.70595D-06, 0.10245D-05,
     & 0.13570D-05, 0.16851D-05, 0.19958D-05, 0.22794D-05, 0.25298D-05,
     & 0.27436D-05, 0.29193D-05, 0.30570D-05, 0.31566D-05, 0.32169D-05,
     & 0.32285D-05, 0.31672D-05, 0.32178D-05, 0.32532D-05, 0.32751D-05,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.57340D-08, 0.65490D-07, 0.20418D-06, 0.40975D-06, 0.66044D-06,
     & 0.93926D-06, 0.12290D-05, 0.15161D-05, 0.17899D-05, 0.20438D-05,
     & 0.22733D-05, 0.24763D-05, 0.26521D-05, 0.28011D-05, 0.29248D-05,
     & 0.30251D-05, 0.31038D-05, 0.31628D-05, 0.32034D-05, 0.32265D-05,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.95922D-08, 0.71954D-07, 0.20007D-06, 0.38236D-06,
     & 0.60032D-06, 0.84043D-06, 0.10895D-05, 0.13370D-05, 0.15747D-05,
     & 0.17975D-05, 0.20023D-05, 0.21871D-05, 0.23516D-05, 0.24964D-05,
     & 0.26224D-05, 0.27309D-05, 0.28236D-05, 0.29022D-05, 0.29682D-05,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.12402D-07, 0.72255D-07, 0.18602D-06,
     & 0.34324D-06, 0.52919D-06, 0.73238D-06, 0.94312D-06, 0.11529D-05,
     & 0.13559D-05, 0.15476D-05, 0.17256D-05, 0.18886D-05, 0.20361D-05,
     & 0.21683D-05, 0.22860D-05, 0.23902D-05, 0.24818D-05, 0.25622D-05,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.38910D-10, 0.13334D-07, 0.66757D-07,
     & 0.16395D-06, 0.29549D-06, 0.45102D-06, 0.61968D-06, 0.79456D-06,
     & 0.96939D-06, 0.11391D-05, 0.13007D-05, 0.14521D-05, 0.15916D-05,
     & 0.17193D-05, 0.18352D-05, 0.19398D-05, 0.20334D-05, 0.21170D-05,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.10684D-09, 0.12236D-07,
     & 0.56643D-07, 0.13632D-06, 0.24302D-06, 0.36981D-06, 0.50679D-06,
     & 0.64888D-06, 0.79137D-06, 0.93053D-06, 0.10636D-05, 0.11889D-05,
     & 0.13055D-05, 0.14131D-05, 0.15112D-05, 0.16003D-05, 0.16809D-05,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.89600D-10,
     & 0.95595D-08, 0.43702D-07, 0.10576D-06, 0.18943D-06, 0.28877D-06,
     & 0.39798D-06, 0.51071D-06, 0.62406D-06, 0.73523D-06, 0.84220D-06,
     & 0.94348D-06, 0.10382D-05, 0.11258D-05, 0.12065D-05, 0.12801D-05/

      data ((c_grdl(i,j),j=1,30),i=41,50)/
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.17572D-10, 0.61540D-08, 0.30090D-07, 0.75358D-07, 0.13817D-06,
     & 0.21308D-06, 0.29638D-06, 0.38401D-06, 0.47178D-06, 0.55818D-06,
     & 0.64167D-06, 0.72110D-06, 0.79580D-06, 0.86533D-06, 0.92952D-06,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.29858D-08, 0.17782D-07, 0.47801D-07,
     & 0.91947D-07, 0.14620D-06, 0.20674D-06, 0.27130D-06, 0.33755D-06,
     & 0.40295D-06, 0.46608D-06, 0.52642D-06, 0.58338D-06, 0.63663D-06,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.83897D-09, 0.83145D-08,
     & 0.25872D-07, 0.53789D-07, 0.90164D-07, 0.13212D-06, 0.17727D-06,
     & 0.22401D-06, 0.27104D-06, 0.31732D-06, 0.36201D-06, 0.40392D-06,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.24959D-10,
     & 0.25016D-08, 0.10874D-07, 0.26011D-07, 0.47435D-07, 0.73873D-07,
     & 0.10335D-06, 0.13455D-06, 0.16620D-06, 0.19759D-06, 0.22821D-06,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.19417D-09, 0.27459D-08, 0.91282D-08, 0.19506D-07,
     & 0.33426D-07, 0.50165D-07, 0.68836D-07, 0.88418D-07, 0.10825D-06,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.10312D-09, 0.15095D-08,
     & 0.49813D-08, 0.10579D-07, 0.18050D-07, 0.27009D-07, 0.37034D-07,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.25462D-09, 0.13603D-08, 0.34372D-08, 0.64174D-08,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.14786D-10,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00/

      DO I=1,50
        DO J=1,30
          UDS_GRD(I,J) = UDS_GRDL(I,J)
          G_GRD(I,J)   = G_GRDL(I,J)
          C_GRD(I,J)   = C_GRDL(I,J)
        ENDDO
      ENDDO

      RETURN
      END
      SUBROUTINE I_NLO_Q3G1
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      PARAMETER (NQ2MAX=30,NZMAX=50)
      DIMENSION UDS_GRD(NZMAX,NQ2MAX),C_GRD(NZMAX,NQ2MAX),
     &          G_GRD(NZMAX,NQ2MAX)
      DIMENSION UDS_GRDL(NZMAX,NQ2MAX),C_GRDL(NZMAX,NQ2MAX),
     &          G_GRDL(NZMAX,NQ2MAX)
      COMMON /CPARAM/ UDS_GRD,C_GRD,G_GRD 

      data ((uds_grdl(i,j),j=1,30),i=1,10)/
     & 0.22512D-03, 0.23099D-02, 0.44126D-02, 0.65304D-02, 0.86608D-02,
     & 0.10801D-01, 0.12948D-01, 0.15102D-01, 0.17263D-01, 0.19427D-01,
     & 0.21594D-01, 0.23763D-01, 0.25933D-01, 0.28102D-01, 0.30270D-01,
     & 0.32437D-01, 0.34600D-01, 0.36761D-01, 0.38919D-01, 0.41072D-01,
     & 0.43219D-01, 0.45361D-01, 0.47497D-01, 0.49626D-01, 0.51749D-01,
     & 0.53867D-01, 0.55978D-01, 0.58080D-01, 0.60173D-01, 0.62258D-01,
     & 0.24680D-03, 0.22735D-02, 0.43129D-02, 0.63626D-02, 0.84205D-02,
     & 0.10485D-01, 0.12553D-01, 0.14624D-01, 0.16698D-01, 0.18773D-01,
     & 0.20847D-01, 0.22921D-01, 0.24993D-01, 0.27061D-01, 0.29126D-01,
     & 0.31186D-01, 0.33242D-01, 0.35292D-01, 0.37339D-01, 0.39378D-01,
     & 0.41410D-01, 0.43434D-01, 0.45450D-01, 0.47458D-01, 0.49457D-01,
     & 0.51453D-01, 0.53438D-01, 0.55413D-01, 0.57379D-01, 0.59335D-01,
     & 0.27352D-03, 0.22426D-02, 0.42196D-02, 0.62023D-02, 0.81892D-02,
     & 0.10179D-01, 0.12170D-01, 0.14160D-01, 0.16150D-01, 0.18137D-01,
     & 0.20121D-01, 0.22102D-01, 0.24079D-01, 0.26050D-01, 0.28015D-01,
     & 0.29973D-01, 0.31924D-01, 0.33868D-01, 0.35807D-01, 0.37737D-01,
     & 0.39657D-01, 0.41568D-01, 0.43469D-01, 0.45361D-01, 0.47243D-01,
     & 0.49119D-01, 0.50985D-01, 0.52839D-01, 0.54683D-01, 0.56515D-01,
     & 0.30663D-03, 0.22192D-02, 0.41348D-02, 0.60521D-02, 0.79696D-02,
     & 0.98863D-02, 0.11802D-01, 0.13714D-01, 0.15622D-01, 0.17524D-01,
     & 0.19421D-01, 0.21312D-01, 0.23198D-01, 0.25075D-01, 0.26943D-01,
     & 0.28803D-01, 0.30653D-01, 0.32495D-01, 0.34331D-01, 0.36155D-01,
     & 0.37969D-01, 0.39771D-01, 0.41563D-01, 0.43343D-01, 0.45113D-01,
     & 0.46877D-01, 0.48628D-01, 0.50367D-01, 0.52094D-01, 0.53809D-01,
     & 0.34785D-03, 0.22042D-02, 0.40593D-02, 0.59119D-02, 0.77611D-02,
     & 0.96063D-02, 0.11448D-01, 0.13283D-01, 0.15111D-01, 0.16930D-01,
     & 0.18741D-01, 0.20546D-01, 0.22342D-01, 0.24128D-01, 0.25903D-01,
     & 0.27667D-01, 0.29420D-01, 0.31163D-01, 0.32899D-01, 0.34621D-01,
     & 0.36332D-01, 0.38030D-01, 0.39715D-01, 0.41389D-01, 0.43051D-01,
     & 0.44706D-01, 0.46347D-01, 0.47976D-01, 0.49591D-01, 0.51194D-01,
     & 0.39944D-03, 0.21999D-02, 0.39950D-02, 0.57838D-02, 0.75658D-02,
     & 0.93406D-02, 0.11110D-01, 0.12870D-01, 0.14619D-01, 0.16357D-01,
     & 0.18085D-01, 0.19805D-01, 0.21514D-01, 0.23211D-01, 0.24895D-01,
     & 0.26566D-01, 0.28225D-01, 0.29872D-01, 0.31511D-01, 0.33136D-01,
     & 0.34746D-01, 0.36344D-01, 0.37928D-01, 0.39498D-01, 0.41057D-01,
     & 0.42607D-01, 0.44143D-01, 0.45666D-01, 0.47174D-01, 0.48669D-01,
     & 0.46435D-03, 0.22095D-02, 0.39453D-02, 0.56713D-02, 0.73871D-02,
     & 0.90929D-02, 0.10791D-01, 0.12477D-01, 0.14149D-01, 0.15809D-01,
     & 0.17455D-01, 0.19092D-01, 0.20717D-01, 0.22327D-01, 0.23923D-01,
     & 0.25504D-01, 0.27072D-01, 0.28627D-01, 0.30172D-01, 0.31702D-01,
     & 0.33217D-01, 0.34718D-01, 0.36204D-01, 0.37675D-01, 0.39134D-01,
     & 0.40584D-01, 0.42020D-01, 0.43440D-01, 0.44847D-01, 0.46239D-01,
     & 0.54647D-03, 0.22370D-02, 0.39145D-02, 0.55788D-02, 0.72299D-02,
     & 0.88683D-02, 0.10497D-01, 0.12110D-01, 0.13708D-01, 0.15291D-01,
     & 0.16859D-01, 0.18416D-01, 0.19959D-01, 0.21485D-01, 0.22996D-01,
     & 0.24491D-01, 0.25971D-01, 0.27437D-01, 0.28892D-01, 0.30331D-01,
     & 0.31754D-01, 0.33161D-01, 0.34554D-01, 0.35931D-01, 0.37294D-01,
     & 0.38649D-01, 0.39988D-01, 0.41312D-01, 0.42621D-01, 0.43915D-01,
     & 0.65085D-03, 0.22878D-02, 0.39083D-02, 0.55125D-02, 0.71005D-02,
     & 0.86733D-02, 0.10234D-01, 0.11778D-01, 0.13303D-01, 0.14812D-01,
     & 0.16304D-01, 0.17783D-01, 0.19247D-01, 0.20693D-01, 0.22121D-01,
     & 0.23533D-01, 0.24929D-01, 0.26310D-01, 0.27678D-01, 0.29030D-01,
     & 0.30365D-01, 0.31684D-01, 0.32986D-01, 0.34274D-01, 0.35547D-01,
     & 0.36810D-01, 0.38057D-01, 0.39289D-01, 0.40505D-01, 0.41707D-01,
     & 0.78410D-03, 0.23679D-02, 0.39319D-02, 0.54767D-02, 0.70027D-02,
     & 0.85109D-02, 0.10005D-01, 0.11480D-01, 0.12935D-01, 0.14371D-01,
     & 0.15789D-01, 0.17192D-01, 0.18579D-01, 0.19946D-01, 0.21296D-01,
     & 0.22627D-01, 0.23940D-01, 0.25239D-01, 0.26524D-01, 0.27791D-01,
     & 0.29041D-01, 0.30274D-01, 0.31491D-01, 0.32692D-01, 0.33877D-01,
     & 0.35053D-01, 0.36212D-01, 0.37355D-01, 0.38483D-01, 0.39596D-01/

      data ((uds_grdl(i,j),j=1,30),i=11,20)/
     & 0.95485D-03, 0.24860D-02, 0.39944D-02, 0.54807D-02, 0.69456D-02,
     & 0.83906D-02, 0.98198D-02, 0.11227D-01, 0.12613D-01, 0.13978D-01,
     & 0.15324D-01, 0.16653D-01, 0.17965D-01, 0.19257D-01, 0.20529D-01,
     & 0.21782D-01, 0.23016D-01, 0.24235D-01, 0.25439D-01, 0.26626D-01,
     & 0.27794D-01, 0.28944D-01, 0.30078D-01, 0.31196D-01, 0.32298D-01,
     & 0.33389D-01, 0.34464D-01, 0.35523D-01, 0.36566D-01, 0.37595D-01,
     & 0.11744D-02, 0.26540D-02, 0.41079D-02, 0.55371D-02, 0.69426D-02,
     & 0.83259D-02, 0.96916D-02, 0.11033D-01, 0.12352D-01, 0.13649D-01,
     & 0.14925D-01, 0.16183D-01, 0.17423D-01, 0.18641D-01, 0.19839D-01,
     & 0.21016D-01, 0.22175D-01, 0.23317D-01, 0.24445D-01, 0.25553D-01,
     & 0.26643D-01, 0.27715D-01, 0.28770D-01, 0.29808D-01, 0.30831D-01,
     & 0.31842D-01, 0.32837D-01, 0.33816D-01, 0.34779D-01, 0.35727D-01,
     & 0.14573D-02, 0.28854D-02, 0.42852D-02, 0.56578D-02, 0.70045D-02,
     & 0.83271D-02, 0.96300D-02, 0.10907D-01, 0.12160D-01, 0.13390D-01,
     & 0.14597D-01, 0.15786D-01, 0.16955D-01, 0.18102D-01, 0.19227D-01,
     & 0.20332D-01, 0.21417D-01, 0.22484D-01, 0.23537D-01, 0.24570D-01,
     & 0.25584D-01, 0.26580D-01, 0.27559D-01, 0.28521D-01, 0.29467D-01,
     & 0.30401D-01, 0.31319D-01, 0.32221D-01, 0.33107D-01, 0.33978D-01,
     & 0.18219D-02, 0.31993D-02, 0.45457D-02, 0.58629D-02, 0.71519D-02,
     & 0.84150D-02, 0.96566D-02, 0.10871D-01, 0.12060D-01, 0.13225D-01,
     & 0.14365D-01, 0.15486D-01, 0.16587D-01, 0.17664D-01, 0.18720D-01,
     & 0.19754D-01, 0.20769D-01, 0.21765D-01, 0.22745D-01, 0.23706D-01,
     & 0.24647D-01, 0.25571D-01, 0.26477D-01, 0.27366D-01, 0.28239D-01,
     & 0.29100D-01, 0.29944D-01, 0.30773D-01, 0.31586D-01, 0.32384D-01,
     & 0.22915D-02, 0.36187D-02, 0.49128D-02, 0.61754D-02, 0.74081D-02,
     & 0.86129D-02, 0.97945D-02, 0.10948D-01, 0.12074D-01, 0.13175D-01,
     & 0.14251D-01, 0.15307D-01, 0.16340D-01, 0.17351D-01, 0.18339D-01,
     & 0.19305D-01, 0.20251D-01, 0.21178D-01, 0.22089D-01, 0.22980D-01,
     & 0.23852D-01, 0.24705D-01, 0.25541D-01, 0.26361D-01, 0.27164D-01,
     & 0.27954D-01, 0.28729D-01, 0.29487D-01, 0.30231D-01, 0.30959D-01,
     & 0.28965D-02, 0.41720D-02, 0.54123D-02, 0.66191D-02, 0.77943D-02,
     & 0.89400D-02, 0.10061D-01, 0.11152D-01, 0.12216D-01, 0.13252D-01,
     & 0.14264D-01, 0.15254D-01, 0.16221D-01, 0.17165D-01, 0.18085D-01,
     & 0.18984D-01, 0.19862D-01, 0.20721D-01, 0.21563D-01, 0.22386D-01,
     & 0.23189D-01, 0.23974D-01, 0.24742D-01, 0.25493D-01, 0.26228D-01,
     & 0.26950D-01, 0.27656D-01, 0.28346D-01, 0.29022D-01, 0.29683D-01,
     & 0.36714D-02, 0.48945D-02, 0.60804D-02, 0.72311D-02, 0.83485D-02,
     & 0.94350D-02, 0.10495D-01, 0.11524D-01, 0.12525D-01, 0.13498D-01,
     & 0.14446D-01, 0.15370D-01, 0.16272D-01, 0.17150D-01, 0.18004D-01,
     & 0.18837D-01, 0.19648D-01, 0.20441D-01, 0.21216D-01, 0.21971D-01,
     & 0.22707D-01, 0.23426D-01, 0.24127D-01, 0.24811D-01, 0.25480D-01,
     & 0.26135D-01, 0.26775D-01, 0.27400D-01, 0.28010D-01, 0.28606D-01,
     & 0.46583D-02, 0.58267D-02, 0.69561D-02, 0.80489D-02, 0.91069D-02,
     & 0.10133D-01, 0.11130D-01, 0.12097D-01, 0.13034D-01, 0.13943D-01,
     & 0.14825D-01, 0.15684D-01, 0.16520D-01, 0.17331D-01, 0.18119D-01,
     & 0.18886D-01, 0.19631D-01, 0.20356D-01, 0.21064D-01, 0.21753D-01,
     & 0.22423D-01, 0.23076D-01, 0.23711D-01, 0.24330D-01, 0.24933D-01,
     & 0.25523D-01, 0.26098D-01, 0.26658D-01, 0.27204D-01, 0.27737D-01,
     & 0.59035D-02, 0.70131D-02, 0.80823D-02, 0.91135D-02, 0.10109D-01,
     & 0.11071D-01, 0.12003D-01, 0.12904D-01, 0.13774D-01, 0.14617D-01,
     & 0.15432D-01, 0.16224D-01, 0.16992D-01, 0.17735D-01, 0.18456D-01,
     & 0.19154D-01, 0.19832D-01, 0.20490D-01, 0.21131D-01, 0.21752D-01,
     & 0.22356D-01, 0.22942D-01, 0.23511D-01, 0.24064D-01, 0.24603D-01,
     & 0.25127D-01, 0.25637D-01, 0.26133D-01, 0.26616D-01, 0.27086D-01,
     & 0.74537D-02, 0.84981D-02, 0.95010D-02, 0.10465D-01, 0.11392D-01,
     & 0.12285D-01, 0.13147D-01, 0.13978D-01, 0.14778D-01, 0.15550D-01,
     & 0.16294D-01, 0.17015D-01, 0.17711D-01, 0.18384D-01, 0.19034D-01,
     & 0.19662D-01, 0.20270D-01, 0.20858D-01, 0.21429D-01, 0.21981D-01,
     & 0.22516D-01, 0.23034D-01, 0.23536D-01, 0.24022D-01, 0.24494D-01,
     & 0.24953D-01, 0.25397D-01, 0.25828D-01, 0.26246D-01, 0.26652D-01/

      data ((uds_grdl(i,j),j=1,30),i=21,30)/
     & 0.93478D-02, 0.10318D-01, 0.11246D-01, 0.12135D-01, 0.12987D-01,
     & 0.13803D-01, 0.14588D-01, 0.15341D-01, 0.16065D-01, 0.16760D-01,
     & 0.17428D-01, 0.18072D-01, 0.18691D-01, 0.19288D-01, 0.19863D-01,
     & 0.20416D-01, 0.20950D-01, 0.21465D-01, 0.21961D-01, 0.22441D-01,
     & 0.22904D-01, 0.23350D-01, 0.23782D-01, 0.24198D-01, 0.24601D-01,
     & 0.24991D-01, 0.25367D-01, 0.25731D-01, 0.26083D-01, 0.26424D-01,
     & 0.11605D-01, 0.12485D-01, 0.13324D-01, 0.14123D-01, 0.14885D-01,
     & 0.15613D-01, 0.16308D-01, 0.16972D-01, 0.17607D-01, 0.18214D-01,
     & 0.18796D-01, 0.19353D-01, 0.19887D-01, 0.20398D-01, 0.20889D-01,
     & 0.21359D-01, 0.21811D-01, 0.22245D-01, 0.22661D-01, 0.23061D-01,
     & 0.23445D-01, 0.23814D-01, 0.24169D-01, 0.24511D-01, 0.24839D-01,
     & 0.25155D-01, 0.25459D-01, 0.25751D-01, 0.26033D-01, 0.26304D-01,
     & 0.14186D-01, 0.14958D-01, 0.15690D-01, 0.16383D-01, 0.17040D-01,
     & 0.17663D-01, 0.18254D-01, 0.18815D-01, 0.19349D-01, 0.19856D-01,
     & 0.20339D-01, 0.20798D-01, 0.21235D-01, 0.21651D-01, 0.22048D-01,
     & 0.22427D-01, 0.22788D-01, 0.23132D-01, 0.23459D-01, 0.23772D-01,
     & 0.24071D-01, 0.24356D-01, 0.24628D-01, 0.24888D-01, 0.25137D-01,
     & 0.25373D-01, 0.25599D-01, 0.25815D-01, 0.26022D-01, 0.26219D-01,
     & 0.16971D-01, 0.17610D-01, 0.18211D-01, 0.18775D-01, 0.19306D-01,
     & 0.19804D-01, 0.20271D-01, 0.20712D-01, 0.21127D-01, 0.21518D-01,
     & 0.21886D-01, 0.22233D-01, 0.22559D-01, 0.22866D-01, 0.23157D-01,
     & 0.23431D-01, 0.23690D-01, 0.23934D-01, 0.24162D-01, 0.24378D-01,
     & 0.24582D-01, 0.24775D-01, 0.24956D-01, 0.25128D-01, 0.25289D-01,
     & 0.25439D-01, 0.25581D-01, 0.25714D-01, 0.25840D-01, 0.25958D-01,
     & 0.19693D-01, 0.20165D-01, 0.20603D-01, 0.21008D-01, 0.21384D-01,
     & 0.21731D-01, 0.22050D-01, 0.22345D-01, 0.22619D-01, 0.22873D-01,
     & 0.23108D-01, 0.23323D-01, 0.23520D-01, 0.23703D-01, 0.23872D-01,
     & 0.24028D-01, 0.24171D-01, 0.24301D-01, 0.24419D-01, 0.24527D-01,
     & 0.24626D-01, 0.24716D-01, 0.24798D-01, 0.24872D-01, 0.24938D-01,
     & 0.24995D-01, 0.25045D-01, 0.25090D-01, 0.25129D-01, 0.25163D-01,
     & 0.20289D-01, 0.20716D-01, 0.21109D-01, 0.21472D-01, 0.21806D-01,
     & 0.22113D-01, 0.22392D-01, 0.22649D-01, 0.22885D-01, 0.23103D-01,
     & 0.23302D-01, 0.23483D-01, 0.23647D-01, 0.23797D-01, 0.23935D-01,
     & 0.24060D-01, 0.24174D-01, 0.24275D-01, 0.24365D-01, 0.24446D-01,
     & 0.24518D-01, 0.24582D-01, 0.24639D-01, 0.24689D-01, 0.24731D-01,
     & 0.24765D-01, 0.24793D-01, 0.24816D-01, 0.24834D-01, 0.24847D-01,
     & 0.20826D-01, 0.21206D-01, 0.21555D-01, 0.21874D-01, 0.22167D-01,
     & 0.22433D-01, 0.22672D-01, 0.22891D-01, 0.23090D-01, 0.23272D-01,
     & 0.23436D-01, 0.23583D-01, 0.23714D-01, 0.23833D-01, 0.23939D-01,
     & 0.24035D-01, 0.24119D-01, 0.24193D-01, 0.24255D-01, 0.24309D-01,
     & 0.24355D-01, 0.24395D-01, 0.24427D-01, 0.24453D-01, 0.24473D-01,
     & 0.24484D-01, 0.24490D-01, 0.24492D-01, 0.24490D-01, 0.24483D-01,
     & 0.21300D-01, 0.21633D-01, 0.21937D-01, 0.22214D-01, 0.22464D-01,
     & 0.22689D-01, 0.22889D-01, 0.23070D-01, 0.23232D-01, 0.23378D-01,
     & 0.23509D-01, 0.23622D-01, 0.23720D-01, 0.23808D-01, 0.23884D-01,
     & 0.23950D-01, 0.24006D-01, 0.24052D-01, 0.24087D-01, 0.24115D-01,
     & 0.24136D-01, 0.24151D-01, 0.24160D-01, 0.24163D-01, 0.24160D-01,
     & 0.24150D-01, 0.24136D-01, 0.24117D-01, 0.24095D-01, 0.24069D-01,
     & 0.21708D-01, 0.21995D-01, 0.22254D-01, 0.22486D-01, 0.22695D-01,
     & 0.22880D-01, 0.23040D-01, 0.23183D-01, 0.23309D-01, 0.23420D-01,
     & 0.23516D-01, 0.23596D-01, 0.23663D-01, 0.23719D-01, 0.23766D-01,
     & 0.23803D-01, 0.23831D-01, 0.23850D-01, 0.23860D-01, 0.23862D-01,
     & 0.23859D-01, 0.23851D-01, 0.23837D-01, 0.23818D-01, 0.23794D-01,
     & 0.23763D-01, 0.23728D-01, 0.23691D-01, 0.23650D-01, 0.23606D-01,
     & 0.22049D-01, 0.22289D-01, 0.22502D-01, 0.22692D-01, 0.22858D-01,
     & 0.23003D-01, 0.23124D-01, 0.23230D-01, 0.23320D-01, 0.23396D-01,
     & 0.23458D-01, 0.23506D-01, 0.23542D-01, 0.23568D-01, 0.23585D-01,
     & 0.23594D-01, 0.23596D-01, 0.23589D-01, 0.23573D-01, 0.23551D-01,
     & 0.23525D-01, 0.23493D-01, 0.23457D-01, 0.23417D-01, 0.23373D-01,
     & 0.23322D-01, 0.23268D-01, 0.23212D-01, 0.23153D-01, 0.23092D-01/

      data ((uds_grdl(i,j),j=1,30),i=31,40)/
     & 0.22320D-01, 0.22513D-01, 0.22681D-01, 0.22827D-01, 0.22952D-01,
     & 0.23057D-01, 0.23140D-01, 0.23208D-01, 0.23262D-01, 0.23304D-01,
     & 0.23334D-01, 0.23350D-01, 0.23355D-01, 0.23352D-01, 0.23341D-01,
     & 0.23323D-01, 0.23298D-01, 0.23266D-01, 0.23225D-01, 0.23180D-01,
     & 0.23131D-01, 0.23078D-01, 0.23021D-01, 0.22961D-01, 0.22897D-01,
     & 0.22827D-01, 0.22755D-01, 0.22682D-01, 0.22606D-01, 0.22529D-01,
     & 0.22520D-01, 0.22666D-01, 0.22789D-01, 0.22892D-01, 0.22975D-01,
     & 0.23040D-01, 0.23085D-01, 0.23117D-01, 0.23136D-01, 0.23144D-01,
     & 0.23142D-01, 0.23127D-01, 0.23102D-01, 0.23070D-01, 0.23032D-01,
     & 0.22988D-01, 0.22938D-01, 0.22881D-01, 0.22817D-01, 0.22750D-01,
     & 0.22679D-01, 0.22605D-01, 0.22528D-01, 0.22449D-01, 0.22366D-01,
     & 0.22279D-01, 0.22190D-01, 0.22099D-01, 0.22008D-01, 0.21915D-01,
     & 0.22648D-01, 0.22746D-01, 0.22824D-01, 0.22884D-01, 0.22927D-01,
     & 0.22953D-01, 0.22960D-01, 0.22955D-01, 0.22941D-01, 0.22916D-01,
     & 0.22882D-01, 0.22837D-01, 0.22783D-01, 0.22724D-01, 0.22659D-01,
     & 0.22589D-01, 0.22515D-01, 0.22435D-01, 0.22348D-01, 0.22259D-01,
     & 0.22168D-01, 0.22074D-01, 0.21979D-01, 0.21881D-01, 0.21781D-01,
     & 0.21677D-01, 0.21572D-01, 0.21466D-01, 0.21360D-01, 0.21253D-01,
     & 0.22702D-01, 0.22753D-01, 0.22787D-01, 0.22804D-01, 0.22806D-01,
     & 0.22793D-01, 0.22763D-01, 0.22724D-01, 0.22675D-01, 0.22618D-01,
     & 0.22554D-01, 0.22480D-01, 0.22398D-01, 0.22312D-01, 0.22222D-01,
     & 0.22128D-01, 0.22030D-01, 0.21928D-01, 0.21820D-01, 0.21711D-01,
     & 0.21600D-01, 0.21488D-01, 0.21374D-01, 0.21259D-01, 0.21143D-01,
     & 0.21023D-01, 0.20903D-01, 0.20783D-01, 0.20663D-01, 0.20543D-01,
     & 0.22682D-01, 0.22687D-01, 0.22676D-01, 0.22651D-01, 0.22614D-01,
     & 0.22563D-01, 0.22497D-01, 0.22423D-01, 0.22342D-01, 0.22254D-01,
     & 0.22160D-01, 0.22057D-01, 0.21949D-01, 0.21837D-01, 0.21722D-01,
     & 0.21605D-01, 0.21485D-01, 0.21362D-01, 0.21234D-01, 0.21106D-01,
     & 0.20976D-01, 0.20847D-01, 0.20716D-01, 0.20585D-01, 0.20454D-01,
     & 0.20320D-01, 0.20186D-01, 0.20053D-01, 0.19920D-01, 0.19788D-01,
     & 0.22582D-01, 0.22539D-01, 0.22484D-01, 0.22417D-01, 0.22340D-01,
     & 0.22252D-01, 0.22150D-01, 0.22042D-01, 0.21929D-01, 0.21811D-01,
     & 0.21688D-01, 0.21558D-01, 0.21424D-01, 0.21287D-01, 0.21149D-01,
     & 0.21010D-01, 0.20869D-01, 0.20725D-01, 0.20579D-01, 0.20432D-01,
     & 0.20286D-01, 0.20140D-01, 0.19994D-01, 0.19849D-01, 0.19703D-01,
     & 0.19556D-01, 0.19409D-01, 0.19264D-01, 0.19120D-01, 0.18977D-01,
     & 0.22407D-01, 0.22319D-01, 0.22220D-01, 0.22112D-01, 0.21996D-01,
     & 0.21871D-01, 0.21735D-01, 0.21594D-01, 0.21450D-01, 0.21303D-01,
     & 0.21152D-01, 0.20997D-01, 0.20838D-01, 0.20678D-01, 0.20518D-01,
     & 0.20357D-01, 0.20196D-01, 0.20035D-01, 0.19870D-01, 0.19707D-01,
     & 0.19545D-01, 0.19384D-01, 0.19224D-01, 0.19065D-01, 0.18907D-01,
     & 0.18748D-01, 0.18590D-01, 0.18434D-01, 0.18280D-01, 0.18127D-01,
     & 0.22157D-01, 0.22022D-01, 0.21881D-01, 0.21732D-01, 0.21578D-01,
     & 0.21417D-01, 0.21247D-01, 0.21075D-01, 0.20902D-01, 0.20726D-01,
     & 0.20550D-01, 0.20369D-01, 0.20187D-01, 0.20005D-01, 0.19825D-01,
     & 0.19645D-01, 0.19465D-01, 0.19286D-01, 0.19106D-01, 0.18927D-01,
     & 0.18751D-01, 0.18576D-01, 0.18403D-01, 0.18232D-01, 0.18063D-01,
     & 0.17893D-01, 0.17726D-01, 0.17560D-01, 0.17397D-01, 0.17237D-01,
     & 0.21828D-01, 0.21648D-01, 0.21464D-01, 0.21276D-01, 0.21084D-01,
     & 0.20889D-01, 0.20686D-01, 0.20484D-01, 0.20282D-01, 0.20080D-01,
     & 0.19878D-01, 0.19674D-01, 0.19470D-01, 0.19268D-01, 0.19068D-01,
     & 0.18870D-01, 0.18673D-01, 0.18478D-01, 0.18283D-01, 0.18091D-01,
     & 0.17902D-01, 0.17715D-01, 0.17531D-01, 0.17349D-01, 0.17170D-01,
     & 0.16991D-01, 0.16815D-01, 0.16641D-01, 0.16471D-01, 0.16303D-01,
     & 0.21420D-01, 0.21195D-01, 0.20969D-01, 0.20742D-01, 0.20514D-01,
     & 0.20285D-01, 0.20051D-01, 0.19819D-01, 0.19590D-01, 0.19362D-01,
     & 0.19137D-01, 0.18911D-01, 0.18686D-01, 0.18465D-01, 0.18247D-01,
     & 0.18032D-01, 0.17821D-01, 0.17611D-01, 0.17403D-01, 0.17198D-01,
     & 0.16998D-01, 0.16800D-01, 0.16606D-01, 0.16415D-01, 0.16227D-01,
     & 0.16041D-01, 0.15857D-01, 0.15678D-01, 0.15501D-01, 0.15328D-01/

      data ((uds_grdl(i,j),j=1,30),i=41,50)/
     & 0.20930D-01, 0.20661D-01, 0.20393D-01, 0.20128D-01, 0.19864D-01,
     & 0.19602D-01, 0.19338D-01, 0.19079D-01, 0.18823D-01, 0.18572D-01,
     & 0.18324D-01, 0.18077D-01, 0.17834D-01, 0.17596D-01, 0.17361D-01,
     & 0.17132D-01, 0.16906D-01, 0.16684D-01, 0.16464D-01, 0.16249D-01,
     & 0.16038D-01, 0.15832D-01, 0.15629D-01, 0.15431D-01, 0.15236D-01,
     & 0.15043D-01, 0.14855D-01, 0.14670D-01, 0.14489D-01, 0.14312D-01,
     & 0.20356D-01, 0.20042D-01, 0.19734D-01, 0.19431D-01, 0.19133D-01,
     & 0.18840D-01, 0.18547D-01, 0.18261D-01, 0.17981D-01, 0.17707D-01,
     & 0.17438D-01, 0.17173D-01, 0.16912D-01, 0.16658D-01, 0.16409D-01,
     & 0.16166D-01, 0.15928D-01, 0.15695D-01, 0.15466D-01, 0.15241D-01,
     & 0.15022D-01, 0.14809D-01, 0.14599D-01, 0.14395D-01, 0.14195D-01,
     & 0.13998D-01, 0.13805D-01, 0.13617D-01, 0.13433D-01, 0.13253D-01,
     & 0.19691D-01, 0.19334D-01, 0.18985D-01, 0.18646D-01, 0.18314D-01,
     & 0.17990D-01, 0.17670D-01, 0.17359D-01, 0.17056D-01, 0.16761D-01,
     & 0.16473D-01, 0.16190D-01, 0.15915D-01, 0.15646D-01, 0.15385D-01,
     & 0.15130D-01, 0.14882D-01, 0.14640D-01, 0.14402D-01, 0.14171D-01,
     & 0.13946D-01, 0.13726D-01, 0.13512D-01, 0.13304D-01, 0.13100D-01,
     & 0.12900D-01, 0.12705D-01, 0.12515D-01, 0.12330D-01, 0.12149D-01,
     & 0.18925D-01, 0.18525D-01, 0.18137D-01, 0.17761D-01, 0.17397D-01,
     & 0.17044D-01, 0.16698D-01, 0.16363D-01, 0.16039D-01, 0.15724D-01,
     & 0.15419D-01, 0.15121D-01, 0.14832D-01, 0.14552D-01, 0.14279D-01,
     & 0.14015D-01, 0.13759D-01, 0.13509D-01, 0.13266D-01, 0.13030D-01,
     & 0.12800D-01, 0.12577D-01, 0.12360D-01, 0.12149D-01, 0.11943D-01,
     & 0.11743D-01, 0.11548D-01, 0.11358D-01, 0.11173D-01, 0.10993D-01,
     & 0.18031D-01, 0.17588D-01, 0.17162D-01, 0.16751D-01, 0.16356D-01,
     & 0.15974D-01, 0.15604D-01, 0.15248D-01, 0.14904D-01, 0.14573D-01,
     & 0.14252D-01, 0.13942D-01, 0.13642D-01, 0.13352D-01, 0.13071D-01,
     & 0.12800D-01, 0.12538D-01, 0.12284D-01, 0.12037D-01, 0.11798D-01,
     & 0.11567D-01, 0.11343D-01, 0.11125D-01, 0.10914D-01, 0.10710D-01,
     & 0.10511D-01, 0.10318D-01, 0.10130D-01, 0.99483D-02, 0.97713D-02,
     & 0.16981D-01, 0.16493D-01, 0.16027D-01, 0.15581D-01, 0.15155D-01,
     & 0.14747D-01, 0.14356D-01, 0.13981D-01, 0.13621D-01, 0.13276D-01,
     & 0.12944D-01, 0.12624D-01, 0.12317D-01, 0.12022D-01, 0.11737D-01,
     & 0.11463D-01, 0.11198D-01, 0.10943D-01, 0.10697D-01, 0.10460D-01,
     & 0.10230D-01, 0.10008D-01, 0.97939D-02, 0.95864D-02, 0.93857D-02,
     & 0.91919D-02, 0.90041D-02, 0.88221D-02, 0.86456D-02, 0.84745D-02,
     & 0.15730D-01, 0.15213D-01, 0.14721D-01, 0.14251D-01, 0.13804D-01,
     & 0.13376D-01, 0.12968D-01, 0.12578D-01, 0.12205D-01, 0.11848D-01,
     & 0.11506D-01, 0.11179D-01, 0.10866D-01, 0.10565D-01, 0.10277D-01,
     & 0.99995D-02, 0.97332D-02, 0.94775D-02, 0.92324D-02, 0.89963D-02,
     & 0.87689D-02, 0.85499D-02, 0.83388D-02, 0.81353D-02, 0.79392D-02,
     & 0.77510D-02, 0.75692D-02, 0.73935D-02, 0.72237D-02, 0.70595D-02,
     & 0.14416D-01, 0.13778D-01, 0.13179D-01, 0.12618D-01, 0.12090D-01,
     & 0.11596D-01, 0.11137D-01, 0.10703D-01, 0.10293D-01, 0.99058D-02,
     & 0.95392D-02, 0.91955D-02, 0.88715D-02, 0.85632D-02, 0.82700D-02,
     & 0.79909D-02, 0.77253D-02, 0.74736D-02, 0.72365D-02, 0.70097D-02,
     & 0.67927D-02, 0.65850D-02, 0.63864D-02, 0.61963D-02, 0.60146D-02,
     & 0.58432D-02, 0.56785D-02, 0.55202D-02, 0.53679D-02, 0.52215D-02,
     & 0.97887D-02, 0.92603D-02, 0.87687D-02, 0.83116D-02, 0.78867D-02,
     & 0.74930D-02, 0.71344D-02, 0.67981D-02, 0.64826D-02, 0.61868D-02,
     & 0.59094D-02, 0.56534D-02, 0.54148D-02, 0.51892D-02, 0.49760D-02,
     & 0.47746D-02, 0.45842D-02, 0.44056D-02, 0.42398D-02, 0.40820D-02,
     & 0.39319D-02, 0.37890D-02, 0.36531D-02, 0.35238D-02, 0.34011D-02,
     & 0.32872D-02, 0.31781D-02, 0.30737D-02, 0.29738D-02, 0.28782D-02,
     & 0.61584D-03, 0.57915D-03, 0.54515D-03, 0.51366D-03, 0.48451D-03,
     & 0.45764D-03, 0.43338D-03, 0.41071D-03, 0.38952D-03, 0.36973D-03,
     & 0.35126D-03, 0.33433D-03, 0.31864D-03, 0.30387D-03, 0.28995D-03,
     & 0.27684D-03, 0.26450D-03, 0.25299D-03, 0.24238D-03, 0.23232D-03,
     & 0.22277D-03, 0.21372D-03, 0.20513D-03, 0.19699D-03, 0.18929D-03,
     & 0.18221D-03, 0.17545D-03, 0.16900D-03, 0.16284D-03, 0.15696D-03/

      data ((g_grdl(i,j),j=1,30),i=1,10)/
     & 0.66473D+00, 0.72970D+00, 0.79455D+00, 0.85925D+00, 0.92376D+00,
     & 0.98811D+00, 0.10525D+01, 0.11165D+01, 0.11800D+01, 0.12432D+01,
     & 0.13059D+01, 0.13684D+01, 0.14306D+01, 0.14921D+01, 0.15532D+01,
     & 0.16137D+01, 0.16736D+01, 0.17331D+01, 0.17924D+01, 0.18510D+01,
     & 0.19089D+01, 0.19663D+01, 0.20231D+01, 0.20792D+01, 0.21349D+01,
     & 0.21903D+01, 0.22451D+01, 0.22992D+01, 0.23527D+01, 0.24056D+01,
     & 0.66468D+00, 0.72693D+00, 0.78889D+00, 0.85055D+00, 0.91187D+00,
     & 0.97291D+00, 0.10339D+01, 0.10943D+01, 0.11542D+01, 0.12136D+01,
     & 0.12725D+01, 0.13311D+01, 0.13893D+01, 0.14469D+01, 0.15038D+01,
     & 0.15601D+01, 0.16158D+01, 0.16711D+01, 0.17260D+01, 0.17802D+01,
     & 0.18338D+01, 0.18868D+01, 0.19391D+01, 0.19908D+01, 0.20420D+01,
     & 0.20929D+01, 0.21431D+01, 0.21927D+01, 0.22417D+01, 0.22900D+01,
     & 0.66466D+00, 0.72417D+00, 0.78324D+00, 0.84187D+00, 0.90004D+00,
     & 0.95780D+00, 0.10154D+01, 0.10723D+01, 0.11287D+01, 0.11845D+01,
     & 0.12396D+01, 0.12944D+01, 0.13488D+01, 0.14024D+01, 0.14553D+01,
     & 0.15077D+01, 0.15593D+01, 0.16105D+01, 0.16613D+01, 0.17113D+01,
     & 0.17607D+01, 0.18095D+01, 0.18575D+01, 0.19050D+01, 0.19519D+01,
     & 0.19985D+01, 0.20444D+01, 0.20897D+01, 0.21343D+01, 0.21784D+01,
     & 0.66471D+00, 0.72151D+00, 0.77773D+00, 0.83337D+00, 0.88844D+00,
     & 0.94300D+00, 0.99728D+00, 0.10509D+01, 0.11037D+01, 0.11560D+01,
     & 0.12075D+01, 0.12587D+01, 0.13093D+01, 0.13591D+01, 0.14083D+01,
     & 0.14567D+01, 0.15045D+01, 0.15518D+01, 0.15986D+01, 0.16447D+01,
     & 0.16901D+01, 0.17348D+01, 0.17789D+01, 0.18223D+01, 0.18651D+01,
     & 0.19077D+01, 0.19495D+01, 0.19907D+01, 0.20313D+01, 0.20712D+01,
     & 0.66471D+00, 0.71877D+00, 0.77213D+00, 0.82481D+00, 0.87680D+00,
     & 0.92820D+00, 0.97922D+00, 0.10295D+01, 0.10789D+01, 0.11277D+01,
     & 0.11758D+01, 0.12233D+01, 0.12703D+01, 0.13165D+01, 0.13620D+01,
     & 0.14067D+01, 0.14507D+01, 0.14942D+01, 0.15372D+01, 0.15795D+01,
     & 0.16210D+01, 0.16619D+01, 0.17021D+01, 0.17417D+01, 0.17807D+01,
     & 0.18193D+01, 0.18573D+01, 0.18946D+01, 0.19314D+01, 0.19675D+01,
     & 0.66466D+00, 0.71598D+00, 0.76648D+00, 0.81619D+00, 0.86514D+00,
     & 0.91340D+00, 0.96120D+00, 0.10082D+01, 0.10543D+01, 0.10997D+01,
     & 0.11443D+01, 0.11884D+01, 0.12319D+01, 0.12745D+01, 0.13164D+01,
     & 0.13575D+01, 0.13980D+01, 0.14378D+01, 0.14771D+01, 0.15157D+01,
     & 0.15536D+01, 0.15908D+01, 0.16274D+01, 0.16633D+01, 0.16986D+01,
     & 0.17336D+01, 0.17679D+01, 0.18015D+01, 0.18346D+01, 0.18670D+01,
     & 0.66456D+00, 0.71313D+00, 0.76079D+00, 0.80757D+00, 0.85350D+00,
     & 0.89867D+00, 0.94332D+00, 0.98707D+00, 0.10300D+01, 0.10720D+01,
     & 0.11133D+01, 0.11540D+01, 0.11941D+01, 0.12333D+01, 0.12717D+01,
     & 0.13094D+01, 0.13463D+01, 0.13827D+01, 0.14185D+01, 0.14536D+01,
     & 0.14880D+01, 0.15217D+01, 0.15547D+01, 0.15872D+01, 0.16190D+01,
     & 0.16504D+01, 0.16812D+01, 0.17114D+01, 0.17410D+01, 0.17700D+01,
     & 0.66450D+00, 0.71031D+00, 0.75512D+00, 0.79898D+00, 0.84194D+00,
     & 0.88406D+00, 0.92560D+00, 0.96620D+00, 0.10059D+01, 0.10448D+01,
     & 0.10828D+01, 0.11202D+01, 0.11570D+01, 0.11929D+01, 0.12280D+01,
     & 0.12623D+01, 0.12959D+01, 0.13289D+01, 0.13614D+01, 0.13932D+01,
     & 0.14242D+01, 0.14546D+01, 0.14843D+01, 0.15134D+01, 0.15419D+01,
     & 0.15701D+01, 0.15976D+01, 0.16245D+01, 0.16508D+01, 0.16766D+01,
     & 0.66450D+00, 0.70756D+00, 0.74956D+00, 0.79055D+00, 0.83057D+00,
     & 0.86972D+00, 0.90821D+00, 0.94573D+00, 0.98235D+00, 0.10181D+01,
     & 0.10530D+01, 0.10873D+01, 0.11208D+01, 0.11535D+01, 0.11854D+01,
     & 0.12166D+01, 0.12470D+01, 0.12768D+01, 0.13061D+01, 0.13347D+01,
     & 0.13625D+01, 0.13898D+01, 0.14163D+01, 0.14423D+01, 0.14677D+01,
     & 0.14927D+01, 0.15172D+01, 0.15410D+01, 0.15643D+01, 0.15870D+01,
     & 0.66441D+00, 0.70471D+00, 0.74388D+00, 0.78200D+00, 0.81912D+00,
     & 0.85531D+00, 0.89079D+00, 0.92530D+00, 0.95887D+00, 0.99156D+00,
     & 0.10234D+01, 0.10546D+01, 0.10850D+01, 0.11146D+01, 0.11435D+01,
     & 0.11716D+01, 0.11989D+01, 0.12257D+01, 0.12519D+01, 0.12774D+01,
     & 0.13022D+01, 0.13264D+01, 0.13500D+01, 0.13730D+01, 0.13955D+01,
     & 0.14175D+01, 0.14390D+01, 0.14599D+01, 0.14804D+01, 0.15003D+01/

      data ((g_grdl(i,j),j=1,30),i=11,20)/
     & 0.66425D+00, 0.70176D+00, 0.73811D+00, 0.77337D+00, 0.80759D+00,
     & 0.84086D+00, 0.87338D+00, 0.90491D+00, 0.93551D+00, 0.96522D+00,
     & 0.99408D+00, 0.10223D+01, 0.10497D+01, 0.10763D+01, 0.11022D+01,
     & 0.11273D+01, 0.11517D+01, 0.11755D+01, 0.11988D+01, 0.12214D+01,
     & 0.12433D+01, 0.12647D+01, 0.12854D+01, 0.13056D+01, 0.13253D+01,
     & 0.13445D+01, 0.13632D+01, 0.13814D+01, 0.13991D+01, 0.14163D+01,
     & 0.66414D+00, 0.69889D+00, 0.73245D+00, 0.76488D+00, 0.79627D+00,
     & 0.82668D+00, 0.85630D+00, 0.88494D+00, 0.91265D+00, 0.93947D+00,
     & 0.96546D+00, 0.99074D+00, 0.10153D+01, 0.10391D+01, 0.10621D+01,
     & 0.10844D+01, 0.11060D+01, 0.11270D+01, 0.11475D+01, 0.11673D+01,
     & 0.11865D+01, 0.12052D+01, 0.12232D+01, 0.12408D+01, 0.12578D+01,
     & 0.12744D+01, 0.12905D+01, 0.13062D+01, 0.13213D+01, 0.13361D+01,
     & 0.66394D+00, 0.69590D+00, 0.72666D+00, 0.75628D+00, 0.78485D+00,
     & 0.81243D+00, 0.83919D+00, 0.86499D+00, 0.88986D+00, 0.91386D+00,
     & 0.93704D+00, 0.95951D+00, 0.98127D+00, 0.10023D+01, 0.10225D+01,
     & 0.10421D+01, 0.10610D+01, 0.10794D+01, 0.10972D+01, 0.11143D+01,
     & 0.11310D+01, 0.11470D+01, 0.11625D+01, 0.11776D+01, 0.11921D+01,
     & 0.12063D+01, 0.12199D+01, 0.12331D+01, 0.12459D+01, 0.12583D+01,
     & 0.66392D+00, 0.69310D+00, 0.72108D+00, 0.74793D+00, 0.77372D+00,
     & 0.79853D+00, 0.82250D+00, 0.84552D+00, 0.86764D+00, 0.88891D+00,
     & 0.90938D+00, 0.92915D+00, 0.94822D+00, 0.96655D+00, 0.98420D+00,
     & 0.10012D+01, 0.10176D+01, 0.10333D+01, 0.10486D+01, 0.10633D+01,
     & 0.10775D+01, 0.10911D+01, 0.11042D+01, 0.11169D+01, 0.11291D+01,
     & 0.11409D+01, 0.11523D+01, 0.11633D+01, 0.11739D+01, 0.11841D+01,
     & 0.66409D+00, 0.69049D+00, 0.71570D+00, 0.73980D+00, 0.76285D+00,
     & 0.78492D+00, 0.80615D+00, 0.82646D+00, 0.84590D+00, 0.86453D+00,
     & 0.88237D+00, 0.89953D+00, 0.91600D+00, 0.93178D+00, 0.94691D+00,
     & 0.96141D+00, 0.97533D+00, 0.98869D+00, 0.10016D+01, 0.10139D+01,
     & 0.10257D+01, 0.10371D+01, 0.10479D+01, 0.10584D+01, 0.10684D+01,
     & 0.10781D+01, 0.10873D+01, 0.10962D+01, 0.11047D+01, 0.11129D+01,
     & 0.66384D+00, 0.68744D+00, 0.70987D+00, 0.73121D+00, 0.75153D+00,
     & 0.77090D+00, 0.78942D+00, 0.80706D+00, 0.82387D+00, 0.83989D+00,
     & 0.85517D+00, 0.86979D+00, 0.88374D+00, 0.89705D+00, 0.90975D+00,
     & 0.92186D+00, 0.93343D+00, 0.94447D+00, 0.95503D+00, 0.96510D+00,
     & 0.97471D+00, 0.98389D+00, 0.99264D+00, 0.10010D+01, 0.10090D+01,
     & 0.10166D+01, 0.10238D+01, 0.10307D+01, 0.10373D+01, 0.10436D+01,
     & 0.66367D+00, 0.68446D+00, 0.70412D+00, 0.72273D+00, 0.74035D+00,
     & 0.75705D+00, 0.77291D+00, 0.78793D+00, 0.80217D+00, 0.81566D+00,
     & 0.82846D+00, 0.84061D+00, 0.85214D+00, 0.86307D+00, 0.87343D+00,
     & 0.88326D+00, 0.89257D+00, 0.90141D+00, 0.90978D+00, 0.91771D+00,
     & 0.92523D+00, 0.93235D+00, 0.93909D+00, 0.94548D+00, 0.95151D+00,
     & 0.95720D+00, 0.96259D+00, 0.96767D+00, 0.97247D+00, 0.97700D+00,
     & 0.66346D+00, 0.68143D+00, 0.69832D+00, 0.71420D+00, 0.72914D+00,
     & 0.74320D+00, 0.75643D+00, 0.76888D+00, 0.78060D+00, 0.79163D+00,
     & 0.80201D+00, 0.81177D+00, 0.82095D+00, 0.82958D+00, 0.83770D+00,
     & 0.84533D+00, 0.85250D+00, 0.85922D+00, 0.86551D+00, 0.87141D+00,
     & 0.87695D+00, 0.88213D+00, 0.88697D+00, 0.89150D+00, 0.89572D+00,
     & 0.89962D+00, 0.90325D+00, 0.90663D+00, 0.90976D+00, 0.91266D+00,
     & 0.66324D+00, 0.67836D+00, 0.69247D+00, 0.70563D+00, 0.71790D+00,
     & 0.72934D+00, 0.73998D+00, 0.74991D+00, 0.75916D+00, 0.76778D+00,
     & 0.77580D+00, 0.78324D+00, 0.79014D+00, 0.79656D+00, 0.80251D+00,
     & 0.80803D+00, 0.81314D+00, 0.81785D+00, 0.82216D+00, 0.82613D+00,
     & 0.82978D+00, 0.83312D+00, 0.83618D+00, 0.83896D+00, 0.84148D+00,
     & 0.84370D+00, 0.84570D+00, 0.84748D+00, 0.84906D+00, 0.85044D+00,
     & 0.66295D+00, 0.67521D+00, 0.68651D+00, 0.69694D+00, 0.70655D+00,
     & 0.71538D+00, 0.72346D+00, 0.73089D+00, 0.73771D+00, 0.74396D+00,
     & 0.74967D+00, 0.75485D+00, 0.75954D+00, 0.76381D+00, 0.76767D+00,
     & 0.77116D+00, 0.77429D+00, 0.77706D+00, 0.77947D+00, 0.78160D+00,
     & 0.78346D+00, 0.78507D+00, 0.78643D+00, 0.78757D+00, 0.78848D+00,
     & 0.78912D+00, 0.78959D+00, 0.78988D+00, 0.79002D+00, 0.79000D+00/

      data ((g_grdl(i,j),j=1,30),i=21,30)/
     & 0.66257D+00, 0.67190D+00, 0.68036D+00, 0.68803D+00, 0.69495D+00,
     & 0.70117D+00, 0.70668D+00, 0.71163D+00, 0.71603D+00, 0.71994D+00,
     & 0.72338D+00, 0.72633D+00, 0.72886D+00, 0.73103D+00, 0.73286D+00,
     & 0.73437D+00, 0.73558D+00, 0.73649D+00, 0.73707D+00, 0.73744D+00,
     & 0.73758D+00, 0.73753D+00, 0.73728D+00, 0.73685D+00, 0.73624D+00,
     & 0.73540D+00, 0.73443D+00, 0.73333D+00, 0.73211D+00, 0.73078D+00,
     & 0.66155D+00, 0.66786D+00, 0.67341D+00, 0.67825D+00, 0.68244D+00,
     & 0.68600D+00, 0.68892D+00, 0.69135D+00, 0.69334D+00, 0.69490D+00,
     & 0.69607D+00, 0.69681D+00, 0.69719D+00, 0.69728D+00, 0.69711D+00,
     & 0.69668D+00, 0.69602D+00, 0.69511D+00, 0.69393D+00, 0.69258D+00,
     & 0.69107D+00, 0.68942D+00, 0.68763D+00, 0.68571D+00, 0.68366D+00,
     & 0.68141D+00, 0.67908D+00, 0.67666D+00, 0.67417D+00, 0.67161D+00,
     & 0.66072D+00, 0.66389D+00, 0.66642D+00, 0.66836D+00, 0.66974D+00,
     & 0.67059D+00, 0.67086D+00, 0.67075D+00, 0.67029D+00, 0.66949D+00,
     & 0.66838D+00, 0.66691D+00, 0.66515D+00, 0.66319D+00, 0.66104D+00,
     & 0.65870D+00, 0.65620D+00, 0.65351D+00, 0.65060D+00, 0.64760D+00,
     & 0.64450D+00, 0.64131D+00, 0.63803D+00, 0.63468D+00, 0.63126D+00,
     & 0.62767D+00, 0.62405D+00, 0.62040D+00, 0.61672D+00, 0.61302D+00,
     & 0.65988D+00, 0.65971D+00, 0.65904D+00, 0.65790D+00, 0.65633D+00,
     & 0.65433D+00, 0.65184D+00, 0.64909D+00, 0.64608D+00, 0.64285D+00,
     & 0.63940D+00, 0.63567D+00, 0.63173D+00, 0.62768D+00, 0.62353D+00,
     & 0.61927D+00, 0.61493D+00, 0.61046D+00, 0.60584D+00, 0.60120D+00,
     & 0.59653D+00, 0.59183D+00, 0.58711D+00, 0.58238D+00, 0.57762D+00,
     & 0.57276D+00, 0.56791D+00, 0.56309D+00, 0.55829D+00, 0.55350D+00,
     & 0.65807D+00, 0.65418D+00, 0.64995D+00, 0.64541D+00, 0.64059D+00,
     & 0.63549D+00, 0.63000D+00, 0.62438D+00, 0.61864D+00, 0.61280D+00,
     & 0.60686D+00, 0.60073D+00, 0.59450D+00, 0.58827D+00, 0.58202D+00,
     & 0.57578D+00, 0.56954D+00, 0.56326D+00, 0.55691D+00, 0.55061D+00,
     & 0.54436D+00, 0.53816D+00, 0.53202D+00, 0.52593D+00, 0.51987D+00,
     & 0.51377D+00, 0.50776D+00, 0.50182D+00, 0.49595D+00, 0.49016D+00,
     & 0.65751D+00, 0.65264D+00, 0.64748D+00, 0.64206D+00, 0.63640D+00,
     & 0.63049D+00, 0.62422D+00, 0.61786D+00, 0.61142D+00, 0.60491D+00,
     & 0.59833D+00, 0.59160D+00, 0.58479D+00, 0.57800D+00, 0.57124D+00,
     & 0.56450D+00, 0.55779D+00, 0.55107D+00, 0.54429D+00, 0.53759D+00,
     & 0.53096D+00, 0.52440D+00, 0.51791D+00, 0.51149D+00, 0.50514D+00,
     & 0.49875D+00, 0.49246D+00, 0.48626D+00, 0.48015D+00, 0.47413D+00,
     & 0.65690D+00, 0.65105D+00, 0.64495D+00, 0.63864D+00, 0.63213D+00,
     & 0.62541D+00, 0.61837D+00, 0.61128D+00, 0.60414D+00, 0.59697D+00,
     & 0.58977D+00, 0.58244D+00, 0.57506D+00, 0.56774D+00, 0.56046D+00,
     & 0.55324D+00, 0.54607D+00, 0.53892D+00, 0.53173D+00, 0.52463D+00,
     & 0.51764D+00, 0.51073D+00, 0.50392D+00, 0.49719D+00, 0.49054D+00,
     & 0.48387D+00, 0.47732D+00, 0.47088D+00, 0.46454D+00, 0.45831D+00,
     & 0.65625D+00, 0.64940D+00, 0.64236D+00, 0.63516D+00, 0.62780D+00,
     & 0.62027D+00, 0.61246D+00, 0.60463D+00, 0.59680D+00, 0.58897D+00,
     & 0.58115D+00, 0.57323D+00, 0.56528D+00, 0.55743D+00, 0.54965D+00,
     & 0.54195D+00, 0.53433D+00, 0.52676D+00, 0.51916D+00, 0.51169D+00,
     & 0.50434D+00, 0.49710D+00, 0.48997D+00, 0.48294D+00, 0.47601D+00,
     & 0.46909D+00, 0.46230D+00, 0.45562D+00, 0.44907D+00, 0.44263D+00,
     & 0.65554D+00, 0.64767D+00, 0.63968D+00, 0.63157D+00, 0.62335D+00,
     & 0.61501D+00, 0.60642D+00, 0.59786D+00, 0.58934D+00, 0.58085D+00,
     & 0.57241D+00, 0.56390D+00, 0.55540D+00, 0.54702D+00, 0.53875D+00,
     & 0.53058D+00, 0.52253D+00, 0.51453D+00, 0.50655D+00, 0.49872D+00,
     & 0.49102D+00, 0.48345D+00, 0.47602D+00, 0.46871D+00, 0.46151D+00,
     & 0.45434D+00, 0.44732D+00, 0.44043D+00, 0.43368D+00, 0.42705D+00,
     & 0.65475D+00, 0.64586D+00, 0.63690D+00, 0.62787D+00, 0.61878D+00,
     & 0.60962D+00, 0.60024D+00, 0.59094D+00, 0.58172D+00, 0.57258D+00,
     & 0.56352D+00, 0.55442D+00, 0.54537D+00, 0.53646D+00, 0.52770D+00,
     & 0.51908D+00, 0.51059D+00, 0.50219D+00, 0.49383D+00, 0.48564D+00,
     & 0.47761D+00, 0.46973D+00, 0.46201D+00, 0.45442D+00, 0.44697D+00,
     & 0.43957D+00, 0.43233D+00, 0.42524D+00, 0.41830D+00, 0.41150D+00/

      data ((g_grdl(i,j),j=1,30),i=31,40)/
     & 0.65389D+00, 0.64394D+00, 0.63398D+00, 0.62401D+00, 0.61404D+00,
     & 0.60405D+00, 0.59387D+00, 0.58383D+00, 0.57391D+00, 0.56410D+00,
     & 0.55442D+00, 0.54474D+00, 0.53514D+00, 0.52572D+00, 0.51647D+00,
     & 0.50740D+00, 0.49848D+00, 0.48969D+00, 0.48096D+00, 0.47242D+00,
     & 0.46406D+00, 0.45589D+00, 0.44788D+00, 0.44004D+00, 0.43235D+00,
     & 0.42473D+00, 0.41728D+00, 0.41001D+00, 0.40289D+00, 0.39594D+00,
     & 0.65293D+00, 0.64189D+00, 0.63091D+00, 0.61998D+00, 0.60911D+00,
     & 0.59826D+00, 0.58728D+00, 0.57647D+00, 0.56584D+00, 0.55537D+00,
     & 0.54506D+00, 0.53479D+00, 0.52464D+00, 0.51471D+00, 0.50499D+00,
     & 0.49546D+00, 0.48613D+00, 0.47695D+00, 0.46786D+00, 0.45899D+00,
     & 0.45032D+00, 0.44186D+00, 0.43358D+00, 0.42549D+00, 0.41758D+00,
     & 0.40975D+00, 0.40212D+00, 0.39467D+00, 0.38740D+00, 0.38030D+00,
     & 0.65186D+00, 0.63970D+00, 0.62766D+00, 0.61574D+00, 0.60394D+00,
     & 0.59222D+00, 0.58041D+00, 0.56884D+00, 0.55748D+00, 0.54634D+00,
     & 0.53540D+00, 0.52454D+00, 0.51384D+00, 0.50340D+00, 0.49319D+00,
     & 0.48323D+00, 0.47348D+00, 0.46392D+00, 0.45448D+00, 0.44528D+00,
     & 0.43632D+00, 0.42758D+00, 0.41905D+00, 0.41073D+00, 0.40260D+00,
     & 0.39459D+00, 0.38678D+00, 0.37917D+00, 0.37176D+00, 0.36453D+00,
     & 0.65068D+00, 0.63734D+00, 0.62420D+00, 0.61125D+00, 0.59849D+00,
     & 0.58587D+00, 0.57322D+00, 0.56086D+00, 0.54877D+00, 0.53695D+00,
     & 0.52537D+00, 0.51392D+00, 0.50268D+00, 0.49172D+00, 0.48105D+00,
     & 0.47064D+00, 0.46050D+00, 0.45056D+00, 0.44078D+00, 0.43127D+00,
     & 0.42202D+00, 0.41302D+00, 0.40425D+00, 0.39571D+00, 0.38738D+00,
     & 0.37919D+00, 0.37123D+00, 0.36349D+00, 0.35595D+00, 0.34861D+00,
     & 0.64937D+00, 0.63481D+00, 0.62053D+00, 0.60651D+00, 0.59276D+00,
     & 0.57921D+00, 0.56569D+00, 0.55252D+00, 0.53968D+00, 0.52716D+00,
     & 0.51494D+00, 0.50290D+00, 0.49110D+00, 0.47963D+00, 0.46849D+00,
     & 0.45765D+00, 0.44711D+00, 0.43681D+00, 0.42671D+00, 0.41690D+00,
     & 0.40737D+00, 0.39812D+00, 0.38912D+00, 0.38038D+00, 0.37187D+00,
     & 0.36353D+00, 0.35543D+00, 0.34757D+00, 0.33992D+00, 0.33249D+00,
     & 0.64774D+00, 0.63188D+00, 0.61639D+00, 0.60125D+00, 0.58645D+00,
     & 0.57194D+00, 0.55753D+00, 0.54353D+00, 0.52992D+00, 0.51669D+00,
     & 0.50381D+00, 0.49117D+00, 0.47882D+00, 0.46685D+00, 0.45524D+00,
     & 0.44398D+00, 0.43304D+00, 0.42239D+00, 0.41198D+00, 0.40188D+00,
     & 0.39210D+00, 0.38262D+00, 0.37341D+00, 0.36448D+00, 0.35581D+00,
     & 0.34734D+00, 0.33912D+00, 0.33116D+00, 0.32343D+00, 0.31592D+00,
     & 0.64596D+00, 0.62872D+00, 0.61196D+00, 0.59565D+00, 0.57976D+00,
     & 0.56423D+00, 0.54889D+00, 0.53403D+00, 0.51964D+00, 0.50568D+00,
     & 0.49213D+00, 0.47888D+00, 0.46598D+00, 0.45351D+00, 0.44144D+00,
     & 0.42975D+00, 0.41844D+00, 0.40745D+00, 0.39674D+00, 0.38638D+00,
     & 0.37635D+00, 0.36665D+00, 0.35726D+00, 0.34817D+00, 0.33935D+00,
     & 0.33077D+00, 0.32247D+00, 0.31442D+00, 0.30663D+00, 0.29908D+00,
     & 0.64391D+00, 0.62520D+00, 0.60708D+00, 0.58952D+00, 0.57248D+00,
     & 0.55591D+00, 0.53960D+00, 0.52385D+00, 0.50864D+00, 0.49394D+00,
     & 0.47972D+00, 0.46586D+00, 0.45241D+00, 0.43944D+00, 0.42691D+00,
     & 0.41482D+00, 0.40314D+00, 0.39182D+00, 0.38084D+00, 0.37023D+00,
     & 0.35999D+00, 0.35010D+00, 0.34054D+00, 0.33130D+00, 0.32237D+00,
     & 0.31371D+00, 0.30534D+00, 0.29725D+00, 0.28942D+00, 0.28185D+00,
     & 0.64148D+00, 0.62119D+00, 0.60162D+00, 0.58273D+00, 0.56447D+00,
     & 0.54678D+00, 0.52946D+00, 0.51280D+00, 0.49675D+00, 0.48129D+00,
     & 0.46638D+00, 0.45191D+00, 0.43791D+00, 0.42444D+00, 0.41147D+00,
     & 0.39899D+00, 0.38695D+00, 0.37533D+00, 0.36409D+00, 0.35326D+00,
     & 0.34283D+00, 0.33277D+00, 0.32308D+00, 0.31373D+00, 0.30471D+00,
     & 0.29600D+00, 0.28760D+00, 0.27949D+00, 0.27166D+00, 0.26410D+00,
     & 0.63857D+00, 0.61657D+00, 0.59543D+00, 0.57511D+00, 0.55556D+00,
     & 0.53669D+00, 0.51831D+00, 0.50069D+00, 0.48377D+00, 0.46752D+00,
     & 0.45191D+00, 0.43682D+00, 0.42228D+00, 0.40833D+00, 0.39493D+00,
     & 0.38206D+00, 0.36970D+00, 0.35780D+00, 0.34634D+00, 0.33532D+00,
     & 0.32473D+00, 0.31454D+00, 0.30474D+00, 0.29531D+00, 0.28624D+00,
     & 0.27753D+00, 0.26913D+00, 0.26104D+00, 0.25325D+00, 0.24573D+00/

      data ((g_grdl(i,j),j=1,30),i=41,50)/
     & 0.63505D+00, 0.61117D+00, 0.58833D+00, 0.56646D+00, 0.54551D+00,
     & 0.52539D+00, 0.50591D+00, 0.48730D+00, 0.46949D+00, 0.45244D+00,
     & 0.43611D+00, 0.42042D+00, 0.40535D+00, 0.39094D+00, 0.37713D+00,
     & 0.36391D+00, 0.35124D+00, 0.33909D+00, 0.32745D+00, 0.31628D+00,
     & 0.30557D+00, 0.29529D+00, 0.28543D+00, 0.27597D+00, 0.26688D+00,
     & 0.25821D+00, 0.24986D+00, 0.24184D+00, 0.23412D+00, 0.22670D+00,
     & 0.63069D+00, 0.60475D+00, 0.58005D+00, 0.55651D+00, 0.53404D+00,
     & 0.51258D+00, 0.49194D+00, 0.47228D+00, 0.45354D+00, 0.43567D+00,
     & 0.41862D+00, 0.40232D+00, 0.38676D+00, 0.37190D+00, 0.35771D+00,
     & 0.34417D+00, 0.33123D+00, 0.31888D+00, 0.30711D+00, 0.29585D+00,
     & 0.28507D+00, 0.27476D+00, 0.26488D+00, 0.25544D+00, 0.24640D+00,
     & 0.23782D+00, 0.22958D+00, 0.22168D+00, 0.21410D+00, 0.20682D+00,
     & 0.62515D+00, 0.59691D+00, 0.57014D+00, 0.54475D+00, 0.52065D+00,
     & 0.49774D+00, 0.47589D+00, 0.45515D+00, 0.43547D+00, 0.41677D+00,
     & 0.39900D+00, 0.38213D+00, 0.36609D+00, 0.35083D+00, 0.33632D+00,
     & 0.32250D+00, 0.30935D+00, 0.29686D+00, 0.28503D+00, 0.27375D+00,
     & 0.26297D+00, 0.25269D+00, 0.24288D+00, 0.23352D+00, 0.22460D+00,
     & 0.21619D+00, 0.20813D+00, 0.20042D+00, 0.19304D+00, 0.18597D+00,
     & 0.61784D+00, 0.58702D+00, 0.55796D+00, 0.53053D+00, 0.50463D+00,
     & 0.48017D+00, 0.45704D+00, 0.43519D+00, 0.41453D+00, 0.39500D+00,
     & 0.37653D+00, 0.35913D+00, 0.34269D+00, 0.32711D+00, 0.31233D+00,
     & 0.29833D+00, 0.28505D+00, 0.27251D+00, 0.26073D+00, 0.24952D+00,
     & 0.23885D+00, 0.22871D+00, 0.21906D+00, 0.20988D+00, 0.20116D+00,
     & 0.19303D+00, 0.18525D+00, 0.17783D+00, 0.17074D+00, 0.16397D+00,
     & 0.60751D+00, 0.57375D+00, 0.54210D+00, 0.51241D+00, 0.48456D+00,
     & 0.45844D+00, 0.43404D+00, 0.41109D+00, 0.38950D+00, 0.36920D+00,
     & 0.35011D+00, 0.33229D+00, 0.31558D+00, 0.29981D+00, 0.28491D+00,
     & 0.27086D+00, 0.25759D+00, 0.24515D+00, 0.23357D+00, 0.22259D+00,
     & 0.21218D+00, 0.20230D+00, 0.19295D+00, 0.18409D+00, 0.17572D+00,
     & 0.16798D+00, 0.16062D+00, 0.15360D+00, 0.14692D+00, 0.14056D+00,
     & 0.59285D+00, 0.55564D+00, 0.52101D+00, 0.48878D+00, 0.45879D+00,
     & 0.43095D+00, 0.40539D+00, 0.38148D+00, 0.35911D+00, 0.33820D+00,
     & 0.31866D+00, 0.30063D+00, 0.28389D+00, 0.26814D+00, 0.25334D+00,
     & 0.23944D+00, 0.22639D+00, 0.21424D+00, 0.20306D+00, 0.19250D+00,
     & 0.18252D+00, 0.17310D+00, 0.16421D+00, 0.15583D+00, 0.14795D+00,
     & 0.14078D+00, 0.13396D+00, 0.12749D+00, 0.12136D+00, 0.11554D+00,
     & 0.57180D+00, 0.53155D+00, 0.49423D+00, 0.45965D+00, 0.42763D+00,
     & 0.39804D+00, 0.37103D+00, 0.34590D+00, 0.32256D+00, 0.30089D+00,
     & 0.28079D+00, 0.26249D+00, 0.24567D+00, 0.22997D+00, 0.21531D+00,
     & 0.20164D+00, 0.18891D+00, 0.17721D+00, 0.16665D+00, 0.15673D+00,
     & 0.14742D+00, 0.13869D+00, 0.13050D+00, 0.12284D+00, 0.11570D+00,
     & 0.10936D+00, 0.10337D+00, 0.97712D-01, 0.92370D-01, 0.87332D-01,
     & 0.55100D+00, 0.50007D+00, 0.45383D+00, 0.41194D+00, 0.37412D+00,
     & 0.34034D+00, 0.31163D+00, 0.28532D+00, 0.26126D+00, 0.23930D+00,
     & 0.21932D+00, 0.20196D+00, 0.18656D+00, 0.17231D+00, 0.15916D+00,
     & 0.14704D+00, 0.13591D+00, 0.12590D+00, 0.11725D+00, 0.10917D+00,
     & 0.10164D+00, 0.94639D-01, 0.88131D-01, 0.82096D-01, 0.76544D-01,
     & 0.71812D-01, 0.67360D-01, 0.63176D-01, 0.59249D-01, 0.55568D-01,
     & 0.38752D+00, 0.34483D+00, 0.30654D+00, 0.27232D+00, 0.24189D+00,
     & 0.21531D+00, 0.19388D+00, 0.17444D+00, 0.15685D+00, 0.14102D+00,
     & 0.12682D+00, 0.11496D+00, 0.10475D+00, 0.95393D-01, 0.86832D-01,
     & 0.79025D-01, 0.71932D-01, 0.65696D-01, 0.60523D-01, 0.55724D-01,
     & 0.51282D-01, 0.47179D-01, 0.43399D-01, 0.39927D-01, 0.36771D-01,
     & 0.34198D-01, 0.31790D-01, 0.29539D-01, 0.27438D-01, 0.25480D-01,
     & 0.24863D-01, 0.21885D-01, 0.19226D-01, 0.16865D-01, 0.14779D-01,
     & 0.12975D-01, 0.11557D-01, 0.10276D-01, 0.91253D-02, 0.80954D-02,
     & 0.71791D-02, 0.64303D-02, 0.57976D-02, 0.52202D-02, 0.46950D-02,
     & 0.42190D-02, 0.37896D-02, 0.34175D-02, 0.31174D-02, 0.28404D-02,
     & 0.25851D-02, 0.23507D-02, 0.21360D-02, 0.19401D-02, 0.17637D-02,
     & 0.16251D-02, 0.14958D-02, 0.13754D-02, 0.12637D-02, 0.11601D-02/

      data ((c_grdl(i,j),j=1,30),i=1,10)/
     & 0.14587D-02, 0.14612D-02, 0.14634D-02, 0.14653D-02, 0.14670D-02,
     & 0.14685D-02, 0.14699D-02, 0.14711D-02, 0.14721D-02, 0.14731D-02,
     & 0.14739D-02, 0.14746D-02, 0.14753D-02, 0.14759D-02, 0.14764D-02,
     & 0.14768D-02, 0.14772D-02, 0.14776D-02, 0.14779D-02, 0.14782D-02,
     & 0.14784D-02, 0.14786D-02, 0.14788D-02, 0.14790D-02, 0.14792D-02,
     & 0.14793D-02, 0.14794D-02, 0.14795D-02, 0.14796D-02, 0.14797D-02,
     & 0.14519D-02, 0.14548D-02, 0.14574D-02, 0.14597D-02, 0.14618D-02,
     & 0.14636D-02, 0.14652D-02, 0.14666D-02, 0.14678D-02, 0.14689D-02,
     & 0.14699D-02, 0.14708D-02, 0.14715D-02, 0.14722D-02, 0.14728D-02,
     & 0.14733D-02, 0.14738D-02, 0.14742D-02, 0.14746D-02, 0.14749D-02,
     & 0.14752D-02, 0.14755D-02, 0.14757D-02, 0.14759D-02, 0.14761D-02,
     & 0.14762D-02, 0.14764D-02, 0.14765D-02, 0.14766D-02, 0.14767D-02,
     & 0.14438D-02, 0.14473D-02, 0.14504D-02, 0.14531D-02, 0.14555D-02,
     & 0.14577D-02, 0.14595D-02, 0.14612D-02, 0.14627D-02, 0.14640D-02,
     & 0.14652D-02, 0.14662D-02, 0.14671D-02, 0.14679D-02, 0.14686D-02,
     & 0.14692D-02, 0.14698D-02, 0.14703D-02, 0.14707D-02, 0.14711D-02,
     & 0.14714D-02, 0.14718D-02, 0.14720D-02, 0.14723D-02, 0.14725D-02,
     & 0.14727D-02, 0.14728D-02, 0.14730D-02, 0.14731D-02, 0.14732D-02,
     & 0.14343D-02, 0.14384D-02, 0.14421D-02, 0.14453D-02, 0.14481D-02,
     & 0.14507D-02, 0.14529D-02, 0.14549D-02, 0.14566D-02, 0.14582D-02,
     & 0.14595D-02, 0.14608D-02, 0.14618D-02, 0.14628D-02, 0.14636D-02,
     & 0.14644D-02, 0.14650D-02, 0.14656D-02, 0.14661D-02, 0.14666D-02,
     & 0.14670D-02, 0.14673D-02, 0.14677D-02, 0.14679D-02, 0.14682D-02,
     & 0.14684D-02, 0.14686D-02, 0.14688D-02, 0.14689D-02, 0.14691D-02,
     & 0.14229D-02, 0.14279D-02, 0.14322D-02, 0.14360D-02, 0.14394D-02,
     & 0.14424D-02, 0.14450D-02, 0.14474D-02, 0.14495D-02, 0.14513D-02,
     & 0.14529D-02, 0.14543D-02, 0.14556D-02, 0.14567D-02, 0.14577D-02,
     & 0.14586D-02, 0.14594D-02, 0.14601D-02, 0.14607D-02, 0.14612D-02,
     & 0.14617D-02, 0.14621D-02, 0.14625D-02, 0.14628D-02, 0.14631D-02,
     & 0.14634D-02, 0.14636D-02, 0.14638D-02, 0.14640D-02, 0.14642D-02,
     & 0.14095D-02, 0.14153D-02, 0.14205D-02, 0.14250D-02, 0.14290D-02,
     & 0.14326D-02, 0.14357D-02, 0.14385D-02, 0.14410D-02, 0.14431D-02,
     & 0.14450D-02, 0.14467D-02, 0.14482D-02, 0.14496D-02, 0.14508D-02,
     & 0.14518D-02, 0.14527D-02, 0.14535D-02, 0.14543D-02, 0.14549D-02,
     & 0.14555D-02, 0.14560D-02, 0.14564D-02, 0.14568D-02, 0.14572D-02,
     & 0.14575D-02, 0.14577D-02, 0.14580D-02, 0.14582D-02, 0.14584D-02,
     & 0.16432D-02, 0.15849D-02, 0.14988D-02, 0.14120D-02, 0.14167D-02,
     & 0.14210D-02, 0.14247D-02, 0.14280D-02, 0.14309D-02, 0.14335D-02,
     & 0.14357D-02, 0.14377D-02, 0.14395D-02, 0.14411D-02, 0.14425D-02,
     & 0.14437D-02, 0.14448D-02, 0.14458D-02, 0.14467D-02, 0.14474D-02,
     & 0.14481D-02, 0.14487D-02, 0.14492D-02, 0.14497D-02, 0.14501D-02,
     & 0.14505D-02, 0.14508D-02, 0.14511D-02, 0.14513D-02, 0.14515D-02,
     & 0.17441D-02, 0.17067D-02, 0.16662D-02, 0.16209D-02, 0.15666D-02,
     & 0.14853D-02, 0.14116D-02, 0.14155D-02, 0.14189D-02, 0.14220D-02,
     & 0.14247D-02, 0.14271D-02, 0.14292D-02, 0.14311D-02, 0.14327D-02,
     & 0.14342D-02, 0.14355D-02, 0.14366D-02, 0.14376D-02, 0.14385D-02,
     & 0.14393D-02, 0.14400D-02, 0.14407D-02, 0.14412D-02, 0.14417D-02,
     & 0.14421D-02, 0.14425D-02, 0.14429D-02, 0.14432D-02, 0.14434D-02,
     & 0.18192D-02, 0.17904D-02, 0.17604D-02, 0.17291D-02, 0.16965D-02,
     & 0.16620D-02, 0.16248D-02, 0.15832D-02, 0.15322D-02, 0.14396D-02,
     & 0.14116D-02, 0.14144D-02, 0.14169D-02, 0.14191D-02, 0.14211D-02,
     & 0.14228D-02, 0.14244D-02, 0.14257D-02, 0.14269D-02, 0.14280D-02,
     & 0.14290D-02, 0.14298D-02, 0.14305D-02, 0.14312D-02, 0.14318D-02,
     & 0.14323D-02, 0.14327D-02, 0.14331D-02, 0.14335D-02, 0.14338D-02,
     & 0.18786D-02, 0.18558D-02, 0.18321D-02, 0.18076D-02, 0.17826D-02,
     & 0.17573D-02, 0.17317D-02, 0.17057D-02, 0.16794D-02, 0.16525D-02,
     & 0.16248D-02, 0.15957D-02, 0.15643D-02, 0.15284D-02, 0.14809D-02,
     & 0.14094D-02, 0.14112D-02, 0.14128D-02, 0.14142D-02, 0.14155D-02,
     & 0.14166D-02, 0.14176D-02, 0.14185D-02, 0.14193D-02, 0.14200D-02,
     & 0.14206D-02, 0.14211D-02, 0.14216D-02, 0.14220D-02, 0.14224D-02/

      data ((c_grdl(i,j),j=1,30),i=11,20)/
     & 0.19244D-02, 0.19071D-02, 0.18887D-02, 0.18694D-02, 0.18497D-02,
     & 0.18296D-02, 0.18096D-02, 0.17896D-02, 0.17699D-02, 0.17506D-02,
     & 0.17317D-02, 0.17132D-02, 0.16952D-02, 0.16777D-02, 0.16607D-02,
     & 0.16442D-02, 0.16282D-02, 0.16126D-02, 0.15974D-02, 0.15826D-02,
     & 0.15681D-02, 0.15537D-02, 0.15395D-02, 0.15253D-02, 0.15108D-02,
     & 0.14958D-02, 0.14796D-02, 0.14610D-02, 0.14345D-02, 0.14089D-02,
     & 0.19555D-02, 0.19446D-02, 0.19317D-02, 0.19175D-02, 0.19024D-02,
     & 0.18868D-02, 0.18710D-02, 0.18552D-02, 0.18396D-02, 0.18244D-02,
     & 0.18096D-02, 0.17953D-02, 0.17817D-02, 0.17687D-02, 0.17564D-02,
     & 0.17448D-02, 0.17339D-02, 0.17237D-02, 0.17142D-02, 0.17054D-02,
     & 0.16972D-02, 0.16897D-02, 0.16827D-02, 0.16764D-02, 0.16705D-02,
     & 0.16652D-02, 0.16604D-02, 0.16560D-02, 0.16520D-02, 0.16483D-02,
     & 0.19692D-02, 0.19659D-02, 0.19597D-02, 0.19513D-02, 0.19414D-02,
     & 0.19304D-02, 0.19187D-02, 0.19067D-02, 0.18946D-02, 0.18827D-02,
     & 0.18710D-02, 0.18597D-02, 0.18489D-02, 0.18386D-02, 0.18289D-02,
     & 0.18198D-02, 0.18113D-02, 0.18034D-02, 0.17961D-02, 0.17894D-02,
     & 0.17832D-02, 0.17776D-02, 0.17724D-02, 0.17677D-02, 0.17635D-02,
     & 0.17596D-02, 0.17561D-02, 0.17530D-02, 0.17502D-02, 0.17477D-02,
     & 0.19614D-02, 0.19675D-02, 0.19695D-02, 0.19682D-02, 0.19645D-02,
     & 0.19589D-02, 0.19520D-02, 0.19443D-02, 0.19360D-02, 0.19274D-02,
     & 0.19187D-02, 0.19101D-02, 0.19018D-02, 0.18938D-02, 0.18862D-02,
     & 0.18791D-02, 0.18724D-02, 0.18661D-02, 0.18604D-02, 0.18550D-02,
     & 0.18501D-02, 0.18457D-02, 0.18416D-02, 0.18379D-02, 0.18345D-02,
     & 0.18315D-02, 0.18287D-02, 0.18263D-02, 0.18241D-02, 0.18221D-02,
     & 0.19273D-02, 0.19449D-02, 0.19569D-02, 0.19644D-02, 0.19684D-02,
     & 0.19695D-02, 0.19684D-02, 0.19657D-02, 0.19619D-02, 0.19572D-02,
     & 0.19520D-02, 0.19466D-02, 0.19410D-02, 0.19354D-02, 0.19300D-02,
     & 0.19247D-02, 0.19197D-02, 0.19150D-02, 0.19106D-02, 0.19065D-02,
     & 0.19027D-02, 0.18993D-02, 0.18961D-02, 0.18932D-02, 0.18906D-02,
     & 0.18882D-02, 0.18861D-02, 0.18841D-02, 0.18824D-02, 0.18809D-02,
     & 0.18615D-02, 0.18929D-02, 0.19171D-02, 0.19352D-02, 0.19485D-02,
     & 0.19579D-02, 0.19640D-02, 0.19676D-02, 0.19692D-02, 0.19694D-02,
     & 0.19684D-02, 0.19666D-02, 0.19643D-02, 0.19616D-02, 0.19587D-02,
     & 0.19557D-02, 0.19527D-02, 0.19497D-02, 0.19469D-02, 0.19442D-02,
     & 0.19416D-02, 0.19392D-02, 0.19370D-02, 0.19350D-02, 0.19331D-02,
     & 0.19314D-02, 0.19299D-02, 0.19284D-02, 0.19272D-02, 0.19260D-02,
     & 0.17578D-02, 0.18055D-02, 0.18443D-02, 0.18753D-02, 0.18999D-02,
     & 0.19191D-02, 0.19340D-02, 0.19453D-02, 0.19537D-02, 0.19597D-02,
     & 0.19640D-02, 0.19668D-02, 0.19684D-02, 0.19693D-02, 0.19695D-02,
     & 0.19692D-02, 0.19686D-02, 0.19677D-02, 0.19668D-02, 0.19657D-02,
     & 0.19646D-02, 0.19635D-02, 0.19624D-02, 0.19614D-02, 0.19604D-02,
     & 0.19595D-02, 0.19586D-02, 0.19578D-02, 0.19571D-02, 0.19565D-02,
     & 0.16134D-02, 0.16784D-02, 0.17324D-02, 0.17785D-02, 0.18165D-02,
     & 0.18477D-02, 0.18731D-02, 0.18936D-02, 0.19102D-02, 0.19234D-02,
     & 0.19340D-02, 0.19424D-02, 0.19490D-02, 0.19541D-02, 0.19581D-02,
     & 0.19612D-02, 0.19636D-02, 0.19653D-02, 0.19666D-02, 0.19676D-02,
     & 0.19683D-02, 0.19688D-02, 0.19691D-02, 0.19693D-02, 0.19694D-02,
     & 0.19695D-02, 0.19695D-02, 0.19694D-02, 0.19694D-02, 0.19693D-02,
     & 0.14198D-02, 0.15062D-02, 0.15795D-02, 0.16415D-02, 0.16936D-02,
     & 0.17374D-02, 0.17751D-02, 0.18067D-02, 0.18330D-02, 0.18549D-02,
     & 0.18731D-02, 0.18882D-02, 0.19007D-02, 0.19111D-02, 0.19197D-02,
     & 0.19269D-02, 0.19328D-02, 0.19378D-02, 0.19419D-02, 0.19454D-02,
     & 0.19483D-02, 0.19507D-02, 0.19528D-02, 0.19545D-02, 0.19559D-02,
     & 0.19572D-02, 0.19582D-02, 0.19591D-02, 0.19599D-02, 0.19605D-02,
     & 0.18585D-02, 0.17843D-02, 0.13754D-02, 0.14568D-02, 0.15266D-02,
     & 0.15861D-02, 0.16369D-02, 0.16800D-02, 0.17164D-02, 0.17480D-02,
     & 0.17751D-02, 0.17982D-02, 0.18178D-02, 0.18345D-02, 0.18487D-02,
     & 0.18608D-02, 0.18711D-02, 0.18798D-02, 0.18874D-02, 0.18938D-02,
     & 0.18994D-02, 0.19041D-02, 0.19083D-02, 0.19118D-02, 0.19149D-02,
     & 0.19176D-02, 0.19199D-02, 0.19219D-02, 0.19237D-02, 0.19252D-02/

      data ((c_grdl(i,j),j=1,30),i=21,30)/
     & 0.16429D-02, 0.17809D-02, 0.18512D-02, 0.18450D-02, 0.17352D-02,
     & 0.13841D-02, 0.14507D-02, 0.15082D-02, 0.15577D-02, 0.16003D-02,
     & 0.16369D-02, 0.16684D-02, 0.16954D-02, 0.17185D-02, 0.17389D-02,
     & 0.17567D-02, 0.17721D-02, 0.17854D-02, 0.17970D-02, 0.18070D-02,
     & 0.18157D-02, 0.18233D-02, 0.18299D-02, 0.18357D-02, 0.18407D-02,
     & 0.18451D-02, 0.18490D-02, 0.18523D-02, 0.18553D-02, 0.18579D-02,
     & 0.17196D-02, 0.17399D-02, 0.17187D-02, 0.17073D-02, 0.18060D-02,
     & 0.18543D-02, 0.18488D-02, 0.17802D-02, 0.15970D-02, 0.14025D-02,
     & 0.14508D-02, 0.14926D-02, 0.15290D-02, 0.15606D-02, 0.15881D-02,
     & 0.16119D-02, 0.16328D-02, 0.16509D-02, 0.16667D-02, 0.16805D-02,
     & 0.16925D-02, 0.17030D-02, 0.17121D-02, 0.17202D-02, 0.17272D-02,
     & 0.17336D-02, 0.17393D-02, 0.17442D-02, 0.17486D-02, 0.17524D-02,
     & 0.96152D-03, 0.14072D-02, 0.16959D-02, 0.17218D-02, 0.17124D-02,
     & 0.16994D-02, 0.16972D-02, 0.17835D-02, 0.18366D-02, 0.18583D-02,
     & 0.18488D-02, 0.18075D-02, 0.17279D-02, 0.15752D-02, 0.13866D-02,
     & 0.14179D-02, 0.14453D-02, 0.14693D-02, 0.14904D-02, 0.15089D-02,
     & 0.15251D-02, 0.15393D-02, 0.15518D-02, 0.15628D-02, 0.15725D-02,
     & 0.15811D-02, 0.15886D-02, 0.15952D-02, 0.16011D-02, 0.16062D-02,
     & 0.60430D-05, 0.30458D-03, 0.73593D-03, 0.11557D-02, 0.15033D-02,
     & 0.17103D-02, 0.17344D-02, 0.17399D-02, 0.17368D-02, 0.16109D-02,
     & 0.16972D-02, 0.17624D-02, 0.18087D-02, 0.18390D-02, 0.18550D-02,
     & 0.18587D-02, 0.18516D-02, 0.18352D-02, 0.18108D-02, 0.17789D-02,
     & 0.17395D-02, 0.16920D-02, 0.16336D-02, 0.15558D-02, 0.13664D-02,
     & 0.13775D-02, 0.13873D-02, 0.13960D-02, 0.14036D-02, 0.14104D-02,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.89809D-04, 0.41383D-03,
     & 0.77930D-03, 0.11237D-02, 0.14172D-02, 0.16301D-02, 0.17241D-02,
     & 0.17344D-02, 0.17321D-02, 0.17156D-02, 0.17368D-02, 0.16911D-02,
     & 0.16394D-02, 0.16880D-02, 0.17274D-02, 0.17592D-02, 0.17844D-02,
     & 0.18042D-02, 0.18197D-02, 0.18317D-02, 0.18407D-02, 0.18473D-02,
     & 0.18520D-02, 0.18552D-02, 0.18573D-02, 0.18584D-02, 0.18588D-02,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.17446D-03,
     & 0.50406D-03, 0.84938D-03, 0.11678D-02, 0.14352D-02, 0.16295D-02,
     & 0.17240D-02, 0.17387D-02, 0.17184D-02, 0.17362D-02, 0.17312D-02,
     & 0.17323D-02, 0.16635D-02, 0.16392D-02, 0.16791D-02, 0.17122D-02,
     & 0.17395D-02, 0.17619D-02, 0.17803D-02, 0.17952D-02, 0.18074D-02,
     & 0.18173D-02, 0.18253D-02, 0.18319D-02, 0.18372D-02, 0.18415D-02,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.19558D-04,
     & 0.26078D-03, 0.58306D-03, 0.90494D-03, 0.11986D-02, 0.14428D-02,
     & 0.16221D-02, 0.17194D-02, 0.17343D-02, 0.16972D-02, 0.17376D-02,
     & 0.17128D-02, 0.17349D-02, 0.17309D-02, 0.16821D-02, 0.16243D-02,
     & 0.16578D-02, 0.16860D-02, 0.17099D-02, 0.17299D-02, 0.17468D-02,
     & 0.17610D-02, 0.17729D-02, 0.17830D-02, 0.17915D-02, 0.17987D-02,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.72152D-04, 0.34013D-03, 0.64884D-03, 0.94721D-03, 0.12166D-02,
     & 0.14403D-02, 0.16078D-02, 0.17081D-02, 0.17207D-02, 0.17299D-02,
     & 0.17203D-02, 0.17400D-02, 0.17170D-02, 0.17344D-02, 0.17348D-02,
     & 0.17142D-02, 0.16375D-02, 0.16249D-02, 0.16492D-02, 0.16700D-02,
     & 0.16879D-02, 0.17032D-02, 0.17164D-02, 0.17277D-02, 0.17374D-02,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.13645D-03, 0.40871D-03, 0.69999D-03, 0.97530D-03,
     & 0.12221D-02, 0.14280D-02, 0.15858D-02, 0.16892D-02, 0.17193D-02,
     & 0.17374D-02, 0.16894D-02, 0.17268D-02, 0.17399D-02, 0.17120D-02,
     & 0.17293D-02, 0.17366D-02, 0.17337D-02, 0.17198D-02, 0.16894D-02,
     & 0.16021D-02, 0.16202D-02, 0.16359D-02, 0.16496D-02, 0.16614D-02,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.12406D-04, 0.19957D-03, 0.46451D-03, 0.73653D-03,
     & 0.98963D-03, 0.12159D-02, 0.14059D-02, 0.15555D-02, 0.16610D-02,
     & 0.17193D-02, 0.17192D-02, 0.17385D-02, 0.16929D-02, 0.17236D-02,
     & 0.17387D-02, 0.17342D-02, 0.17162D-02, 0.17283D-02, 0.17350D-02,
     & 0.17368D-02, 0.17342D-02, 0.17275D-02, 0.17166D-02, 0.17008D-02/

      data ((c_grdl(i,j),j=1,30),i=31,40)/
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.46643D-04, 0.25515D-03, 0.50665D-03,
     & 0.75878D-03, 0.99081D-03, 0.11982D-02, 0.13741D-02, 0.15161D-02,
     & 0.16227D-02, 0.16934D-02, 0.17245D-02, 0.17240D-02, 0.17387D-02,
     & 0.16960D-02, 0.17107D-02, 0.17292D-02, 0.17387D-02, 0.17385D-02,
     & 0.17156D-02, 0.17169D-02, 0.17249D-02, 0.17306D-02, 0.17343D-02,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.87516D-04, 0.29970D-03,
     & 0.53516D-03, 0.76732D-03, 0.97947D-03, 0.11696D-02, 0.13324D-02,
     & 0.14672D-02, 0.15733D-02, 0.16512D-02, 0.17024D-02, 0.17245D-02,
     & 0.17180D-02, 0.17361D-02, 0.17358D-02, 0.16852D-02, 0.17058D-02,
     & 0.17209D-02, 0.17312D-02, 0.17374D-02, 0.17399D-02, 0.17387D-02,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.23997D-05, 0.12587D-03,
     & 0.33198D-03, 0.55026D-03, 0.76286D-03, 0.95625D-03, 0.11304D-02,
     & 0.12813D-02, 0.14084D-02, 0.15124D-02, 0.15941D-02, 0.16551D-02,
     & 0.16971D-02, 0.17211D-02, 0.17149D-02, 0.17220D-02, 0.17349D-02,
     & 0.17386D-02, 0.17318D-02, 0.16956D-02, 0.16962D-02, 0.17072D-02,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.16239D-04,
     & 0.15681D-03, 0.35174D-03, 0.55226D-03, 0.74613D-03, 0.92186D-03,
     & 0.10812D-02, 0.12206D-02, 0.13398D-02, 0.14402D-02, 0.15227D-02,
     & 0.15886D-02, 0.16397D-02, 0.16779D-02, 0.17044D-02, 0.17205D-02,
     & 0.17237D-02, 0.17084D-02, 0.17218D-02, 0.17309D-02, 0.17363D-02,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.33867D-04, 0.17807D-03, 0.35912D-03, 0.54182D-03, 0.71792D-03,
     & 0.87772D-03, 0.10225D-02, 0.11506D-02, 0.12620D-02, 0.13574D-02,
     & 0.14382D-02, 0.15055D-02, 0.15609D-02, 0.16056D-02, 0.16412D-02,
     & 0.16692D-02, 0.16905D-02, 0.17061D-02, 0.17169D-02, 0.17231D-02,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.49163D-04, 0.18890D-03, 0.35475D-03, 0.52020D-03,
     & 0.67913D-03, 0.82425D-03, 0.95484D-03, 0.10722D-02, 0.11754D-02,
     & 0.12650D-02, 0.13422D-02, 0.14084D-02, 0.14646D-02, 0.15120D-02,
     & 0.15517D-02, 0.15848D-02, 0.16122D-02, 0.16348D-02, 0.16534D-02,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.58969D-04, 0.18940D-03, 0.33947D-03,
     & 0.48836D-03, 0.63069D-03, 0.76208D-03, 0.87974D-03, 0.98610D-03,
     & 0.10807D-02, 0.11640D-02, 0.12368D-02, 0.12998D-02, 0.13545D-02,
     & 0.14018D-02, 0.14425D-02, 0.14774D-02, 0.15073D-02, 0.15329D-02,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.51448D-06, 0.62042D-04, 0.18023D-03,
     & 0.31444D-03, 0.44740D-03, 0.57372D-03, 0.69182D-03, 0.79842D-03,
     & 0.89339D-03, 0.97938D-03, 0.10558D-02, 0.11230D-02, 0.11824D-02,
     & 0.12343D-02, 0.12796D-02, 0.13190D-02, 0.13536D-02, 0.13837D-02,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.12994D-05, 0.58413D-04,
     & 0.16252D-03, 0.28085D-03, 0.39849D-03, 0.51006D-03, 0.61477D-03,
     & 0.71060D-03, 0.79660D-03, 0.87312D-03, 0.94180D-03, 0.10035D-02,
     & 0.10580D-02, 0.11060D-02, 0.11485D-02, 0.11860D-02, 0.12189D-02,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.11086D-05,
     & 0.49043D-04, 0.13779D-03, 0.24024D-03, 0.34323D-03, 0.44140D-03,
     & 0.53262D-03, 0.61758D-03, 0.69466D-03, 0.76379D-03, 0.82523D-03,
     & 0.87993D-03, 0.92886D-03, 0.97286D-03, 0.10118D-02, 0.10463D-02/

      data ((c_grdl(i,j),j=1,30),i=41,50)/
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.24245D-06, 0.35669D-04, 0.10799D-03, 0.19443D-03, 0.28316D-03,
     & 0.36843D-03, 0.44818D-03, 0.52153D-03, 0.58924D-03, 0.65080D-03,
     & 0.70600D-03, 0.75531D-03, 0.79913D-03, 0.83803D-03, 0.87259D-03,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.20775D-04, 0.75736D-04, 0.14594D-03,
     & 0.22025D-03, 0.29310D-03, 0.36181D-03, 0.42546D-03, 0.48364D-03,
     & 0.53651D-03, 0.58502D-03, 0.62873D-03, 0.66787D-03, 0.70277D-03,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.76270D-05, 0.44377D-04,
     & 0.97662D-04, 0.15730D-03, 0.21757D-03, 0.27556D-03, 0.32970D-03,
     & 0.37958D-03, 0.42522D-03, 0.46649D-03, 0.50368D-03, 0.53727D-03,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.33875D-06,
     & 0.18141D-04, 0.53747D-04, 0.98013D-04, 0.14523D-03, 0.19203D-03,
     & 0.23683D-03, 0.27863D-03, 0.31706D-03, 0.35210D-03, 0.38378D-03,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.22090D-05, 0.19486D-04, 0.47452D-04, 0.80689D-04,
     & 0.11575D-03, 0.15050D-03, 0.18374D-03, 0.21492D-03, 0.24375D-03,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.12586D-05, 0.12229D-04,
     & 0.30525D-04, 0.52708D-04, 0.76520D-04, 0.10051D-03, 0.12378D-03,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.27972D-05, 0.11260D-04, 0.23120D-04, 0.36780D-04,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.20540D-06,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00/

      DO I=1,50
        DO J=1,30
          UDS_GRD(I,J) = UDS_GRDL(I,J)
          G_GRD(I,J)   = G_GRDL(I,J)
          C_GRD(I,J)   = C_GRDL(I,J)
        ENDDO
      ENDDO

      RETURN
      END


      SUBROUTINE I_NLO_Q3G3
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      PARAMETER (NQ2MAX=30,NZMAX=50)
      DIMENSION UDS_GRD(NZMAX,NQ2MAX),C_GRD(NZMAX,NQ2MAX),
     &          G_GRD(NZMAX,NQ2MAX)
      DIMENSION UDS_GRDL(NZMAX,NQ2MAX),C_GRDL(NZMAX,NQ2MAX),
     &          G_GRDL(NZMAX,NQ2MAX)
      COMMON /CPARAM/ UDS_GRD,C_GRD,G_GRD 
      data ((uds_grdl(i,j),j=1,30),i=1,10)/
     & 0.94426D-03, 0.30654D-02, 0.52006D-02, 0.73473D-02, 0.95031D-02,
     & 0.11665D-01, 0.13829D-01, 0.15999D-01, 0.18173D-01, 0.20350D-01,
     & 0.22529D-01, 0.24707D-01, 0.26884D-01, 0.29061D-01, 0.31237D-01,
     & 0.33411D-01, 0.35582D-01, 0.37751D-01, 0.39914D-01, 0.42074D-01,
     & 0.44229D-01, 0.46380D-01, 0.48526D-01, 0.50666D-01, 0.52800D-01,
     & 0.54928D-01, 0.57050D-01, 0.59165D-01, 0.61273D-01, 0.63373D-01,
     & 0.97765D-03, 0.30258D-02, 0.50834D-02, 0.71481D-02, 0.92181D-02,
     & 0.11291D-01, 0.13363D-01, 0.15438D-01, 0.17514D-01, 0.19590D-01,
     & 0.21665D-01, 0.23737D-01, 0.25806D-01, 0.27872D-01, 0.29935D-01,
     & 0.31994D-01, 0.34048D-01, 0.36098D-01, 0.38141D-01, 0.40179D-01,
     & 0.42211D-01, 0.44236D-01, 0.46255D-01, 0.48266D-01, 0.50270D-01,
     & 0.52269D-01, 0.54259D-01, 0.56241D-01, 0.58214D-01, 0.60180D-01,
     & 0.10176D-02, 0.29924D-02, 0.49722D-02, 0.69554D-02, 0.89404D-02,
     & 0.10925D-01, 0.12908D-01, 0.14889D-01, 0.16869D-01, 0.18846D-01,
     & 0.20820D-01, 0.22789D-01, 0.24753D-01, 0.26712D-01, 0.28665D-01,
     & 0.30613D-01, 0.32554D-01, 0.34489D-01, 0.36416D-01, 0.38337D-01,
     & 0.40250D-01, 0.42154D-01, 0.44051D-01, 0.45939D-01, 0.47819D-01,
     & 0.49693D-01, 0.51557D-01, 0.53412D-01, 0.55257D-01, 0.57094D-01,
     & 0.10656D-02, 0.29676D-02, 0.48707D-02, 0.67736D-02, 0.86750D-02,
     & 0.10574D-01, 0.12468D-01, 0.14358D-01, 0.16244D-01, 0.18125D-01,
     & 0.20000D-01, 0.21869D-01, 0.23731D-01, 0.25586D-01, 0.27434D-01,
     & 0.29274D-01, 0.31106D-01, 0.32931D-01, 0.34748D-01, 0.36555D-01,
     & 0.38354D-01, 0.40143D-01, 0.41924D-01, 0.43694D-01, 0.45456D-01,
     & 0.47210D-01, 0.48955D-01, 0.50689D-01, 0.52412D-01, 0.54126D-01,
     & 0.11233D-02, 0.29519D-02, 0.47783D-02, 0.66011D-02, 0.84196D-02,
     & 0.10233D-01, 0.12040D-01, 0.13840D-01, 0.15634D-01, 0.17421D-01,
     & 0.19200D-01, 0.20971D-01, 0.22734D-01, 0.24488D-01, 0.26234D-01,
     & 0.27969D-01, 0.29696D-01, 0.31414D-01, 0.33123D-01, 0.34822D-01,
     & 0.36511D-01, 0.38189D-01, 0.39858D-01, 0.41515D-01, 0.43163D-01,
     & 0.44804D-01, 0.46433D-01, 0.48051D-01, 0.49659D-01, 0.51255D-01,
     & 0.11927D-02, 0.29477D-02, 0.46973D-02, 0.64405D-02, 0.81768D-02,
     & 0.99055D-02, 0.11626D-01, 0.13339D-01, 0.15042D-01, 0.16736D-01,
     & 0.18421D-01, 0.20097D-01, 0.21764D-01, 0.23420D-01, 0.25066D-01,
     & 0.26701D-01, 0.28325D-01, 0.29940D-01, 0.31545D-01, 0.33140D-01,
     & 0.34723D-01, 0.36294D-01, 0.37855D-01, 0.39404D-01, 0.40943D-01,
     & 0.42474D-01, 0.43994D-01, 0.45502D-01, 0.46998D-01, 0.48483D-01,
     & 0.12765D-02, 0.29581D-02, 0.46314D-02, 0.62957D-02, 0.79507D-02,
     & 0.95962D-02, 0.11232D-01, 0.12858D-01, 0.14473D-01, 0.16077D-01,
     & 0.17670D-01, 0.19253D-01, 0.20826D-01, 0.22387D-01, 0.23936D-01,
     & 0.25474D-01, 0.26999D-01, 0.28515D-01, 0.30020D-01, 0.31514D-01,
     & 0.32995D-01, 0.34465D-01, 0.35922D-01, 0.37368D-01, 0.38803D-01,
     & 0.40229D-01, 0.41644D-01, 0.43046D-01, 0.44437D-01, 0.45816D-01,
     & 0.13782D-02, 0.29864D-02, 0.45839D-02, 0.61702D-02, 0.77453D-02,
     & 0.93091D-02, 0.10863D-01, 0.12404D-01, 0.13932D-01, 0.15449D-01,
     & 0.16953D-01, 0.18447D-01, 0.19929D-01, 0.21398D-01, 0.22855D-01,
     & 0.24298D-01, 0.25730D-01, 0.27150D-01, 0.28560D-01, 0.29957D-01,
     & 0.31341D-01, 0.32713D-01, 0.34072D-01, 0.35420D-01, 0.36755D-01,
     & 0.38083D-01, 0.39398D-01, 0.40701D-01, 0.41991D-01, 0.43269D-01,
     & 0.15020D-02, 0.30380D-02, 0.45610D-02, 0.60711D-02, 0.75681D-02,
     & 0.90524D-02, 0.10525D-01, 0.11984D-01, 0.13430D-01, 0.14862D-01,
     & 0.16281D-01, 0.17688D-01, 0.19083D-01, 0.20464D-01, 0.21832D-01,
     & 0.23186D-01, 0.24527D-01, 0.25856D-01, 0.27175D-01, 0.28481D-01,
     & 0.29773D-01, 0.31052D-01, 0.32318D-01, 0.33572D-01, 0.34815D-01,
     & 0.36048D-01, 0.37269D-01, 0.38478D-01, 0.39674D-01, 0.40858D-01,
     & 0.16530D-02, 0.31172D-02, 0.45667D-02, 0.60015D-02, 0.74218D-02,
     & 0.88281D-02, 0.10222D-01, 0.11601D-01, 0.12966D-01, 0.14316D-01,
     & 0.15651D-01, 0.16975D-01, 0.18286D-01, 0.19582D-01, 0.20864D-01,
     & 0.22132D-01, 0.23387D-01, 0.24629D-01, 0.25861D-01, 0.27079D-01,
     & 0.28283D-01, 0.29474D-01, 0.30652D-01, 0.31817D-01, 0.32970D-01,
     & 0.34115D-01, 0.35247D-01, 0.36366D-01, 0.37472D-01, 0.38567D-01/

      data ((uds_grdl(i,j),j=1,30),i=11,20)/
     & 0.18379D-02, 0.32311D-02, 0.46081D-02, 0.59692D-02, 0.73146D-02,
     & 0.86450D-02, 0.99625D-02, 0.11264D-01, 0.12550D-01, 0.13821D-01,
     & 0.15077D-01, 0.16321D-01, 0.17551D-01, 0.18766D-01, 0.19966D-01,
     & 0.21152D-01, 0.22324D-01, 0.23483D-01, 0.24632D-01, 0.25766D-01,
     & 0.26887D-01, 0.27994D-01, 0.29088D-01, 0.30169D-01, 0.31238D-01,
     & 0.32298D-01, 0.33346D-01, 0.34380D-01, 0.35403D-01, 0.36413D-01,
     & 0.20652D-02, 0.33897D-02, 0.46967D-02, 0.59867D-02, 0.72601D-02,
     & 0.85177D-02, 0.97619D-02, 0.10989D-01, 0.12201D-01, 0.13396D-01,
     & 0.14576D-01, 0.15744D-01, 0.16897D-01, 0.18035D-01, 0.19158D-01,
     & 0.20266D-01, 0.21360D-01, 0.22442D-01, 0.23512D-01, 0.24568D-01,
     & 0.25610D-01, 0.26639D-01, 0.27654D-01, 0.28656D-01, 0.29647D-01,
     & 0.30629D-01, 0.31597D-01, 0.32553D-01, 0.33497D-01, 0.34429D-01,
     & 0.23450D-02, 0.36026D-02, 0.48416D-02, 0.60628D-02, 0.72666D-02,
     & 0.84541D-02, 0.96277D-02, 0.10784D-01, 0.11924D-01, 0.13047D-01,
     & 0.14154D-01, 0.15249D-01, 0.16330D-01, 0.17394D-01, 0.18444D-01,
     & 0.19478D-01, 0.20498D-01, 0.21506D-01, 0.22502D-01, 0.23484D-01,
     & 0.24452D-01, 0.25406D-01, 0.26347D-01, 0.27275D-01, 0.28192D-01,
     & 0.29099D-01, 0.29994D-01, 0.30876D-01, 0.31746D-01, 0.32605D-01,
     & 0.26907D-02, 0.38843D-02, 0.50585D-02, 0.62143D-02, 0.73521D-02,
     & 0.84732D-02, 0.95802D-02, 0.10669D-01, 0.11742D-01, 0.12797D-01,
     & 0.13837D-01, 0.14863D-01, 0.15875D-01, 0.16871D-01, 0.17852D-01,
     & 0.18818D-01, 0.19769D-01, 0.20708D-01, 0.21635D-01, 0.22548D-01,
     & 0.23447D-01, 0.24332D-01, 0.25205D-01, 0.26064D-01, 0.26912D-01,
     & 0.27751D-01, 0.28577D-01, 0.29391D-01, 0.30193D-01, 0.30983D-01,
     & 0.31180D-02, 0.42514D-02, 0.53650D-02, 0.64595D-02, 0.75357D-02,
     & 0.85947D-02, 0.96396D-02, 0.10666D-01, 0.11676D-01, 0.12668D-01,
     & 0.13644D-01, 0.14607D-01, 0.15556D-01, 0.16489D-01, 0.17405D-01,
     & 0.18307D-01, 0.19195D-01, 0.20069D-01, 0.20933D-01, 0.21782D-01,
     & 0.22617D-01, 0.23438D-01, 0.24247D-01, 0.25043D-01, 0.25827D-01,
     & 0.26602D-01, 0.27365D-01, 0.28116D-01, 0.28855D-01, 0.29582D-01,
     & 0.36461D-02, 0.47226D-02, 0.57787D-02, 0.68153D-02, 0.78332D-02,
     & 0.88336D-02, 0.98198D-02, 0.10788D-01, 0.11738D-01, 0.12671D-01,
     & 0.13587D-01, 0.14490D-01, 0.15379D-01, 0.16252D-01, 0.17109D-01,
     & 0.17951D-01, 0.18778D-01, 0.19592D-01, 0.20396D-01, 0.21185D-01,
     & 0.21960D-01, 0.22722D-01, 0.23470D-01, 0.24207D-01, 0.24931D-01,
     & 0.25647D-01, 0.26351D-01, 0.27042D-01, 0.27722D-01, 0.28390D-01,
     & 0.42979D-02, 0.53226D-02, 0.63264D-02, 0.73101D-02, 0.82747D-02,
     & 0.92215D-02, 0.10154D-01, 0.11068D-01, 0.11963D-01, 0.12842D-01,
     & 0.13703D-01, 0.14552D-01, 0.15386D-01, 0.16203D-01, 0.17005D-01,
     & 0.17791D-01, 0.18563D-01, 0.19322D-01, 0.20070D-01, 0.20804D-01,
     & 0.21524D-01, 0.22230D-01, 0.22924D-01, 0.23605D-01, 0.24274D-01,
     & 0.24935D-01, 0.25583D-01, 0.26220D-01, 0.26845D-01, 0.27460D-01,
     & 0.50996D-02, 0.60778D-02, 0.70343D-02, 0.79703D-02, 0.88865D-02,
     & 0.97846D-02, 0.10668D-01, 0.11533D-01, 0.12378D-01, 0.13207D-01,
     & 0.14018D-01, 0.14816D-01, 0.15599D-01, 0.16366D-01, 0.17116D-01,
     & 0.17852D-01, 0.18572D-01, 0.19280D-01, 0.19977D-01, 0.20659D-01,
     & 0.21327D-01, 0.21982D-01, 0.22624D-01, 0.23254D-01, 0.23872D-01,
     & 0.24481D-01, 0.25079D-01, 0.25664D-01, 0.26238D-01, 0.26802D-01,
     & 0.60794D-02, 0.70165D-02, 0.79312D-02, 0.88244D-02, 0.96973D-02,
     & 0.10551D-01, 0.11391D-01, 0.12210D-01, 0.13011D-01, 0.13793D-01,
     & 0.14558D-01, 0.15310D-01, 0.16046D-01, 0.16765D-01, 0.17468D-01,
     & 0.18156D-01, 0.18829D-01, 0.19488D-01, 0.20137D-01, 0.20770D-01,
     & 0.21390D-01, 0.21997D-01, 0.22590D-01, 0.23172D-01, 0.23741D-01,
     & 0.24302D-01, 0.24851D-01, 0.25388D-01, 0.25914D-01, 0.26429D-01,
     & 0.72644D-02, 0.81660D-02, 0.90439D-02, 0.98994D-02, 0.10733D-01,
     & 0.11548D-01, 0.12347D-01, 0.13125D-01, 0.13884D-01, 0.14624D-01,
     & 0.15346D-01, 0.16054D-01, 0.16746D-01, 0.17420D-01, 0.18079D-01,
     & 0.18721D-01, 0.19348D-01, 0.19962D-01, 0.20564D-01, 0.21152D-01,
     & 0.21725D-01, 0.22285D-01, 0.22832D-01, 0.23366D-01, 0.23889D-01,
     & 0.24402D-01, 0.24904D-01, 0.25394D-01, 0.25873D-01, 0.26341D-01/

      data ((uds_grdl(i,j),j=1,30),i=21,30)/
     & 0.86753D-02, 0.95469D-02, 0.10393D-01, 0.11215D-01, 0.12015D-01,
     & 0.12793D-01, 0.13555D-01, 0.14295D-01, 0.15014D-01, 0.15714D-01,
     & 0.16395D-01, 0.17061D-01, 0.17710D-01, 0.18342D-01, 0.18956D-01,
     & 0.19554D-01, 0.20137D-01, 0.20705D-01, 0.21262D-01, 0.21803D-01,
     & 0.22330D-01, 0.22843D-01, 0.23343D-01, 0.23831D-01, 0.24306D-01,
     & 0.24773D-01, 0.25227D-01, 0.25669D-01, 0.26101D-01, 0.26521D-01,
     & 0.10317D-01, 0.11161D-01, 0.11978D-01, 0.12768D-01, 0.13534D-01,
     & 0.14276D-01, 0.15001D-01, 0.15702D-01, 0.16381D-01, 0.17040D-01,
     & 0.17678D-01, 0.18300D-01, 0.18905D-01, 0.19491D-01, 0.20059D-01,
     & 0.20610D-01, 0.21144D-01, 0.21665D-01, 0.22172D-01, 0.22664D-01,
     & 0.23142D-01, 0.23605D-01, 0.24055D-01, 0.24492D-01, 0.24917D-01,
     & 0.25332D-01, 0.25735D-01, 0.26127D-01, 0.26507D-01, 0.26877D-01,
     & 0.12159D-01, 0.12978D-01, 0.13766D-01, 0.14525D-01, 0.15256D-01,
     & 0.15962D-01, 0.16647D-01, 0.17307D-01, 0.17943D-01, 0.18556D-01,
     & 0.19148D-01, 0.19722D-01, 0.20278D-01, 0.20813D-01, 0.21330D-01,
     & 0.21829D-01, 0.22311D-01, 0.22778D-01, 0.23230D-01, 0.23667D-01,
     & 0.24089D-01, 0.24497D-01, 0.24891D-01, 0.25272D-01, 0.25641D-01,
     & 0.25999D-01, 0.26345D-01, 0.26679D-01, 0.27003D-01, 0.27316D-01,
     & 0.14115D-01, 0.14905D-01, 0.15660D-01, 0.16383D-01, 0.17074D-01,
     & 0.17736D-01, 0.18373D-01, 0.18983D-01, 0.19567D-01, 0.20127D-01,
     & 0.20663D-01, 0.21179D-01, 0.21674D-01, 0.22148D-01, 0.22603D-01,
     & 0.23040D-01, 0.23458D-01, 0.23860D-01, 0.24247D-01, 0.24618D-01,
     & 0.24973D-01, 0.25315D-01, 0.25642D-01, 0.25957D-01, 0.26259D-01,
     & 0.26549D-01, 0.26827D-01, 0.27095D-01, 0.27351D-01, 0.27597D-01,
     & 0.15999D-01, 0.16746D-01, 0.17453D-01, 0.18122D-01, 0.18755D-01,
     & 0.19356D-01, 0.19926D-01, 0.20467D-01, 0.20979D-01, 0.21464D-01,
     & 0.21925D-01, 0.22362D-01, 0.22776D-01, 0.23169D-01, 0.23542D-01,
     & 0.23896D-01, 0.24232D-01, 0.24550D-01, 0.24851D-01, 0.25137D-01,
     & 0.25408D-01, 0.25665D-01, 0.25908D-01, 0.26139D-01, 0.26357D-01,
     & 0.26563D-01, 0.26757D-01, 0.26942D-01, 0.27116D-01, 0.27281D-01,
     & 0.16408D-01, 0.17141D-01, 0.17833D-01, 0.18485D-01, 0.19100D-01,
     & 0.19682D-01, 0.20231D-01, 0.20750D-01, 0.21240D-01, 0.21704D-01,
     & 0.22141D-01, 0.22555D-01, 0.22946D-01, 0.23315D-01, 0.23664D-01,
     & 0.23994D-01, 0.24306D-01, 0.24600D-01, 0.24877D-01, 0.25138D-01,
     & 0.25386D-01, 0.25619D-01, 0.25839D-01, 0.26046D-01, 0.26241D-01,
     & 0.26423D-01, 0.26595D-01, 0.26757D-01, 0.26909D-01, 0.27052D-01,
     & 0.16776D-01, 0.17493D-01, 0.18166D-01, 0.18800D-01, 0.19396D-01,
     & 0.19956D-01, 0.20483D-01, 0.20980D-01, 0.21447D-01, 0.21887D-01,
     & 0.22301D-01, 0.22690D-01, 0.23056D-01, 0.23400D-01, 0.23725D-01,
     & 0.24030D-01, 0.24317D-01, 0.24586D-01, 0.24838D-01, 0.25075D-01,
     & 0.25298D-01, 0.25507D-01, 0.25703D-01, 0.25886D-01, 0.26058D-01,
     & 0.26217D-01, 0.26365D-01, 0.26504D-01, 0.26634D-01, 0.26754D-01,
     & 0.17100D-01, 0.17798D-01, 0.18453D-01, 0.19066D-01, 0.19640D-01,
     & 0.20178D-01, 0.20682D-01, 0.21154D-01, 0.21596D-01, 0.22011D-01,
     & 0.22400D-01, 0.22763D-01, 0.23103D-01, 0.23422D-01, 0.23721D-01,
     & 0.24000D-01, 0.24262D-01, 0.24505D-01, 0.24731D-01, 0.24943D-01,
     & 0.25141D-01, 0.25325D-01, 0.25496D-01, 0.25656D-01, 0.25804D-01,
     & 0.25939D-01, 0.26064D-01, 0.26180D-01, 0.26286D-01, 0.26385D-01,
     & 0.17378D-01, 0.18056D-01, 0.18689D-01, 0.19279D-01, 0.19830D-01,
     & 0.20344D-01, 0.20821D-01, 0.21267D-01, 0.21684D-01, 0.22072D-01,
     & 0.22435D-01, 0.22771D-01, 0.23084D-01, 0.23375D-01, 0.23647D-01,
     & 0.23900D-01, 0.24135D-01, 0.24353D-01, 0.24553D-01, 0.24738D-01,
     & 0.24910D-01, 0.25070D-01, 0.25217D-01, 0.25352D-01, 0.25476D-01,
     & 0.25586D-01, 0.25688D-01, 0.25781D-01, 0.25865D-01, 0.25941D-01,
     & 0.17610D-01, 0.18265D-01, 0.18873D-01, 0.19439D-01, 0.19964D-01,
     & 0.20452D-01, 0.20901D-01, 0.21320D-01, 0.21708D-01, 0.22069D-01,
     & 0.22404D-01, 0.22712D-01, 0.22996D-01, 0.23259D-01, 0.23503D-01,
     & 0.23729D-01, 0.23937D-01, 0.24128D-01, 0.24301D-01, 0.24460D-01,
     & 0.24606D-01, 0.24739D-01, 0.24861D-01, 0.24972D-01, 0.25072D-01,
     & 0.25158D-01, 0.25236D-01, 0.25306D-01, 0.25368D-01, 0.25422D-01/

      data ((uds_grdl(i,j),j=1,30),i=31,40)/
     & 0.17793D-01, 0.18422D-01, 0.19004D-01, 0.19542D-01, 0.20040D-01,
     & 0.20499D-01, 0.20919D-01, 0.21307D-01, 0.21666D-01, 0.21998D-01,
     & 0.22303D-01, 0.22581D-01, 0.22836D-01, 0.23070D-01, 0.23286D-01,
     & 0.23483D-01, 0.23663D-01, 0.23827D-01, 0.23972D-01, 0.24104D-01,
     & 0.24224D-01, 0.24332D-01, 0.24429D-01, 0.24515D-01, 0.24590D-01,
     & 0.24653D-01, 0.24707D-01, 0.24754D-01, 0.24793D-01, 0.24826D-01,
     & 0.17928D-01, 0.18527D-01, 0.19080D-01, 0.19588D-01, 0.20055D-01,
     & 0.20483D-01, 0.20871D-01, 0.21228D-01, 0.21556D-01, 0.21856D-01,
     & 0.22131D-01, 0.22378D-01, 0.22602D-01, 0.22806D-01, 0.22992D-01,
     & 0.23160D-01, 0.23312D-01, 0.23447D-01, 0.23564D-01, 0.23669D-01,
     & 0.23763D-01, 0.23845D-01, 0.23916D-01, 0.23978D-01, 0.24029D-01,
     & 0.24068D-01, 0.24099D-01, 0.24123D-01, 0.24140D-01, 0.24151D-01,
     & 0.18012D-01, 0.18579D-01, 0.19099D-01, 0.19574D-01, 0.20008D-01,
     & 0.20403D-01, 0.20757D-01, 0.21080D-01, 0.21374D-01, 0.21642D-01,
     & 0.21884D-01, 0.22099D-01, 0.22291D-01, 0.22464D-01, 0.22620D-01,
     & 0.22758D-01, 0.22881D-01, 0.22988D-01, 0.23077D-01, 0.23154D-01,
     & 0.23221D-01, 0.23277D-01, 0.23323D-01, 0.23360D-01, 0.23388D-01,
     & 0.23403D-01, 0.23411D-01, 0.23413D-01, 0.23409D-01, 0.23399D-01,
     & 0.18046D-01, 0.18576D-01, 0.19059D-01, 0.19498D-01, 0.19896D-01,
     & 0.20255D-01, 0.20573D-01, 0.20860D-01, 0.21120D-01, 0.21353D-01,
     & 0.21562D-01, 0.21744D-01, 0.21903D-01, 0.22044D-01, 0.22168D-01,
     & 0.22277D-01, 0.22370D-01, 0.22448D-01, 0.22509D-01, 0.22559D-01,
     & 0.22599D-01, 0.22629D-01, 0.22650D-01, 0.22663D-01, 0.22667D-01,
     & 0.22659D-01, 0.22645D-01, 0.22625D-01, 0.22600D-01, 0.22570D-01,
     & 0.18029D-01, 0.18519D-01, 0.18962D-01, 0.19362D-01, 0.19722D-01,
     & 0.20042D-01, 0.20321D-01, 0.20570D-01, 0.20793D-01, 0.20991D-01,
     & 0.21165D-01, 0.21312D-01, 0.21437D-01, 0.21545D-01, 0.21638D-01,
     & 0.21716D-01, 0.21780D-01, 0.21828D-01, 0.21861D-01, 0.21883D-01,
     & 0.21897D-01, 0.21902D-01, 0.21898D-01, 0.21887D-01, 0.21868D-01,
     & 0.21838D-01, 0.21802D-01, 0.21761D-01, 0.21716D-01, 0.21667D-01,
     & 0.17957D-01, 0.18400D-01, 0.18799D-01, 0.19155D-01, 0.19472D-01,
     & 0.19750D-01, 0.19987D-01, 0.20197D-01, 0.20381D-01, 0.20540D-01,
     & 0.20678D-01, 0.20789D-01, 0.20879D-01, 0.20954D-01, 0.21015D-01,
     & 0.21061D-01, 0.21095D-01, 0.21115D-01, 0.21119D-01, 0.21114D-01,
     & 0.21101D-01, 0.21081D-01, 0.21053D-01, 0.21019D-01, 0.20978D-01,
     & 0.20926D-01, 0.20869D-01, 0.20808D-01, 0.20744D-01, 0.20676D-01,
     & 0.17833D-01, 0.18227D-01, 0.18577D-01, 0.18886D-01, 0.19157D-01,
     & 0.19392D-01, 0.19585D-01, 0.19752D-01, 0.19896D-01, 0.20017D-01,
     & 0.20116D-01, 0.20191D-01, 0.20246D-01, 0.20287D-01, 0.20315D-01,
     & 0.20330D-01, 0.20334D-01, 0.20325D-01, 0.20301D-01, 0.20270D-01,
     & 0.20232D-01, 0.20187D-01, 0.20136D-01, 0.20079D-01, 0.20017D-01,
     & 0.19944D-01, 0.19868D-01, 0.19788D-01, 0.19706D-01, 0.19621D-01,
     & 0.17656D-01, 0.17995D-01, 0.18292D-01, 0.18551D-01, 0.18773D-01,
     & 0.18961D-01, 0.19108D-01, 0.19232D-01, 0.19333D-01, 0.19414D-01,
     & 0.19475D-01, 0.19513D-01, 0.19532D-01, 0.19539D-01, 0.19535D-01,
     & 0.19519D-01, 0.19494D-01, 0.19456D-01, 0.19405D-01, 0.19348D-01,
     & 0.19286D-01, 0.19218D-01, 0.19145D-01, 0.19067D-01, 0.18984D-01,
     & 0.18892D-01, 0.18797D-01, 0.18701D-01, 0.18602D-01, 0.18501D-01,
     & 0.17424D-01, 0.17703D-01, 0.17943D-01, 0.18147D-01, 0.18318D-01,
     & 0.18455D-01, 0.18555D-01, 0.18633D-01, 0.18691D-01, 0.18730D-01,
     & 0.18753D-01, 0.18753D-01, 0.18736D-01, 0.18710D-01, 0.18673D-01,
     & 0.18627D-01, 0.18572D-01, 0.18508D-01, 0.18431D-01, 0.18349D-01,
     & 0.18263D-01, 0.18173D-01, 0.18079D-01, 0.17981D-01, 0.17879D-01,
     & 0.17770D-01, 0.17659D-01, 0.17546D-01, 0.17433D-01, 0.17318D-01,
     & 0.17136D-01, 0.17350D-01, 0.17528D-01, 0.17674D-01, 0.17789D-01,
     & 0.17875D-01, 0.17924D-01, 0.17955D-01, 0.17969D-01, 0.17966D-01,
     & 0.17949D-01, 0.17912D-01, 0.17860D-01, 0.17799D-01, 0.17731D-01,
     & 0.17655D-01, 0.17572D-01, 0.17481D-01, 0.17380D-01, 0.17274D-01,
     & 0.17166D-01, 0.17056D-01, 0.16942D-01, 0.16826D-01, 0.16708D-01,
     & 0.16582D-01, 0.16456D-01, 0.16330D-01, 0.16203D-01, 0.16076D-01/

      data ((uds_grdl(i,j),j=1,30),i=41,50)/
     & 0.16789D-01, 0.16932D-01, 0.17044D-01, 0.17128D-01, 0.17185D-01,
     & 0.17215D-01, 0.17213D-01, 0.17196D-01, 0.17164D-01, 0.17119D-01,
     & 0.17062D-01, 0.16988D-01, 0.16901D-01, 0.16808D-01, 0.16709D-01,
     & 0.16604D-01, 0.16495D-01, 0.16379D-01, 0.16254D-01, 0.16128D-01,
     & 0.15999D-01, 0.15870D-01, 0.15739D-01, 0.15606D-01, 0.15473D-01,
     & 0.15333D-01, 0.15195D-01, 0.15057D-01, 0.14919D-01, 0.14782D-01,
     & 0.16381D-01, 0.16449D-01, 0.16490D-01, 0.16508D-01, 0.16504D-01,
     & 0.16478D-01, 0.16423D-01, 0.16357D-01, 0.16279D-01, 0.16192D-01,
     & 0.16095D-01, 0.15984D-01, 0.15863D-01, 0.15739D-01, 0.15611D-01,
     & 0.15479D-01, 0.15345D-01, 0.15205D-01, 0.15059D-01, 0.14913D-01,
     & 0.14767D-01, 0.14621D-01, 0.14474D-01, 0.14328D-01, 0.14181D-01,
     & 0.14031D-01, 0.13882D-01, 0.13734D-01, 0.13588D-01, 0.13443D-01,
     & 0.15906D-01, 0.15893D-01, 0.15860D-01, 0.15809D-01, 0.15741D-01,
     & 0.15656D-01, 0.15548D-01, 0.15432D-01, 0.15309D-01, 0.15180D-01,
     & 0.15045D-01, 0.14898D-01, 0.14746D-01, 0.14592D-01, 0.14436D-01,
     & 0.14280D-01, 0.14122D-01, 0.13962D-01, 0.13798D-01, 0.13635D-01,
     & 0.13473D-01, 0.13313D-01, 0.13154D-01, 0.12996D-01, 0.12839D-01,
     & 0.12680D-01, 0.12523D-01, 0.12369D-01, 0.12217D-01, 0.12067D-01,
     & 0.15354D-01, 0.15256D-01, 0.15145D-01, 0.15023D-01, 0.14890D-01,
     & 0.14746D-01, 0.14585D-01, 0.14421D-01, 0.14253D-01, 0.14083D-01,
     & 0.13911D-01, 0.13731D-01, 0.13549D-01, 0.13368D-01, 0.13188D-01,
     & 0.13009D-01, 0.12831D-01, 0.12653D-01, 0.12474D-01, 0.12297D-01,
     & 0.12123D-01, 0.11951D-01, 0.11782D-01, 0.11616D-01, 0.11452D-01,
     & 0.11287D-01, 0.11126D-01, 0.10968D-01, 0.10812D-01, 0.10660D-01,
     & 0.14702D-01, 0.14517D-01, 0.14327D-01, 0.14133D-01, 0.13935D-01,
     & 0.13733D-01, 0.13520D-01, 0.13310D-01, 0.13100D-01, 0.12892D-01,
     & 0.12685D-01, 0.12476D-01, 0.12268D-01, 0.12063D-01, 0.11862D-01,
     & 0.11664D-01, 0.11470D-01, 0.11278D-01, 0.11087D-01, 0.10900D-01,
     & 0.10717D-01, 0.10538D-01, 0.10363D-01, 0.10192D-01, 0.10024D-01,
     & 0.98574D-02, 0.96949D-02, 0.95362D-02, 0.93811D-02, 0.92296D-02,
     & 0.13924D-01, 0.13650D-01, 0.13381D-01, 0.13115D-01, 0.12854D-01,
     & 0.12595D-01, 0.12335D-01, 0.12082D-01, 0.11835D-01, 0.11593D-01,
     & 0.11356D-01, 0.11123D-01, 0.10893D-01, 0.10670D-01, 0.10453D-01,
     & 0.10242D-01, 0.10036D-01, 0.98346D-02, 0.96369D-02, 0.94448D-02,
     & 0.92580D-02, 0.90763D-02, 0.88996D-02, 0.87275D-02, 0.85599D-02,
     & 0.83956D-02, 0.82359D-02, 0.80805D-02, 0.79293D-02, 0.77821D-02,
     & 0.12983D-01, 0.12632D-01, 0.12292D-01, 0.11964D-01, 0.11647D-01,
     & 0.11339D-01, 0.11037D-01, 0.10746D-01, 0.10466D-01, 0.10194D-01,
     & 0.99323D-02, 0.96773D-02, 0.94302D-02, 0.91918D-02, 0.89616D-02,
     & 0.87393D-02, 0.85245D-02, 0.83166D-02, 0.81148D-02, 0.79199D-02,
     & 0.77316D-02, 0.75495D-02, 0.73734D-02, 0.72030D-02, 0.70381D-02,
     & 0.68783D-02, 0.67236D-02, 0.65738D-02, 0.64287D-02, 0.62882D-02,
     & 0.11996D-01, 0.11499D-01, 0.11030D-01, 0.10589D-01, 0.10173D-01,
     & 0.97804D-02, 0.94134D-02, 0.90655D-02, 0.87356D-02, 0.84226D-02,
     & 0.81257D-02, 0.78455D-02, 0.75801D-02, 0.73271D-02, 0.70858D-02,
     & 0.68557D-02, 0.66362D-02, 0.64274D-02, 0.62297D-02, 0.60403D-02,
     & 0.58588D-02, 0.56848D-02, 0.55182D-02, 0.53584D-02, 0.52054D-02,
     & 0.50604D-02, 0.49209D-02, 0.47867D-02, 0.46575D-02, 0.45331D-02,
     & 0.81943D-02, 0.77597D-02, 0.73551D-02, 0.69784D-02, 0.66277D-02,
     & 0.63024D-02, 0.60053D-02, 0.57264D-02, 0.54645D-02, 0.52187D-02,
     & 0.49881D-02, 0.47746D-02, 0.45755D-02, 0.43871D-02, 0.42089D-02,
     & 0.40404D-02, 0.38810D-02, 0.37314D-02, 0.35922D-02, 0.34597D-02,
     & 0.33335D-02, 0.32134D-02, 0.30991D-02, 0.29904D-02, 0.28871D-02,
     & 0.27910D-02, 0.26990D-02, 0.26109D-02, 0.25266D-02, 0.24459D-02,
     & 0.51728D-03, 0.48644D-03, 0.45786D-03, 0.43138D-03, 0.40686D-03,
     & 0.38426D-03, 0.36384D-03, 0.34475D-03, 0.32691D-03, 0.31024D-03,
     & 0.29468D-03, 0.28041D-03, 0.26720D-03, 0.25475D-03, 0.24302D-03,
     & 0.23197D-03, 0.22158D-03, 0.21188D-03, 0.20294D-03, 0.19447D-03,
     & 0.18642D-03, 0.17880D-03, 0.17157D-03, 0.16471D-03, 0.15823D-03,
     & 0.15228D-03, 0.14659D-03, 0.14116D-03, 0.13598D-03, 0.13103D-03/

      data ((g_grdl(i,j),j=1,30),i=1,10)/
     & 0.76535D+00, 0.82656D+00, 0.88784D+00, 0.94915D+00, 0.10105D+01,
     & 0.10717D+01, 0.11331D+01, 0.11942D+01, 0.12552D+01, 0.13159D+01,
     & 0.13763D+01, 0.14366D+01, 0.14967D+01, 0.15564D+01, 0.16157D+01,
     & 0.16747D+01, 0.17333D+01, 0.17915D+01, 0.18496D+01, 0.19072D+01,
     & 0.19643D+01, 0.20209D+01, 0.20771D+01, 0.21329D+01, 0.21882D+01,
     & 0.22434D+01, 0.22980D+01, 0.23521D+01, 0.24058D+01, 0.24589D+01,
     & 0.76009D+00, 0.81820D+00, 0.87626D+00, 0.93422D+00, 0.99205D+00,
     & 0.10498D+01, 0.11075D+01, 0.11649D+01, 0.12220D+01, 0.12788D+01,
     & 0.13353D+01, 0.13915D+01, 0.14475D+01, 0.15031D+01, 0.15582D+01,
     & 0.16129D+01, 0.16672D+01, 0.17212D+01, 0.17749D+01, 0.18280D+01,
     & 0.18807D+01, 0.19329D+01, 0.19846D+01, 0.20359D+01, 0.20867D+01,
     & 0.21373D+01, 0.21874D+01, 0.22370D+01, 0.22860D+01, 0.23346D+01,
     & 0.75400D+00, 0.80903D+00, 0.86389D+00, 0.91854D+00, 0.97297D+00,
     & 0.10272D+01, 0.10813D+01, 0.11351D+01, 0.11885D+01, 0.12415D+01,
     & 0.12941D+01, 0.13465D+01, 0.13986D+01, 0.14502D+01, 0.15013D+01,
     & 0.15519D+01, 0.16021D+01, 0.16519D+01, 0.17014D+01, 0.17504D+01,
     & 0.17989D+01, 0.18469D+01, 0.18943D+01, 0.19413D+01, 0.19879D+01,
     & 0.20342D+01, 0.20800D+01, 0.21252D+01, 0.21700D+01, 0.22142D+01,
     & 0.74700D+00, 0.79902D+00, 0.85075D+00, 0.90218D+00, 0.95331D+00,
     & 0.10041D+01, 0.10548D+01, 0.11051D+01, 0.11549D+01, 0.12043D+01,
     & 0.12532D+01, 0.13019D+01, 0.13502D+01, 0.13980D+01, 0.14452D+01,
     & 0.14920D+01, 0.15382D+01, 0.15841D+01, 0.16297D+01, 0.16747D+01,
     & 0.17192D+01, 0.17632D+01, 0.18066D+01, 0.18496D+01, 0.18921D+01,
     & 0.19344D+01, 0.19761D+01, 0.20173D+01, 0.20581D+01, 0.20983D+01,
     & 0.73881D+00, 0.78783D+00, 0.83648D+00, 0.88474D+00, 0.93261D+00,
     & 0.98014D+00, 0.10275D+01, 0.10743D+01, 0.11206D+01, 0.11665D+01,
     & 0.12119D+01, 0.12569D+01, 0.13016D+01, 0.13457D+01, 0.13893D+01,
     & 0.14323D+01, 0.14749D+01, 0.15170D+01, 0.15588D+01, 0.16000D+01,
     & 0.16407D+01, 0.16809D+01, 0.17205D+01, 0.17597D+01, 0.17984D+01,
     & 0.18368D+01, 0.18748D+01, 0.19122D+01, 0.19490D+01, 0.19854D+01,
     & 0.72926D+00, 0.77532D+00, 0.82092D+00, 0.86608D+00, 0.91078D+00,
     & 0.95507D+00, 0.99913D+00, 0.10426D+01, 0.10856D+01, 0.11281D+01,
     & 0.11700D+01, 0.12116D+01, 0.12528D+01, 0.12934D+01, 0.13334D+01,
     & 0.13729D+01, 0.14119D+01, 0.14504D+01, 0.14886D+01, 0.15263D+01,
     & 0.15634D+01, 0.15999D+01, 0.16360D+01, 0.16716D+01, 0.17067D+01,
     & 0.17415D+01, 0.17758D+01, 0.18096D+01, 0.18429D+01, 0.18757D+01,
     & 0.71818D+00, 0.76134D+00, 0.80398D+00, 0.84610D+00, 0.88772D+00,
     & 0.92888D+00, 0.96976D+00, 0.10101D+01, 0.10498D+01, 0.10890D+01,
     & 0.11276D+01, 0.11659D+01, 0.12037D+01, 0.12409D+01, 0.12776D+01,
     & 0.13137D+01, 0.13493D+01, 0.13845D+01, 0.14193D+01, 0.14535D+01,
     & 0.14872D+01, 0.15204D+01, 0.15531D+01, 0.15852D+01, 0.16169D+01,
     & 0.16484D+01, 0.16793D+01, 0.17097D+01, 0.17397D+01, 0.17691D+01,
     & 0.70548D+00, 0.74580D+00, 0.78554D+00, 0.82471D+00, 0.86335D+00,
     & 0.90149D+00, 0.93931D+00, 0.97651D+00, 0.10131D+01, 0.10492D+01,
     & 0.10847D+01, 0.11198D+01, 0.11544D+01, 0.11885D+01, 0.12219D+01,
     & 0.12549D+01, 0.12873D+01, 0.13192D+01, 0.13508D+01, 0.13818D+01,
     & 0.14123D+01, 0.14423D+01, 0.14718D+01, 0.15008D+01, 0.15293D+01,
     & 0.15576D+01, 0.15854D+01, 0.16126D+01, 0.16395D+01, 0.16658D+01,
     & 0.69102D+00, 0.72859D+00, 0.76554D+00, 0.80188D+00, 0.83765D+00,
     & 0.87290D+00, 0.90779D+00, 0.94205D+00, 0.97571D+00, 0.10088D+01,
     & 0.10413D+01, 0.10734D+01, 0.11050D+01, 0.11361D+01, 0.11665D+01,
     & 0.11965D+01, 0.12259D+01, 0.12548D+01, 0.12834D+01, 0.13114D+01,
     & 0.13389D+01, 0.13659D+01, 0.13924D+01, 0.14185D+01, 0.14441D+01,
     & 0.14694D+01, 0.14942D+01, 0.15186D+01, 0.15425D+01, 0.15660D+01,
     & 0.67442D+00, 0.70931D+00, 0.74355D+00, 0.77717D+00, 0.81018D+00,
     & 0.84264D+00, 0.87472D+00, 0.90617D+00, 0.93700D+00, 0.96724D+00,
     & 0.99691D+00, 0.10262D+01, 0.10549D+01, 0.10831D+01, 0.11108D+01,
     & 0.11378D+01, 0.11644D+01, 0.11905D+01, 0.12162D+01, 0.12415D+01,
     & 0.12662D+01, 0.12904D+01, 0.13141D+01, 0.13374D+01, 0.13602D+01,
     & 0.13828D+01, 0.14049D+01, 0.14266D+01, 0.14478D+01, 0.14687D+01/

      data ((g_grdl(i,j),j=1,30),i=11,20)/
     & 0.65553D+00, 0.68784D+00, 0.71948D+00, 0.75046D+00, 0.78083D+00,
     & 0.81064D+00, 0.84005D+00, 0.86881D+00, 0.89696D+00, 0.92451D+00,
     & 0.95150D+00, 0.97808D+00, 0.10042D+01, 0.10297D+01, 0.10546D+01,
     & 0.10790D+01, 0.11029D+01, 0.11264D+01, 0.11495D+01, 0.11721D+01,
     & 0.11942D+01, 0.12158D+01, 0.12370D+01, 0.12577D+01, 0.12780D+01,
     & 0.12980D+01, 0.13176D+01, 0.13368D+01, 0.13555D+01, 0.13739D+01,
     & 0.63436D+00, 0.66424D+00, 0.69343D+00, 0.72195D+00, 0.74985D+00,
     & 0.77717D+00, 0.80408D+00, 0.83034D+00, 0.85599D+00, 0.88105D+00,
     & 0.90555D+00, 0.92963D+00, 0.95322D+00, 0.97625D+00, 0.99873D+00,
     & 0.10207D+01, 0.10422D+01, 0.10632D+01, 0.10839D+01, 0.11040D+01,
     & 0.11238D+01, 0.11430D+01, 0.11618D+01, 0.11802D+01, 0.11982D+01,
     & 0.12159D+01, 0.12332D+01, 0.12501D+01, 0.12666D+01, 0.12827D+01,
     & 0.61065D+00, 0.63823D+00, 0.66511D+00, 0.69131D+00, 0.71689D+00,
     & 0.74188D+00, 0.76645D+00, 0.79038D+00, 0.81369D+00, 0.83643D+00,
     & 0.85861D+00, 0.88037D+00, 0.90165D+00, 0.92237D+00, 0.94257D+00,
     & 0.96227D+00, 0.98147D+00, 0.10003D+01, 0.10187D+01, 0.10367D+01,
     & 0.10542D+01, 0.10712D+01, 0.10879D+01, 0.11041D+01, 0.11200D+01,
     & 0.11355D+01, 0.11507D+01, 0.11656D+01, 0.11800D+01, 0.11941D+01,
     & 0.58471D+00, 0.61018D+00, 0.63494D+00, 0.65903D+00, 0.68247D+00,
     & 0.70534D+00, 0.72777D+00, 0.74957D+00, 0.77076D+00, 0.79138D+00,
     & 0.81146D+00, 0.83111D+00, 0.85030D+00, 0.86894D+00, 0.88707D+00,
     & 0.90471D+00, 0.92188D+00, 0.93864D+00, 0.95506D+00, 0.97102D+00,
     & 0.98654D+00, 0.10016D+01, 0.10163D+01, 0.10307D+01, 0.10446D+01,
     & 0.10583D+01, 0.10716D+01, 0.10845D+01, 0.10972D+01, 0.11094D+01,
     & 0.55665D+00, 0.58022D+00, 0.60306D+00, 0.62523D+00, 0.64676D+00,
     & 0.66770D+00, 0.68821D+00, 0.70809D+00, 0.72737D+00, 0.74608D+00,
     & 0.76426D+00, 0.78202D+00, 0.79932D+00, 0.81609D+00, 0.83236D+00,
     & 0.84816D+00, 0.86350D+00, 0.87845D+00, 0.89305D+00, 0.90721D+00,
     & 0.92096D+00, 0.93430D+00, 0.94726D+00, 0.95986D+00, 0.97210D+00,
     & 0.98409D+00, 0.99572D+00, 0.10070D+01, 0.10180D+01, 0.10286D+01,
     & 0.52613D+00, 0.54801D+00, 0.56916D+00, 0.58963D+00, 0.60945D+00,
     & 0.62869D+00, 0.64748D+00, 0.66565D+00, 0.68322D+00, 0.70025D+00,
     & 0.71674D+00, 0.73281D+00, 0.74842D+00, 0.76352D+00, 0.77814D+00,
     & 0.79230D+00, 0.80601D+00, 0.81934D+00, 0.83233D+00, 0.84490D+00,
     & 0.85707D+00, 0.86885D+00, 0.88026D+00, 0.89133D+00, 0.90206D+00,
     & 0.91253D+00, 0.92267D+00, 0.93249D+00, 0.94200D+00, 0.95121D+00,
     & 0.49425D+00, 0.51470D+00, 0.53441D+00, 0.55344D+00, 0.57181D+00,
     & 0.58959D+00, 0.60690D+00, 0.62360D+00, 0.63972D+00, 0.65528D+00,
     & 0.67031D+00, 0.68492D+00, 0.69907D+00, 0.71273D+00, 0.72591D+00,
     & 0.73865D+00, 0.75095D+00, 0.76287D+00, 0.77445D+00, 0.78563D+00,
     & 0.79642D+00, 0.80684D+00, 0.81691D+00, 0.82664D+00, 0.83605D+00,
     & 0.84520D+00, 0.85404D+00, 0.86257D+00, 0.87081D+00, 0.87877D+00,
     & 0.46174D+00, 0.48105D+00, 0.49961D+00, 0.51746D+00, 0.53465D+00,
     & 0.55123D+00, 0.56733D+00, 0.58281D+00, 0.59770D+00, 0.61204D+00,
     & 0.62585D+00, 0.63923D+00, 0.65215D+00, 0.66458D+00, 0.67654D+00,
     & 0.68806D+00, 0.69915D+00, 0.70986D+00, 0.72024D+00, 0.73022D+00,
     & 0.73983D+00, 0.74908D+00, 0.75798D+00, 0.76656D+00, 0.77483D+00,
     & 0.78284D+00, 0.79054D+00, 0.79796D+00, 0.80509D+00, 0.81196D+00,
     & 0.42992D+00, 0.44841D+00, 0.46612D+00, 0.48310D+00, 0.49939D+00,
     & 0.51505D+00, 0.53020D+00, 0.54472D+00, 0.55864D+00, 0.57199D+00,
     & 0.58481D+00, 0.59718D+00, 0.60909D+00, 0.62051D+00, 0.63145D+00,
     & 0.64196D+00, 0.65204D+00, 0.66174D+00, 0.67109D+00, 0.68006D+00,
     & 0.68866D+00, 0.69690D+00, 0.70481D+00, 0.71240D+00, 0.71968D+00,
     & 0.72670D+00, 0.73342D+00, 0.73987D+00, 0.74604D+00, 0.75196D+00,
     & 0.40060D+00, 0.41861D+00, 0.43580D+00, 0.45221D+00, 0.46790D+00,
     & 0.48292D+00, 0.49738D+00, 0.51119D+00, 0.52438D+00, 0.53698D+00,
     & 0.54903D+00, 0.56061D+00, 0.57171D+00, 0.58230D+00, 0.59242D+00,
     & 0.60209D+00, 0.61133D+00, 0.62017D+00, 0.62866D+00, 0.63676D+00,
     & 0.64449D+00, 0.65188D+00, 0.65892D+00, 0.66565D+00, 0.67207D+00,
     & 0.67822D+00, 0.68408D+00, 0.68966D+00, 0.69499D+00, 0.70006D+00/

      data ((g_grdl(i,j),j=1,30),i=21,30)/
     & 0.37631D+00, 0.39420D+00, 0.41119D+00, 0.42735D+00, 0.44272D+00,
     & 0.45736D+00, 0.47139D+00, 0.48472D+00, 0.49740D+00, 0.50945D+00,
     & 0.52093D+00, 0.53188D+00, 0.54233D+00, 0.55226D+00, 0.56169D+00,
     & 0.57065D+00, 0.57918D+00, 0.58729D+00, 0.59501D+00, 0.60235D+00,
     & 0.60931D+00, 0.61592D+00, 0.62218D+00, 0.62812D+00, 0.63376D+00,
     & 0.63910D+00, 0.64416D+00, 0.64894D+00, 0.65347D+00, 0.65774D+00,
     & 0.36045D+00, 0.37858D+00, 0.39570D+00, 0.41189D+00, 0.42721D+00,
     & 0.44171D+00, 0.45551D+00, 0.46854D+00, 0.48087D+00, 0.49252D+00,
     & 0.50354D+00, 0.51398D+00, 0.52386D+00, 0.53320D+00, 0.54201D+00,
     & 0.55033D+00, 0.55818D+00, 0.56558D+00, 0.57257D+00, 0.57915D+00,
     & 0.58535D+00, 0.59118D+00, 0.59667D+00, 0.60181D+00, 0.60665D+00,
     & 0.61116D+00, 0.61538D+00, 0.61934D+00, 0.62303D+00, 0.62648D+00,
     & 0.35918D+00, 0.37788D+00, 0.39543D+00, 0.41190D+00, 0.42737D+00,
     & 0.44190D+00, 0.45559D+00, 0.46843D+00, 0.48047D+00, 0.49176D+00,
     & 0.50234D+00, 0.51227D+00, 0.52156D+00, 0.53026D+00, 0.53840D+00,
     & 0.54600D+00, 0.55310D+00, 0.55971D+00, 0.56585D+00, 0.57156D+00,
     & 0.57688D+00, 0.58181D+00, 0.58638D+00, 0.59061D+00, 0.59450D+00,
     & 0.59803D+00, 0.60127D+00, 0.60424D+00, 0.60696D+00, 0.60942D+00,
     & 0.38130D+00, 0.40079D+00, 0.41890D+00, 0.43574D+00, 0.45138D+00,
     & 0.46590D+00, 0.47937D+00, 0.49186D+00, 0.50345D+00, 0.51418D+00,
     & 0.52410D+00, 0.53323D+00, 0.54164D+00, 0.54939D+00, 0.55653D+00,
     & 0.56309D+00, 0.56909D+00, 0.57455D+00, 0.57946D+00, 0.58393D+00,
     & 0.58799D+00, 0.59165D+00, 0.59493D+00, 0.59786D+00, 0.60043D+00,
     & 0.60259D+00, 0.60447D+00, 0.60609D+00, 0.60745D+00, 0.60857D+00,
     & 0.44281D+00, 0.46279D+00, 0.48108D+00, 0.49782D+00, 0.51309D+00,
     & 0.52699D+00, 0.53952D+00, 0.55092D+00, 0.56127D+00, 0.57063D+00,
     & 0.57906D+00, 0.58652D+00, 0.59314D+00, 0.59905D+00, 0.60429D+00,
     & 0.60891D+00, 0.61294D+00, 0.61635D+00, 0.61912D+00, 0.62144D+00,
     & 0.62336D+00, 0.62488D+00, 0.62604D+00, 0.62684D+00, 0.62729D+00,
     & 0.62725D+00, 0.62696D+00, 0.62644D+00, 0.62570D+00, 0.62474D+00,
     & 0.46653D+00, 0.48646D+00, 0.50462D+00, 0.52113D+00, 0.53611D+00,
     & 0.54964D+00, 0.56168D+00, 0.57257D+00, 0.58237D+00, 0.59115D+00,
     & 0.59897D+00, 0.60577D+00, 0.61169D+00, 0.61690D+00, 0.62144D+00,
     & 0.62534D+00, 0.62866D+00, 0.63134D+00, 0.63335D+00, 0.63494D+00,
     & 0.63612D+00, 0.63691D+00, 0.63734D+00, 0.63743D+00, 0.63717D+00,
     & 0.63641D+00, 0.63541D+00, 0.63419D+00, 0.63277D+00, 0.63114D+00,
     & 0.49370D+00, 0.51343D+00, 0.53130D+00, 0.54746D+00, 0.56201D+00,
     & 0.57503D+00, 0.58647D+00, 0.59672D+00, 0.60585D+00, 0.61394D+00,
     & 0.62104D+00, 0.62707D+00, 0.63218D+00, 0.63659D+00, 0.64032D+00,
     & 0.64341D+00, 0.64591D+00, 0.64778D+00, 0.64894D+00, 0.64969D+00,
     & 0.65005D+00, 0.65003D+00, 0.64966D+00, 0.64896D+00, 0.64792D+00,
     & 0.64636D+00, 0.64459D+00, 0.64261D+00, 0.64044D+00, 0.63809D+00,
     & 0.52454D+00, 0.54394D+00, 0.56139D+00, 0.57705D+00, 0.59102D+00,
     & 0.60339D+00, 0.61407D+00, 0.62352D+00, 0.63183D+00, 0.63907D+00,
     & 0.64531D+00, 0.65041D+00, 0.65458D+00, 0.65805D+00, 0.66084D+00,
     & 0.66301D+00, 0.66458D+00, 0.66551D+00, 0.66572D+00, 0.66554D+00,
     & 0.66498D+00, 0.66406D+00, 0.66280D+00, 0.66123D+00, 0.65933D+00,
     & 0.65691D+00, 0.65429D+00, 0.65149D+00, 0.64852D+00, 0.64539D+00,
     & 0.55930D+00, 0.57815D+00, 0.59498D+00, 0.60993D+00, 0.62314D+00,
     & 0.63468D+00, 0.64441D+00, 0.65290D+00, 0.66022D+00, 0.66646D+00,
     & 0.67168D+00, 0.67573D+00, 0.67881D+00, 0.68120D+00, 0.68293D+00,
     & 0.68404D+00, 0.68456D+00, 0.68444D+00, 0.68359D+00, 0.68236D+00,
     & 0.68078D+00, 0.67886D+00, 0.67663D+00, 0.67410D+00, 0.67126D+00,
     & 0.66789D+00, 0.66435D+00, 0.66066D+00, 0.65682D+00, 0.65283D+00,
     & 0.59824D+00, 0.61632D+00, 0.63231D+00, 0.64636D+00, 0.65859D+00,
     & 0.66908D+00, 0.67765D+00, 0.68496D+00, 0.69110D+00, 0.69615D+00,
     & 0.70017D+00, 0.70296D+00, 0.70479D+00, 0.70593D+00, 0.70644D+00,
     & 0.70635D+00, 0.70568D+00, 0.70438D+00, 0.70234D+00, 0.69995D+00,
     & 0.69723D+00, 0.69421D+00, 0.69090D+00, 0.68732D+00, 0.68346D+00,
     & 0.67906D+00, 0.67452D+00, 0.66986D+00, 0.66508D+00, 0.66019D+00/

      data ((g_grdl(i,j),j=1,30),i=31,40)/
     & 0.64164D+00, 0.65866D+00, 0.67353D+00, 0.68639D+00, 0.69739D+00,
     & 0.70658D+00, 0.71374D+00, 0.71965D+00, 0.72439D+00, 0.72803D+00,
     & 0.73065D+00, 0.73201D+00, 0.73239D+00, 0.73212D+00, 0.73124D+00,
     & 0.72979D+00, 0.72779D+00, 0.72517D+00, 0.72180D+00, 0.71812D+00,
     & 0.71415D+00, 0.70992D+00, 0.70543D+00, 0.70069D+00, 0.69571D+00,
     & 0.69020D+00, 0.68459D+00, 0.67888D+00, 0.67310D+00, 0.66723D+00,
     & 0.68981D+00, 0.70543D+00, 0.71885D+00, 0.73021D+00, 0.73968D+00,
     & 0.74729D+00, 0.75276D+00, 0.75699D+00, 0.76007D+00, 0.76207D+00,
     & 0.76305D+00, 0.76276D+00, 0.76148D+00, 0.75960D+00, 0.75714D+00,
     & 0.75415D+00, 0.75066D+00, 0.74656D+00, 0.74172D+00, 0.73662D+00,
     & 0.73127D+00, 0.72570D+00, 0.71992D+00, 0.71394D+00, 0.70774D+00,
     & 0.70103D+00, 0.69426D+00, 0.68744D+00, 0.68057D+00, 0.67367D+00,
     & 0.74305D+00, 0.75687D+00, 0.76845D+00, 0.77796D+00, 0.78555D+00,
     & 0.79125D+00, 0.79470D+00, 0.79695D+00, 0.79807D+00, 0.79815D+00,
     & 0.79725D+00, 0.79504D+00, 0.79188D+00, 0.78817D+00, 0.78393D+00,
     & 0.77922D+00, 0.77405D+00, 0.76831D+00, 0.76184D+00, 0.75517D+00,
     & 0.74832D+00, 0.74130D+00, 0.73411D+00, 0.72676D+00, 0.71925D+00,
     & 0.71125D+00, 0.70324D+00, 0.69523D+00, 0.68722D+00, 0.67921D+00,
     & 0.80163D+00, 0.81316D+00, 0.82245D+00, 0.82967D+00, 0.83497D+00,
     & 0.83838D+00, 0.83945D+00, 0.83938D+00, 0.83824D+00, 0.83610D+00,
     & 0.83303D+00, 0.82867D+00, 0.82337D+00, 0.81760D+00, 0.81138D+00,
     & 0.80474D+00, 0.79770D+00, 0.79015D+00, 0.78190D+00, 0.77351D+00,
     & 0.76501D+00, 0.75640D+00, 0.74769D+00, 0.73888D+00, 0.72995D+00,
     & 0.72057D+00, 0.71125D+00, 0.70197D+00, 0.69275D+00, 0.68357D+00,
     & 0.86585D+00, 0.87456D+00, 0.88106D+00, 0.88551D+00, 0.88807D+00,
     & 0.88874D+00, 0.88700D+00, 0.88421D+00, 0.88042D+00, 0.87572D+00,
     & 0.87016D+00, 0.86332D+00, 0.85561D+00, 0.84751D+00, 0.83906D+00,
     & 0.83027D+00, 0.82117D+00, 0.81161D+00, 0.80141D+00, 0.79116D+00,
     & 0.78087D+00, 0.77054D+00, 0.76019D+00, 0.74981D+00, 0.73937D+00,
     & 0.72854D+00, 0.71782D+00, 0.70721D+00, 0.69672D+00, 0.68633D+00,
     & 0.93637D+00, 0.94153D+00, 0.94456D+00, 0.94562D+00, 0.94486D+00,
     & 0.94227D+00, 0.93724D+00, 0.93127D+00, 0.92443D+00, 0.91677D+00,
     & 0.90836D+00, 0.89872D+00, 0.88830D+00, 0.87760D+00, 0.86665D+00,
     & 0.85547D+00, 0.84408D+00, 0.83232D+00, 0.81998D+00, 0.80770D+00,
     & 0.79546D+00, 0.78328D+00, 0.77115D+00, 0.75908D+00, 0.74704D+00,
     & 0.73466D+00, 0.72247D+00, 0.71046D+00, 0.69862D+00, 0.68696D+00,
     & 0.10130D+01, 0.10139D+01, 0.10128D+01, 0.10098D+01, 0.10051D+01,
     & 0.99867D+00, 0.98980D+00, 0.98015D+00, 0.96978D+00, 0.95875D+00,
     & 0.94709D+00, 0.93430D+00, 0.92082D+00, 0.90722D+00, 0.89351D+00,
     & 0.87970D+00, 0.86579D+00, 0.85163D+00, 0.83699D+00, 0.82251D+00,
     & 0.80820D+00, 0.79404D+00, 0.78003D+00, 0.76617D+00, 0.75243D+00,
     & 0.73844D+00, 0.72473D+00, 0.71127D+00, 0.69806D+00, 0.68510D+00,
     & 0.10962D+01, 0.10918D+01, 0.10856D+01, 0.10778D+01, 0.10685D+01,
     & 0.10575D+01, 0.10442D+01, 0.10304D+01, 0.10160D+01, 0.10011D+01,
     & 0.98577D+00, 0.96944D+00, 0.95259D+00, 0.93578D+00, 0.91903D+00,
     & 0.90233D+00, 0.88569D+00, 0.86893D+00, 0.85182D+00, 0.83500D+00,
     & 0.81846D+00, 0.80220D+00, 0.78621D+00, 0.77048D+00, 0.75496D+00,
     & 0.73932D+00, 0.72403D+00, 0.70909D+00, 0.69448D+00, 0.68019D+00,
     & 0.11861D+01, 0.11753D+01, 0.11631D+01, 0.11495D+01, 0.11347D+01,
     & 0.11185D+01, 0.11000D+01, 0.10813D+01, 0.10623D+01, 0.10431D+01,
     & 0.10236D+01, 0.10034D+01, 0.98276D+00, 0.96242D+00, 0.94233D+00,
     & 0.92248D+00, 0.90287D+00, 0.88331D+00, 0.86356D+00, 0.84425D+00,
     & 0.82538D+00, 0.80691D+00, 0.78885D+00, 0.77116D+00, 0.75381D+00,
     & 0.73648D+00, 0.71960D+00, 0.70316D+00, 0.68714D+00, 0.67153D+00,
     & 0.12827D+01, 0.12644D+01, 0.12449D+01, 0.12245D+01, 0.12032D+01,
     & 0.11808D+01, 0.11565D+01, 0.11321D+01, 0.11079D+01, 0.10837D+01,
     & 0.10596D+01, 0.10350D+01, 0.10103D+01, 0.98603D+00, 0.96230D+00,
     & 0.93903D+00, 0.91622D+00, 0.89367D+00, 0.87114D+00, 0.84923D+00,
     & 0.82791D+00, 0.80716D+00, 0.78695D+00, 0.76726D+00, 0.74805D+00,
     & 0.72903D+00, 0.71057D+00, 0.69265D+00, 0.67525D+00, 0.65834D+00/

      data ((g_grdl(i,j),j=1,30),i=41,50)/
     & 0.13861D+01, 0.13585D+01, 0.13305D+01, 0.13020D+01, 0.12730D+01,
     & 0.12435D+01, 0.12124D+01, 0.11818D+01, 0.11516D+01, 0.11219D+01,
     & 0.10927D+01, 0.10632D+01, 0.10340D+01, 0.10056D+01, 0.97790D+00,
     & 0.95097D+00, 0.92475D+00, 0.89904D+00, 0.87362D+00, 0.84900D+00,
     & 0.82516D+00, 0.80206D+00, 0.77967D+00, 0.75795D+00, 0.73687D+00,
     & 0.71619D+00, 0.69619D+00, 0.67683D+00, 0.65810D+00, 0.63996D+00,
     & 0.14958D+01, 0.14573D+01, 0.14190D+01, 0.13809D+01, 0.13431D+01,
     & 0.13053D+01, 0.12664D+01, 0.12286D+01, 0.11918D+01, 0.11558D+01,
     & 0.11208D+01, 0.10860D+01, 0.10519D+01, 0.10189D+01, 0.98707D+00,
     & 0.95624D+00, 0.92641D+00, 0.89741D+00, 0.86903D+00, 0.84167D+00,
     & 0.81529D+00, 0.78984D+00, 0.76529D+00, 0.74159D+00, 0.71869D+00,
     & 0.69646D+00, 0.67502D+00, 0.65434D+00, 0.63439D+00, 0.61514D+00,
     & 0.16109D+01, 0.15591D+01, 0.15086D+01, 0.14592D+01, 0.14110D+01,
     & 0.13636D+01, 0.13159D+01, 0.12700D+01, 0.12257D+01, 0.11829D+01,
     & 0.11415D+01, 0.11010D+01, 0.10616D+01, 0.10239D+01, 0.98758D+00,
     & 0.95269D+00, 0.91913D+00, 0.88678D+00, 0.85545D+00, 0.82539D+00,
     & 0.79652D+00, 0.76880D+00, 0.74217D+00, 0.71659D+00, 0.69201D+00,
     & 0.66839D+00, 0.64570D+00, 0.62388D+00, 0.60290D+00, 0.58273D+00,
     & 0.17293D+01, 0.16617D+01, 0.15967D+01, 0.15340D+01, 0.14735D+01,
     & 0.14150D+01, 0.13574D+01, 0.13023D+01, 0.12496D+01, 0.11991D+01,
     & 0.11508D+01, 0.11042D+01, 0.10594D+01, 0.10166D+01, 0.97578D+00,
     & 0.93676D+00, 0.89948D+00, 0.86384D+00, 0.82976D+00, 0.79718D+00,
     & 0.76604D+00, 0.73627D+00, 0.70781D+00, 0.68061D+00, 0.65461D+00,
     & 0.62994D+00, 0.60631D+00, 0.58366D+00, 0.56197D+00, 0.54118D+00,
     & 0.18460D+01, 0.17592D+01, 0.16767D+01, 0.15983D+01, 0.15236D+01,
     & 0.14524D+01, 0.13837D+01, 0.13186D+01, 0.12569D+01, 0.11983D+01,
     & 0.11427D+01, 0.10898D+01, 0.10396D+01, 0.99196D+00, 0.94671D+00,
     & 0.90377D+00, 0.86301D+00, 0.82441D+00, 0.78798D+00, 0.75331D+00,
     & 0.72033D+00, 0.68895D+00, 0.65910D+00, 0.63071D+00, 0.60375D+00,
     & 0.57852D+00, 0.55444D+00, 0.53144D+00, 0.50949D+00, 0.48855D+00,
     & 0.19554D+01, 0.18452D+01, 0.17417D+01, 0.16445D+01, 0.15533D+01,
     & 0.14677D+01, 0.13874D+01, 0.13119D+01, 0.12409D+01, 0.11740D+01,
     & 0.11112D+01, 0.10524D+01, 0.99718D+00, 0.94510D+00, 0.89597D+00,
     & 0.84963D+00, 0.80593D+00, 0.76494D+00, 0.72680D+00, 0.69067D+00,
     & 0.65646D+00, 0.62407D+00, 0.59343D+00, 0.56446D+00, 0.53712D+00,
     & 0.51195D+00, 0.48802D+00, 0.46526D+00, 0.44363D+00, 0.42308D+00,
     & 0.20489D+01, 0.19149D+01, 0.17899D+01, 0.16732D+01, 0.15643D+01,
     & 0.14627D+01, 0.13681D+01, 0.12797D+01, 0.11973D+01, 0.11205D+01,
     & 0.10489D+01, 0.98298D+00, 0.92196D+00, 0.86485D+00, 0.81143D+00,
     & 0.76151D+00, 0.71490D+00, 0.67185D+00, 0.63271D+00, 0.59590D+00,
     & 0.56128D+00, 0.52877D+00, 0.49825D+00, 0.46964D+00, 0.44294D+00,
     & 0.41907D+00, 0.39649D+00, 0.37515D+00, 0.35498D+00, 0.33594D+00,
     & 0.21643D+01, 0.19663D+01, 0.17864D+01, 0.16233D+01, 0.14759D+01,
     & 0.13440D+01, 0.12316D+01, 0.11286D+01, 0.10342D+01, 0.94809D+00,
     & 0.86965D+00, 0.80137D+00, 0.74066D+00, 0.68449D+00, 0.63262D+00,
     & 0.58481D+00, 0.54084D+00, 0.50131D+00, 0.46704D+00, 0.43504D+00,
     & 0.40521D+00, 0.37745D+00, 0.35164D+00, 0.32771D+00, 0.30567D+00,
     & 0.28685D+00, 0.26915D+00, 0.25251D+00, 0.23688D+00, 0.22223D+00,
     & 0.16121D+01, 0.14345D+01, 0.12751D+01, 0.11327D+01, 0.10060D+01,
     & 0.89542D+00, 0.80626D+00, 0.72536D+00, 0.65220D+00, 0.58631D+00,
     & 0.52723D+00, 0.47790D+00, 0.43545D+00, 0.39653D+00, 0.36092D+00,
     & 0.32845D+00, 0.29894D+00, 0.27301D+00, 0.25150D+00, 0.23155D+00,
     & 0.21308D+00, 0.19602D+00, 0.18030D+00, 0.16586D+00, 0.15274D+00,
     & 0.14205D+00, 0.13204D+00, 0.12268D+00, 0.11395D+00, 0.10581D+00,
     & 0.10658D+00, 0.93837D-01, 0.82464D-01, 0.72359D-01, 0.63433D-01,
     & 0.55712D-01, 0.49637D-01, 0.44154D-01, 0.39222D-01, 0.34809D-01,
     & 0.30882D-01, 0.27671D-01, 0.24956D-01, 0.22479D-01, 0.20224D-01,
     & 0.18181D-01, 0.16338D-01, 0.14740D-01, 0.13450D-01, 0.12259D-01,
     & 0.11161D-01, 0.10153D-01, 0.92295D-02, 0.83868D-02, 0.76280D-02,
     & 0.70305D-02, 0.64734D-02, 0.59549D-02, 0.54733D-02, 0.50270D-02/

      data ((c_grdl(i,j),j=1,30),i=1,10)/
     & 0.48607D-02, 0.48645D-02, 0.48678D-02, 0.48708D-02, 0.48733D-02,
     & 0.48756D-02, 0.48776D-02, 0.48794D-02, 0.48809D-02, 0.48823D-02,
     & 0.48835D-02, 0.48846D-02, 0.48855D-02, 0.48863D-02, 0.48871D-02,
     & 0.48877D-02, 0.48883D-02, 0.48888D-02, 0.48892D-02, 0.48896D-02,
     & 0.48900D-02, 0.48903D-02, 0.48906D-02, 0.48908D-02, 0.48911D-02,
     & 0.48912D-02, 0.48914D-02, 0.48916D-02, 0.48917D-02, 0.48918D-02,
     & 0.48502D-02, 0.48547D-02, 0.48588D-02, 0.48623D-02, 0.48654D-02,
     & 0.48681D-02, 0.48705D-02, 0.48726D-02, 0.48745D-02, 0.48762D-02,
     & 0.48776D-02, 0.48789D-02, 0.48800D-02, 0.48810D-02, 0.48819D-02,
     & 0.48827D-02, 0.48833D-02, 0.48840D-02, 0.48845D-02, 0.48850D-02,
     & 0.48854D-02, 0.48858D-02, 0.48861D-02, 0.48864D-02, 0.48866D-02,
     & 0.48869D-02, 0.48871D-02, 0.48873D-02, 0.48874D-02, 0.48876D-02,
     & 0.48374D-02, 0.48430D-02, 0.48478D-02, 0.48521D-02, 0.48559D-02,
     & 0.48591D-02, 0.48620D-02, 0.48646D-02, 0.48668D-02, 0.48688D-02,
     & 0.48705D-02, 0.48721D-02, 0.48734D-02, 0.48746D-02, 0.48757D-02,
     & 0.48766D-02, 0.48774D-02, 0.48782D-02, 0.48788D-02, 0.48794D-02,
     & 0.48799D-02, 0.48803D-02, 0.48807D-02, 0.48811D-02, 0.48814D-02,
     & 0.48817D-02, 0.48819D-02, 0.48821D-02, 0.48823D-02, 0.48825D-02,
     & 0.48219D-02, 0.48287D-02, 0.48346D-02, 0.48398D-02, 0.48443D-02,
     & 0.48483D-02, 0.48518D-02, 0.48549D-02, 0.48576D-02, 0.48599D-02,
     & 0.48620D-02, 0.48639D-02, 0.48655D-02, 0.48670D-02, 0.48682D-02,
     & 0.48693D-02, 0.48703D-02, 0.48712D-02, 0.48720D-02, 0.48727D-02,
     & 0.48733D-02, 0.48738D-02, 0.48743D-02, 0.48747D-02, 0.48751D-02,
     & 0.48754D-02, 0.48757D-02, 0.48760D-02, 0.48762D-02, 0.48764D-02,
     & 0.48031D-02, 0.48113D-02, 0.48185D-02, 0.48248D-02, 0.48303D-02,
     & 0.48351D-02, 0.48394D-02, 0.48431D-02, 0.48464D-02, 0.48492D-02,
     & 0.48518D-02, 0.48540D-02, 0.48560D-02, 0.48577D-02, 0.48593D-02,
     & 0.48606D-02, 0.48618D-02, 0.48629D-02, 0.48638D-02, 0.48646D-02,
     & 0.48653D-02, 0.48660D-02, 0.48666D-02, 0.48671D-02, 0.48675D-02,
     & 0.48679D-02, 0.48682D-02, 0.48686D-02, 0.48688D-02, 0.48691D-02,
     & 0.47800D-02, 0.47901D-02, 0.47989D-02, 0.48066D-02, 0.48133D-02,
     & 0.48192D-02, 0.48243D-02, 0.48289D-02, 0.48328D-02, 0.48363D-02,
     & 0.48394D-02, 0.48421D-02, 0.48445D-02, 0.48466D-02, 0.48484D-02,
     & 0.48501D-02, 0.48515D-02, 0.48528D-02, 0.48539D-02, 0.48549D-02,
     & 0.48558D-02, 0.48565D-02, 0.48572D-02, 0.48578D-02, 0.48584D-02,
     & 0.48589D-02, 0.48593D-02, 0.48597D-02, 0.48600D-02, 0.48603D-02,
     & 0.49173D-02, 0.48860D-02, 0.48356D-02, 0.47843D-02, 0.47925D-02,
     & 0.47997D-02, 0.48060D-02, 0.48115D-02, 0.48164D-02, 0.48206D-02,
     & 0.48243D-02, 0.48276D-02, 0.48305D-02, 0.48331D-02, 0.48353D-02,
     & 0.48373D-02, 0.48390D-02, 0.48406D-02, 0.48419D-02, 0.48431D-02,
     & 0.48442D-02, 0.48451D-02, 0.48460D-02, 0.48467D-02, 0.48474D-02,
     & 0.48479D-02, 0.48485D-02, 0.48489D-02, 0.48493D-02, 0.48497D-02,
     & 0.49647D-02, 0.49483D-02, 0.49290D-02, 0.49056D-02, 0.48757D-02,
     & 0.48274D-02, 0.47836D-02, 0.47903D-02, 0.47963D-02, 0.48014D-02,
     & 0.48060D-02, 0.48100D-02, 0.48135D-02, 0.48166D-02, 0.48194D-02,
     & 0.48218D-02, 0.48239D-02, 0.48258D-02, 0.48274D-02, 0.48289D-02,
     & 0.48302D-02, 0.48313D-02, 0.48323D-02, 0.48332D-02, 0.48340D-02,
     & 0.48347D-02, 0.48353D-02, 0.48359D-02, 0.48364D-02, 0.48368D-02,
     & 0.49912D-02, 0.49822D-02, 0.49712D-02, 0.49583D-02, 0.49436D-02,
     & 0.49269D-02, 0.49077D-02, 0.48850D-02, 0.48557D-02, 0.47986D-02,
     & 0.47836D-02, 0.47885D-02, 0.47928D-02, 0.47966D-02, 0.47999D-02,
     & 0.48029D-02, 0.48055D-02, 0.48078D-02, 0.48098D-02, 0.48116D-02,
     & 0.48132D-02, 0.48145D-02, 0.48158D-02, 0.48169D-02, 0.48178D-02,
     & 0.48187D-02, 0.48194D-02, 0.48201D-02, 0.48207D-02, 0.48212D-02,
     & 0.50019D-02, 0.49994D-02, 0.49945D-02, 0.49878D-02, 0.49795D-02,
     & 0.49700D-02, 0.49594D-02, 0.49479D-02, 0.49354D-02, 0.49221D-02,
     & 0.49077D-02, 0.48920D-02, 0.48743D-02, 0.48534D-02, 0.48246D-02,
     & 0.47798D-02, 0.47830D-02, 0.47858D-02, 0.47882D-02, 0.47904D-02,
     & 0.47923D-02, 0.47940D-02, 0.47955D-02, 0.47969D-02, 0.47981D-02,
     & 0.47991D-02, 0.48000D-02, 0.48008D-02, 0.48015D-02, 0.48022D-02/

      data ((c_grdl(i,j),j=1,30),i=11,20)/
     & 0.49972D-02, 0.50011D-02, 0.50022D-02, 0.50012D-02, 0.49983D-02,
     & 0.49939D-02, 0.49884D-02, 0.49820D-02, 0.49749D-02, 0.49673D-02,
     & 0.49594D-02, 0.49513D-02, 0.49430D-02, 0.49346D-02, 0.49262D-02,
     & 0.49178D-02, 0.49095D-02, 0.49012D-02, 0.48929D-02, 0.48847D-02,
     & 0.48765D-02, 0.48683D-02, 0.48600D-02, 0.48516D-02, 0.48429D-02,
     & 0.48338D-02, 0.48238D-02, 0.48122D-02, 0.47953D-02, 0.47789D-02,
     & 0.53209D-02, 0.52939D-02, 0.52643D-02, 0.49992D-02, 0.50016D-02,
     & 0.50022D-02, 0.50013D-02, 0.49993D-02, 0.49962D-02, 0.49926D-02,
     & 0.49884D-02, 0.49839D-02, 0.49792D-02, 0.49744D-02, 0.49697D-02,
     & 0.49650D-02, 0.49604D-02, 0.49560D-02, 0.49517D-02, 0.49477D-02,
     & 0.49439D-02, 0.49404D-02, 0.49370D-02, 0.49340D-02, 0.49311D-02,
     & 0.49285D-02, 0.49260D-02, 0.49238D-02, 0.49218D-02, 0.49199D-02,
     & 0.53658D-02, 0.53512D-02, 0.53322D-02, 0.53102D-02, 0.52863D-02,
     & 0.52613D-02, 0.49989D-02, 0.50011D-02, 0.50021D-02, 0.50021D-02,
     & 0.50013D-02, 0.50000D-02, 0.49981D-02, 0.49960D-02, 0.49937D-02,
     & 0.49913D-02, 0.49889D-02, 0.49865D-02, 0.49841D-02, 0.49819D-02,
     & 0.49797D-02, 0.49777D-02, 0.49758D-02, 0.49741D-02, 0.49724D-02,
     & 0.49709D-02, 0.49696D-02, 0.49683D-02, 0.49672D-02, 0.49661D-02,
     & 0.53739D-02, 0.53757D-02, 0.53706D-02, 0.53604D-02, 0.53466D-02,
     & 0.53301D-02, 0.53121D-02, 0.52931D-02, 0.52738D-02, 0.52546D-02,
     & 0.49989D-02, 0.50007D-02, 0.50016D-02, 0.50021D-02, 0.50022D-02,
     & 0.50020D-02, 0.50015D-02, 0.50008D-02, 0.50001D-02, 0.49992D-02,
     & 0.49984D-02, 0.49975D-02, 0.49967D-02, 0.49959D-02, 0.49951D-02,
     & 0.49944D-02, 0.49937D-02, 0.49930D-02, 0.49925D-02, 0.49919D-02,
     & 0.53342D-02, 0.53572D-02, 0.53703D-02, 0.53756D-02, 0.53749D-02,
     & 0.53698D-02, 0.53614D-02, 0.53507D-02, 0.53386D-02, 0.53255D-02,
     & 0.53121D-02, 0.52986D-02, 0.52854D-02, 0.52726D-02, 0.52604D-02,
     & 0.49971D-02, 0.49986D-02, 0.49998D-02, 0.50006D-02, 0.50012D-02,
     & 0.50015D-02, 0.50018D-02, 0.50020D-02, 0.50021D-02, 0.50022D-02,
     & 0.50022D-02, 0.50022D-02, 0.50022D-02, 0.50021D-02, 0.50020D-02,
     & 0.52326D-02, 0.52831D-02, 0.53197D-02, 0.53449D-02, 0.53615D-02,
     & 0.53711D-02, 0.53754D-02, 0.53756D-02, 0.53728D-02, 0.53678D-02,
     & 0.53614D-02, 0.53540D-02, 0.53460D-02, 0.53378D-02, 0.53295D-02,
     & 0.53215D-02, 0.53137D-02, 0.53063D-02, 0.52994D-02, 0.52929D-02,
     & 0.52869D-02, 0.52813D-02, 0.52762D-02, 0.52716D-02, 0.52674D-02,
     & 0.52636D-02, 0.52601D-02, 0.52570D-02, 0.52542D-02, 0.49966D-02,
     & 0.50497D-02, 0.51364D-02, 0.52038D-02, 0.52552D-02, 0.52939D-02,
     & 0.53227D-02, 0.53433D-02, 0.53577D-02, 0.53671D-02, 0.53727D-02,
     & 0.53754D-02, 0.53759D-02, 0.53748D-02, 0.53725D-02, 0.53695D-02,
     & 0.53660D-02, 0.53622D-02, 0.53583D-02, 0.53544D-02, 0.53506D-02,
     & 0.53469D-02, 0.53434D-02, 0.53402D-02, 0.53371D-02, 0.53343D-02,
     & 0.53317D-02, 0.53294D-02, 0.53272D-02, 0.53252D-02, 0.53235D-02,
     & 0.47777D-02, 0.49034D-02, 0.50024D-02, 0.50877D-02, 0.51558D-02,
     & 0.52096D-02, 0.52516D-02, 0.52842D-02, 0.53095D-02, 0.53288D-02,
     & 0.53433D-02, 0.53541D-02, 0.53620D-02, 0.53676D-02, 0.53714D-02,
     & 0.53738D-02, 0.53752D-02, 0.53758D-02, 0.53759D-02, 0.53756D-02,
     & 0.53750D-02, 0.53742D-02, 0.53733D-02, 0.53723D-02, 0.53714D-02,
     & 0.53704D-02, 0.53694D-02, 0.53686D-02, 0.53677D-02, 0.53669D-02,
     & 0.47636D-02, 0.45576D-02, 0.47098D-02, 0.48329D-02, 0.49316D-02,
     & 0.50118D-02, 0.50815D-02, 0.51384D-02, 0.51844D-02, 0.52216D-02,
     & 0.52516D-02, 0.52757D-02, 0.52952D-02, 0.53109D-02, 0.53235D-02,
     & 0.53336D-02, 0.53417D-02, 0.53483D-02, 0.53536D-02, 0.53578D-02,
     & 0.53612D-02, 0.53640D-02, 0.53662D-02, 0.53679D-02, 0.53694D-02,
     & 0.53705D-02, 0.53714D-02, 0.53722D-02, 0.53728D-02, 0.53733D-02,
     & 0.51658D-02, 0.50069D-02, 0.46836D-02, 0.44510D-02, 0.46007D-02,
     & 0.47232D-02, 0.48239D-02, 0.49063D-02, 0.49732D-02, 0.50316D-02,
     & 0.50815D-02, 0.51233D-02, 0.51581D-02, 0.51870D-02, 0.52112D-02,
     & 0.52314D-02, 0.52483D-02, 0.52625D-02, 0.52745D-02, 0.52845D-02,
     & 0.52931D-02, 0.53004D-02, 0.53067D-02, 0.53120D-02, 0.53165D-02,
     & 0.53204D-02, 0.53237D-02, 0.53266D-02, 0.53291D-02, 0.53313D-02/

      data ((c_grdl(i,j),j=1,30),i=21,30)/
     & 0.48002D-02, 0.50556D-02, 0.51651D-02, 0.51300D-02, 0.49145D-02,
     & 0.46997D-02, 0.44376D-02, 0.45619D-02, 0.46652D-02, 0.47516D-02,
     & 0.48239D-02, 0.48844D-02, 0.49348D-02, 0.49770D-02, 0.50145D-02,
     & 0.50477D-02, 0.50760D-02, 0.51002D-02, 0.51211D-02, 0.51390D-02,
     & 0.51544D-02, 0.51677D-02, 0.51791D-02, 0.51891D-02, 0.51977D-02,
     & 0.52052D-02, 0.52117D-02, 0.52174D-02, 0.52223D-02, 0.52267D-02,
     & 0.49335D-02, 0.49726D-02, 0.49291D-02, 0.49239D-02, 0.50971D-02,
     & 0.51684D-02, 0.51387D-02, 0.49991D-02, 0.46634D-02, 0.47333D-02,
     & 0.44376D-02, 0.45287D-02, 0.46058D-02, 0.46711D-02, 0.47271D-02,
     & 0.47748D-02, 0.48159D-02, 0.48510D-02, 0.48812D-02, 0.49071D-02,
     & 0.49295D-02, 0.49488D-02, 0.49655D-02, 0.49799D-02, 0.49926D-02,
     & 0.50047D-02, 0.50153D-02, 0.50245D-02, 0.50327D-02, 0.50398D-02,
     & 0.31999D-02, 0.42940D-02, 0.48867D-02, 0.49374D-02, 0.49210D-02,
     & 0.48938D-02, 0.49048D-02, 0.50601D-02, 0.51452D-02, 0.51705D-02,
     & 0.51386D-02, 0.50521D-02, 0.49011D-02, 0.46242D-02, 0.47044D-02,
     & 0.47603D-02, 0.48068D-02, 0.44783D-02, 0.45238D-02, 0.45633D-02,
     & 0.45975D-02, 0.46273D-02, 0.46532D-02, 0.46758D-02, 0.46956D-02,
     & 0.47130D-02, 0.47282D-02, 0.47415D-02, 0.47532D-02, 0.47635D-02,
     & 0.25784D-04, 0.11802D-02, 0.25716D-02, 0.37008D-02, 0.45040D-02,
     & 0.49136D-02, 0.49603D-02, 0.49725D-02, 0.49637D-02, 0.47366D-02,
     & 0.49049D-02, 0.50238D-02, 0.51015D-02, 0.51486D-02, 0.51691D-02,
     & 0.51667D-02, 0.51454D-02, 0.51084D-02, 0.50586D-02, 0.49966D-02,
     & 0.49226D-02, 0.48353D-02, 0.47294D-02, 0.45894D-02, 0.46666D-02,
     & 0.46875D-02, 0.47056D-02, 0.47214D-02, 0.47352D-02, 0.47472D-02,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.37022D-03, 0.15584D-02,
     & 0.26974D-02, 0.36204D-02, 0.43163D-02, 0.47625D-02, 0.49400D-02,
     & 0.49603D-02, 0.49592D-02, 0.49267D-02, 0.49634D-02, 0.48788D-02,
     & 0.47933D-02, 0.48874D-02, 0.49612D-02, 0.50182D-02, 0.50615D-02,
     & 0.50943D-02, 0.51188D-02, 0.51376D-02, 0.51511D-02, 0.51602D-02,
     & 0.51660D-02, 0.51692D-02, 0.51705D-02, 0.51704D-02, 0.51692D-02,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.70083D-03,
     & 0.18550D-02, 0.28947D-02, 0.37309D-02, 0.43564D-02, 0.47612D-02,
     & 0.49397D-02, 0.49683D-02, 0.49341D-02, 0.49650D-02, 0.49542D-02,
     & 0.49545D-02, 0.48292D-02, 0.47928D-02, 0.48704D-02, 0.49331D-02,
     & 0.49832D-02, 0.50230D-02, 0.50546D-02, 0.50795D-02, 0.50994D-02,
     & 0.51151D-02, 0.51278D-02, 0.51380D-02, 0.51460D-02, 0.51522D-02,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.82758D-04,
     & 0.10225D-02, 0.21069D-02, 0.30463D-02, 0.38067D-02, 0.43731D-02,
     & 0.47469D-02, 0.49308D-02, 0.49607D-02, 0.48942D-02, 0.49688D-02,
     & 0.49218D-02, 0.49608D-02, 0.49517D-02, 0.48625D-02, 0.47633D-02,
     & 0.48292D-02, 0.48837D-02, 0.49287D-02, 0.49657D-02, 0.49962D-02,
     & 0.50213D-02, 0.50421D-02, 0.50592D-02, 0.50733D-02, 0.50853D-02,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.29914D-03, 0.13055D-02, 0.23109D-02, 0.31614D-02, 0.38503D-02,
     & 0.43676D-02, 0.47188D-02, 0.49094D-02, 0.49355D-02, 0.49521D-02,
     & 0.49377D-02, 0.49725D-02, 0.49293D-02, 0.49598D-02, 0.49594D-02,
     & 0.49208D-02, 0.47824D-02, 0.47646D-02, 0.48126D-02, 0.48530D-02,
     & 0.48873D-02, 0.49162D-02, 0.49408D-02, 0.49616D-02, 0.49793D-02,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.55445D-03, 0.15411D-02, 0.24653D-02, 0.32368D-02,
     & 0.38638D-02, 0.43404D-02, 0.46751D-02, 0.48744D-02, 0.49316D-02,
     & 0.49663D-02, 0.48794D-02, 0.49495D-02, 0.49725D-02, 0.49203D-02,
     & 0.49510D-02, 0.49635D-02, 0.49573D-02, 0.49311D-02, 0.48757D-02,
     & 0.47188D-02, 0.47553D-02, 0.47865D-02, 0.48133D-02, 0.48364D-02,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.52700D-04, 0.79587D-03, 0.17267D-02, 0.25734D-02,
     & 0.32750D-02, 0.38487D-02, 0.42912D-02, 0.46133D-02, 0.48215D-02,
     & 0.49308D-02, 0.49328D-02, 0.49680D-02, 0.48860D-02, 0.49436D-02,
     & 0.49708D-02, 0.49612D-02, 0.49278D-02, 0.49491D-02, 0.49609D-02,
     & 0.49636D-02, 0.49582D-02, 0.49454D-02, 0.49252D-02, 0.48964D-02/

      data ((c_grdl(i,j),j=1,30),i=31,40)/
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.19512D-03, 0.10020D-02, 0.18634D-02,
     & 0.26382D-02, 0.32781D-02, 0.38057D-02, 0.42193D-02, 0.45312D-02,
     & 0.47480D-02, 0.48822D-02, 0.49406D-02, 0.49417D-02, 0.49683D-02,
     & 0.48905D-02, 0.49197D-02, 0.49540D-02, 0.49708D-02, 0.49694D-02,
     & 0.49269D-02, 0.49291D-02, 0.49433D-02, 0.49532D-02, 0.49597D-02,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.36103D-03, 0.11628D-02,
     & 0.19548D-02, 0.26629D-02, 0.32479D-02, 0.37353D-02, 0.41239D-02,
     & 0.44264D-02, 0.46497D-02, 0.48029D-02, 0.48984D-02, 0.49407D-02,
     & 0.49305D-02, 0.49640D-02, 0.49628D-02, 0.48713D-02, 0.49105D-02,
     & 0.49387D-02, 0.49576D-02, 0.49686D-02, 0.49726D-02, 0.49698D-02,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.10275D-04, 0.51310D-03,
     & 0.12770D-02, 0.20030D-02, 0.26500D-02, 0.31858D-02, 0.36373D-02,
     & 0.40048D-02, 0.42968D-02, 0.45233D-02, 0.46917D-02, 0.48102D-02,
     & 0.48889D-02, 0.49340D-02, 0.49237D-02, 0.49381D-02, 0.49617D-02,
     & 0.49682D-02, 0.49555D-02, 0.48899D-02, 0.48923D-02, 0.49132D-02,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.68834D-04,
     & 0.63325D-03, 0.13460D-02, 0.20093D-02, 0.26014D-02, 0.30926D-02,
     & 0.35129D-02, 0.38601D-02, 0.41410D-02, 0.43674D-02, 0.45450D-02,
     & 0.46806D-02, 0.47809D-02, 0.48534D-02, 0.49022D-02, 0.49330D-02,
     & 0.49393D-02, 0.49125D-02, 0.49376D-02, 0.49544D-02, 0.49643D-02,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.14238D-03, 0.71457D-03, 0.13716D-02, 0.19760D-02, 0.25185D-02,
     & 0.29727D-02, 0.33615D-02, 0.36880D-02, 0.39594D-02, 0.41813D-02,
     & 0.43629D-02, 0.45087D-02, 0.46243D-02, 0.47146D-02, 0.47839D-02,
     & 0.48370D-02, 0.48767D-02, 0.49054D-02, 0.49262D-02, 0.49379D-02,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.20547D-03, 0.75567D-03, 0.13565D-02, 0.19070D-02,
     & 0.24028D-02, 0.28246D-02, 0.31820D-02, 0.34900D-02, 0.37495D-02,
     & 0.39664D-02, 0.41464D-02, 0.42967D-02, 0.44207D-02, 0.45224D-02,
     & 0.46055D-02, 0.46730D-02, 0.47275D-02, 0.47716D-02, 0.48070D-02,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.24559D-03, 0.75754D-03, 0.13032D-02,
     & 0.18044D-02, 0.22552D-02, 0.26478D-02, 0.29782D-02, 0.32656D-02,
     & 0.35117D-02, 0.37213D-02, 0.38990D-02, 0.40482D-02, 0.41748D-02,
     & 0.42820D-02, 0.43724D-02, 0.44485D-02, 0.45125D-02, 0.45664D-02,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.22099D-05, 0.25811D-03, 0.72279D-03,
     & 0.12152D-02, 0.16703D-02, 0.20775D-02, 0.24409D-02, 0.27519D-02,
     & 0.30152D-02, 0.32477D-02, 0.34478D-02, 0.36188D-02, 0.37668D-02,
     & 0.38931D-02, 0.40008D-02, 0.40927D-02, 0.41726D-02, 0.42411D-02,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.55726D-05, 0.24332D-03,
     & 0.65519D-03, 0.10951D-02, 0.15065D-02, 0.18744D-02, 0.22060D-02,
     & 0.24969D-02, 0.27467D-02, 0.29601D-02, 0.31468D-02, 0.33116D-02,
     & 0.34535D-02, 0.35758D-02, 0.36828D-02, 0.37757D-02, 0.38559D-02,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.47558D-05,
     & 0.20498D-03, 0.55966D-03, 0.94727D-03, 0.13164D-02, 0.16505D-02,
     & 0.19467D-02, 0.22147D-02, 0.24494D-02, 0.26527D-02, 0.28274D-02,
     & 0.29787D-02, 0.31117D-02, 0.32303D-02, 0.33336D-02, 0.34234D-02/

      data ((c_grdl(i,j),j=1,30),i=41,50)/
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.10423D-05, 0.14985D-03, 0.44265D-03, 0.77652D-03, 0.11034D-02,
     & 0.14037D-02, 0.16729D-02, 0.19112D-02, 0.21263D-02, 0.23169D-02,
     & 0.24832D-02, 0.26282D-02, 0.27539D-02, 0.28632D-02, 0.29587D-02,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.87856D-04, 0.31363D-03, 0.59130D-03,
     & 0.87329D-03, 0.11391D-02, 0.13809D-02, 0.15974D-02, 0.17891D-02,
     & 0.19591D-02, 0.21130D-02, 0.22492D-02, 0.23688D-02, 0.24736D-02,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.32502D-04, 0.18580D-03,
     & 0.40159D-03, 0.63517D-03, 0.86331D-03, 0.10761D-02, 0.12690D-02,
     & 0.14419D-02, 0.15966D-02, 0.17332D-02, 0.18538D-02, 0.19615D-02,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.14558D-05,
     & 0.76816D-04, 0.22425D-03, 0.40299D-03, 0.58854D-03, 0.76749D-03,
     & 0.93473D-03, 0.10872D-02, 0.12245D-02, 0.13473D-02, 0.14563D-02,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.94609D-05, 0.82458D-04, 0.19844D-03, 0.33360D-03,
     & 0.47330D-03, 0.60894D-03, 0.73611D-03, 0.85344D-03, 0.96020D-03,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.53979D-05, 0.51954D-04,
     & 0.12852D-03, 0.22000D-03, 0.31679D-03, 0.41293D-03, 0.50489D-03,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.11972D-04, 0.47865D-04, 0.97661D-04, 0.15445D-03,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.88314D-06,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00/


      DO I=1,50
        DO J=1,30
          UDS_GRD(I,J) = UDS_GRDL(I,J)
          G_GRD(I,J)   = G_GRDL(I,J)
          C_GRD(I,J)   = C_GRDL(I,J)
        ENDDO
      ENDDO

      RETURN
      END

      SUBROUTINE I_LO_Q3G0
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      PARAMETER (NQ2MAX=30,NZMAX=50)
      DIMENSION UDS_GRD(NZMAX,NQ2MAX),C_GRD(NZMAX,NQ2MAX),
     &          G_GRD(NZMAX,NQ2MAX)
      DIMENSION UDS_GRDL(NZMAX,NQ2MAX),C_GRDL(NZMAX,NQ2MAX),
     &          G_GRDL(NZMAX,NQ2MAX)
      COMMON /CPARAM/ UDS_GRD,C_GRD,G_GRD 
      data ((uds_grdl(i,j),j=1,30),i=1,10)/
     & 0.11851D-01, 0.11861D-01, 0.11877D-01, 0.11899D-01, 0.11925D-01,
     & 0.11956D-01, 0.11990D-01, 0.12029D-01, 0.12072D-01, 0.12118D-01,
     & 0.12169D-01, 0.12221D-01, 0.12277D-01, 0.12336D-01, 0.12398D-01,
     & 0.12463D-01, 0.12531D-01, 0.12600D-01, 0.12672D-01, 0.12746D-01,
     & 0.12822D-01, 0.12900D-01, 0.12980D-01, 0.13062D-01, 0.13146D-01,
     & 0.13231D-01, 0.13318D-01, 0.13406D-01, 0.13496D-01, 0.13587D-01,
     & 0.11907D-01, 0.11918D-01, 0.11935D-01, 0.11957D-01, 0.11984D-01,
     & 0.12015D-01, 0.12048D-01, 0.12087D-01, 0.12129D-01, 0.12174D-01,
     & 0.12223D-01, 0.12274D-01, 0.12328D-01, 0.12385D-01, 0.12445D-01,
     & 0.12507D-01, 0.12572D-01, 0.12639D-01, 0.12707D-01, 0.12778D-01,
     & 0.12851D-01, 0.12925D-01, 0.13002D-01, 0.13080D-01, 0.13159D-01,
     & 0.13240D-01, 0.13322D-01, 0.13406D-01, 0.13491D-01, 0.13577D-01,
     & 0.11974D-01, 0.11986D-01, 0.12004D-01, 0.12026D-01, 0.12053D-01,
     & 0.12084D-01, 0.12118D-01, 0.12155D-01, 0.12197D-01, 0.12241D-01,
     & 0.12289D-01, 0.12339D-01, 0.12391D-01, 0.12446D-01, 0.12504D-01,
     & 0.12564D-01, 0.12626D-01, 0.12690D-01, 0.12756D-01, 0.12823D-01,
     & 0.12893D-01, 0.12964D-01, 0.13036D-01, 0.13111D-01, 0.13186D-01,
     & 0.13263D-01, 0.13341D-01, 0.13420D-01, 0.13500D-01, 0.13582D-01,
     & 0.12054D-01, 0.12067D-01, 0.12086D-01, 0.12109D-01, 0.12136D-01,
     & 0.12167D-01, 0.12201D-01, 0.12238D-01, 0.12279D-01, 0.12323D-01,
     & 0.12369D-01, 0.12417D-01, 0.12468D-01, 0.12522D-01, 0.12578D-01,
     & 0.12635D-01, 0.12695D-01, 0.12757D-01, 0.12820D-01, 0.12884D-01,
     & 0.12951D-01, 0.13018D-01, 0.13088D-01, 0.13158D-01, 0.13230D-01,
     & 0.13303D-01, 0.13377D-01, 0.13452D-01, 0.13528D-01, 0.13604D-01,
     & 0.12146D-01, 0.12162D-01, 0.12182D-01, 0.12206D-01, 0.12233D-01,
     & 0.12264D-01, 0.12298D-01, 0.12335D-01, 0.12375D-01, 0.12418D-01,
     & 0.12463D-01, 0.12511D-01, 0.12560D-01, 0.12612D-01, 0.12666D-01,
     & 0.12722D-01, 0.12779D-01, 0.12838D-01, 0.12899D-01, 0.12961D-01,
     & 0.13024D-01, 0.13089D-01, 0.13155D-01, 0.13222D-01, 0.13290D-01,
     & 0.13359D-01, 0.13429D-01, 0.13500D-01, 0.13572D-01, 0.13645D-01,
     & 0.12255D-01, 0.12272D-01, 0.12293D-01, 0.12318D-01, 0.12346D-01,
     & 0.12377D-01, 0.12412D-01, 0.12448D-01, 0.12488D-01, 0.12530D-01,
     & 0.12575D-01, 0.12621D-01, 0.12670D-01, 0.12720D-01, 0.12772D-01,
     & 0.12826D-01, 0.12881D-01, 0.12938D-01, 0.12996D-01, 0.13055D-01,
     & 0.13116D-01, 0.13178D-01, 0.13241D-01, 0.13304D-01, 0.13369D-01,
     & 0.13435D-01, 0.13501D-01, 0.13568D-01, 0.13636D-01, 0.13705D-01,
     & 0.12382D-01, 0.12401D-01, 0.12423D-01, 0.12449D-01, 0.12478D-01,
     & 0.12510D-01, 0.12544D-01, 0.12581D-01, 0.12620D-01, 0.12662D-01,
     & 0.12706D-01, 0.12751D-01, 0.12799D-01, 0.12848D-01, 0.12898D-01,
     & 0.12950D-01, 0.13004D-01, 0.13058D-01, 0.13114D-01, 0.13171D-01,
     & 0.13229D-01, 0.13288D-01, 0.13348D-01, 0.13408D-01, 0.13470D-01,
     & 0.13532D-01, 0.13595D-01, 0.13659D-01, 0.13723D-01, 0.13787D-01,
     & 0.12532D-01, 0.12553D-01, 0.12577D-01, 0.12604D-01, 0.12633D-01,
     & 0.12666D-01, 0.12701D-01, 0.12738D-01, 0.12777D-01, 0.12818D-01,
     & 0.12861D-01, 0.12906D-01, 0.12952D-01, 0.13000D-01, 0.13049D-01,
     & 0.13100D-01, 0.13151D-01, 0.13204D-01, 0.13258D-01, 0.13312D-01,
     & 0.13368D-01, 0.13424D-01, 0.13481D-01, 0.13539D-01, 0.13597D-01,
     & 0.13656D-01, 0.13716D-01, 0.13776D-01, 0.13837D-01, 0.13898D-01,
     & 0.12710D-01, 0.12733D-01, 0.12758D-01, 0.12786D-01, 0.12817D-01,
     & 0.12850D-01, 0.12886D-01, 0.12923D-01, 0.12962D-01, 0.13003D-01,
     & 0.13046D-01, 0.13090D-01, 0.13135D-01, 0.13182D-01, 0.13230D-01,
     & 0.13278D-01, 0.13328D-01, 0.13379D-01, 0.13431D-01, 0.13483D-01,
     & 0.13537D-01, 0.13590D-01, 0.13645D-01, 0.13700D-01, 0.13755D-01,
     & 0.13812D-01, 0.13868D-01, 0.13925D-01, 0.13982D-01, 0.14040D-01,
     & 0.12917D-01, 0.12942D-01, 0.12969D-01, 0.12999D-01, 0.13031D-01,
     & 0.13065D-01, 0.13101D-01, 0.13139D-01, 0.13178D-01, 0.13219D-01,
     & 0.13261D-01, 0.13304D-01, 0.13349D-01, 0.13394D-01, 0.13441D-01,
     & 0.13488D-01, 0.13536D-01, 0.13586D-01, 0.13635D-01, 0.13686D-01,
     & 0.13737D-01, 0.13788D-01, 0.13840D-01, 0.13893D-01, 0.13945D-01,
     & 0.13999D-01, 0.14052D-01, 0.14106D-01, 0.14160D-01, 0.14214D-01/

      data ((uds_grdl(i,j),j=1,30),i=11,20)/
     & 0.13160D-01, 0.13187D-01, 0.13216D-01, 0.13247D-01, 0.13280D-01,
     & 0.13315D-01, 0.13352D-01, 0.13390D-01, 0.13429D-01, 0.13470D-01,
     & 0.13511D-01, 0.13554D-01, 0.13598D-01, 0.13643D-01, 0.13688D-01,
     & 0.13734D-01, 0.13781D-01, 0.13828D-01, 0.13876D-01, 0.13924D-01,
     & 0.13973D-01, 0.14022D-01, 0.14072D-01, 0.14122D-01, 0.14172D-01,
     & 0.14222D-01, 0.14273D-01, 0.14324D-01, 0.14374D-01, 0.14425D-01,
     & 0.13447D-01, 0.13476D-01, 0.13506D-01, 0.13539D-01, 0.13573D-01,
     & 0.13609D-01, 0.13646D-01, 0.13685D-01, 0.13724D-01, 0.13765D-01,
     & 0.13806D-01, 0.13848D-01, 0.13891D-01, 0.13935D-01, 0.13979D-01,
     & 0.14024D-01, 0.14069D-01, 0.14114D-01, 0.14161D-01, 0.14207D-01,
     & 0.14254D-01, 0.14301D-01, 0.14348D-01, 0.14395D-01, 0.14442D-01,
     & 0.14490D-01, 0.14538D-01, 0.14586D-01, 0.14633D-01, 0.14681D-01,
     & 0.13781D-01, 0.13812D-01, 0.13845D-01, 0.13879D-01, 0.13914D-01,
     & 0.13951D-01, 0.13988D-01, 0.14027D-01, 0.14066D-01, 0.14107D-01,
     & 0.14147D-01, 0.14189D-01, 0.14231D-01, 0.14274D-01, 0.14317D-01,
     & 0.14360D-01, 0.14404D-01, 0.14447D-01, 0.14492D-01, 0.14536D-01,
     & 0.14581D-01, 0.14625D-01, 0.14670D-01, 0.14714D-01, 0.14759D-01,
     & 0.14804D-01, 0.14849D-01, 0.14894D-01, 0.14938D-01, 0.14983D-01,
     & 0.14176D-01, 0.14209D-01, 0.14244D-01, 0.14279D-01, 0.14315D-01,
     & 0.14352D-01, 0.14390D-01, 0.14429D-01, 0.14468D-01, 0.14508D-01,
     & 0.14548D-01, 0.14589D-01, 0.14630D-01, 0.14671D-01, 0.14713D-01,
     & 0.14755D-01, 0.14796D-01, 0.14838D-01, 0.14881D-01, 0.14923D-01,
     & 0.14965D-01, 0.15007D-01, 0.15049D-01, 0.15091D-01, 0.15133D-01,
     & 0.15175D-01, 0.15217D-01, 0.15259D-01, 0.15300D-01, 0.15341D-01,
     & 0.14642D-01, 0.14676D-01, 0.14712D-01, 0.14748D-01, 0.14785D-01,
     & 0.14822D-01, 0.14860D-01, 0.14899D-01, 0.14938D-01, 0.14977D-01,
     & 0.15016D-01, 0.15055D-01, 0.15095D-01, 0.15135D-01, 0.15175D-01,
     & 0.15214D-01, 0.15254D-01, 0.15294D-01, 0.15334D-01, 0.15374D-01,
     & 0.15413D-01, 0.15453D-01, 0.15492D-01, 0.15531D-01, 0.15570D-01,
     & 0.15608D-01, 0.15647D-01, 0.15686D-01, 0.15724D-01, 0.15761D-01,
     & 0.15176D-01, 0.15212D-01, 0.15248D-01, 0.15284D-01, 0.15321D-01,
     & 0.15358D-01, 0.15396D-01, 0.15434D-01, 0.15471D-01, 0.15509D-01,
     & 0.15547D-01, 0.15585D-01, 0.15622D-01, 0.15660D-01, 0.15698D-01,
     & 0.15735D-01, 0.15772D-01, 0.15809D-01, 0.15846D-01, 0.15883D-01,
     & 0.15919D-01, 0.15955D-01, 0.15991D-01, 0.16027D-01, 0.16062D-01,
     & 0.16098D-01, 0.16133D-01, 0.16167D-01, 0.16201D-01, 0.16235D-01,
     & 0.15799D-01, 0.15835D-01, 0.15871D-01, 0.15907D-01, 0.15943D-01,
     & 0.15979D-01, 0.16015D-01, 0.16051D-01, 0.16087D-01, 0.16122D-01,
     & 0.16157D-01, 0.16193D-01, 0.16228D-01, 0.16263D-01, 0.16297D-01,
     & 0.16331D-01, 0.16365D-01, 0.16399D-01, 0.16432D-01, 0.16465D-01,
     & 0.16498D-01, 0.16530D-01, 0.16562D-01, 0.16593D-01, 0.16625D-01,
     & 0.16656D-01, 0.16686D-01, 0.16717D-01, 0.16746D-01, 0.16776D-01,
     & 0.16519D-01, 0.16553D-01, 0.16588D-01, 0.16622D-01, 0.16655D-01,
     & 0.16689D-01, 0.16722D-01, 0.16755D-01, 0.16788D-01, 0.16820D-01,
     & 0.16852D-01, 0.16884D-01, 0.16915D-01, 0.16946D-01, 0.16976D-01,
     & 0.17006D-01, 0.17035D-01, 0.17064D-01, 0.17093D-01, 0.17121D-01,
     & 0.17149D-01, 0.17176D-01, 0.17203D-01, 0.17230D-01, 0.17256D-01,
     & 0.17282D-01, 0.17307D-01, 0.17332D-01, 0.17356D-01, 0.17380D-01,
     & 0.17344D-01, 0.17376D-01, 0.17407D-01, 0.17437D-01, 0.17467D-01,
     & 0.17496D-01, 0.17525D-01, 0.17553D-01, 0.17581D-01, 0.17608D-01,
     & 0.17635D-01, 0.17661D-01, 0.17686D-01, 0.17712D-01, 0.17736D-01,
     & 0.17760D-01, 0.17783D-01, 0.17806D-01, 0.17829D-01, 0.17851D-01,
     & 0.17873D-01, 0.17894D-01, 0.17914D-01, 0.17934D-01, 0.17954D-01,
     & 0.17973D-01, 0.17992D-01, 0.18010D-01, 0.18028D-01, 0.18045D-01,
     & 0.18282D-01, 0.18307D-01, 0.18332D-01, 0.18356D-01, 0.18379D-01,
     & 0.18401D-01, 0.18423D-01, 0.18444D-01, 0.18464D-01, 0.18484D-01,
     & 0.18503D-01, 0.18522D-01, 0.18539D-01, 0.18557D-01, 0.18573D-01,
     & 0.18589D-01, 0.18604D-01, 0.18619D-01, 0.18634D-01, 0.18648D-01,
     & 0.18661D-01, 0.18674D-01, 0.18686D-01, 0.18697D-01, 0.18709D-01,
     & 0.18720D-01, 0.18730D-01, 0.18740D-01, 0.18749D-01, 0.18758D-01/

      data ((uds_grdl(i,j),j=1,30),i=21,30)/
     & 0.19331D-01, 0.19346D-01, 0.19361D-01, 0.19375D-01, 0.19387D-01,
     & 0.19399D-01, 0.19411D-01, 0.19421D-01, 0.19431D-01, 0.19440D-01,
     & 0.19448D-01, 0.19456D-01, 0.19463D-01, 0.19469D-01, 0.19475D-01,
     & 0.19480D-01, 0.19484D-01, 0.19488D-01, 0.19491D-01, 0.19494D-01,
     & 0.19497D-01, 0.19499D-01, 0.19500D-01, 0.19501D-01, 0.19501D-01,
     & 0.19501D-01, 0.19501D-01, 0.19500D-01, 0.19499D-01, 0.19498D-01,
     & 0.20470D-01, 0.20470D-01, 0.20469D-01, 0.20468D-01, 0.20465D-01,
     & 0.20462D-01, 0.20458D-01, 0.20453D-01, 0.20447D-01, 0.20441D-01,
     & 0.20434D-01, 0.20426D-01, 0.20418D-01, 0.20409D-01, 0.20400D-01,
     & 0.20390D-01, 0.20380D-01, 0.20369D-01, 0.20358D-01, 0.20346D-01,
     & 0.20334D-01, 0.20322D-01, 0.20309D-01, 0.20296D-01, 0.20282D-01,
     & 0.20268D-01, 0.20254D-01, 0.20240D-01, 0.20225D-01, 0.20210D-01,
     & 0.21685D-01, 0.21663D-01, 0.21640D-01, 0.21616D-01, 0.21591D-01,
     & 0.21566D-01, 0.21540D-01, 0.21514D-01, 0.21487D-01, 0.21460D-01,
     & 0.21432D-01, 0.21404D-01, 0.21375D-01, 0.21346D-01, 0.21317D-01,
     & 0.21287D-01, 0.21257D-01, 0.21227D-01, 0.21197D-01, 0.21166D-01,
     & 0.21135D-01, 0.21104D-01, 0.21073D-01, 0.21042D-01, 0.21010D-01,
     & 0.20978D-01, 0.20947D-01, 0.20915D-01, 0.20883D-01, 0.20851D-01,
     & 0.22914D-01, 0.22860D-01, 0.22806D-01, 0.22751D-01, 0.22696D-01,
     & 0.22640D-01, 0.22585D-01, 0.22529D-01, 0.22474D-01, 0.22418D-01,
     & 0.22362D-01, 0.22306D-01, 0.22250D-01, 0.22195D-01, 0.22139D-01,
     & 0.22083D-01, 0.22028D-01, 0.21972D-01, 0.21917D-01, 0.21862D-01,
     & 0.21807D-01, 0.21752D-01, 0.21697D-01, 0.21643D-01, 0.21588D-01,
     & 0.21534D-01, 0.21480D-01, 0.21426D-01, 0.21373D-01, 0.21319D-01,
     & 0.24031D-01, 0.23932D-01, 0.23834D-01, 0.23736D-01, 0.23639D-01,
     & 0.23542D-01, 0.23446D-01, 0.23351D-01, 0.23257D-01, 0.23163D-01,
     & 0.23070D-01, 0.22978D-01, 0.22886D-01, 0.22795D-01, 0.22705D-01,
     & 0.22616D-01, 0.22527D-01, 0.22439D-01, 0.22352D-01, 0.22265D-01,
     & 0.22179D-01, 0.22094D-01, 0.22010D-01, 0.21926D-01, 0.21843D-01,
     & 0.21761D-01, 0.21679D-01, 0.21598D-01, 0.21518D-01, 0.21438D-01,
     & 0.24265D-01, 0.24153D-01, 0.24042D-01, 0.23932D-01, 0.23823D-01,
     & 0.23715D-01, 0.23607D-01, 0.23501D-01, 0.23395D-01, 0.23291D-01,
     & 0.23187D-01, 0.23084D-01, 0.22983D-01, 0.22882D-01, 0.22782D-01,
     & 0.22683D-01, 0.22585D-01, 0.22488D-01, 0.22392D-01, 0.22297D-01,
     & 0.22202D-01, 0.22109D-01, 0.22016D-01, 0.21925D-01, 0.21834D-01,
     & 0.21744D-01, 0.21654D-01, 0.21566D-01, 0.21478D-01, 0.21392D-01,
     & 0.24472D-01, 0.24347D-01, 0.24222D-01, 0.24099D-01, 0.23977D-01,
     & 0.23857D-01, 0.23737D-01, 0.23619D-01, 0.23502D-01, 0.23387D-01,
     & 0.23273D-01, 0.23159D-01, 0.23047D-01, 0.22936D-01, 0.22827D-01,
     & 0.22718D-01, 0.22611D-01, 0.22505D-01, 0.22399D-01, 0.22295D-01,
     & 0.22192D-01, 0.22090D-01, 0.21990D-01, 0.21890D-01, 0.21791D-01,
     & 0.21693D-01, 0.21596D-01, 0.21501D-01, 0.21406D-01, 0.21312D-01,
     & 0.24651D-01, 0.24512D-01, 0.24374D-01, 0.24237D-01, 0.24103D-01,
     & 0.23969D-01, 0.23838D-01, 0.23708D-01, 0.23579D-01, 0.23452D-01,
     & 0.23327D-01, 0.23203D-01, 0.23080D-01, 0.22959D-01, 0.22839D-01,
     & 0.22721D-01, 0.22604D-01, 0.22488D-01, 0.22374D-01, 0.22261D-01,
     & 0.22149D-01, 0.22039D-01, 0.21930D-01, 0.21822D-01, 0.21715D-01,
     & 0.21609D-01, 0.21505D-01, 0.21402D-01, 0.21300D-01, 0.21199D-01,
     & 0.24802D-01, 0.24647D-01, 0.24495D-01, 0.24345D-01, 0.24197D-01,
     & 0.24051D-01, 0.23906D-01, 0.23764D-01, 0.23624D-01, 0.23485D-01,
     & 0.23348D-01, 0.23213D-01, 0.23080D-01, 0.22948D-01, 0.22818D-01,
     & 0.22690D-01, 0.22564D-01, 0.22439D-01, 0.22315D-01, 0.22193D-01,
     & 0.22072D-01, 0.21953D-01, 0.21836D-01, 0.21720D-01, 0.21605D-01,
     & 0.21492D-01, 0.21379D-01, 0.21269D-01, 0.21159D-01, 0.21051D-01,
     & 0.24923D-01, 0.24754D-01, 0.24587D-01, 0.24422D-01, 0.24261D-01,
     & 0.24101D-01, 0.23944D-01, 0.23789D-01, 0.23636D-01, 0.23486D-01,
     & 0.23337D-01, 0.23191D-01, 0.23047D-01, 0.22904D-01, 0.22764D-01,
     & 0.22626D-01, 0.22489D-01, 0.22355D-01, 0.22222D-01, 0.22091D-01,
     & 0.21961D-01, 0.21834D-01, 0.21708D-01, 0.21583D-01, 0.21461D-01,
     & 0.21339D-01, 0.21219D-01, 0.21101D-01, 0.20984D-01, 0.20869D-01/

      data ((uds_grdl(i,j),j=1,30),i=31,40)/
     & 0.25014D-01, 0.24829D-01, 0.24647D-01, 0.24468D-01, 0.24292D-01,
     & 0.24119D-01, 0.23948D-01, 0.23781D-01, 0.23615D-01, 0.23453D-01,
     & 0.23293D-01, 0.23135D-01, 0.22979D-01, 0.22826D-01, 0.22675D-01,
     & 0.22527D-01, 0.22380D-01, 0.22236D-01, 0.22094D-01, 0.21953D-01,
     & 0.21815D-01, 0.21679D-01, 0.21544D-01, 0.21412D-01, 0.21281D-01,
     & 0.21152D-01, 0.21024D-01, 0.20899D-01, 0.20775D-01, 0.20652D-01,
     & 0.25075D-01, 0.24874D-01, 0.24676D-01, 0.24482D-01, 0.24292D-01,
     & 0.24104D-01, 0.23920D-01, 0.23739D-01, 0.23561D-01, 0.23386D-01,
     & 0.23213D-01, 0.23044D-01, 0.22877D-01, 0.22713D-01, 0.22551D-01,
     & 0.22392D-01, 0.22236D-01, 0.22082D-01, 0.21930D-01, 0.21780D-01,
     & 0.21633D-01, 0.21488D-01, 0.21345D-01, 0.21204D-01, 0.21065D-01,
     & 0.20928D-01, 0.20793D-01, 0.20660D-01, 0.20529D-01, 0.20399D-01,
     & 0.25105D-01, 0.24886D-01, 0.24673D-01, 0.24463D-01, 0.24258D-01,
     & 0.24056D-01, 0.23857D-01, 0.23663D-01, 0.23471D-01, 0.23284D-01,
     & 0.23099D-01, 0.22918D-01, 0.22739D-01, 0.22564D-01, 0.22391D-01,
     & 0.22222D-01, 0.22055D-01, 0.21891D-01, 0.21730D-01, 0.21571D-01,
     & 0.21414D-01, 0.21260D-01, 0.21109D-01, 0.20960D-01, 0.20813D-01,
     & 0.20668D-01, 0.20525D-01, 0.20385D-01, 0.20246D-01, 0.20110D-01,
     & 0.25103D-01, 0.24867D-01, 0.24637D-01, 0.24411D-01, 0.24190D-01,
     & 0.23973D-01, 0.23760D-01, 0.23551D-01, 0.23346D-01, 0.23146D-01,
     & 0.22949D-01, 0.22755D-01, 0.22565D-01, 0.22378D-01, 0.22194D-01,
     & 0.22014D-01, 0.21837D-01, 0.21663D-01, 0.21492D-01, 0.21324D-01,
     & 0.21158D-01, 0.20995D-01, 0.20835D-01, 0.20678D-01, 0.20523D-01,
     & 0.20370D-01, 0.20220D-01, 0.20072D-01, 0.19927D-01, 0.19783D-01,
     & 0.25069D-01, 0.24815D-01, 0.24568D-01, 0.24325D-01, 0.24088D-01,
     & 0.23856D-01, 0.23628D-01, 0.23405D-01, 0.23186D-01, 0.22972D-01,
     & 0.22762D-01, 0.22556D-01, 0.22354D-01, 0.22155D-01, 0.21961D-01,
     & 0.21770D-01, 0.21582D-01, 0.21398D-01, 0.21217D-01, 0.21039D-01,
     & 0.20864D-01, 0.20693D-01, 0.20524D-01, 0.20359D-01, 0.20196D-01,
     & 0.20035D-01, 0.19877D-01, 0.19722D-01, 0.19569D-01, 0.19419D-01,
     & 0.24996D-01, 0.24723D-01, 0.24457D-01, 0.24197D-01, 0.23944D-01,
     & 0.23695D-01, 0.23452D-01, 0.23214D-01, 0.22981D-01, 0.22753D-01,
     & 0.22530D-01, 0.22311D-01, 0.22096D-01, 0.21886D-01, 0.21680D-01,
     & 0.21478D-01, 0.21280D-01, 0.21086D-01, 0.20895D-01, 0.20707D-01,
     & 0.20523D-01, 0.20343D-01, 0.20166D-01, 0.19992D-01, 0.19821D-01,
     & 0.19652D-01, 0.19487D-01, 0.19325D-01, 0.19165D-01, 0.19008D-01,
     & 0.24890D-01, 0.24598D-01, 0.24313D-01, 0.24035D-01, 0.23764D-01,
     & 0.23499D-01, 0.23240D-01, 0.22986D-01, 0.22739D-01, 0.22497D-01,
     & 0.22260D-01, 0.22028D-01, 0.21801D-01, 0.21578D-01, 0.21361D-01,
     & 0.21148D-01, 0.20939D-01, 0.20734D-01, 0.20533D-01, 0.20336D-01,
     & 0.20143D-01, 0.19953D-01, 0.19767D-01, 0.19585D-01, 0.19406D-01,
     & 0.19230D-01, 0.19057D-01, 0.18887D-01, 0.18721D-01, 0.18557D-01,
     & 0.24748D-01, 0.24435D-01, 0.24130D-01, 0.23833D-01, 0.23544D-01,
     & 0.23262D-01, 0.22985D-01, 0.22716D-01, 0.22453D-01, 0.22197D-01,
     & 0.21946D-01, 0.21701D-01, 0.21460D-01, 0.21226D-01, 0.20996D-01,
     & 0.20772D-01, 0.20552D-01, 0.20337D-01, 0.20126D-01, 0.19919D-01,
     & 0.19716D-01, 0.19518D-01, 0.19324D-01, 0.19133D-01, 0.18946D-01,
     & 0.18762D-01, 0.18582D-01, 0.18405D-01, 0.18232D-01, 0.18062D-01,
     & 0.24566D-01, 0.24231D-01, 0.23905D-01, 0.23588D-01, 0.23279D-01,
     & 0.22979D-01, 0.22685D-01, 0.22399D-01, 0.22121D-01, 0.21849D-01,
     & 0.21584D-01, 0.21324D-01, 0.21071D-01, 0.20824D-01, 0.20582D-01,
     & 0.20346D-01, 0.20115D-01, 0.19889D-01, 0.19668D-01, 0.19451D-01,
     & 0.19240D-01, 0.19032D-01, 0.18829D-01, 0.18630D-01, 0.18436D-01,
     & 0.18244D-01, 0.18057D-01, 0.17873D-01, 0.17693D-01, 0.17516D-01,
     & 0.24340D-01, 0.23981D-01, 0.23633D-01, 0.23295D-01, 0.22966D-01,
     & 0.22647D-01, 0.22335D-01, 0.22032D-01, 0.21736D-01, 0.21449D-01,
     & 0.21169D-01, 0.20895D-01, 0.20628D-01, 0.20367D-01, 0.20113D-01,
     & 0.19865D-01, 0.19623D-01, 0.19386D-01, 0.19155D-01, 0.18928D-01,
     & 0.18707D-01, 0.18491D-01, 0.18279D-01, 0.18072D-01, 0.17869D-01,
     & 0.17670D-01, 0.17476D-01, 0.17285D-01, 0.17099D-01, 0.16916D-01/

      data ((uds_grdl(i,j),j=1,30),i=41,50)/
     & 0.24067D-01, 0.23682D-01, 0.23310D-01, 0.22949D-01, 0.22598D-01,
     & 0.22258D-01, 0.21927D-01, 0.21605D-01, 0.21293D-01, 0.20989D-01,
     & 0.20693D-01, 0.20404D-01, 0.20123D-01, 0.19849D-01, 0.19583D-01,
     & 0.19323D-01, 0.19069D-01, 0.18821D-01, 0.18579D-01, 0.18343D-01,
     & 0.18112D-01, 0.17887D-01, 0.17667D-01, 0.17452D-01, 0.17242D-01,
     & 0.17035D-01, 0.16834D-01, 0.16637D-01, 0.16444D-01, 0.16255D-01,
     & 0.23737D-01, 0.23326D-01, 0.22928D-01, 0.22542D-01, 0.22169D-01,
     & 0.21807D-01, 0.21455D-01, 0.21114D-01, 0.20783D-01, 0.20462D-01,
     & 0.20149D-01, 0.19845D-01, 0.19549D-01, 0.19262D-01, 0.18982D-01,
     & 0.18709D-01, 0.18444D-01, 0.18185D-01, 0.17932D-01, 0.17686D-01,
     & 0.17446D-01, 0.17211D-01, 0.16983D-01, 0.16760D-01, 0.16542D-01,
     & 0.16329D-01, 0.16120D-01, 0.15917D-01, 0.15718D-01, 0.15523D-01,
     & 0.23341D-01, 0.22899D-01, 0.22473D-01, 0.22061D-01, 0.21662D-01,
     & 0.21277D-01, 0.20903D-01, 0.20541D-01, 0.20191D-01, 0.19851D-01,
     & 0.19521D-01, 0.19201D-01, 0.18890D-01, 0.18588D-01, 0.18294D-01,
     & 0.18009D-01, 0.17731D-01, 0.17461D-01, 0.17198D-01, 0.16942D-01,
     & 0.16693D-01, 0.16450D-01, 0.16213D-01, 0.15982D-01, 0.15757D-01,
     & 0.15537D-01, 0.15322D-01, 0.15113D-01, 0.14909D-01, 0.14709D-01,
     & 0.22859D-01, 0.22384D-01, 0.21926D-01, 0.21484D-01, 0.21059D-01,
     & 0.20647D-01, 0.20250D-01, 0.19866D-01, 0.19494D-01, 0.19135D-01,
     & 0.18787D-01, 0.18449D-01, 0.18122D-01, 0.17805D-01, 0.17498D-01,
     & 0.17199D-01, 0.16910D-01, 0.16628D-01, 0.16354D-01, 0.16089D-01,
     & 0.15830D-01, 0.15578D-01, 0.15334D-01, 0.15096D-01, 0.14864D-01,
     & 0.14638D-01, 0.14417D-01, 0.14203D-01, 0.13994D-01, 0.13789D-01,
     & 0.22249D-01, 0.21735D-01, 0.21242D-01, 0.20768D-01, 0.20312D-01,
     & 0.19872D-01, 0.19449D-01, 0.19040D-01, 0.18647D-01, 0.18267D-01,
     & 0.17899D-01, 0.17545D-01, 0.17201D-01, 0.16869D-01, 0.16548D-01,
     & 0.16237D-01, 0.15935D-01, 0.15643D-01, 0.15360D-01, 0.15085D-01,
     & 0.14818D-01, 0.14559D-01, 0.14307D-01, 0.14063D-01, 0.13825D-01,
     & 0.13594D-01, 0.13370D-01, 0.13151D-01, 0.12938D-01, 0.12731D-01,
     & 0.21465D-01, 0.20905D-01, 0.20370D-01, 0.19858D-01, 0.19367D-01,
     & 0.18896D-01, 0.18445D-01, 0.18011D-01, 0.17595D-01, 0.17193D-01,
     & 0.16807D-01, 0.16435D-01, 0.16077D-01, 0.15731D-01, 0.15398D-01,
     & 0.15075D-01, 0.14763D-01, 0.14462D-01, 0.14171D-01, 0.13890D-01,
     & 0.13617D-01, 0.13352D-01, 0.13096D-01, 0.12848D-01, 0.12607D-01,
     & 0.12373D-01, 0.12147D-01, 0.11926D-01, 0.11712D-01, 0.11504D-01,
     & 0.20439D-01, 0.19848D-01, 0.19283D-01, 0.18743D-01, 0.18225D-01,
     & 0.17728D-01, 0.17251D-01, 0.16793D-01, 0.16354D-01, 0.15932D-01,
     & 0.15526D-01, 0.15135D-01, 0.14759D-01, 0.14397D-01, 0.14049D-01,
     & 0.13712D-01, 0.13388D-01, 0.13076D-01, 0.12774D-01, 0.12483D-01,
     & 0.12202D-01, 0.11930D-01, 0.11668D-01, 0.11413D-01, 0.11167D-01,
     & 0.10930D-01, 0.10700D-01, 0.10477D-01, 0.10261D-01, 0.10052D-01,
     & 0.19387D-01, 0.18634D-01, 0.17922D-01, 0.17251D-01, 0.16616D-01,
     & 0.16017D-01, 0.15455D-01, 0.14921D-01, 0.14413D-01, 0.13931D-01,
     & 0.13473D-01, 0.13039D-01, 0.12628D-01, 0.12235D-01, 0.11860D-01,
     & 0.11501D-01, 0.11158D-01, 0.10831D-01, 0.10521D-01, 0.10223D-01,
     & 0.99372D-02, 0.96628D-02, 0.93993D-02, 0.91462D-02, 0.89034D-02,
     & 0.86724D-02, 0.84499D-02, 0.82353D-02, 0.80285D-02, 0.78291D-02,
     & 0.13489D-01, 0.12851D-01, 0.12255D-01, 0.11696D-01, 0.11172D-01,
     & 0.10683D-01, 0.10230D-01, 0.98028D-02, 0.94000D-02, 0.90200D-02,
     & 0.86614D-02, 0.83266D-02, 0.80117D-02, 0.77128D-02, 0.74290D-02,
     & 0.71595D-02, 0.69036D-02, 0.66617D-02, 0.64347D-02, 0.62179D-02,
     & 0.60109D-02, 0.58132D-02, 0.56244D-02, 0.54441D-02, 0.52721D-02,
     & 0.51106D-02, 0.49557D-02, 0.48069D-02, 0.46641D-02, 0.45271D-02,
     & 0.86035D-03, 0.81571D-03, 0.77405D-03, 0.73516D-03, 0.69887D-03,
     & 0.66509D-03, 0.63404D-03, 0.60485D-03, 0.57739D-03, 0.55158D-03,
     & 0.52732D-03, 0.50478D-03, 0.48370D-03, 0.46373D-03, 0.44483D-03,
     & 0.42694D-03, 0.41000D-03, 0.39406D-03, 0.37919D-03, 0.36503D-03,
     & 0.35154D-03, 0.33869D-03, 0.32646D-03, 0.31480D-03, 0.30373D-03,
     & 0.29340D-03, 0.28351D-03, 0.27404D-03, 0.26498D-03, 0.25629D-03/

      data ((g_grdl(i,j),j=1,30),i=1,10)/
     & 0.10003D-07, 0.48421D-02, 0.97179D-02, 0.14621D-01, 0.19547D-01,
     & 0.24491D-01, 0.29451D-01, 0.34419D-01, 0.39392D-01, 0.44367D-01,
     & 0.49342D-01, 0.54318D-01, 0.59291D-01, 0.64255D-01, 0.69207D-01,
     & 0.74148D-01, 0.79075D-01, 0.83990D-01, 0.88897D-01, 0.93783D-01,
     & 0.98650D-01, 0.10349D+00, 0.10832D+00, 0.11312D+00, 0.11790D+00,
     & 0.12267D+00, 0.12742D+00, 0.13214D+00, 0.13683D+00, 0.14149D+00,
     & 0.10002D-07, 0.46868D-02, 0.93934D-02, 0.14115D-01, 0.18848D-01,
     & 0.23588D-01, 0.28336D-01, 0.33083D-01, 0.37825D-01, 0.42561D-01,
     & 0.47288D-01, 0.52011D-01, 0.56725D-01, 0.61421D-01, 0.66100D-01,
     & 0.70761D-01, 0.75403D-01, 0.80028D-01, 0.84641D-01, 0.89229D-01,
     & 0.93792D-01, 0.98329D-01, 0.10284D+00, 0.10733D+00, 0.11179D+00,
     & 0.11624D+00, 0.12066D+00, 0.12505D+00, 0.12941D+00, 0.13375D+00,
     & 0.10002D-07, 0.45302D-02, 0.90674D-02, 0.13608D-01, 0.18148D-01,
     & 0.22687D-01, 0.27227D-01, 0.31756D-01, 0.36272D-01, 0.40773D-01,
     & 0.45259D-01, 0.49735D-01, 0.54195D-01, 0.58632D-01, 0.63046D-01,
     & 0.67436D-01, 0.71802D-01, 0.76148D-01, 0.80477D-01, 0.84778D-01,
     & 0.89049D-01, 0.93291D-01, 0.97505D-01, 0.10169D+00, 0.10585D+00,
     & 0.10999D+00, 0.11410D+00, 0.11818D+00, 0.12223D+00, 0.12624D+00,
     & 0.10003D-07, 0.43728D-02, 0.87407D-02, 0.13101D-01, 0.17452D-01,
     & 0.21792D-01, 0.26126D-01, 0.30441D-01, 0.34735D-01, 0.39007D-01,
     & 0.43258D-01, 0.47493D-01, 0.51708D-01, 0.55893D-01, 0.60051D-01,
     & 0.64180D-01, 0.68280D-01, 0.72356D-01, 0.76413D-01, 0.80437D-01,
     & 0.84429D-01, 0.88388D-01, 0.92316D-01, 0.96213D-01, 0.10008D+00,
     & 0.10393D+00, 0.10775D+00, 0.11153D+00, 0.11528D+00, 0.11899D+00,
     & 0.10003D-07, 0.42130D-02, 0.84102D-02, 0.12590D-01, 0.16751D-01,
     & 0.20893D-01, 0.25023D-01, 0.29126D-01, 0.33201D-01, 0.37249D-01,
     & 0.41268D-01, 0.45268D-01, 0.49242D-01, 0.53182D-01, 0.57089D-01,
     & 0.60964D-01, 0.64807D-01, 0.68622D-01, 0.72415D-01, 0.76172D-01,
     & 0.79893D-01, 0.83580D-01, 0.87233D-01, 0.90853D-01, 0.94441D-01,
     & 0.98011D-01, 0.10154D+00, 0.10504D+00, 0.10851D+00, 0.11194D+00,
     & 0.10003D-07, 0.40507D-02, 0.80757D-02, 0.12074D-01, 0.16046D-01,
     & 0.19991D-01, 0.23918D-01, 0.27811D-01, 0.31670D-01, 0.35496D-01,
     & 0.39289D-01, 0.43058D-01, 0.46796D-01, 0.50497D-01, 0.54161D-01,
     & 0.57789D-01, 0.61381D-01, 0.64943D-01, 0.68480D-01, 0.71979D-01,
     & 0.75440D-01, 0.78865D-01, 0.82253D-01, 0.85607D-01, 0.88927D-01,
     & 0.92228D-01, 0.95491D-01, 0.98718D-01, 0.10191D+00, 0.10507D+00,
     & 0.10002D-07, 0.38860D-02, 0.77373D-02, 0.11554D-01, 0.15336D-01,
     & 0.19085D-01, 0.22810D-01, 0.26496D-01, 0.30142D-01, 0.33750D-01,
     & 0.37320D-01, 0.40862D-01, 0.44370D-01, 0.47837D-01, 0.51264D-01,
     & 0.54652D-01, 0.58002D-01, 0.61319D-01, 0.64609D-01, 0.67858D-01,
     & 0.71068D-01, 0.74240D-01, 0.77374D-01, 0.80473D-01, 0.83536D-01,
     & 0.86579D-01, 0.89584D-01, 0.92551D-01, 0.95483D-01, 0.98379D-01,
     & 0.10001D-07, 0.37195D-02, 0.73961D-02, 0.11031D-01, 0.14624D-01,
     & 0.18179D-01, 0.21705D-01, 0.25185D-01, 0.28622D-01, 0.32016D-01,
     & 0.35369D-01, 0.38690D-01, 0.41973D-01, 0.45213D-01, 0.48410D-01,
     & 0.51566D-01, 0.54682D-01, 0.57762D-01, 0.60814D-01, 0.63824D-01,
     & 0.66793D-01, 0.69722D-01, 0.72614D-01, 0.75468D-01, 0.78287D-01,
     & 0.81083D-01, 0.83841D-01, 0.86562D-01, 0.89247D-01, 0.91896D-01,
     & 0.10002D-07, 0.35513D-02, 0.70527D-02, 0.10506D-01, 0.13912D-01,
     & 0.17274D-01, 0.20602D-01, 0.23881D-01, 0.27112D-01, 0.30297D-01,
     & 0.33437D-01, 0.36543D-01, 0.39608D-01, 0.42628D-01, 0.45603D-01,
     & 0.48534D-01, 0.51424D-01, 0.54277D-01, 0.57099D-01, 0.59879D-01,
     & 0.62617D-01, 0.65315D-01, 0.67974D-01, 0.70596D-01, 0.73181D-01,
     & 0.75744D-01, 0.78268D-01, 0.80754D-01, 0.83205D-01, 0.85621D-01,
     & 0.10001D-07, 0.33798D-02, 0.67037D-02, 0.99737D-02, 0.13192D-01,
     & 0.16361D-01, 0.19492D-01, 0.22571D-01, 0.25598D-01, 0.28577D-01,
     & 0.31508D-01, 0.34402D-01, 0.37253D-01, 0.40057D-01, 0.42814D-01,
     & 0.45528D-01, 0.48198D-01, 0.50830D-01, 0.53430D-01, 0.55987D-01,
     & 0.58502D-01, 0.60977D-01, 0.63413D-01, 0.65811D-01, 0.68173D-01,
     & 0.70511D-01, 0.72811D-01, 0.75074D-01, 0.77301D-01, 0.79494D-01/

      data ((g_grdl(i,j),j=1,30),i=11,20)/
     & 0.99996D-08, 0.32054D-02, 0.63496D-02, 0.94353D-02, 0.12465D-01,
     & 0.15442D-01, 0.18377D-01, 0.21257D-01, 0.24083D-01, 0.26858D-01,
     & 0.29583D-01, 0.32269D-01, 0.34911D-01, 0.37505D-01, 0.40051D-01,
     & 0.42552D-01, 0.45009D-01, 0.47427D-01, 0.49813D-01, 0.52155D-01,
     & 0.54455D-01, 0.56715D-01, 0.58936D-01, 0.61120D-01, 0.63268D-01,
     & 0.65391D-01, 0.67477D-01, 0.69527D-01, 0.71542D-01, 0.73523D-01,
     & 0.99991D-08, 0.30293D-02, 0.59932D-02, 0.88949D-02, 0.11737D-01,
     & 0.14524D-01, 0.17265D-01, 0.19949D-01, 0.22578D-01, 0.25154D-01,
     & 0.27679D-01, 0.30163D-01, 0.32601D-01, 0.34991D-01, 0.37333D-01,
     & 0.39629D-01, 0.41881D-01, 0.44095D-01, 0.46274D-01, 0.48411D-01,
     & 0.50506D-01, 0.52562D-01, 0.54579D-01, 0.56559D-01, 0.58505D-01,
     & 0.60424D-01, 0.62308D-01, 0.64156D-01, 0.65970D-01, 0.67752D-01,
     & 0.99973D-08, 0.28498D-02, 0.56310D-02, 0.83470D-02, 0.11001D-01,
     & 0.13597D-01, 0.16146D-01, 0.18635D-01, 0.21068D-01, 0.23448D-01,
     & 0.25775D-01, 0.28060D-01, 0.30300D-01, 0.32490D-01, 0.34633D-01,
     & 0.36730D-01, 0.38784D-01, 0.40798D-01, 0.42779D-01, 0.44717D-01,
     & 0.46615D-01, 0.48473D-01, 0.50295D-01, 0.52080D-01, 0.53831D-01,
     & 0.55556D-01, 0.57246D-01, 0.58903D-01, 0.60526D-01, 0.62119D-01,
     & 0.99985D-08, 0.26693D-02, 0.52677D-02, 0.77989D-02, 0.10266D-01,
     & 0.12674D-01, 0.15033D-01, 0.17331D-01, 0.19573D-01, 0.21761D-01,
     & 0.23896D-01, 0.25989D-01, 0.28035D-01, 0.30033D-01, 0.31984D-01,
     & 0.33890D-01, 0.35753D-01, 0.37577D-01, 0.39367D-01, 0.41116D-01,
     & 0.42826D-01, 0.44498D-01, 0.46134D-01, 0.47734D-01, 0.49302D-01,
     & 0.50843D-01, 0.52351D-01, 0.53827D-01, 0.55272D-01, 0.56686D-01,
     & 0.10003D-07, 0.24873D-02, 0.49022D-02, 0.72487D-02, 0.95305D-02,
     & 0.11752D-01, 0.13922D-01, 0.16033D-01, 0.18087D-01, 0.20087D-01,
     & 0.22035D-01, 0.23940D-01, 0.25799D-01, 0.27611D-01, 0.29376D-01,
     & 0.31098D-01, 0.32777D-01, 0.34419D-01, 0.36026D-01, 0.37594D-01,
     & 0.39124D-01, 0.40618D-01, 0.42077D-01, 0.43502D-01, 0.44895D-01,
     & 0.46263D-01, 0.47599D-01, 0.48904D-01, 0.50180D-01, 0.51427D-01,
     & 0.10001D-07, 0.23012D-02, 0.45296D-02, 0.66894D-02, 0.87842D-02,
     & 0.10819D-01, 0.12801D-01, 0.14724D-01, 0.16592D-01, 0.18406D-01,
     & 0.20171D-01, 0.21891D-01, 0.23566D-01, 0.25195D-01, 0.26780D-01,
     & 0.28322D-01, 0.29824D-01, 0.31288D-01, 0.32720D-01, 0.34113D-01,
     & 0.35471D-01, 0.36793D-01, 0.38083D-01, 0.39340D-01, 0.40568D-01,
     & 0.41770D-01, 0.42942D-01, 0.44086D-01, 0.45202D-01, 0.46291D-01,
     & 0.10001D-07, 0.21138D-02, 0.41554D-02, 0.61287D-02, 0.80379D-02,
     & 0.98871D-02, 0.11684D-01, 0.13423D-01, 0.15108D-01, 0.16742D-01,
     & 0.18326D-01, 0.19867D-01, 0.21365D-01, 0.22817D-01, 0.24228D-01,
     & 0.25598D-01, 0.26929D-01, 0.28224D-01, 0.29487D-01, 0.30714D-01,
     & 0.31907D-01, 0.33068D-01, 0.34197D-01, 0.35296D-01, 0.36366D-01,
     & 0.37413D-01, 0.38431D-01, 0.39423D-01, 0.40389D-01, 0.41330D-01,
     & 0.10001D-07, 0.19251D-02, 0.37792D-02, 0.55665D-02, 0.72910D-02,
     & 0.89569D-02, 0.10571D-01, 0.12129D-01, 0.13635D-01, 0.15091D-01,
     & 0.16501D-01, 0.17868D-01, 0.19193D-01, 0.20475D-01, 0.21718D-01,
     & 0.22922D-01, 0.24089D-01, 0.25223D-01, 0.26325D-01, 0.27394D-01,
     & 0.28431D-01, 0.29438D-01, 0.30415D-01, 0.31364D-01, 0.32287D-01,
     & 0.33187D-01, 0.34061D-01, 0.34910D-01, 0.35736D-01, 0.36539D-01,
     & 0.10002D-07, 0.17356D-02, 0.34024D-02, 0.50048D-02, 0.65463D-02,
     & 0.80312D-02, 0.94652D-02, 0.10846D-01, 0.12178D-01, 0.13461D-01,
     & 0.14701D-01, 0.15899D-01, 0.17058D-01, 0.18177D-01, 0.19258D-01,
     & 0.20304D-01, 0.21315D-01, 0.22294D-01, 0.23244D-01, 0.24163D-01,
     & 0.25052D-01, 0.25914D-01, 0.26748D-01, 0.27557D-01, 0.28342D-01,
     & 0.29104D-01, 0.29843D-01, 0.30559D-01, 0.31255D-01, 0.31929D-01,
     & 0.10003D-07, 0.15460D-02, 0.30263D-02, 0.44450D-02, 0.58059D-02,
     & 0.71126D-02, 0.83701D-02, 0.95780D-02, 0.10739D-01, 0.11856D-01,
     & 0.12930D-01, 0.13967D-01, 0.14965D-01, 0.15928D-01, 0.16855D-01,
     & 0.17749D-01, 0.18612D-01, 0.19445D-01, 0.20250D-01, 0.21027D-01,
     & 0.21777D-01, 0.22502D-01, 0.23203D-01, 0.23881D-01, 0.24536D-01,
     & 0.25170D-01, 0.25784D-01, 0.26378D-01, 0.26952D-01, 0.27508D-01/

      data ((g_grdl(i,j),j=1,30),i=21,30)/
     & 0.10004D-07, 0.13567D-02, 0.26517D-02, 0.38888D-02, 0.50717D-02,
     & 0.62036D-02, 0.72885D-02, 0.83276D-02, 0.93233D-02, 0.10278D-01,
     & 0.11194D-01, 0.12074D-01, 0.12920D-01, 0.13732D-01, 0.14512D-01,
     & 0.15263D-01, 0.15984D-01, 0.16679D-01, 0.17348D-01, 0.17992D-01,
     & 0.18612D-01, 0.19210D-01, 0.19785D-01, 0.20341D-01, 0.20876D-01,
     & 0.21392D-01, 0.21889D-01, 0.22370D-01, 0.22833D-01, 0.23281D-01,
     & 0.99978D-08, 0.11673D-02, 0.22776D-02, 0.33345D-02, 0.43415D-02,
     & 0.53015D-02, 0.62174D-02, 0.70917D-02, 0.79268D-02, 0.87249D-02,
     & 0.94880D-02, 0.10218D-01, 0.10916D-01, 0.11585D-01, 0.12226D-01,
     & 0.12840D-01, 0.13428D-01, 0.13993D-01, 0.14534D-01, 0.15053D-01,
     & 0.15551D-01, 0.16030D-01, 0.16490D-01, 0.16931D-01, 0.17356D-01,
     & 0.17763D-01, 0.18154D-01, 0.18530D-01, 0.18893D-01, 0.19241D-01,
     & 0.99975D-08, 0.98178D-03, 0.19120D-02, 0.27942D-02, 0.36312D-02,
     & 0.44257D-02, 0.51797D-02, 0.58968D-02, 0.65792D-02, 0.72288D-02,
     & 0.78475D-02, 0.84361D-02, 0.89969D-02, 0.95319D-02, 0.10043D-01,
     & 0.10530D-01, 0.10996D-01, 0.11440D-01, 0.11863D-01, 0.12268D-01,
     & 0.12656D-01, 0.13026D-01, 0.13381D-01, 0.13720D-01, 0.14044D-01,
     & 0.14353D-01, 0.14650D-01, 0.14933D-01, 0.15205D-01, 0.15466D-01,
     & 0.10002D-07, 0.80078D-03, 0.15562D-02, 0.22693D-02, 0.29428D-02,
     & 0.35788D-02, 0.41784D-02, 0.47463D-02, 0.52842D-02, 0.57940D-02,
     & 0.62771D-02, 0.67339D-02, 0.71667D-02, 0.75778D-02, 0.79684D-02,
     & 0.83396D-02, 0.86924D-02, 0.90270D-02, 0.93436D-02, 0.96451D-02,
     & 0.99322D-02, 0.10206D-01, 0.10466D-01, 0.10713D-01, 0.10949D-01,
     & 0.11171D-01, 0.11383D-01, 0.11585D-01, 0.11778D-01, 0.11962D-01,
     & 0.10001D-07, 0.62346D-03, 0.12085D-02, 0.17578D-02, 0.22736D-02,
     & 0.27576D-02, 0.32100D-02, 0.36362D-02, 0.40377D-02, 0.44160D-02,
     & 0.47725D-02, 0.51066D-02, 0.54209D-02, 0.57178D-02, 0.59983D-02,
     & 0.62633D-02, 0.65135D-02, 0.67491D-02, 0.69696D-02, 0.71785D-02,
     & 0.73762D-02, 0.75632D-02, 0.77401D-02, 0.79072D-02, 0.80650D-02,
     & 0.82118D-02, 0.83508D-02, 0.84825D-02, 0.86071D-02, 0.87249D-02,
     & 0.10001D-07, 0.58174D-03, 0.11269D-02, 0.16379D-02, 0.21169D-02,
     & 0.25657D-02, 0.29841D-02, 0.33777D-02, 0.37480D-02, 0.40963D-02,
     & 0.44239D-02, 0.47302D-02, 0.50177D-02, 0.52889D-02, 0.55447D-02,
     & 0.57859D-02, 0.60133D-02, 0.62269D-02, 0.64262D-02, 0.66147D-02,
     & 0.67927D-02, 0.69608D-02, 0.71195D-02, 0.72692D-02, 0.74101D-02,
     & 0.75406D-02, 0.76640D-02, 0.77806D-02, 0.78907D-02, 0.79945D-02,
     & 0.10000D-07, 0.54198D-03, 0.10491D-02, 0.15237D-02, 0.19680D-02,
     & 0.23834D-02, 0.27697D-02, 0.31325D-02, 0.34733D-02, 0.37933D-02,
     & 0.40938D-02, 0.43741D-02, 0.46365D-02, 0.48837D-02, 0.51164D-02,
     & 0.53355D-02, 0.55416D-02, 0.57347D-02, 0.59143D-02, 0.60839D-02,
     & 0.62437D-02, 0.63944D-02, 0.65363D-02, 0.66699D-02, 0.67953D-02,
     & 0.69109D-02, 0.70199D-02, 0.71227D-02, 0.72195D-02, 0.73106D-02,
     & 0.10000D-07, 0.50405D-03, 0.97498D-03, 0.14150D-02, 0.18263D-02,
     & 0.22100D-02, 0.25659D-02, 0.28996D-02, 0.32126D-02, 0.35060D-02,
     & 0.37810D-02, 0.40368D-02, 0.42757D-02, 0.45003D-02, 0.47115D-02,
     & 0.49099D-02, 0.50962D-02, 0.52703D-02, 0.54317D-02, 0.55837D-02,
     & 0.57267D-02, 0.58613D-02, 0.59877D-02, 0.61064D-02, 0.62176D-02,
     & 0.63195D-02, 0.64154D-02, 0.65056D-02, 0.65903D-02, 0.66697D-02,
     & 0.10000D-07, 0.46783D-03, 0.90424D-03, 0.13114D-02, 0.16912D-02,
     & 0.20449D-02, 0.23720D-02, 0.26783D-02, 0.29650D-02, 0.32333D-02,
     & 0.34843D-02, 0.37171D-02, 0.39339D-02, 0.41375D-02, 0.43285D-02,
     & 0.45076D-02, 0.46755D-02, 0.48318D-02, 0.49763D-02, 0.51120D-02,
     & 0.52395D-02, 0.53592D-02, 0.54714D-02, 0.55764D-02, 0.56744D-02,
     & 0.57638D-02, 0.58477D-02, 0.59263D-02, 0.60000D-02, 0.60689D-02,
     & 0.10000D-07, 0.43320D-03, 0.83667D-03, 0.12124D-02, 0.15624D-02,
     & 0.18876D-02, 0.21874D-02, 0.24676D-02, 0.27296D-02, 0.29742D-02,
     & 0.32026D-02, 0.34138D-02, 0.36100D-02, 0.37938D-02, 0.39660D-02,
     & 0.41271D-02, 0.42778D-02, 0.44177D-02, 0.45464D-02, 0.46671D-02,
     & 0.47802D-02, 0.48861D-02, 0.49851D-02, 0.50776D-02, 0.51636D-02,
     & 0.52414D-02, 0.53143D-02, 0.53825D-02, 0.54461D-02, 0.55054D-02/

      data ((g_grdl(i,j),j=1,30),i=31,40)/
     & 0.10000D-07, 0.40008D-03, 0.77209D-03, 0.11180D-02, 0.14394D-02,
     & 0.17376D-02, 0.20115D-02, 0.22672D-02, 0.25056D-02, 0.27280D-02,
     & 0.29351D-02, 0.31260D-02, 0.33028D-02, 0.34682D-02, 0.36228D-02,
     & 0.37671D-02, 0.39017D-02, 0.40263D-02, 0.41404D-02, 0.42472D-02,
     & 0.43470D-02, 0.44402D-02, 0.45272D-02, 0.46080D-02, 0.46830D-02,
     & 0.47504D-02, 0.48133D-02, 0.48719D-02, 0.49263D-02, 0.49769D-02,
     & 0.10000D-07, 0.36837D-03, 0.71033D-03, 0.10277D-02, 0.13221D-02,
     & 0.15945D-02, 0.18439D-02, 0.20762D-02, 0.22926D-02, 0.24938D-02,
     & 0.26810D-02, 0.28527D-02, 0.30114D-02, 0.31595D-02, 0.32976D-02,
     & 0.34263D-02, 0.35460D-02, 0.36564D-02, 0.37570D-02, 0.38509D-02,
     & 0.39385D-02, 0.40200D-02, 0.40958D-02, 0.41661D-02, 0.42310D-02,
     & 0.42888D-02, 0.43426D-02, 0.43925D-02, 0.44386D-02, 0.44813D-02,
     & 0.10000D-07, 0.33801D-03, 0.65123D-03, 0.94137D-03, 0.12100D-02,
     & 0.14579D-02, 0.16841D-02, 0.18944D-02, 0.20898D-02, 0.22712D-02,
     & 0.24395D-02, 0.25933D-02, 0.27350D-02, 0.28669D-02, 0.29897D-02,
     & 0.31037D-02, 0.32096D-02, 0.33068D-02, 0.33949D-02, 0.34770D-02,
     & 0.35532D-02, 0.36240D-02, 0.36896D-02, 0.37502D-02, 0.38059D-02,
     & 0.38550D-02, 0.39005D-02, 0.39425D-02, 0.39812D-02, 0.40167D-02,
     & 0.10000D-07, 0.30896D-03, 0.59474D-03, 0.85892D-03, 0.11030D-02,
     & 0.13277D-02, 0.15318D-02, 0.17213D-02, 0.18969D-02, 0.20597D-02,
     & 0.22102D-02, 0.23473D-02, 0.24730D-02, 0.25899D-02, 0.26983D-02,
     & 0.27988D-02, 0.28918D-02, 0.29768D-02, 0.30534D-02, 0.31245D-02,
     & 0.31904D-02, 0.32514D-02, 0.33076D-02, 0.33593D-02, 0.34066D-02,
     & 0.34479D-02, 0.34859D-02, 0.35208D-02, 0.35528D-02, 0.35820D-02,
     & 0.10001D-07, 0.28118D-03, 0.54076D-03, 0.78024D-03, 0.10010D-02,
     & 0.12037D-02, 0.13870D-02, 0.15568D-02, 0.17138D-02, 0.18590D-02,
     & 0.19929D-02, 0.21143D-02, 0.22252D-02, 0.23280D-02, 0.24232D-02,
     & 0.25111D-02, 0.25922D-02, 0.26661D-02, 0.27321D-02, 0.27932D-02,
     & 0.28496D-02, 0.29016D-02, 0.29493D-02, 0.29930D-02, 0.30328D-02,
     & 0.30669D-02, 0.30982D-02, 0.31268D-02, 0.31528D-02, 0.31763D-02,
     & 0.99998D-08, 0.25435D-03, 0.48868D-03, 0.70439D-03, 0.90274D-03,
     & 0.10844D-02, 0.12478D-02, 0.13988D-02, 0.15383D-02, 0.16667D-02,
     & 0.17850D-02, 0.18916D-02, 0.19886D-02, 0.20782D-02, 0.21610D-02,
     & 0.22372D-02, 0.23072D-02, 0.23706D-02, 0.24269D-02, 0.24787D-02,
     & 0.25265D-02, 0.25702D-02, 0.26102D-02, 0.26466D-02, 0.26795D-02,
     & 0.27072D-02, 0.27325D-02, 0.27554D-02, 0.27760D-02, 0.27945D-02,
     & 0.99996D-08, 0.22875D-03, 0.43904D-03, 0.63217D-03, 0.80930D-03,
     & 0.97099D-03, 0.11157D-02, 0.12492D-02, 0.13720D-02, 0.14849D-02,
     & 0.15885D-02, 0.16813D-02, 0.17654D-02, 0.18429D-02, 0.19142D-02,
     & 0.19796D-02, 0.20395D-02, 0.20934D-02, 0.21407D-02, 0.21842D-02,
     & 0.22241D-02, 0.22604D-02, 0.22934D-02, 0.23233D-02, 0.23500D-02,
     & 0.23720D-02, 0.23920D-02, 0.24099D-02, 0.24258D-02, 0.24399D-02,
     & 0.99998D-08, 0.20426D-03, 0.39161D-03, 0.56324D-03, 0.72023D-03,
     & 0.86304D-03, 0.99012D-03, 0.11070D-02, 0.12143D-02, 0.13126D-02,
     & 0.14025D-02, 0.14825D-02, 0.15546D-02, 0.16209D-02, 0.16816D-02,
     & 0.17371D-02, 0.17877D-02, 0.18329D-02, 0.18722D-02, 0.19081D-02,
     & 0.19408D-02, 0.19705D-02, 0.19972D-02, 0.20212D-02, 0.20425D-02,
     & 0.20596D-02, 0.20748D-02, 0.20884D-02, 0.21002D-02, 0.21105D-02,
     & 0.99999D-08, 0.18079D-03, 0.34621D-03, 0.49735D-03, 0.63518D-03,
     & 0.76010D-03, 0.87053D-03, 0.97181D-03, 0.10645D-02, 0.11492D-02,
     & 0.12263D-02, 0.12944D-02, 0.13555D-02, 0.14114D-02, 0.14624D-02,
     & 0.15088D-02, 0.15508D-02, 0.15882D-02, 0.16202D-02, 0.16492D-02,
     & 0.16756D-02, 0.16992D-02, 0.17204D-02, 0.17392D-02, 0.17557D-02,
     & 0.17684D-02, 0.17796D-02, 0.17893D-02, 0.17977D-02, 0.18046D-02,
     & 0.99999D-08, 0.15833D-03, 0.30281D-03, 0.43443D-03, 0.55408D-03,
     & 0.66207D-03, 0.75683D-03, 0.84348D-03, 0.92253D-03, 0.99444D-03,
     & 0.10597D-02, 0.11169D-02, 0.11677D-02, 0.12141D-02, 0.12562D-02,
     & 0.12943D-02, 0.13287D-02, 0.13588D-02, 0.13843D-02, 0.14073D-02,
     & 0.14279D-02, 0.14463D-02, 0.14626D-02, 0.14769D-02, 0.14891D-02,
     & 0.14981D-02, 0.15058D-02, 0.15123D-02, 0.15177D-02, 0.15219D-02/

      data ((g_grdl(i,j),j=1,30),i=41,50)/
     & 0.10000D-07, 0.13686D-03, 0.26139D-03, 0.37448D-03, 0.47691D-03,
     & 0.56895D-03, 0.64901D-03, 0.72199D-03, 0.78831D-03, 0.84840D-03,
     & 0.90263D-03, 0.94975D-03, 0.99127D-03, 0.10289D-02, 0.10630D-02,
     & 0.10937D-02, 0.11211D-02, 0.11449D-02, 0.11646D-02, 0.11822D-02,
     & 0.11978D-02, 0.12116D-02, 0.12236D-02, 0.12340D-02, 0.12426D-02,
     & 0.12484D-02, 0.12533D-02, 0.12571D-02, 0.12600D-02, 0.12620D-02,
     & 0.10000D-07, 0.11635D-03, 0.22190D-03, 0.31740D-03, 0.40357D-03,
     & 0.48059D-03, 0.54692D-03, 0.60716D-03, 0.66168D-03, 0.71084D-03,
     & 0.75497D-03, 0.79287D-03, 0.82593D-03, 0.85577D-03, 0.88258D-03,
     & 0.90652D-03, 0.92774D-03, 0.94592D-03, 0.96053D-03, 0.97347D-03,
     & 0.98481D-03, 0.99464D-03, 0.10030D-02, 0.10101D-02, 0.10157D-02,
     & 0.10189D-02, 0.10214D-02, 0.10231D-02, 0.10241D-02, 0.10244D-02,
     & 0.10000D-07, 0.96818D-04, 0.18433D-03, 0.26321D-03, 0.33405D-03,
     & 0.39700D-03, 0.45058D-03, 0.49902D-03, 0.54266D-03, 0.58179D-03,
     & 0.61670D-03, 0.64625D-03, 0.67172D-03, 0.69455D-03, 0.71490D-03,
     & 0.73291D-03, 0.74870D-03, 0.76197D-03, 0.77224D-03, 0.78119D-03,
     & 0.78889D-03, 0.79539D-03, 0.80076D-03, 0.80505D-03, 0.80824D-03,
     & 0.80947D-03, 0.81011D-03, 0.81017D-03, 0.80970D-03, 0.80871D-03,
     & 0.10000D-07, 0.78260D-04, 0.14872D-03, 0.21194D-03, 0.26842D-03,
     & 0.31826D-03, 0.36007D-03, 0.39770D-03, 0.43139D-03, 0.46142D-03,
     & 0.48799D-03, 0.51009D-03, 0.52884D-03, 0.54550D-03, 0.56021D-03,
     & 0.57307D-03, 0.58418D-03, 0.59327D-03, 0.59992D-03, 0.60557D-03,
     & 0.61028D-03, 0.61408D-03, 0.61703D-03, 0.61917D-03, 0.62047D-03,
     & 0.62021D-03, 0.61953D-03, 0.61846D-03, 0.61700D-03, 0.61518D-03,
     & 0.10000D-07, 0.60731D-04, 0.11515D-03, 0.16373D-03, 0.20687D-03,
     & 0.24461D-03, 0.27570D-03, 0.30351D-03, 0.32825D-03, 0.35011D-03,
     & 0.36928D-03, 0.38486D-03, 0.39778D-03, 0.40914D-03, 0.41904D-03,
     & 0.42754D-03, 0.43474D-03, 0.44039D-03, 0.44414D-03, 0.44718D-03,
     & 0.44955D-03, 0.45128D-03, 0.45240D-03, 0.45295D-03, 0.45291D-03,
     & 0.45168D-03, 0.45018D-03, 0.44842D-03, 0.44642D-03, 0.44419D-03,
     & 0.10000D-07, 0.44309D-04, 0.83796D-04, 0.11881D-03, 0.14968D-03,
     & 0.17640D-03, 0.19788D-03, 0.21695D-03, 0.23377D-03, 0.24847D-03,
     & 0.26120D-03, 0.27121D-03, 0.27925D-03, 0.28620D-03, 0.29214D-03,
     & 0.29711D-03, 0.30117D-03, 0.30413D-03, 0.30570D-03, 0.30682D-03,
     & 0.30751D-03, 0.30779D-03, 0.30768D-03, 0.30721D-03, 0.30635D-03,
     & 0.30465D-03, 0.30281D-03, 0.30083D-03, 0.29872D-03, 0.29648D-03,
     & 0.10000D-07, 0.29027D-04, 0.54707D-04, 0.77291D-04, 0.96997D-04,
     & 0.11380D-03, 0.12682D-03, 0.13827D-03, 0.14824D-03, 0.15684D-03,
     & 0.16415D-03, 0.16961D-03, 0.17376D-03, 0.17726D-03, 0.18015D-03,
     & 0.18246D-03, 0.18422D-03, 0.18529D-03, 0.18548D-03, 0.18543D-03,
     & 0.18515D-03, 0.18464D-03, 0.18393D-03, 0.18303D-03, 0.18191D-03,
     & 0.18029D-03, 0.17862D-03, 0.17689D-03, 0.17510D-03, 0.17327D-03,
     & 0.10000D-07, 0.15062D-04, 0.28303D-04, 0.39862D-04, 0.49859D-04,
     & 0.58271D-04, 0.64555D-04, 0.70030D-04, 0.74748D-04, 0.78757D-04,
     & 0.82103D-04, 0.84459D-04, 0.86135D-04, 0.87499D-04, 0.88571D-04,
     & 0.89366D-04, 0.89901D-04, 0.90097D-04, 0.89848D-04, 0.89497D-04,
     & 0.89049D-04, 0.88509D-04, 0.87883D-04, 0.87174D-04, 0.86376D-04,
     & 0.85340D-04, 0.84289D-04, 0.83225D-04, 0.82147D-04, 0.81058D-04,
     & 0.10000D-07, 0.49081D-05, 0.91811D-05, 0.12874D-04, 0.16029D-04,
     & 0.18633D-04, 0.20471D-04, 0.22049D-04, 0.23385D-04, 0.24493D-04,
     & 0.25389D-04, 0.25950D-04, 0.26290D-04, 0.26541D-04, 0.26708D-04,
     & 0.26796D-04, 0.26810D-04, 0.26723D-04, 0.26497D-04, 0.26249D-04,
     & 0.25979D-04, 0.25689D-04, 0.25380D-04, 0.25053D-04, 0.24705D-04,
     & 0.24290D-04, 0.23877D-04, 0.23465D-04, 0.23056D-04, 0.22649D-04,
     & 0.10000D-07, 0.12592D-06, 0.22440D-06, 0.30673D-06, 0.37406D-06,
     & 0.42564D-06, 0.45367D-06, 0.47582D-06, 0.49250D-06, 0.50407D-06,
     & 0.51084D-06, 0.50885D-06, 0.50154D-06, 0.49307D-06, 0.48351D-06,
     & 0.47292D-06, 0.46137D-06, 0.44808D-06, 0.43202D-06, 0.41622D-06,
     & 0.40068D-06, 0.38537D-06, 0.37030D-06, 0.35546D-06, 0.34076D-06,
     & 0.32525D-06, 0.31029D-06, 0.29587D-06, 0.28194D-06, 0.26851D-06/

      data ((c_grdl(i,j),j=1,30),i=1,10)/
     & 0.18780D-05, 0.18839D-05, 0.18892D-05, 0.18938D-05, 0.18979D-05,
     & 0.19016D-05, 0.19048D-05, 0.19076D-05, 0.19102D-05, 0.19124D-05,
     & 0.19144D-05, 0.19162D-05, 0.19177D-05, 0.19191D-05, 0.19203D-05,
     & 0.19214D-05, 0.19224D-05, 0.19232D-05, 0.19240D-05, 0.19247D-05,
     & 0.19253D-05, 0.19258D-05, 0.19262D-05, 0.19267D-05, 0.19270D-05,
     & 0.19273D-05, 0.19276D-05, 0.19279D-05, 0.19281D-05, 0.19283D-05,
     & 0.18619D-05, 0.18689D-05, 0.18750D-05, 0.18805D-05, 0.18854D-05,
     & 0.18896D-05, 0.18934D-05, 0.18968D-05, 0.18998D-05, 0.19024D-05,
     & 0.19048D-05, 0.19069D-05, 0.19087D-05, 0.19103D-05, 0.19118D-05,
     & 0.19130D-05, 0.19142D-05, 0.19152D-05, 0.19161D-05, 0.19169D-05,
     & 0.19176D-05, 0.19182D-05, 0.19187D-05, 0.19192D-05, 0.19196D-05,
     & 0.19200D-05, 0.19204D-05, 0.19207D-05, 0.19209D-05, 0.19212D-05,
     & 0.18429D-05, 0.18511D-05, 0.18584D-05, 0.18649D-05, 0.18706D-05,
     & 0.18756D-05, 0.18801D-05, 0.18841D-05, 0.18876D-05, 0.18907D-05,
     & 0.18934D-05, 0.18959D-05, 0.18981D-05, 0.19000D-05, 0.19017D-05,
     & 0.19032D-05, 0.19045D-05, 0.19057D-05, 0.19067D-05, 0.19077D-05,
     & 0.19085D-05, 0.19092D-05, 0.19099D-05, 0.19104D-05, 0.19110D-05,
     & 0.19114D-05, 0.19118D-05, 0.19122D-05, 0.19125D-05, 0.19127D-05,
     & 0.18206D-05, 0.18303D-05, 0.18388D-05, 0.18464D-05, 0.18531D-05,
     & 0.18591D-05, 0.18644D-05, 0.18690D-05, 0.18732D-05, 0.18768D-05,
     & 0.18801D-05, 0.18830D-05, 0.18855D-05, 0.18878D-05, 0.18898D-05,
     & 0.18916D-05, 0.18931D-05, 0.18945D-05, 0.18958D-05, 0.18969D-05,
     & 0.18978D-05, 0.18987D-05, 0.18994D-05, 0.19001D-05, 0.19007D-05,
     & 0.19012D-05, 0.19017D-05, 0.19021D-05, 0.19025D-05, 0.19028D-05,
     & 0.17943D-05, 0.18057D-05, 0.18158D-05, 0.18247D-05, 0.18326D-05,
     & 0.18396D-05, 0.18458D-05, 0.18513D-05, 0.18562D-05, 0.18605D-05,
     & 0.18644D-05, 0.18678D-05, 0.18708D-05, 0.18734D-05, 0.18758D-05,
     & 0.18779D-05, 0.18797D-05, 0.18814D-05, 0.18828D-05, 0.18841D-05,
     & 0.18852D-05, 0.18863D-05, 0.18872D-05, 0.18879D-05, 0.18887D-05,
     & 0.18893D-05, 0.18898D-05, 0.18903D-05, 0.18908D-05, 0.18911D-05,
     & 0.17634D-05, 0.17768D-05, 0.17886D-05, 0.17991D-05, 0.18084D-05,
     & 0.18167D-05, 0.18240D-05, 0.18305D-05, 0.18362D-05, 0.18413D-05,
     & 0.18458D-05, 0.18498D-05, 0.18534D-05, 0.18565D-05, 0.18593D-05,
     & 0.18617D-05, 0.18639D-05, 0.18659D-05, 0.18676D-05, 0.18691D-05,
     & 0.18704D-05, 0.18716D-05, 0.18727D-05, 0.18736D-05, 0.18744D-05,
     & 0.18752D-05, 0.18758D-05, 0.18764D-05, 0.18769D-05, 0.18774D-05,
     & 0.25004D-05, 0.23172D-05, 0.20454D-05, 0.17690D-05, 0.17800D-05,
     & 0.17897D-05, 0.17983D-05, 0.18060D-05, 0.18127D-05, 0.18187D-05,
     & 0.18240D-05, 0.18287D-05, 0.18329D-05, 0.18366D-05, 0.18399D-05,
     & 0.18428D-05, 0.18453D-05, 0.18476D-05, 0.18496D-05, 0.18514D-05,
     & 0.18530D-05, 0.18544D-05, 0.18556D-05, 0.18567D-05, 0.18577D-05,
     & 0.18586D-05, 0.18593D-05, 0.18600D-05, 0.18606D-05, 0.18612D-05,
     & 0.28123D-05, 0.26980D-05, 0.25725D-05, 0.24307D-05, 0.22596D-05,
     & 0.20028D-05, 0.17681D-05, 0.17771D-05, 0.17850D-05, 0.17921D-05,
     & 0.17983D-05, 0.18039D-05, 0.18088D-05, 0.18131D-05, 0.18170D-05,
     & 0.18204D-05, 0.18234D-05, 0.18261D-05, 0.18285D-05, 0.18306D-05,
     & 0.18324D-05, 0.18341D-05, 0.18356D-05, 0.18369D-05, 0.18380D-05,
     & 0.18390D-05, 0.18399D-05, 0.18407D-05, 0.18414D-05, 0.18421D-05,
     & 0.30358D-05, 0.29514D-05, 0.28616D-05, 0.27667D-05, 0.26663D-05,
     & 0.25592D-05, 0.24429D-05, 0.23118D-05, 0.21509D-05, 0.18587D-05,
     & 0.17681D-05, 0.17746D-05, 0.17804D-05, 0.17855D-05, 0.17900D-05,
     & 0.17941D-05, 0.17976D-05, 0.18008D-05, 0.18036D-05, 0.18060D-05,
     & 0.18082D-05, 0.18102D-05, 0.18119D-05, 0.18134D-05, 0.18148D-05,
     & 0.18160D-05, 0.18171D-05, 0.18180D-05, 0.18188D-05, 0.18196D-05,
     & 0.32014D-05, 0.31394D-05, 0.30726D-05, 0.30019D-05, 0.29283D-05,
     & 0.28524D-05, 0.27745D-05, 0.26949D-05, 0.26134D-05, 0.25297D-05,
     & 0.24429D-05, 0.23514D-05, 0.22523D-05, 0.21388D-05, 0.19889D-05,
     & 0.17631D-05, 0.17673D-05, 0.17710D-05, 0.17743D-05, 0.17772D-05,
     & 0.17798D-05, 0.17821D-05, 0.17841D-05, 0.17859D-05, 0.17875D-05,
     & 0.17889D-05, 0.17901D-05, 0.17912D-05, 0.17922D-05, 0.17931D-05/

      data ((c_grdl(i,j),j=1,30),i=11,20)/
     & 0.33147D-05, 0.32744D-05, 0.32279D-05, 0.31767D-05, 0.31223D-05,
     & 0.30657D-05, 0.30077D-05, 0.29491D-05, 0.28904D-05, 0.28321D-05,
     & 0.27745D-05, 0.27179D-05, 0.26624D-05, 0.26082D-05, 0.25553D-05,
     & 0.25038D-05, 0.24535D-05, 0.24046D-05, 0.23568D-05, 0.23100D-05,
     & 0.22642D-05, 0.22190D-05, 0.21740D-05, 0.21290D-05, 0.20833D-05,
     & 0.20358D-05, 0.19848D-05, 0.19260D-05, 0.18426D-05, 0.17619D-05,
     & 0.33730D-05, 0.33559D-05, 0.33307D-05, 0.32991D-05, 0.32627D-05,
     & 0.32229D-05, 0.31810D-05, 0.31378D-05, 0.30941D-05, 0.30505D-05,
     & 0.30077D-05, 0.29659D-05, 0.29255D-05, 0.28867D-05, 0.28497D-05,
     & 0.28146D-05, 0.27814D-05, 0.27503D-05, 0.27211D-05, 0.26940D-05,
     & 0.26687D-05, 0.26454D-05, 0.26239D-05, 0.26041D-05, 0.25859D-05,
     & 0.25693D-05, 0.25542D-05, 0.25404D-05, 0.25279D-05, 0.25166D-05,
     & 0.33696D-05, 0.33790D-05, 0.33775D-05, 0.33671D-05, 0.33500D-05,
     & 0.33278D-05, 0.33018D-05, 0.32732D-05, 0.32431D-05, 0.32121D-05,
     & 0.31810D-05, 0.31502D-05, 0.31202D-05, 0.30913D-05, 0.30637D-05,
     & 0.30375D-05, 0.30128D-05, 0.29897D-05, 0.29683D-05, 0.29484D-05,
     & 0.29301D-05, 0.29132D-05, 0.28978D-05, 0.28838D-05, 0.28710D-05,
     & 0.28594D-05, 0.28489D-05, 0.28394D-05, 0.28309D-05, 0.28232D-05,
     & 0.32954D-05, 0.33352D-05, 0.33609D-05, 0.33749D-05, 0.33796D-05,
     & 0.33768D-05, 0.33682D-05, 0.33554D-05, 0.33394D-05, 0.33213D-05,
     & 0.33018D-05, 0.32816D-05, 0.32613D-05, 0.32411D-05, 0.32215D-05,
     & 0.32026D-05, 0.31847D-05, 0.31678D-05, 0.31520D-05, 0.31372D-05,
     & 0.31236D-05, 0.31111D-05, 0.30996D-05, 0.30891D-05, 0.30796D-05,
     & 0.30709D-05, 0.30631D-05, 0.30560D-05, 0.30496D-05, 0.30439D-05,
     & 0.31420D-05, 0.32154D-05, 0.32718D-05, 0.33136D-05, 0.33432D-05,
     & 0.33628D-05, 0.33742D-05, 0.33791D-05, 0.33790D-05, 0.33750D-05,
     & 0.33682D-05, 0.33594D-05, 0.33493D-05, 0.33383D-05, 0.33269D-05,
     & 0.33155D-05, 0.33042D-05, 0.32933D-05, 0.32828D-05, 0.32729D-05,
     & 0.32636D-05, 0.32549D-05, 0.32469D-05, 0.32396D-05, 0.32328D-05,
     & 0.32267D-05, 0.32211D-05, 0.32160D-05, 0.32114D-05, 0.32073D-05,
     & 0.29032D-05, 0.30125D-05, 0.31020D-05, 0.31742D-05, 0.32317D-05,
     & 0.32765D-05, 0.33107D-05, 0.33360D-05, 0.33542D-05, 0.33665D-05,
     & 0.33742D-05, 0.33783D-05, 0.33796D-05, 0.33789D-05, 0.33766D-05,
     & 0.33732D-05, 0.33691D-05, 0.33647D-05, 0.33600D-05, 0.33552D-05,
     & 0.33505D-05, 0.33459D-05, 0.33416D-05, 0.33374D-05, 0.33336D-05,
     & 0.33300D-05, 0.33267D-05, 0.33237D-05, 0.33209D-05, 0.33184D-05,
     & 0.25769D-05, 0.27220D-05, 0.28459D-05, 0.29505D-05, 0.30377D-05,
     & 0.31099D-05, 0.31690D-05, 0.32170D-05, 0.32557D-05, 0.32865D-05,
     & 0.33107D-05, 0.33296D-05, 0.33441D-05, 0.33551D-05, 0.33633D-05,
     & 0.33693D-05, 0.33735D-05, 0.33763D-05, 0.33781D-05, 0.33792D-05,
     & 0.33796D-05, 0.33796D-05, 0.33793D-05, 0.33787D-05, 0.33781D-05,
     & 0.33773D-05, 0.33765D-05, 0.33757D-05, 0.33749D-05, 0.33741D-05,
     & 0.21692D-05, 0.23466D-05, 0.25030D-05, 0.26389D-05, 0.27565D-05,
     & 0.28571D-05, 0.29427D-05, 0.30150D-05, 0.30758D-05, 0.31266D-05,
     & 0.31690D-05, 0.32043D-05, 0.32336D-05, 0.32579D-05, 0.32779D-05,
     & 0.32945D-05, 0.33081D-05, 0.33193D-05, 0.33286D-05, 0.33363D-05,
     & 0.33426D-05, 0.33479D-05, 0.33522D-05, 0.33558D-05, 0.33588D-05,
     & 0.33613D-05, 0.33634D-05, 0.33652D-05, 0.33667D-05, 0.33679D-05,
     & 0.16990D-05, 0.18989D-05, 0.20808D-05, 0.22444D-05, 0.23896D-05,
     & 0.25174D-05, 0.26287D-05, 0.27254D-05, 0.28091D-05, 0.28810D-05,
     & 0.29427D-05, 0.29956D-05, 0.30407D-05, 0.30793D-05, 0.31122D-05,
     & 0.31403D-05, 0.31643D-05, 0.31848D-05, 0.32025D-05, 0.32176D-05,
     & 0.32305D-05, 0.32417D-05, 0.32513D-05, 0.32596D-05, 0.32667D-05,
     & 0.32729D-05, 0.32783D-05, 0.32830D-05, 0.32870D-05, 0.32906D-05,
     & 0.30531D-05, 0.28863D-05, 0.16017D-05, 0.17829D-05, 0.19484D-05,
     & 0.20979D-05, 0.22319D-05, 0.23509D-05, 0.24560D-05, 0.25482D-05,
     & 0.26287D-05, 0.26991D-05, 0.27605D-05, 0.28140D-05, 0.28604D-05,
     & 0.29008D-05, 0.29358D-05, 0.29663D-05, 0.29928D-05, 0.30158D-05,
     & 0.30359D-05, 0.30534D-05, 0.30687D-05, 0.30820D-05, 0.30937D-05,
     & 0.31039D-05, 0.31128D-05, 0.31207D-05, 0.31276D-05, 0.31336D-05/

      data ((c_grdl(i,j),j=1,30),i=21,30)/
     & 0.23190D-05, 0.27357D-05, 0.29983D-05, 0.30407D-05, 0.27429D-05,
     & 0.16204D-05, 0.17689D-05, 0.19038D-05, 0.20254D-05, 0.21346D-05,
     & 0.22319D-05, 0.23183D-05, 0.23948D-05, 0.24622D-05, 0.25216D-05,
     & 0.25737D-05, 0.26196D-05, 0.26599D-05, 0.26954D-05, 0.27265D-05,
     & 0.27539D-05, 0.27780D-05, 0.27992D-05, 0.28178D-05, 0.28342D-05,
     & 0.28486D-05, 0.28613D-05, 0.28725D-05, 0.28824D-05, 0.28911D-05,
     & 0.25745D-05, 0.26373D-05, 0.25793D-05, 0.25043D-05, 0.28219D-05,
     & 0.30136D-05, 0.30474D-05, 0.28747D-05, 0.23139D-05, 0.16608D-05,
     & 0.17689D-05, 0.18666D-05, 0.19543D-05, 0.20327D-05, 0.21029D-05,
     & 0.21653D-05, 0.22208D-05, 0.22700D-05, 0.23136D-05, 0.23523D-05,
     & 0.23865D-05, 0.24167D-05, 0.24435D-05, 0.24671D-05, 0.24880D-05,
     & 0.25065D-05, 0.25228D-05, 0.25372D-05, 0.25499D-05, 0.25612D-05,
     & 0.88133D-06, 0.17490D-05, 0.25062D-05, 0.25842D-05, 0.25532D-05,
     & 0.25186D-05, 0.24742D-05, 0.27447D-05, 0.29359D-05, 0.30392D-05,
     & 0.30473D-05, 0.29503D-05, 0.27210D-05, 0.22447D-05, 0.16259D-05,
     & 0.16947D-05, 0.17565D-05, 0.18118D-05, 0.18613D-05, 0.19054D-05,
     & 0.19447D-05, 0.19797D-05, 0.20108D-05, 0.20384D-05, 0.20630D-05,
     & 0.20849D-05, 0.21043D-05, 0.21215D-05, 0.21367D-05, 0.21503D-05,
     & 0.53557D-09, 0.12340D-06, 0.54952D-06, 0.12225D-05, 0.19789D-05,
     & 0.25498D-05, 0.26236D-05, 0.26377D-05, 0.26329D-05, 0.22314D-05,
     & 0.24743D-05, 0.26749D-05, 0.28316D-05, 0.29451D-05, 0.30177D-05,
     & 0.30521D-05, 0.30517D-05, 0.30198D-05, 0.29590D-05, 0.28710D-05,
     & 0.27560D-05, 0.26116D-05, 0.24295D-05, 0.21830D-05, 0.15824D-05,
     & 0.16062D-05, 0.16274D-05, 0.16463D-05, 0.16631D-05, 0.16780D-05,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.18356D-07, 0.20570D-06,
     & 0.60737D-06, 0.11626D-05, 0.17721D-05, 0.23138D-05, 0.25912D-05,
     & 0.26235D-05, 0.26108D-05, 0.25630D-05, 0.26333D-05, 0.24923D-05,
     & 0.23093D-05, 0.24472D-05, 0.25651D-05, 0.26646D-05, 0.27475D-05,
     & 0.28158D-05, 0.28715D-05, 0.29164D-05, 0.29522D-05, 0.29803D-05,
     & 0.30021D-05, 0.30188D-05, 0.30313D-05, 0.30404D-05, 0.30468D-05,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.50389D-07,
     & 0.28817D-06, 0.70734D-06, 0.12456D-05, 0.18142D-05, 0.23120D-05,
     & 0.25912D-05, 0.26357D-05, 0.25678D-05, 0.26273D-05, 0.26132D-05,
     & 0.26210D-05, 0.24042D-05, 0.23086D-05, 0.24215D-05, 0.25190D-05,
     & 0.26025D-05, 0.26735D-05, 0.27337D-05, 0.27843D-05, 0.28269D-05,
     & 0.28626D-05, 0.28925D-05, 0.29174D-05, 0.29381D-05, 0.29554D-05,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.22845D-08,
     & 0.95710D-07, 0.36883D-06, 0.79233D-06, 0.13052D-05, 0.18321D-05,
     & 0.22915D-05, 0.25771D-05, 0.26207D-05, 0.25036D-05, 0.26288D-05,
     & 0.25545D-05, 0.26259D-05, 0.26167D-05, 0.24637D-05, 0.22677D-05,
     & 0.23605D-05, 0.24415D-05, 0.25119D-05, 0.25727D-05, 0.26252D-05,
     & 0.26704D-05, 0.27094D-05, 0.27428D-05, 0.27717D-05, 0.27965D-05,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.13373D-07, 0.14810D-06, 0.44226D-06, 0.85840D-06, 0.13405D-05,
     & 0.18262D-05, 0.22518D-05, 0.25432D-05, 0.25779D-05, 0.26096D-05,
     & 0.25737D-05, 0.26377D-05, 0.25675D-05, 0.26239D-05, 0.26283D-05,
     & 0.25652D-05, 0.23211D-05, 0.22694D-05, 0.23366D-05, 0.23954D-05,
     & 0.24470D-05, 0.24921D-05, 0.25314D-05, 0.25658D-05, 0.25958D-05,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.34345D-07, 0.20143D-06, 0.50387D-06, 0.90370D-06,
     & 0.13517D-05, 0.17973D-05, 0.21921D-05, 0.24857D-05, 0.25760D-05,
     & 0.26311D-05, 0.24808D-05, 0.25938D-05, 0.26377D-05, 0.25519D-05,
     & 0.26073D-05, 0.26320D-05, 0.26252D-05, 0.25827D-05, 0.24870D-05,
     & 0.22078D-05, 0.22566D-05, 0.22997D-05, 0.23376D-05, 0.23709D-05,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.12845D-08, 0.62364D-07, 0.25040D-06, 0.55031D-06,
     & 0.92727D-06, 0.13392D-05, 0.17461D-05, 0.21118D-05, 0.24024D-05,
     & 0.25770D-05, 0.25734D-05, 0.26355D-05, 0.24910D-05, 0.25838D-05,
     & 0.26327D-05, 0.26213D-05, 0.25650D-05, 0.26038D-05, 0.26261D-05,
     & 0.26333D-05, 0.26266D-05, 0.26066D-05, 0.25728D-05, 0.25231D-05/

      data ((c_grdl(i,j),j=1,30),i=31,40)/
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.72516D-08, 0.92381D-07, 0.29070D-06,
     & 0.57961D-06, 0.92922D-06, 0.13044D-05, 0.16738D-05, 0.20110D-05,
     & 0.22930D-05, 0.24986D-05, 0.25925D-05, 0.25882D-05, 0.26358D-05,
     & 0.25026D-05, 0.25442D-05, 0.26017D-05, 0.26328D-05, 0.26339D-05,
     & 0.25635D-05, 0.25671D-05, 0.25930D-05, 0.26114D-05, 0.26236D-05,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.17676D-07, 0.12018D-06,
     & 0.31915D-06, 0.59108D-06, 0.91052D-06, 0.12489D-05, 0.15820D-05,
     & 0.18904D-05, 0.21586D-05, 0.23742D-05, 0.25261D-05, 0.25925D-05,
     & 0.25697D-05, 0.26268D-05, 0.26277D-05, 0.24685D-05, 0.25294D-05,
     & 0.25754D-05, 0.26079D-05, 0.26283D-05, 0.26373D-05, 0.26345D-05,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.18373D-09, 0.30361D-07,
     & 0.14227D-06, 0.33453D-06, 0.58507D-06, 0.87285D-06, 0.11750D-05,
     & 0.14727D-05, 0.17518D-05, 0.20016D-05, 0.22144D-05, 0.23851D-05,
     & 0.25100D-05, 0.25822D-05, 0.25622D-05, 0.25821D-05, 0.26226D-05,
     & 0.26357D-05, 0.26154D-05, 0.25015D-05, 0.25008D-05, 0.25336D-05,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.18017D-08,
     & 0.42605D-07, 0.15658D-06, 0.33658D-06, 0.56285D-06, 0.81848D-06,
     & 0.10852D-05, 0.13486D-05, 0.15981D-05, 0.18260D-05, 0.20275D-05,
     & 0.21995D-05, 0.23410D-05, 0.24518D-05, 0.25321D-05, 0.25806D-05,
     & 0.25899D-05, 0.25406D-05, 0.25813D-05, 0.26097D-05, 0.26274D-05,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.46955D-08, 0.52036D-07, 0.16209D-06, 0.32596D-06, 0.52640D-06,
     & 0.75001D-06, 0.98247D-06, 0.12129D-05, 0.14327D-05, 0.16367D-05,
     & 0.18212D-05, 0.19844D-05, 0.21258D-05, 0.22458D-05, 0.23454D-05,
     & 0.24262D-05, 0.24896D-05, 0.25370D-05, 0.25697D-05, 0.25883D-05,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.77974D-08, 0.57132D-07, 0.15881D-06, 0.30406D-06,
     & 0.47826D-06, 0.67066D-06, 0.87059D-06, 0.10691D-05, 0.12599D-05,
     & 0.14388D-05, 0.16032D-05, 0.17518D-05, 0.18841D-05, 0.20006D-05,
     & 0.21019D-05, 0.21893D-05, 0.22639D-05, 0.23272D-05, 0.23804D-05,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.10046D-07, 0.57370D-07, 0.14762D-06,
     & 0.27286D-06, 0.42137D-06, 0.58403D-06, 0.75311D-06, 0.92143D-06,
     & 0.10843D-05, 0.12382D-05, 0.13810D-05, 0.15119D-05, 0.16304D-05,
     & 0.17367D-05, 0.18314D-05, 0.19151D-05, 0.19888D-05, 0.20535D-05,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.34160D-10, 0.10791D-07, 0.53035D-07,
     & 0.13004D-06, 0.23485D-06, 0.35891D-06, 0.49376D-06, 0.63387D-06,
     & 0.77421D-06, 0.91037D-06, 0.10400D-05, 0.11615D-05, 0.12735D-05,
     & 0.13760D-05, 0.14691D-05, 0.15530D-05, 0.16283D-05, 0.16955D-05,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.92776D-10, 0.99134D-08,
     & 0.45065D-07, 0.10805D-06, 0.19305D-06, 0.29403D-06, 0.40346D-06,
     & 0.51714D-06, 0.63131D-06, 0.74299D-06, 0.84980D-06, 0.95031D-06,
     & 0.10439D-05, 0.11302D-05, 0.12089D-05, 0.12805D-05, 0.13452D-05,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.77970D-10,
     & 0.77712D-08, 0.34864D-07, 0.83838D-07, 0.15034D-06, 0.22949D-06,
     & 0.31655D-06, 0.40659D-06, 0.49727D-06, 0.58632D-06, 0.67208D-06,
     & 0.75339D-06, 0.82940D-06, 0.89972D-06, 0.96440D-06, 0.10235D-05/

      data ((c_grdl(i,j),j=1,30),i=41,50)/
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.15536D-10, 0.50342D-08, 0.24105D-07, 0.59815D-07, 0.10952D-06,
     & 0.16919D-06, 0.23555D-06, 0.30538D-06, 0.37548D-06, 0.44455D-06,
     & 0.51136D-06, 0.57499D-06, 0.63486D-06, 0.69064D-06, 0.74219D-06,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.24693D-08, 0.14336D-07, 0.38098D-07,
     & 0.72919D-07, 0.11590D-06, 0.16413D-06, 0.21557D-06, 0.26833D-06,
     & 0.32052D-06, 0.37093D-06, 0.41915D-06, 0.46471D-06, 0.50733D-06,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.70775D-09, 0.67723D-08,
     & 0.20763D-07, 0.42817D-07, 0.71509D-07, 0.10473D-06, 0.14065D-06,
     & 0.17790D-06, 0.21537D-06, 0.25223D-06, 0.28780D-06, 0.32130D-06,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.22002D-10,
     & 0.20745D-08, 0.88241D-08, 0.20873D-07, 0.37809D-07, 0.58644D-07,
     & 0.81936D-07, 0.10665D-06, 0.13183D-06, 0.15684D-06, 0.18125D-06,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.16734D-09, 0.22738D-08, 0.74253D-08, 0.15708D-07,
     & 0.26746D-07, 0.39962D-07, 0.54675D-07, 0.70129D-07, 0.85815D-07,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.89582D-10, 0.12617D-08,
     & 0.40877D-08, 0.85877D-08, 0.14550D-07, 0.21664D-07, 0.29598D-07,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.21863D-09, 0.11389D-08, 0.28364D-08, 0.52465D-08,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.13090D-10,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00/
      
      DO I=1,50
        DO J=1,30
          UDS_GRD(I,J) = UDS_GRDL(I,J)
          G_GRD(I,J)   = G_GRDL(I,J)
          C_GRD(I,J)   = C_GRDL(I,J)
        ENDDO
      ENDDO

      RETURN
      END

      SUBROUTINE I_LO_Q3G1
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      PARAMETER (NQ2MAX=30,NZMAX=50)
      DIMENSION UDS_GRD(NZMAX,NQ2MAX),C_GRD(NZMAX,NQ2MAX),
     &          G_GRD(NZMAX,NQ2MAX)
      DIMENSION UDS_GRDL(NZMAX,NQ2MAX),C_GRDL(NZMAX,NQ2MAX),
     &          G_GRDL(NZMAX,NQ2MAX)
      COMMON /CPARAM/ UDS_GRD,C_GRD,G_GRD 
      data ((uds_grdl(i,j),j=1,30),i=1,10)/
     & 0.49423D-03, 0.17306D-02, 0.30223D-02, 0.43643D-02, 0.57517D-02,
     & 0.71793D-02, 0.86382D-02, 0.10135D-01, 0.11665D-01, 0.13227D-01,
     & 0.14818D-01, 0.16431D-01, 0.18066D-01, 0.19725D-01, 0.21405D-01,
     & 0.23104D-01, 0.24821D-01, 0.26554D-01, 0.28298D-01, 0.30057D-01,
     & 0.31829D-01, 0.33611D-01, 0.35404D-01, 0.37206D-01, 0.39016D-01,
     & 0.40834D-01, 0.42657D-01, 0.44487D-01, 0.46321D-01, 0.48160D-01,
     & 0.52294D-03, 0.17560D-02, 0.30382D-02, 0.43650D-02, 0.57320D-02,
     & 0.71347D-02, 0.85659D-02, 0.10030D-01, 0.11523D-01, 0.13043D-01,
     & 0.14587D-01, 0.16151D-01, 0.17735D-01, 0.19337D-01, 0.20957D-01,
     & 0.22593D-01, 0.24243D-01, 0.25906D-01, 0.27580D-01, 0.29264D-01,
     & 0.30957D-01, 0.32659D-01, 0.34369D-01, 0.36084D-01, 0.37806D-01,
     & 0.39534D-01, 0.41266D-01, 0.43001D-01, 0.44738D-01, 0.46478D-01,
     & 0.55769D-03, 0.17869D-02, 0.30594D-02, 0.43708D-02, 0.57175D-02,
     & 0.70954D-02, 0.84989D-02, 0.99301D-02, 0.11386D-01, 0.12865D-01,
     & 0.14364D-01, 0.15880D-01, 0.17412D-01, 0.18960D-01, 0.20521D-01,
     & 0.22095D-01, 0.23680D-01, 0.25275D-01, 0.26879D-01, 0.28491D-01,
     & 0.30109D-01, 0.31732D-01, 0.33361D-01, 0.34994D-01, 0.36630D-01,
     & 0.38271D-01, 0.39914D-01, 0.41558D-01, 0.43203D-01, 0.44847D-01,
     & 0.59992D-03, 0.18249D-02, 0.30874D-02, 0.43834D-02, 0.57098D-02,
     & 0.70630D-02, 0.84390D-02, 0.98379D-02, 0.11257D-01, 0.12695D-01,
     & 0.14150D-01, 0.15619D-01, 0.17101D-01, 0.18595D-01, 0.20100D-01,
     & 0.21613D-01, 0.23135D-01, 0.24664D-01, 0.26201D-01, 0.27742D-01,
     & 0.29287D-01, 0.30835D-01, 0.32385D-01, 0.33938D-01, 0.35492D-01,
     & 0.37050D-01, 0.38607D-01, 0.40164D-01, 0.41719D-01, 0.43272D-01,
     & 0.65136D-03, 0.18714D-02, 0.31233D-02, 0.44035D-02, 0.57092D-02,
     & 0.70375D-02, 0.83858D-02, 0.97523D-02, 0.11135D-01, 0.12533D-01,
     & 0.13943D-01, 0.15366D-01, 0.16798D-01, 0.18239D-01, 0.19688D-01,
     & 0.21142D-01, 0.22602D-01, 0.24067D-01, 0.25537D-01, 0.27009D-01,
     & 0.28483D-01, 0.29957D-01, 0.31432D-01, 0.32907D-01, 0.34381D-01,
     & 0.35858D-01, 0.37333D-01, 0.38804D-01, 0.40273D-01, 0.41738D-01,
     & 0.71430D-03, 0.19287D-02, 0.31692D-02, 0.44332D-02, 0.57178D-02,
     & 0.70209D-02, 0.83410D-02, 0.96751D-02, 0.11022D-01, 0.12379D-01,
     & 0.13745D-01, 0.15122D-01, 0.16505D-01, 0.17894D-01, 0.19287D-01,
     & 0.20683D-01, 0.22082D-01, 0.23484D-01, 0.24890D-01, 0.26295D-01,
     & 0.27699D-01, 0.29101D-01, 0.30502D-01, 0.31901D-01, 0.33298D-01,
     & 0.34697D-01, 0.36091D-01, 0.37481D-01, 0.38866D-01, 0.40247D-01,
     & 0.79165D-03, 0.19994D-02, 0.32280D-02, 0.44750D-02, 0.57382D-02,
     & 0.70158D-02, 0.83074D-02, 0.96088D-02, 0.10919D-01, 0.12236D-01,
     & 0.13559D-01, 0.14890D-01, 0.16225D-01, 0.17562D-01, 0.18900D-01,
     & 0.20239D-01, 0.21579D-01, 0.22919D-01, 0.24261D-01, 0.25600D-01,
     & 0.26936D-01, 0.28269D-01, 0.29598D-01, 0.30924D-01, 0.32246D-01,
     & 0.33568D-01, 0.34885D-01, 0.36195D-01, 0.37500D-01, 0.38798D-01,
     & 0.88719D-03, 0.20875D-02, 0.33035D-02, 0.45331D-02, 0.57744D-02,
     & 0.70263D-02, 0.82891D-02, 0.95579D-02, 0.10832D-01, 0.12109D-01,
     & 0.13389D-01, 0.14674D-01, 0.15962D-01, 0.17248D-01, 0.18533D-01,
     & 0.19816D-01, 0.21097D-01, 0.22377D-01, 0.23657D-01, 0.24933D-01,
     & 0.26203D-01, 0.27468D-01, 0.28728D-01, 0.29982D-01, 0.31232D-01,
     & 0.32480D-01, 0.33722D-01, 0.34956D-01, 0.36183D-01, 0.37403D-01,
     & 0.10057D-02, 0.21977D-02, 0.34004D-02, 0.46120D-02, 0.58312D-02,
     & 0.70570D-02, 0.82909D-02, 0.95270D-02, 0.10764D-01, 0.12003D-01,
     & 0.13241D-01, 0.14481D-01, 0.15721D-01, 0.16957D-01, 0.18190D-01,
     & 0.19418D-01, 0.20643D-01, 0.21864D-01, 0.23084D-01, 0.24297D-01,
     & 0.25503D-01, 0.26703D-01, 0.27896D-01, 0.29081D-01, 0.30261D-01,
     & 0.31439D-01, 0.32608D-01, 0.33769D-01, 0.34921D-01, 0.36065D-01,
     & 0.11533D-02, 0.23357D-02, 0.35239D-02, 0.47166D-02, 0.59127D-02,
     & 0.71118D-02, 0.83159D-02, 0.95188D-02, 0.10720D-01, 0.11918D-01,
     & 0.13114D-01, 0.14309D-01, 0.15502D-01, 0.16689D-01, 0.17869D-01,
     & 0.19044D-01, 0.20212D-01, 0.21376D-01, 0.22536D-01, 0.23688D-01,
     & 0.24832D-01, 0.25967D-01, 0.27094D-01, 0.28213D-01, 0.29325D-01,
     & 0.30433D-01, 0.31533D-01, 0.32622D-01, 0.33702D-01, 0.34773D-01/

      data ((uds_grdl(i,j),j=1,30),i=11,20)/
     & 0.13379D-02, 0.25092D-02, 0.36816D-02, 0.48543D-02, 0.60264D-02,
     & 0.71979D-02, 0.83716D-02, 0.95405D-02, 0.10704D-01, 0.11863D-01,
     & 0.13016D-01, 0.14167D-01, 0.15312D-01, 0.16450D-01, 0.17579D-01,
     & 0.18700D-01, 0.19813D-01, 0.20920D-01, 0.22022D-01, 0.23114D-01,
     & 0.24196D-01, 0.25269D-01, 0.26332D-01, 0.27386D-01, 0.28431D-01,
     & 0.29473D-01, 0.30504D-01, 0.31524D-01, 0.32534D-01, 0.33534D-01,
     & 0.15697D-02, 0.27285D-02, 0.38841D-02, 0.50357D-02, 0.61830D-02,
     & 0.73264D-02, 0.84690D-02, 0.96037D-02, 0.10730D-01, 0.11849D-01,
     & 0.12960D-01, 0.14066D-01, 0.15165D-01, 0.16254D-01, 0.17333D-01,
     & 0.18401D-01, 0.19460D-01, 0.20511D-01, 0.21556D-01, 0.22590D-01,
     & 0.23613D-01, 0.24625D-01, 0.25626D-01, 0.26617D-01, 0.27598D-01,
     & 0.28574D-01, 0.29540D-01, 0.30493D-01, 0.31437D-01, 0.32369D-01,
     & 0.18615D-02, 0.30060D-02, 0.41428D-02, 0.52719D-02, 0.63931D-02,
     & 0.75070D-02, 0.86173D-02, 0.97167D-02, 0.10806D-01, 0.11884D-01,
     & 0.12952D-01, 0.14013D-01, 0.15065D-01, 0.16105D-01, 0.17133D-01,
     & 0.18150D-01, 0.19155D-01, 0.20151D-01, 0.21140D-01, 0.22116D-01,
     & 0.23080D-01, 0.24032D-01, 0.24972D-01, 0.25901D-01, 0.26820D-01,
     & 0.27733D-01, 0.28634D-01, 0.29523D-01, 0.30401D-01, 0.31268D-01,
     & 0.22298D-02, 0.33581D-02, 0.44749D-02, 0.55802D-02, 0.66742D-02,
     & 0.77578D-02, 0.88350D-02, 0.98987D-02, 0.10949D-01, 0.11987D-01,
     & 0.13012D-01, 0.14029D-01, 0.15034D-01, 0.16026D-01, 0.17004D-01,
     & 0.17969D-01, 0.18922D-01, 0.19863D-01, 0.20797D-01, 0.21717D-01,
     & 0.22623D-01, 0.23517D-01, 0.24399D-01, 0.25268D-01, 0.26126D-01,
     & 0.26978D-01, 0.27817D-01, 0.28644D-01, 0.29459D-01, 0.30262D-01,
     & 0.26946D-02, 0.38048D-02, 0.48997D-02, 0.59796D-02, 0.70450D-02,
     & 0.80971D-02, 0.91400D-02, 0.10167D-01, 0.11178D-01, 0.12175D-01,
     & 0.13157D-01, 0.14128D-01, 0.15087D-01, 0.16031D-01, 0.16959D-01,
     & 0.17873D-01, 0.18774D-01, 0.19662D-01, 0.20541D-01, 0.21405D-01,
     & 0.22256D-01, 0.23092D-01, 0.23916D-01, 0.24727D-01, 0.25527D-01,
     & 0.26319D-01, 0.27097D-01, 0.27864D-01, 0.28618D-01, 0.29360D-01,
     & 0.32820D-02, 0.43705D-02, 0.54401D-02, 0.64915D-02, 0.75254D-02,
     & 0.85433D-02, 0.95494D-02, 0.10537D-01, 0.11507D-01, 0.12461D-01,
     & 0.13398D-01, 0.14323D-01, 0.15233D-01, 0.16127D-01, 0.17005D-01,
     & 0.17867D-01, 0.18715D-01, 0.19549D-01, 0.20373D-01, 0.21181D-01,
     & 0.21975D-01, 0.22755D-01, 0.23521D-01, 0.24274D-01, 0.25015D-01,
     & 0.25748D-01, 0.26467D-01, 0.27173D-01, 0.27868D-01, 0.28550D-01,
     & 0.40218D-02, 0.50852D-02, 0.61265D-02, 0.71466D-02, 0.81465D-02,
     & 0.91278D-02, 0.10095D-01, 0.11042D-01, 0.11969D-01, 0.12877D-01,
     & 0.13768D-01, 0.14645D-01, 0.15506D-01, 0.16350D-01, 0.17176D-01,
     & 0.17986D-01, 0.18780D-01, 0.19560D-01, 0.20329D-01, 0.21082D-01,
     & 0.21819D-01, 0.22542D-01, 0.23252D-01, 0.23947D-01, 0.24630D-01,
     & 0.25304D-01, 0.25965D-01, 0.26613D-01, 0.27248D-01, 0.27871D-01,
     & 0.49496D-02, 0.59836D-02, 0.69925D-02, 0.79775D-02, 0.89398D-02,
     & 0.98811D-02, 0.10806D-01, 0.11709D-01, 0.12590D-01, 0.13452D-01,
     & 0.14294D-01, 0.15121D-01, 0.15931D-01, 0.16722D-01, 0.17495D-01,
     & 0.18251D-01, 0.18991D-01, 0.19716D-01, 0.20428D-01, 0.21125D-01,
     & 0.21806D-01, 0.22472D-01, 0.23124D-01, 0.23762D-01, 0.24387D-01,
     & 0.25002D-01, 0.25604D-01, 0.26194D-01, 0.26771D-01, 0.27335D-01,
     & 0.61045D-02, 0.71036D-02, 0.80749D-02, 0.90199D-02, 0.99401D-02,
     & 0.10837D-01, 0.11716D-01, 0.12570D-01, 0.13403D-01, 0.14214D-01,
     & 0.15004D-01, 0.15778D-01, 0.16534D-01, 0.17271D-01, 0.17989D-01,
     & 0.18689D-01, 0.19372D-01, 0.20040D-01, 0.20695D-01, 0.21333D-01,
     & 0.21956D-01, 0.22564D-01, 0.23157D-01, 0.23737D-01, 0.24303D-01,
     & 0.24860D-01, 0.25403D-01, 0.25933D-01, 0.26452D-01, 0.26958D-01,
     & 0.75259D-02, 0.84829D-02, 0.94099D-02, 0.10309D-01, 0.11180D-01,
     & 0.12027D-01, 0.12854D-01, 0.13655D-01, 0.14433D-01, 0.15189D-01,
     & 0.15924D-01, 0.16640D-01, 0.17338D-01, 0.18016D-01, 0.18675D-01,
     & 0.19316D-01, 0.19940D-01, 0.20547D-01, 0.21142D-01, 0.21719D-01,
     & 0.22282D-01, 0.22829D-01, 0.23362D-01, 0.23881D-01, 0.24387D-01,
     & 0.24883D-01, 0.25366D-01, 0.25836D-01, 0.26295D-01, 0.26741D-01/

      data ((uds_grdl(i,j),j=1,30),i=21,30)/
     & 0.92464D-02, 0.10152D-01, 0.11026D-01, 0.11869D-01, 0.12685D-01,
     & 0.13474D-01, 0.14241D-01, 0.14982D-01, 0.15699D-01, 0.16393D-01,
     & 0.17065D-01, 0.17718D-01, 0.18352D-01, 0.18966D-01, 0.19561D-01,
     & 0.20137D-01, 0.20697D-01, 0.21240D-01, 0.21769D-01, 0.22282D-01,
     & 0.22780D-01, 0.23263D-01, 0.23732D-01, 0.24188D-01, 0.24631D-01,
     & 0.25063D-01, 0.25482D-01, 0.25890D-01, 0.26286D-01, 0.26671D-01,
     & 0.11281D-01, 0.12122D-01, 0.12929D-01, 0.13705D-01, 0.14453D-01,
     & 0.15172D-01, 0.15868D-01, 0.16538D-01, 0.17184D-01, 0.17806D-01,
     & 0.18406D-01, 0.18987D-01, 0.19548D-01, 0.20090D-01, 0.20613D-01,
     & 0.21117D-01, 0.21605D-01, 0.22077D-01, 0.22534D-01, 0.22976D-01,
     & 0.23403D-01, 0.23816D-01, 0.24215D-01, 0.24601D-01, 0.24975D-01,
     & 0.25339D-01, 0.25690D-01, 0.26030D-01, 0.26359D-01, 0.26678D-01,
     & 0.13598D-01, 0.14358D-01, 0.15084D-01, 0.15778D-01, 0.16443D-01,
     & 0.17080D-01, 0.17692D-01, 0.18279D-01, 0.18841D-01, 0.19380D-01,
     & 0.19898D-01, 0.20395D-01, 0.20874D-01, 0.21333D-01, 0.21774D-01,
     & 0.22198D-01, 0.22605D-01, 0.22997D-01, 0.23375D-01, 0.23738D-01,
     & 0.24087D-01, 0.24422D-01, 0.24746D-01, 0.25056D-01, 0.25356D-01,
     & 0.25644D-01, 0.25922D-01, 0.26190D-01, 0.26447D-01, 0.26695D-01,
     & 0.16096D-01, 0.16754D-01, 0.17379D-01, 0.17972D-01, 0.18536D-01,
     & 0.19073D-01, 0.19584D-01, 0.20071D-01, 0.20534D-01, 0.20975D-01,
     & 0.21395D-01, 0.21796D-01, 0.22178D-01, 0.22542D-01, 0.22889D-01,
     & 0.23220D-01, 0.23536D-01, 0.23837D-01, 0.24125D-01, 0.24399D-01,
     & 0.24661D-01, 0.24911D-01, 0.25149D-01, 0.25376D-01, 0.25593D-01,
     & 0.25800D-01, 0.25997D-01, 0.26185D-01, 0.26364D-01, 0.26535D-01,
     & 0.18549D-01, 0.19077D-01, 0.19573D-01, 0.20039D-01, 0.20477D-01,
     & 0.20890D-01, 0.21277D-01, 0.21641D-01, 0.21984D-01, 0.22307D-01,
     & 0.22611D-01, 0.22896D-01, 0.23165D-01, 0.23417D-01, 0.23655D-01,
     & 0.23878D-01, 0.24088D-01, 0.24285D-01, 0.24470D-01, 0.24643D-01,
     & 0.24805D-01, 0.24958D-01, 0.25101D-01, 0.25235D-01, 0.25360D-01,
     & 0.25475D-01, 0.25583D-01, 0.25684D-01, 0.25778D-01, 0.25865D-01,
     & 0.19092D-01, 0.19583D-01, 0.20044D-01, 0.20475D-01, 0.20878D-01,
     & 0.21257D-01, 0.21610D-01, 0.21942D-01, 0.22253D-01, 0.22544D-01,
     & 0.22816D-01, 0.23071D-01, 0.23309D-01, 0.23532D-01, 0.23741D-01,
     & 0.23936D-01, 0.24119D-01, 0.24289D-01, 0.24446D-01, 0.24593D-01,
     & 0.24730D-01, 0.24858D-01, 0.24977D-01, 0.25087D-01, 0.25188D-01,
     & 0.25281D-01, 0.25366D-01, 0.25445D-01, 0.25517D-01, 0.25583D-01,
     & 0.19582D-01, 0.20037D-01, 0.20461D-01, 0.20857D-01, 0.21226D-01,
     & 0.21569D-01, 0.21889D-01, 0.22188D-01, 0.22466D-01, 0.22726D-01,
     & 0.22967D-01, 0.23191D-01, 0.23399D-01, 0.23593D-01, 0.23773D-01,
     & 0.23941D-01, 0.24096D-01, 0.24239D-01, 0.24370D-01, 0.24492D-01,
     & 0.24604D-01, 0.24707D-01, 0.24801D-01, 0.24888D-01, 0.24967D-01,
     & 0.25037D-01, 0.25100D-01, 0.25157D-01, 0.25209D-01, 0.25254D-01,
     & 0.20019D-01, 0.20436D-01, 0.20824D-01, 0.21183D-01, 0.21517D-01,
     & 0.21826D-01, 0.22112D-01, 0.22377D-01, 0.22623D-01, 0.22851D-01,
     & 0.23062D-01, 0.23255D-01, 0.23434D-01, 0.23598D-01, 0.23750D-01,
     & 0.23890D-01, 0.24018D-01, 0.24134D-01, 0.24240D-01, 0.24336D-01,
     & 0.24423D-01, 0.24502D-01, 0.24573D-01, 0.24637D-01, 0.24693D-01,
     & 0.24741D-01, 0.24783D-01, 0.24820D-01, 0.24851D-01, 0.24877D-01,
     & 0.20399D-01, 0.20778D-01, 0.21129D-01, 0.21452D-01, 0.21750D-01,
     & 0.22025D-01, 0.22276D-01, 0.22508D-01, 0.22722D-01, 0.22918D-01,
     & 0.23098D-01, 0.23261D-01, 0.23409D-01, 0.23545D-01, 0.23669D-01,
     & 0.23781D-01, 0.23883D-01, 0.23973D-01, 0.24053D-01, 0.24124D-01,
     & 0.24187D-01, 0.24243D-01, 0.24291D-01, 0.24333D-01, 0.24367D-01,
     & 0.24394D-01, 0.24415D-01, 0.24431D-01, 0.24443D-01, 0.24450D-01,
     & 0.20722D-01, 0.21062D-01, 0.21375D-01, 0.21661D-01, 0.21924D-01,
     & 0.22164D-01, 0.22381D-01, 0.22579D-01, 0.22760D-01, 0.22925D-01,
     & 0.23074D-01, 0.23207D-01, 0.23326D-01, 0.23433D-01, 0.23529D-01,
     & 0.23614D-01, 0.23690D-01, 0.23755D-01, 0.23809D-01, 0.23856D-01,
     & 0.23896D-01, 0.23928D-01, 0.23954D-01, 0.23974D-01, 0.23987D-01,
     & 0.23993D-01, 0.23995D-01, 0.23991D-01, 0.23984D-01, 0.23972D-01/

      data ((uds_grdl(i,j),j=1,30),i=31,40)/
     & 0.20984D-01, 0.21285D-01, 0.21559D-01, 0.21809D-01, 0.22036D-01,
     & 0.22241D-01, 0.22423D-01, 0.22588D-01, 0.22737D-01, 0.22870D-01,
     & 0.22988D-01, 0.23092D-01, 0.23182D-01, 0.23261D-01, 0.23329D-01,
     & 0.23388D-01, 0.23437D-01, 0.23477D-01, 0.23507D-01, 0.23530D-01,
     & 0.23547D-01, 0.23557D-01, 0.23561D-01, 0.23560D-01, 0.23553D-01,
     & 0.23539D-01, 0.23521D-01, 0.23499D-01, 0.23473D-01, 0.23444D-01,
     & 0.21184D-01, 0.21446D-01, 0.21682D-01, 0.21894D-01, 0.22085D-01,
     & 0.22255D-01, 0.22403D-01, 0.22534D-01, 0.22651D-01, 0.22752D-01,
     & 0.22841D-01, 0.22914D-01, 0.22976D-01, 0.23027D-01, 0.23068D-01,
     & 0.23101D-01, 0.23125D-01, 0.23141D-01, 0.23147D-01, 0.23146D-01,
     & 0.23141D-01, 0.23129D-01, 0.23112D-01, 0.23091D-01, 0.23064D-01,
     & 0.23031D-01, 0.22994D-01, 0.22954D-01, 0.22910D-01, 0.22864D-01,
     & 0.21322D-01, 0.21544D-01, 0.21741D-01, 0.21916D-01, 0.22070D-01,
     & 0.22205D-01, 0.22318D-01, 0.22417D-01, 0.22501D-01, 0.22572D-01,
     & 0.22630D-01, 0.22674D-01, 0.22707D-01, 0.22731D-01, 0.22746D-01,
     & 0.22753D-01, 0.22753D-01, 0.22744D-01, 0.22727D-01, 0.22704D-01,
     & 0.22676D-01, 0.22644D-01, 0.22607D-01, 0.22566D-01, 0.22520D-01,
     & 0.22468D-01, 0.22413D-01, 0.22356D-01, 0.22296D-01, 0.22234D-01,
     & 0.21397D-01, 0.21578D-01, 0.21736D-01, 0.21873D-01, 0.21991D-01,
     & 0.22090D-01, 0.22169D-01, 0.22235D-01, 0.22287D-01, 0.22327D-01,
     & 0.22355D-01, 0.22371D-01, 0.22376D-01, 0.22373D-01, 0.22362D-01,
     & 0.22344D-01, 0.22319D-01, 0.22288D-01, 0.22248D-01, 0.22203D-01,
     & 0.22155D-01, 0.22102D-01, 0.22046D-01, 0.21985D-01, 0.21922D-01,
     & 0.21852D-01, 0.21781D-01, 0.21707D-01, 0.21631D-01, 0.21554D-01,
     & 0.21407D-01, 0.21547D-01, 0.21666D-01, 0.21765D-01, 0.21846D-01,
     & 0.21910D-01, 0.21956D-01, 0.21988D-01, 0.22009D-01, 0.22018D-01,
     & 0.22018D-01, 0.22005D-01, 0.21982D-01, 0.21953D-01, 0.21917D-01,
     & 0.21875D-01, 0.21827D-01, 0.21772D-01, 0.21711D-01, 0.21645D-01,
     & 0.21577D-01, 0.21505D-01, 0.21429D-01, 0.21351D-01, 0.21270D-01,
     & 0.21184D-01, 0.21097D-01, 0.21007D-01, 0.20917D-01, 0.20825D-01,
     & 0.21347D-01, 0.21445D-01, 0.21524D-01, 0.21585D-01, 0.21629D-01,
     & 0.21658D-01, 0.21669D-01, 0.21668D-01, 0.21657D-01, 0.21637D-01,
     & 0.21607D-01, 0.21567D-01, 0.21517D-01, 0.21462D-01, 0.21401D-01,
     & 0.21336D-01, 0.21265D-01, 0.21189D-01, 0.21106D-01, 0.21020D-01,
     & 0.20932D-01, 0.20842D-01, 0.20749D-01, 0.20654D-01, 0.20556D-01,
     & 0.20454D-01, 0.20351D-01, 0.20248D-01, 0.20143D-01, 0.20038D-01,
     & 0.21222D-01, 0.21278D-01, 0.21317D-01, 0.21339D-01, 0.21347D-01,
     & 0.21341D-01, 0.21318D-01, 0.21285D-01, 0.21243D-01, 0.21193D-01,
     & 0.21135D-01, 0.21067D-01, 0.20991D-01, 0.20911D-01, 0.20827D-01,
     & 0.20738D-01, 0.20645D-01, 0.20548D-01, 0.20445D-01, 0.20341D-01,
     & 0.20234D-01, 0.20126D-01, 0.20016D-01, 0.19905D-01, 0.19792D-01,
     & 0.19676D-01, 0.19559D-01, 0.19442D-01, 0.19324D-01, 0.19207D-01,
     & 0.21030D-01, 0.21043D-01, 0.21042D-01, 0.21026D-01, 0.20997D-01,
     & 0.20955D-01, 0.20899D-01, 0.20834D-01, 0.20762D-01, 0.20682D-01,
     & 0.20596D-01, 0.20502D-01, 0.20401D-01, 0.20297D-01, 0.20189D-01,
     & 0.20078D-01, 0.19965D-01, 0.19848D-01, 0.19726D-01, 0.19603D-01,
     & 0.19479D-01, 0.19354D-01, 0.19229D-01, 0.19103D-01, 0.18976D-01,
     & 0.18845D-01, 0.18716D-01, 0.18586D-01, 0.18457D-01, 0.18329D-01,
     & 0.20768D-01, 0.20739D-01, 0.20697D-01, 0.20642D-01, 0.20576D-01,
     & 0.20500D-01, 0.20410D-01, 0.20313D-01, 0.20211D-01, 0.20103D-01,
     & 0.19990D-01, 0.19869D-01, 0.19744D-01, 0.19616D-01, 0.19486D-01,
     & 0.19354D-01, 0.19221D-01, 0.19085D-01, 0.18945D-01, 0.18805D-01,
     & 0.18665D-01, 0.18525D-01, 0.18385D-01, 0.18245D-01, 0.18104D-01,
     & 0.17962D-01, 0.17820D-01, 0.17680D-01, 0.17540D-01, 0.17402D-01,
     & 0.20436D-01, 0.20363D-01, 0.20279D-01, 0.20186D-01, 0.20083D-01,
     & 0.19972D-01, 0.19849D-01, 0.19721D-01, 0.19589D-01, 0.19453D-01,
     & 0.19313D-01, 0.19168D-01, 0.19018D-01, 0.18868D-01, 0.18717D-01,
     & 0.18565D-01, 0.18413D-01, 0.18259D-01, 0.18102D-01, 0.17946D-01,
     & 0.17791D-01, 0.17636D-01, 0.17483D-01, 0.17330D-01, 0.17178D-01,
     & 0.17024D-01, 0.16872D-01, 0.16722D-01, 0.16573D-01, 0.16426D-01/

      data ((uds_grdl(i,j),j=1,30),i=41,50)/
     & 0.20030D-01, 0.19913D-01, 0.19787D-01, 0.19654D-01, 0.19515D-01,
     & 0.19369D-01, 0.19212D-01, 0.19054D-01, 0.18892D-01, 0.18729D-01,
     & 0.18564D-01, 0.18394D-01, 0.18222D-01, 0.18050D-01, 0.17879D-01,
     & 0.17708D-01, 0.17538D-01, 0.17367D-01, 0.17195D-01, 0.17024D-01,
     & 0.16855D-01, 0.16688D-01, 0.16523D-01, 0.16359D-01, 0.16196D-01,
     & 0.16033D-01, 0.15871D-01, 0.15712D-01, 0.15556D-01, 0.15401D-01,
     & 0.19547D-01, 0.19384D-01, 0.19217D-01, 0.19045D-01, 0.18868D-01,
     & 0.18687D-01, 0.18498D-01, 0.18309D-01, 0.18119D-01, 0.17929D-01,
     & 0.17739D-01, 0.17546D-01, 0.17352D-01, 0.17160D-01, 0.16970D-01,
     & 0.16781D-01, 0.16594D-01, 0.16408D-01, 0.16221D-01, 0.16037D-01,
     & 0.15856D-01, 0.15677D-01, 0.15501D-01, 0.15327D-01, 0.15155D-01,
     & 0.14983D-01, 0.14814D-01, 0.14648D-01, 0.14485D-01, 0.14324D-01,
     & 0.18980D-01, 0.18771D-01, 0.18561D-01, 0.18348D-01, 0.18134D-01,
     & 0.17919D-01, 0.17698D-01, 0.17478D-01, 0.17261D-01, 0.17045D-01,
     & 0.16830D-01, 0.16615D-01, 0.16401D-01, 0.16190D-01, 0.15982D-01,
     & 0.15777D-01, 0.15575D-01, 0.15374D-01, 0.15175D-01, 0.14980D-01,
     & 0.14787D-01, 0.14599D-01, 0.14413D-01, 0.14231D-01, 0.14051D-01,
     & 0.13873D-01, 0.13698D-01, 0.13527D-01, 0.13358D-01, 0.13193D-01,
     & 0.18318D-01, 0.18062D-01, 0.17808D-01, 0.17554D-01, 0.17303D-01,
     & 0.17052D-01, 0.16800D-01, 0.16551D-01, 0.16306D-01, 0.16065D-01,
     & 0.15827D-01, 0.15591D-01, 0.15357D-01, 0.15129D-01, 0.14904D-01,
     & 0.14684D-01, 0.14468D-01, 0.14256D-01, 0.14046D-01, 0.13840D-01,
     & 0.13639D-01, 0.13442D-01, 0.13249D-01, 0.13060D-01, 0.12875D-01,
     & 0.12692D-01, 0.12513D-01, 0.12337D-01, 0.12166D-01, 0.11998D-01,
     & 0.17534D-01, 0.17228D-01, 0.16929D-01, 0.16634D-01, 0.16345D-01,
     & 0.16060D-01, 0.15776D-01, 0.15498D-01, 0.15227D-01, 0.14962D-01,
     & 0.14703D-01, 0.14447D-01, 0.14196D-01, 0.13952D-01, 0.13713D-01,
     & 0.13480D-01, 0.13253D-01, 0.13030D-01, 0.12812D-01, 0.12598D-01,
     & 0.12390D-01, 0.12187D-01, 0.11990D-01, 0.11796D-01, 0.11608D-01,
     & 0.11422D-01, 0.11241D-01, 0.11065D-01, 0.10892D-01, 0.10724D-01,
     & 0.16597D-01, 0.16239D-01, 0.15891D-01, 0.15554D-01, 0.15225D-01,
     & 0.14905D-01, 0.14591D-01, 0.14287D-01, 0.13992D-01, 0.13705D-01,
     & 0.13426D-01, 0.13154D-01, 0.12889D-01, 0.12631D-01, 0.12382D-01,
     & 0.12139D-01, 0.11903D-01, 0.11674D-01, 0.11450D-01, 0.11232D-01,
     & 0.11021D-01, 0.10816D-01, 0.10616D-01, 0.10422D-01, 0.10232D-01,
     & 0.10048D-01, 0.98682D-02, 0.96934D-02, 0.95233D-02, 0.93576D-02,
     & 0.15463D-01, 0.15064D-01, 0.14680D-01, 0.14308D-01, 0.13948D-01,
     & 0.13600D-01, 0.13259D-01, 0.12931D-01, 0.12614D-01, 0.12308D-01,
     & 0.12012D-01, 0.11725D-01, 0.11446D-01, 0.11177D-01, 0.10917D-01,
     & 0.10665D-01, 0.10422D-01, 0.10186D-01, 0.99576D-02, 0.97363D-02,
     & 0.95221D-02, 0.93148D-02, 0.91139D-02, 0.89193D-02, 0.87307D-02,
     & 0.85476D-02, 0.83701D-02, 0.81980D-02, 0.80310D-02, 0.78689D-02,
     & 0.14272D-01, 0.13746D-01, 0.13248D-01, 0.12775D-01, 0.12328D-01,
     & 0.11903D-01, 0.11500D-01, 0.11117D-01, 0.10753D-01, 0.10405D-01,
     & 0.10074D-01, 0.97593D-02, 0.94593D-02, 0.91724D-02, 0.88978D-02,
     & 0.86351D-02, 0.83835D-02, 0.81431D-02, 0.79139D-02, 0.76937D-02,
     & 0.74822D-02, 0.72790D-02, 0.70836D-02, 0.68958D-02, 0.67154D-02,
     & 0.65432D-02, 0.63772D-02, 0.62171D-02, 0.60627D-02, 0.59137D-02,
     & 0.97399D-02, 0.92902D-02, 0.88685D-02, 0.84728D-02, 0.81015D-02,
     & 0.77537D-02, 0.74306D-02, 0.71256D-02, 0.68375D-02, 0.65655D-02,
     & 0.63085D-02, 0.60678D-02, 0.58412D-02, 0.56258D-02, 0.54212D-02,
     & 0.52268D-02, 0.50420D-02, 0.48671D-02, 0.47027D-02, 0.45457D-02,
     & 0.43956D-02, 0.42522D-02, 0.41152D-02, 0.39843D-02, 0.38594D-02,
     & 0.37419D-02, 0.36292D-02, 0.35209D-02, 0.34170D-02, 0.33171D-02,
     & 0.61454D-03, 0.58290D-03, 0.55335D-03, 0.52575D-03, 0.49997D-03,
     & 0.47594D-03, 0.45381D-03, 0.43299D-03, 0.41341D-03, 0.39498D-03,
     & 0.37765D-03, 0.36154D-03, 0.34646D-03, 0.33217D-03, 0.31864D-03,
     & 0.30582D-03, 0.29369D-03, 0.28227D-03, 0.27161D-03, 0.26146D-03,
     & 0.25179D-03, 0.24257D-03, 0.23380D-03, 0.22544D-03, 0.21749D-03,
     & 0.21008D-03, 0.20299D-03, 0.19619D-03, 0.18968D-03, 0.18345D-03/

      data ((g_grdl(i,j),j=1,30),i=1,10)/
     & 0.80813D+00, 0.88720D+00, 0.96575D+00, 0.10438D+01, 0.11213D+01,
     & 0.11982D+01, 0.12748D+01, 0.13508D+01, 0.14260D+01, 0.15006D+01,
     & 0.15745D+01, 0.16479D+01, 0.17208D+01, 0.17928D+01, 0.18641D+01,
     & 0.19347D+01, 0.20046D+01, 0.20738D+01, 0.21425D+01, 0.22105D+01,
     & 0.22777D+01, 0.23441D+01, 0.24098D+01, 0.24748D+01, 0.25391D+01,
     & 0.26030D+01, 0.26662D+01, 0.27286D+01, 0.27902D+01, 0.28512D+01,
     & 0.80806D+00, 0.88348D+00, 0.95821D+00, 0.10323D+01, 0.11056D+01,
     & 0.11784D+01, 0.12507D+01, 0.13222D+01, 0.13930D+01, 0.14629D+01,
     & 0.15322D+01, 0.16008D+01, 0.16688D+01, 0.17360D+01, 0.18024D+01,
     & 0.18680D+01, 0.19328D+01, 0.19970D+01, 0.20606D+01, 0.21235D+01,
     & 0.21855D+01, 0.22467D+01, 0.23072D+01, 0.23670D+01, 0.24260D+01,
     & 0.24847D+01, 0.25426D+01, 0.25997D+01, 0.26561D+01, 0.27118D+01,
     & 0.80804D+00, 0.87981D+00, 0.95076D+00, 0.10209D+01, 0.10902D+01,
     & 0.11589D+01, 0.12269D+01, 0.12941D+01, 0.13605D+01, 0.14260D+01,
     & 0.14907D+01, 0.15547D+01, 0.16181D+01, 0.16806D+01, 0.17422D+01,
     & 0.18030D+01, 0.18630D+01, 0.19223D+01, 0.19811D+01, 0.20390D+01,
     & 0.20961D+01, 0.21524D+01, 0.22079D+01, 0.22627D+01, 0.23168D+01,
     & 0.23705D+01, 0.24234D+01, 0.24755D+01, 0.25269D+01, 0.25776D+01,
     & 0.80810D+00, 0.87625D+00, 0.94346D+00, 0.10097D+01, 0.10751D+01,
     & 0.11397D+01, 0.12037D+01, 0.12666D+01, 0.13287D+01, 0.13899D+01,
     & 0.14502D+01, 0.15098D+01, 0.15687D+01, 0.16266D+01, 0.16837D+01,
     & 0.17399D+01, 0.17953D+01, 0.18500D+01, 0.19041D+01, 0.19573D+01,
     & 0.20098D+01, 0.20614D+01, 0.21122D+01, 0.21623D+01, 0.22117D+01,
     & 0.22606D+01, 0.23088D+01, 0.23562D+01, 0.24029D+01, 0.24489D+01,
     & 0.80810D+00, 0.87264D+00, 0.93612D+00, 0.99859D+00, 0.10601D+01,
     & 0.11206D+01, 0.11805D+01, 0.12394D+01, 0.12973D+01, 0.13542D+01,
     & 0.14103D+01, 0.14656D+01, 0.15201D+01, 0.15737D+01, 0.16264D+01,
     & 0.16782D+01, 0.17291D+01, 0.17794D+01, 0.18290D+01, 0.18777D+01,
     & 0.19257D+01, 0.19728D+01, 0.20192D+01, 0.20648D+01, 0.21097D+01,
     & 0.21541D+01, 0.21978D+01, 0.22407D+01, 0.22830D+01, 0.23245D+01,
     & 0.80804D+00, 0.86896D+00, 0.92875D+00, 0.98743D+00, 0.10451D+01,
     & 0.11017D+01, 0.11576D+01, 0.12124D+01, 0.12663D+01, 0.13191D+01,
     & 0.13710D+01, 0.14221D+01, 0.14724D+01, 0.15217D+01, 0.15702D+01,
     & 0.16177D+01, 0.16644D+01, 0.17104D+01, 0.17557D+01, 0.18002D+01,
     & 0.18438D+01, 0.18867D+01, 0.19288D+01, 0.19701D+01, 0.20108D+01,
     & 0.20510D+01, 0.20904D+01, 0.21291D+01, 0.21671D+01, 0.22045D+01,
     & 0.80792D+00, 0.86524D+00, 0.92135D+00, 0.97628D+00, 0.10301D+01,
     & 0.10829D+01, 0.11349D+01, 0.11858D+01, 0.12356D+01, 0.12844D+01,
     & 0.13323D+01, 0.13793D+01, 0.14256D+01, 0.14708D+01, 0.15152D+01,
     & 0.15586D+01, 0.16012D+01, 0.16431D+01, 0.16843D+01, 0.17247D+01,
     & 0.17643D+01, 0.18030D+01, 0.18411D+01, 0.18784D+01, 0.19150D+01,
     & 0.19511D+01, 0.19865D+01, 0.20212D+01, 0.20553D+01, 0.20887D+01,
     & 0.80784D+00, 0.86158D+00, 0.91405D+00, 0.96530D+00, 0.10154D+01,
     & 0.10644D+01, 0.11126D+01, 0.11596D+01, 0.12056D+01, 0.12505D+01,
     & 0.12944D+01, 0.13376D+01, 0.13799D+01, 0.14212D+01, 0.14617D+01,
     & 0.15012D+01, 0.15399D+01, 0.15778D+01, 0.16152D+01, 0.16516D+01,
     & 0.16873D+01, 0.17222D+01, 0.17564D+01, 0.17899D+01, 0.18227D+01,
     & 0.18550D+01, 0.18866D+01, 0.19176D+01, 0.19479D+01, 0.19776D+01,
     & 0.80784D+00, 0.85804D+00, 0.90691D+00, 0.95453D+00, 0.10010D+01,
     & 0.10463D+01, 0.10907D+01, 0.11340D+01, 0.11762D+01, 0.12174D+01,
     & 0.12576D+01, 0.12970D+01, 0.13355D+01, 0.13731D+01, 0.14098D+01,
     & 0.14456D+01, 0.14805D+01, 0.15147D+01, 0.15483D+01, 0.15811D+01,
     & 0.16131D+01, 0.16444D+01, 0.16749D+01, 0.17048D+01, 0.17340D+01,
     & 0.17627D+01, 0.17908D+01, 0.18182D+01, 0.18450D+01, 0.18713D+01,
     & 0.80773D+00, 0.85439D+00, 0.89969D+00, 0.94371D+00, 0.98652D+00,
     & 0.10282D+01, 0.10690D+01, 0.11086D+01, 0.11472D+01, 0.11847D+01,
     & 0.12212D+01, 0.12570D+01, 0.12918D+01, 0.13258D+01, 0.13588D+01,
     & 0.13910D+01, 0.14224D+01, 0.14531D+01, 0.14831D+01, 0.15123D+01,
     & 0.15408D+01, 0.15686D+01, 0.15957D+01, 0.16222D+01, 0.16480D+01,
     & 0.16733D+01, 0.16980D+01, 0.17221D+01, 0.17456D+01, 0.17686D+01/

      data ((g_grdl(i,j),j=1,30),i=11,20)/
     & 0.80754D+00, 0.85066D+00, 0.89242D+00, 0.93289D+00, 0.97215D+00,
     & 0.10103D+01, 0.10475D+01, 0.10835D+01, 0.11185D+01, 0.11525D+01,
     & 0.11855D+01, 0.12177D+01, 0.12490D+01, 0.12795D+01, 0.13090D+01,
     & 0.13378D+01, 0.13657D+01, 0.13930D+01, 0.14196D+01, 0.14455D+01,
     & 0.14706D+01, 0.14951D+01, 0.15190D+01, 0.15422D+01, 0.15648D+01,
     & 0.15869D+01, 0.16084D+01, 0.16294D+01, 0.16498D+01, 0.16697D+01,
     & 0.80741D+00, 0.84705D+00, 0.88533D+00, 0.92232D+00, 0.95809D+00,
     & 0.99275D+00, 0.10264D+01, 0.10590D+01, 0.10906D+01, 0.11211D+01,
     & 0.11507D+01, 0.11795D+01, 0.12075D+01, 0.12346D+01, 0.12609D+01,
     & 0.12863D+01, 0.13110D+01, 0.13351D+01, 0.13585D+01, 0.13812D+01,
     & 0.14032D+01, 0.14246D+01, 0.14453D+01, 0.14655D+01, 0.14851D+01,
     & 0.15042D+01, 0.15228D+01, 0.15409D+01, 0.15584D+01, 0.15754D+01,
     & 0.80717D+00, 0.84334D+00, 0.87816D+00, 0.91170D+00, 0.94405D+00,
     & 0.97529D+00, 0.10056D+01, 0.10348D+01, 0.10630D+01, 0.10902D+01,
     & 0.11165D+01, 0.11420D+01, 0.11667D+01, 0.11906D+01, 0.12137D+01,
     & 0.12360D+01, 0.12576D+01, 0.12785D+01, 0.12989D+01, 0.13185D+01,
     & 0.13376D+01, 0.13560D+01, 0.13739D+01, 0.13912D+01, 0.14079D+01,
     & 0.14242D+01, 0.14400D+01, 0.14553D+01, 0.14702D+01, 0.14845D+01,
     & 0.80714D+00, 0.83990D+00, 0.87132D+00, 0.90150D+00, 0.93051D+00,
     & 0.95843D+00, 0.98541D+00, 0.10113D+01, 0.10363D+01, 0.10603D+01,
     & 0.10835D+01, 0.11059D+01, 0.11275D+01, 0.11483D+01, 0.11683D+01,
     & 0.11876D+01, 0.12063D+01, 0.12243D+01, 0.12418D+01, 0.12586D+01,
     & 0.12748D+01, 0.12905D+01, 0.13056D+01, 0.13202D+01, 0.13343D+01,
     & 0.13480D+01, 0.13612D+01, 0.13740D+01, 0.13863D+01, 0.13982D+01,
     & 0.80735D+00, 0.83672D+00, 0.86481D+00, 0.89169D+00, 0.91743D+00,
     & 0.94212D+00, 0.96588D+00, 0.98864D+00, 0.10105D+01, 0.10314D+01,
     & 0.10515D+01, 0.10709D+01, 0.10895D+01, 0.11074D+01, 0.11245D+01,
     & 0.11410D+01, 0.11568D+01, 0.11721D+01, 0.11868D+01, 0.12009D+01,
     & 0.12145D+01, 0.12276D+01, 0.12401D+01, 0.12522D+01, 0.12639D+01,
     & 0.12751D+01, 0.12859D+01, 0.12962D+01, 0.13062D+01, 0.13158D+01,
     & 0.80704D+00, 0.83305D+00, 0.85782D+00, 0.88143D+00, 0.90395D+00,
     & 0.92546D+00, 0.94606D+00, 0.96572D+00, 0.98450D+00, 0.10024D+01,
     & 0.10196D+01, 0.10360D+01, 0.10518D+01, 0.10668D+01, 0.10812D+01,
     & 0.10950D+01, 0.11081D+01, 0.11207D+01, 0.11328D+01, 0.11444D+01,
     & 0.11555D+01, 0.11661D+01, 0.11762D+01, 0.11859D+01, 0.11952D+01,
     & 0.12041D+01, 0.12126D+01, 0.12208D+01, 0.12286D+01, 0.12360D+01,
     & 0.80683D+00, 0.82952D+00, 0.85103D+00, 0.87143D+00, 0.89080D+00,
     & 0.90920D+00, 0.92674D+00, 0.94339D+00, 0.95921D+00, 0.97426D+00,
     & 0.98856D+00, 0.10022D+01, 0.10152D+01, 0.10275D+01, 0.10392D+01,
     & 0.10504D+01, 0.10610D+01, 0.10711D+01, 0.10808D+01, 0.10899D+01,
     & 0.10986D+01, 0.11069D+01, 0.11148D+01, 0.11222D+01, 0.11293D+01,
     & 0.11361D+01, 0.11425D+01, 0.11485D+01, 0.11543D+01, 0.11598D+01,
     & 0.80658D+00, 0.82598D+00, 0.84426D+00, 0.86151D+00, 0.87779D+00,
     & 0.89316D+00, 0.90770D+00, 0.92142D+00, 0.93438D+00, 0.94662D+00,
     & 0.95818D+00, 0.96911D+00, 0.97943D+00, 0.98918D+00, 0.99838D+00,
     & 0.10071D+01, 0.10153D+01, 0.10230D+01, 0.10303D+01, 0.10371D+01,
     & 0.10436D+01, 0.10497D+01, 0.10554D+01, 0.10608D+01, 0.10658D+01,
     & 0.10705D+01, 0.10750D+01, 0.10791D+01, 0.10830D+01, 0.10866D+01,
     & 0.80631D+00, 0.82244D+00, 0.83754D+00, 0.85168D+00, 0.86492D+00,
     & 0.87732D+00, 0.88894D+00, 0.89981D+00, 0.90999D+00, 0.91951D+00,
     & 0.92842D+00, 0.93674D+00, 0.94451D+00, 0.95177D+00, 0.95854D+00,
     & 0.96486D+00, 0.97074D+00, 0.97620D+00, 0.98125D+00, 0.98594D+00,
     & 0.99029D+00, 0.99430D+00, 0.99801D+00, 0.10014D+01, 0.10045D+01,
     & 0.10074D+01, 0.10099D+01, 0.10123D+01, 0.10144D+01, 0.10163D+01,
     & 0.80596D+00, 0.81885D+00, 0.83080D+00, 0.84187D+00, 0.85212D+00,
     & 0.86159D+00, 0.87034D+00, 0.87842D+00, 0.88588D+00, 0.89276D+00,
     & 0.89909D+00, 0.90489D+00, 0.91019D+00, 0.91505D+00, 0.91948D+00,
     & 0.92352D+00, 0.92718D+00, 0.93048D+00, 0.93340D+00, 0.93603D+00,
     & 0.93836D+00, 0.94042D+00, 0.94221D+00, 0.94376D+00, 0.94506D+00,
     & 0.94610D+00, 0.94693D+00, 0.94757D+00, 0.94803D+00, 0.94832D+00/

      data ((g_grdl(i,j),j=1,30),i=21,30)/
     & 0.80549D+00, 0.81514D+00, 0.82393D+00, 0.83194D+00, 0.83921D+00,
     & 0.84579D+00, 0.85170D+00, 0.85703D+00, 0.86182D+00, 0.86611D+00,
     & 0.86991D+00, 0.87324D+00, 0.87613D+00, 0.87866D+00, 0.88082D+00,
     & 0.88266D+00, 0.88417D+00, 0.88538D+00, 0.88626D+00, 0.88690D+00,
     & 0.88730D+00, 0.88749D+00, 0.88746D+00, 0.88724D+00, 0.88682D+00,
     & 0.88616D+00, 0.88536D+00, 0.88440D+00, 0.88332D+00, 0.88210D+00,
     & 0.80426D+00, 0.81060D+00, 0.81620D+00, 0.82111D+00, 0.82538D+00,
     & 0.82905D+00, 0.83211D+00, 0.83469D+00, 0.83681D+00, 0.83851D+00,
     & 0.83981D+00, 0.84069D+00, 0.84121D+00, 0.84143D+00, 0.84137D+00,
     & 0.84104D+00, 0.84046D+00, 0.83963D+00, 0.83852D+00, 0.83724D+00,
     & 0.83578D+00, 0.83415D+00, 0.83237D+00, 0.83045D+00, 0.82838D+00,
     & 0.82611D+00, 0.82374D+00, 0.82128D+00, 0.81872D+00, 0.81608D+00,
     & 0.80325D+00, 0.80621D+00, 0.80854D+00, 0.81030D+00, 0.81153D+00,
     & 0.81225D+00, 0.81243D+00, 0.81223D+00, 0.81167D+00, 0.81077D+00,
     & 0.80956D+00, 0.80800D+00, 0.80616D+00, 0.80409D+00, 0.80182D+00,
     & 0.79936D+00, 0.79672D+00, 0.79388D+00, 0.79083D+00, 0.78766D+00,
     & 0.78439D+00, 0.78101D+00, 0.77753D+00, 0.77397D+00, 0.77031D+00,
     & 0.76650D+00, 0.76263D+00, 0.75873D+00, 0.75478D+00, 0.75079D+00,
     & 0.80222D+00, 0.80162D+00, 0.80052D+00, 0.79897D+00, 0.79701D+00,
     & 0.79464D+00, 0.79183D+00, 0.78874D+00, 0.78540D+00, 0.78182D+00,
     & 0.77803D+00, 0.77397D+00, 0.76970D+00, 0.76530D+00, 0.76079D+00,
     & 0.75617D+00, 0.75144D+00, 0.74659D+00, 0.74159D+00, 0.73655D+00,
     & 0.73146D+00, 0.72634D+00, 0.72119D+00, 0.71601D+00, 0.71080D+00,
     & 0.70547D+00, 0.70016D+00, 0.69486D+00, 0.68956D+00, 0.68428D+00,
     & 0.80002D+00, 0.79545D+00, 0.79055D+00, 0.78535D+00, 0.77988D+00,
     & 0.77414D+00, 0.76805D+00, 0.76183D+00, 0.75548D+00, 0.74902D+00,
     & 0.74246D+00, 0.73573D+00, 0.72888D+00, 0.72202D+00, 0.71514D+00,
     & 0.70826D+00, 0.70136D+00, 0.69443D+00, 0.68741D+00, 0.68044D+00,
     & 0.67351D+00, 0.66662D+00, 0.65978D+00, 0.65298D+00, 0.64622D+00,
     & 0.63940D+00, 0.63266D+00, 0.62599D+00, 0.61940D+00, 0.61287D+00,
     & 0.79934D+00, 0.79372D+00, 0.78782D+00, 0.78167D+00, 0.77528D+00,
     & 0.76867D+00, 0.76173D+00, 0.75469D+00, 0.74757D+00, 0.74037D+00,
     & 0.73310D+00, 0.72568D+00, 0.71818D+00, 0.71069D+00, 0.70322D+00,
     & 0.69576D+00, 0.68832D+00, 0.68087D+00, 0.67335D+00, 0.66590D+00,
     & 0.65852D+00, 0.65120D+00, 0.64395D+00, 0.63676D+00, 0.62962D+00,
     & 0.62245D+00, 0.61537D+00, 0.60838D+00, 0.60148D+00, 0.59466D+00,
     & 0.79861D+00, 0.79193D+00, 0.78503D+00, 0.77791D+00, 0.77061D+00,
     & 0.76311D+00, 0.75532D+00, 0.74748D+00, 0.73959D+00, 0.73165D+00,
     & 0.72368D+00, 0.71558D+00, 0.70744D+00, 0.69933D+00, 0.69128D+00,
     & 0.68326D+00, 0.67529D+00, 0.66732D+00, 0.65932D+00, 0.65141D+00,
     & 0.64359D+00, 0.63585D+00, 0.62820D+00, 0.62063D+00, 0.61314D+00,
     & 0.60562D+00, 0.59822D+00, 0.59092D+00, 0.58372D+00, 0.57663D+00,
     & 0.79781D+00, 0.79008D+00, 0.78216D+00, 0.77407D+00, 0.76585D+00,
     & 0.75747D+00, 0.74883D+00, 0.74017D+00, 0.73151D+00, 0.72283D+00,
     & 0.71416D+00, 0.70539D+00, 0.69660D+00, 0.68789D+00, 0.67925D+00,
     & 0.67068D+00, 0.66218D+00, 0.65372D+00, 0.64524D+00, 0.63688D+00,
     & 0.62863D+00, 0.62048D+00, 0.61245D+00, 0.60451D+00, 0.59667D+00,
     & 0.58883D+00, 0.58111D+00, 0.57352D+00, 0.56605D+00, 0.55869D+00,
     & 0.79695D+00, 0.78812D+00, 0.77917D+00, 0.77011D+00, 0.76095D+00,
     & 0.75167D+00, 0.74218D+00, 0.73271D+00, 0.72327D+00, 0.71386D+00,
     & 0.70448D+00, 0.69505D+00, 0.68562D+00, 0.67630D+00, 0.66709D+00,
     & 0.65798D+00, 0.64896D+00, 0.64001D+00, 0.63106D+00, 0.62226D+00,
     & 0.61359D+00, 0.60505D+00, 0.59664D+00, 0.58835D+00, 0.58018D+00,
     & 0.57202D+00, 0.56401D+00, 0.55614D+00, 0.54840D+00, 0.54080D+00,
     & 0.79599D+00, 0.78607D+00, 0.77606D+00, 0.76600D+00, 0.75589D+00,
     & 0.74572D+00, 0.73535D+00, 0.72506D+00, 0.71483D+00, 0.70468D+00,
     & 0.69461D+00, 0.68450D+00, 0.67444D+00, 0.66451D+00, 0.65473D+00,
     & 0.64508D+00, 0.63555D+00, 0.62611D+00, 0.61671D+00, 0.60747D+00,
     & 0.59840D+00, 0.58947D+00, 0.58070D+00, 0.57207D+00, 0.56358D+00,
     & 0.55512D+00, 0.54682D+00, 0.53869D+00, 0.53070D+00, 0.52287D+00/

      data ((g_grdl(i,j),j=1,30),i=31,40)/
     & 0.79494D+00, 0.78388D+00, 0.77280D+00, 0.76171D+00, 0.75064D+00,
     & 0.73954D+00, 0.72829D+00, 0.71716D+00, 0.70615D+00, 0.69525D+00,
     & 0.68447D+00, 0.67369D+00, 0.66299D+00, 0.65247D+00, 0.64211D+00,
     & 0.63193D+00, 0.62190D+00, 0.61198D+00, 0.60213D+00, 0.59247D+00,
     & 0.58300D+00, 0.57370D+00, 0.56457D+00, 0.55562D+00, 0.54681D+00,
     & 0.53807D+00, 0.52950D+00, 0.52112D+00, 0.51290D+00, 0.50484D+00,
     & 0.79377D+00, 0.78153D+00, 0.76934D+00, 0.75721D+00, 0.74514D+00,
     & 0.73310D+00, 0.72095D+00, 0.70897D+00, 0.69716D+00, 0.68551D+00,
     & 0.67401D+00, 0.66255D+00, 0.65121D+00, 0.64009D+00, 0.62917D+00,
     & 0.61845D+00, 0.60792D+00, 0.59754D+00, 0.58725D+00, 0.57717D+00,
     & 0.56731D+00, 0.55765D+00, 0.54818D+00, 0.53891D+00, 0.52981D+00,
     & 0.52079D+00, 0.51198D+00, 0.50335D+00, 0.49491D+00, 0.48666D+00,
     & 0.79248D+00, 0.77902D+00, 0.76568D+00, 0.75246D+00, 0.73937D+00,
     & 0.72636D+00, 0.71329D+00, 0.70044D+00, 0.68781D+00, 0.67539D+00,
     & 0.66317D+00, 0.65103D+00, 0.63905D+00, 0.62732D+00, 0.61584D+00,
     & 0.60459D+00, 0.59356D+00, 0.58271D+00, 0.57199D+00, 0.56151D+00,
     & 0.55127D+00, 0.54125D+00, 0.53146D+00, 0.52188D+00, 0.51250D+00,
     & 0.50323D+00, 0.49417D+00, 0.48533D+00, 0.47669D+00, 0.46824D+00,
     & 0.79104D+00, 0.77630D+00, 0.76177D+00, 0.74742D+00, 0.73326D+00,
     & 0.71926D+00, 0.70524D+00, 0.69150D+00, 0.67804D+00, 0.66484D+00,
     & 0.65189D+00, 0.63906D+00, 0.62643D+00, 0.61410D+00, 0.60205D+00,
     & 0.59028D+00, 0.57876D+00, 0.56745D+00, 0.55630D+00, 0.54543D+00,
     & 0.53482D+00, 0.52447D+00, 0.51436D+00, 0.50449D+00, 0.49484D+00,
     & 0.48532D+00, 0.47605D+00, 0.46700D+00, 0.45817D+00, 0.44956D+00,
     & 0.78945D+00, 0.77338D+00, 0.75760D+00, 0.74208D+00, 0.72682D+00,
     & 0.71178D+00, 0.69679D+00, 0.68214D+00, 0.66782D+00, 0.65381D+00,
     & 0.64011D+00, 0.62659D+00, 0.61331D+00, 0.60037D+00, 0.58775D+00,
     & 0.57545D+00, 0.56345D+00, 0.55169D+00, 0.54012D+00, 0.52887D+00,
     & 0.51790D+00, 0.50722D+00, 0.49682D+00, 0.48667D+00, 0.47677D+00,
     & 0.46703D+00, 0.45755D+00, 0.44832D+00, 0.43933D+00, 0.43056D+00,
     & 0.78747D+00, 0.76999D+00, 0.75287D+00, 0.73612D+00, 0.71970D+00,
     & 0.70358D+00, 0.68757D+00, 0.67198D+00, 0.65678D+00, 0.64196D+00,
     & 0.62750D+00, 0.61326D+00, 0.59933D+00, 0.58578D+00, 0.57260D+00,
     & 0.55978D+00, 0.54729D+00, 0.53509D+00, 0.52312D+00, 0.51150D+00,
     & 0.50019D+00, 0.48920D+00, 0.47851D+00, 0.46810D+00, 0.45797D+00,
     & 0.44803D+00, 0.43837D+00, 0.42897D+00, 0.41984D+00, 0.41095D+00,
     & 0.78530D+00, 0.76632D+00, 0.74781D+00, 0.72976D+00, 0.71213D+00,
     & 0.69488D+00, 0.67781D+00, 0.66124D+00, 0.64513D+00, 0.62947D+00,
     & 0.61422D+00, 0.59927D+00, 0.58467D+00, 0.57051D+00, 0.55677D+00,
     & 0.54342D+00, 0.53046D+00, 0.51782D+00, 0.50547D+00, 0.49348D+00,
     & 0.48185D+00, 0.47057D+00, 0.45961D+00, 0.44897D+00, 0.43862D+00,
     & 0.42850D+00, 0.41868D+00, 0.40915D+00, 0.39989D+00, 0.39090D+00,
     & 0.78281D+00, 0.76222D+00, 0.74222D+00, 0.72278D+00, 0.70388D+00,
     & 0.68544D+00, 0.66728D+00, 0.64969D+00, 0.63264D+00, 0.61611D+00,
     & 0.60007D+00, 0.58439D+00, 0.56912D+00, 0.55435D+00, 0.54005D+00,
     & 0.52619D+00, 0.51276D+00, 0.49970D+00, 0.48697D+00, 0.47465D+00,
     & 0.46272D+00, 0.45116D+00, 0.43996D+00, 0.42910D+00, 0.41856D+00,
     & 0.40830D+00, 0.39835D+00, 0.38871D+00, 0.37936D+00, 0.37029D+00,
     & 0.77986D+00, 0.75754D+00, 0.73594D+00, 0.71503D+00, 0.69476D+00,
     & 0.67507D+00, 0.65575D+00, 0.63711D+00, 0.61909D+00, 0.60166D+00,
     & 0.58481D+00, 0.56839D+00, 0.55245D+00, 0.53707D+00, 0.52221D+00,
     & 0.50784D+00, 0.49396D+00, 0.48050D+00, 0.46742D+00, 0.45478D+00,
     & 0.44257D+00, 0.43076D+00, 0.41934D+00, 0.40830D+00, 0.39760D+00,
     & 0.38722D+00, 0.37718D+00, 0.36746D+00, 0.35805D+00, 0.34894D+00,
     & 0.77631D+00, 0.75212D+00, 0.72881D+00, 0.70631D+00, 0.68459D+00,
     & 0.66357D+00, 0.64304D+00, 0.62328D+00, 0.60425D+00, 0.58590D+00,
     & 0.56820D+00, 0.55103D+00, 0.53442D+00, 0.51843D+00, 0.50301D+00,
     & 0.48816D+00, 0.47383D+00, 0.45999D+00, 0.44659D+00, 0.43367D+00,
     & 0.42121D+00, 0.40919D+00, 0.39758D+00, 0.38638D+00, 0.37557D+00,
     & 0.36511D+00, 0.35501D+00, 0.34526D+00, 0.33583D+00, 0.32672D+00/

      data ((g_grdl(i,j),j=1,30),i=41,50)/
     & 0.77204D+00, 0.74579D+00, 0.72059D+00, 0.69638D+00, 0.67309D+00,
     & 0.65065D+00, 0.62885D+00, 0.60794D+00, 0.58786D+00, 0.56856D+00,
     & 0.55000D+00, 0.53208D+00, 0.51481D+00, 0.49822D+00, 0.48228D+00,
     & 0.46695D+00, 0.45221D+00, 0.43802D+00, 0.42433D+00, 0.41116D+00,
     & 0.39849D+00, 0.38630D+00, 0.37455D+00, 0.36324D+00, 0.35235D+00,
     & 0.34186D+00, 0.33176D+00, 0.32201D+00, 0.31261D+00, 0.30354D+00,
     & 0.76674D+00, 0.73824D+00, 0.71099D+00, 0.68491D+00, 0.65994D+00,
     & 0.63597D+00, 0.61283D+00, 0.59069D+00, 0.56951D+00, 0.54922D+00,
     & 0.52979D+00, 0.51111D+00, 0.49318D+00, 0.47601D+00, 0.45956D+00,
     & 0.44378D+00, 0.42866D+00, 0.41415D+00, 0.40023D+00, 0.38686D+00,
     & 0.37404D+00, 0.36172D+00, 0.34989D+00, 0.33852D+00, 0.32761D+00,
     & 0.31716D+00, 0.30711D+00, 0.29744D+00, 0.28813D+00, 0.27916D+00,
     & 0.76000D+00, 0.72898D+00, 0.69945D+00, 0.67133D+00, 0.64451D+00,
     & 0.61890D+00, 0.59434D+00, 0.57094D+00, 0.54862D+00, 0.52734D+00,
     & 0.50702D+00, 0.48760D+00, 0.46905D+00, 0.45133D+00, 0.43441D+00,
     & 0.41824D+00, 0.40278D+00, 0.38802D+00, 0.37393D+00, 0.36045D+00,
     & 0.34753D+00, 0.33516D+00, 0.32332D+00, 0.31197D+00, 0.30111D+00,
     & 0.29078D+00, 0.28086D+00, 0.27133D+00, 0.26219D+00, 0.25340D+00,
     & 0.75112D+00, 0.71727D+00, 0.68521D+00, 0.65482D+00, 0.62598D+00,
     & 0.59860D+00, 0.57254D+00, 0.54781D+00, 0.52433D+00, 0.50202D+00,
     & 0.48082D+00, 0.46070D+00, 0.44158D+00, 0.42338D+00, 0.40606D+00,
     & 0.38956D+00, 0.37386D+00, 0.35893D+00, 0.34478D+00, 0.33128D+00,
     & 0.31839D+00, 0.30607D+00, 0.29432D+00, 0.28309D+00, 0.27239D+00,
     & 0.26228D+00, 0.25261D+00, 0.24334D+00, 0.23445D+00, 0.22595D+00,
     & 0.73856D+00, 0.70151D+00, 0.66660D+00, 0.63369D+00, 0.60266D+00,
     & 0.57338D+00, 0.54580D+00, 0.51974D+00, 0.49510D+00, 0.47182D+00,
     & 0.44981D+00, 0.42909D+00, 0.40952D+00, 0.39097D+00, 0.37338D+00,
     & 0.35670D+00, 0.34089D+00, 0.32595D+00, 0.31191D+00, 0.29854D+00,
     & 0.28582D+00, 0.27372D+00, 0.26221D+00, 0.25125D+00, 0.24085D+00,
     & 0.23112D+00, 0.22183D+00, 0.21295D+00, 0.20447D+00, 0.19637D+00,
     & 0.72073D+00, 0.67995D+00, 0.64179D+00, 0.60607D+00, 0.57263D+00,
     & 0.54137D+00, 0.51235D+00, 0.48506D+00, 0.45941D+00, 0.43530D+00,
     & 0.41263D+00, 0.39151D+00, 0.37173D+00, 0.35304D+00, 0.33541D+00,
     & 0.31876D+00, 0.30304D+00, 0.28830D+00, 0.27458D+00, 0.26156D+00,
     & 0.24922D+00, 0.23752D+00, 0.22644D+00, 0.21594D+00, 0.20601D+00,
     & 0.19684D+00, 0.18811D+00, 0.17979D+00, 0.17187D+00, 0.16433D+00,
     & 0.69515D+00, 0.65115D+00, 0.61012D+00, 0.57185D+00, 0.53616D+00,
     & 0.50293D+00, 0.47215D+00, 0.44338D+00, 0.41648D+00, 0.39135D+00,
     & 0.36789D+00, 0.34625D+00, 0.32615D+00, 0.30728D+00, 0.28958D+00,
     & 0.27299D+00, 0.25745D+00, 0.24302D+00, 0.22980D+00, 0.21733D+00,
     & 0.20557D+00, 0.19449D+00, 0.18405D+00, 0.17424D+00, 0.16504D+00,
     & 0.15671D+00, 0.14882D+00, 0.14134D+00, 0.13425D+00, 0.12754D+00,
     & 0.66986D+00, 0.61402D+00, 0.56298D+00, 0.51641D+00, 0.47400D+00,
     & 0.43569D+00, 0.40235D+00, 0.37162D+00, 0.34334D+00, 0.31735D+00,
     & 0.29353D+00, 0.27244D+00, 0.25346D+00, 0.23582D+00, 0.21945D+00,
     & 0.20428D+00, 0.19026D+00, 0.17752D+00, 0.16627D+00, 0.15574D+00,
     & 0.14588D+00, 0.13667D+00, 0.12808D+00, 0.12006D+00, 0.11264D+00,
     & 0.10617D+00, 0.10007D+00, 0.94319D-01, 0.88902D-01, 0.83806D-01,
     & 0.47111D+00, 0.42420D+00, 0.38182D+00, 0.34364D+00, 0.30938D+00,
     & 0.27906D+00, 0.25381D+00, 0.23076D+00, 0.20978D+00, 0.19074D+00,
     & 0.17351D+00, 0.15878D+00, 0.14586D+00, 0.13396D+00, 0.12301D+00,
     & 0.11296D+00, 0.10377D+00, 0.95588D-01, 0.88621D-01, 0.82135D-01,
     & 0.76107D-01, 0.70514D-01, 0.65336D-01, 0.60554D-01, 0.56175D-01,
     & 0.52507D-01, 0.49063D-01, 0.45834D-01, 0.42810D-01, 0.39983D-01,
     & 0.30227D-01, 0.26950D-01, 0.24004D-01, 0.21366D-01, 0.19013D-01,
     & 0.16949D-01, 0.15266D-01, 0.13738D-01, 0.12353D-01, 0.11105D-01,
     & 0.99836D-02, 0.90418D-02, 0.82281D-02, 0.74817D-02, 0.67988D-02,
     & 0.61759D-02, 0.56099D-02, 0.51118D-02, 0.46976D-02, 0.43136D-02,
     & 0.39583D-02, 0.36304D-02, 0.33286D-02, 0.30515D-02, 0.27999D-02,
     & 0.25953D-02, 0.24039D-02, 0.22252D-02, 0.20587D-02, 0.19037D-02/

      data ((c_grdl(i,j),j=1,30),i=1,10)/
     & 0.21356D-02, 0.21381D-02, 0.21403D-02, 0.21422D-02, 0.21438D-02,
     & 0.21453D-02, 0.21466D-02, 0.21478D-02, 0.21488D-02, 0.21497D-02,
     & 0.21504D-02, 0.21511D-02, 0.21518D-02, 0.21523D-02, 0.21528D-02,
     & 0.21532D-02, 0.21536D-02, 0.21539D-02, 0.21542D-02, 0.21545D-02,
     & 0.21547D-02, 0.21549D-02, 0.21551D-02, 0.21552D-02, 0.21554D-02,
     & 0.21555D-02, 0.21556D-02, 0.21557D-02, 0.21558D-02, 0.21559D-02,
     & 0.21289D-02, 0.21318D-02, 0.21344D-02, 0.21367D-02, 0.21387D-02,
     & 0.21405D-02, 0.21420D-02, 0.21434D-02, 0.21446D-02, 0.21457D-02,
     & 0.21466D-02, 0.21474D-02, 0.21482D-02, 0.21488D-02, 0.21494D-02,
     & 0.21499D-02, 0.21504D-02, 0.21508D-02, 0.21511D-02, 0.21514D-02,
     & 0.21517D-02, 0.21519D-02, 0.21521D-02, 0.21523D-02, 0.21525D-02,
     & 0.21527D-02, 0.21528D-02, 0.21529D-02, 0.21530D-02, 0.21531D-02,
     & 0.21207D-02, 0.21242D-02, 0.21274D-02, 0.21301D-02, 0.21325D-02,
     & 0.21346D-02, 0.21365D-02, 0.21382D-02, 0.21396D-02, 0.21409D-02,
     & 0.21420D-02, 0.21430D-02, 0.21439D-02, 0.21447D-02, 0.21454D-02,
     & 0.21460D-02, 0.21465D-02, 0.21470D-02, 0.21474D-02, 0.21478D-02,
     & 0.21481D-02, 0.21484D-02, 0.21486D-02, 0.21489D-02, 0.21491D-02,
     & 0.21492D-02, 0.21494D-02, 0.21495D-02, 0.21497D-02, 0.21498D-02,
     & 0.21108D-02, 0.21151D-02, 0.21189D-02, 0.21222D-02, 0.21251D-02,
     & 0.21277D-02, 0.21299D-02, 0.21319D-02, 0.21336D-02, 0.21352D-02,
     & 0.21365D-02, 0.21377D-02, 0.21388D-02, 0.21397D-02, 0.21405D-02,
     & 0.21412D-02, 0.21419D-02, 0.21425D-02, 0.21430D-02, 0.21434D-02,
     & 0.21438D-02, 0.21441D-02, 0.21445D-02, 0.21447D-02, 0.21450D-02,
     & 0.21452D-02, 0.21454D-02, 0.21455D-02, 0.21457D-02, 0.21458D-02,
     & 0.20989D-02, 0.21041D-02, 0.21087D-02, 0.21127D-02, 0.21162D-02,
     & 0.21192D-02, 0.21220D-02, 0.21243D-02, 0.21264D-02, 0.21283D-02,
     & 0.21299D-02, 0.21313D-02, 0.21326D-02, 0.21337D-02, 0.21347D-02,
     & 0.21356D-02, 0.21364D-02, 0.21370D-02, 0.21376D-02, 0.21382D-02,
     & 0.21387D-02, 0.21391D-02, 0.21394D-02, 0.21398D-02, 0.21401D-02,
     & 0.21403D-02, 0.21405D-02, 0.21407D-02, 0.21409D-02, 0.21411D-02,
     & 0.20844D-02, 0.20907D-02, 0.20962D-02, 0.21011D-02, 0.21053D-02,
     & 0.21091D-02, 0.21124D-02, 0.21152D-02, 0.21178D-02, 0.21200D-02,
     & 0.21220D-02, 0.21237D-02, 0.21252D-02, 0.21266D-02, 0.21277D-02,
     & 0.21288D-02, 0.21297D-02, 0.21305D-02, 0.21313D-02, 0.21319D-02,
     & 0.21325D-02, 0.21330D-02, 0.21334D-02, 0.21338D-02, 0.21342D-02,
     & 0.21345D-02, 0.21347D-02, 0.21350D-02, 0.21352D-02, 0.21354D-02,
     & 0.20933D-02, 0.20209D-02, 0.19139D-02, 0.20871D-02, 0.20922D-02,
     & 0.20968D-02, 0.21007D-02, 0.21042D-02, 0.21073D-02, 0.21100D-02,
     & 0.21124D-02, 0.21144D-02, 0.21163D-02, 0.21179D-02, 0.21193D-02,
     & 0.21206D-02, 0.21217D-02, 0.21227D-02, 0.21236D-02, 0.21244D-02,
     & 0.21250D-02, 0.21256D-02, 0.21262D-02, 0.21267D-02, 0.21271D-02,
     & 0.21274D-02, 0.21278D-02, 0.21281D-02, 0.21283D-02, 0.21285D-02,
     & 0.22187D-02, 0.21723D-02, 0.21220D-02, 0.20657D-02, 0.19982D-02,
     & 0.18971D-02, 0.20866D-02, 0.20909D-02, 0.20946D-02, 0.20979D-02,
     & 0.21007D-02, 0.21033D-02, 0.21055D-02, 0.21075D-02, 0.21092D-02,
     & 0.21107D-02, 0.21121D-02, 0.21133D-02, 0.21143D-02, 0.21153D-02,
     & 0.21161D-02, 0.21168D-02, 0.21175D-02, 0.21180D-02, 0.21185D-02,
     & 0.21190D-02, 0.21194D-02, 0.21197D-02, 0.21200D-02, 0.21203D-02,
     & 0.23122D-02, 0.22763D-02, 0.22389D-02, 0.22001D-02, 0.21595D-02,
     & 0.21167D-02, 0.20705D-02, 0.20187D-02, 0.19554D-02, 0.20972D-02,
     & 0.20866D-02, 0.20897D-02, 0.20924D-02, 0.20948D-02, 0.20969D-02,
     & 0.20988D-02, 0.21004D-02, 0.21019D-02, 0.21031D-02, 0.21043D-02,
     & 0.21053D-02, 0.21061D-02, 0.21069D-02, 0.21076D-02, 0.21082D-02,
     & 0.21088D-02, 0.21092D-02, 0.21097D-02, 0.21100D-02, 0.21104D-02,
     & 0.23860D-02, 0.23576D-02, 0.23281D-02, 0.22977D-02, 0.22666D-02,
     & 0.22352D-02, 0.22033D-02, 0.21710D-02, 0.21383D-02, 0.21049D-02,
     & 0.20705D-02, 0.20344D-02, 0.19953D-02, 0.19506D-02, 0.18917D-02,
     & 0.20842D-02, 0.20862D-02, 0.20880D-02, 0.20895D-02, 0.20909D-02,
     & 0.20921D-02, 0.20932D-02, 0.20941D-02, 0.20950D-02, 0.20957D-02,
     & 0.20964D-02, 0.20970D-02, 0.20975D-02, 0.20979D-02, 0.20983D-02/

      data ((c_grdl(i,j),j=1,30),i=11,20)/
     & 0.24430D-02, 0.24215D-02, 0.23985D-02, 0.23746D-02, 0.23500D-02,
     & 0.23251D-02, 0.23001D-02, 0.22753D-02, 0.22509D-02, 0.22268D-02,
     & 0.22033D-02, 0.21803D-02, 0.21580D-02, 0.21362D-02, 0.21151D-02,
     & 0.20946D-02, 0.20747D-02, 0.20553D-02, 0.20365D-02, 0.20180D-02,
     & 0.20000D-02, 0.19822D-02, 0.19645D-02, 0.19468D-02, 0.19288D-02,
     & 0.19101D-02, 0.18900D-02, 0.18668D-02, 0.20949D-02, 0.20837D-02,
     & 0.24818D-02, 0.24681D-02, 0.24521D-02, 0.24344D-02, 0.24156D-02,
     & 0.23962D-02, 0.23765D-02, 0.23569D-02, 0.23375D-02, 0.23185D-02,
     & 0.23001D-02, 0.22824D-02, 0.22655D-02, 0.22493D-02, 0.22340D-02,
     & 0.22196D-02, 0.22061D-02, 0.21934D-02, 0.21816D-02, 0.21707D-02,
     & 0.21605D-02, 0.21511D-02, 0.21425D-02, 0.21346D-02, 0.21273D-02,
     & 0.21207D-02, 0.21147D-02, 0.21092D-02, 0.21042D-02, 0.20997D-02,
     & 0.24989D-02, 0.24948D-02, 0.24870D-02, 0.24765D-02, 0.24641D-02,
     & 0.24504D-02, 0.24359D-02, 0.24209D-02, 0.24058D-02, 0.23910D-02,
     & 0.23765D-02, 0.23625D-02, 0.23491D-02, 0.23363D-02, 0.23242D-02,
     & 0.23129D-02, 0.23023D-02, 0.22925D-02, 0.22834D-02, 0.22750D-02,
     & 0.22674D-02, 0.22603D-02, 0.22539D-02, 0.22481D-02, 0.22428D-02,
     & 0.22380D-02, 0.22337D-02, 0.22298D-02, 0.22263D-02, 0.22232D-02,
     & 0.24894D-02, 0.24970D-02, 0.24993D-02, 0.24977D-02, 0.24930D-02,
     & 0.24861D-02, 0.24775D-02, 0.24678D-02, 0.24574D-02, 0.24467D-02,
     & 0.24359D-02, 0.24252D-02, 0.24148D-02, 0.24049D-02, 0.23955D-02,
     & 0.23866D-02, 0.23782D-02, 0.23705D-02, 0.23633D-02, 0.23567D-02,
     & 0.23506D-02, 0.23450D-02, 0.23399D-02, 0.23353D-02, 0.23311D-02,
     & 0.23274D-02, 0.23240D-02, 0.23209D-02, 0.23181D-02, 0.23157D-02,
     & 0.24473D-02, 0.24691D-02, 0.24839D-02, 0.24932D-02, 0.24980D-02,
     & 0.24993D-02, 0.24980D-02, 0.24946D-02, 0.24898D-02, 0.24839D-02,
     & 0.24775D-02, 0.24706D-02, 0.24636D-02, 0.24567D-02, 0.24499D-02,
     & 0.24434D-02, 0.24372D-02, 0.24313D-02, 0.24258D-02, 0.24207D-02,
     & 0.24160D-02, 0.24117D-02, 0.24077D-02, 0.24042D-02, 0.24009D-02,
     & 0.23979D-02, 0.23953D-02, 0.23929D-02, 0.23907D-02, 0.23888D-02,
     & 0.23655D-02, 0.24045D-02, 0.24346D-02, 0.24571D-02, 0.24736D-02,
     & 0.24851D-02, 0.24927D-02, 0.24971D-02, 0.24991D-02, 0.24992D-02,
     & 0.24980D-02, 0.24957D-02, 0.24928D-02, 0.24894D-02, 0.24858D-02,
     & 0.24820D-02, 0.24783D-02, 0.24746D-02, 0.24710D-02, 0.24676D-02,
     & 0.24645D-02, 0.24615D-02, 0.24587D-02, 0.24562D-02, 0.24539D-02,
     & 0.24517D-02, 0.24498D-02, 0.24480D-02, 0.24464D-02, 0.24450D-02,
     & 0.22364D-02, 0.22959D-02, 0.23441D-02, 0.23827D-02, 0.24132D-02,
     & 0.24371D-02, 0.24555D-02, 0.24695D-02, 0.24799D-02, 0.24874D-02,
     & 0.24927D-02, 0.24961D-02, 0.24981D-02, 0.24991D-02, 0.24993D-02,
     & 0.24990D-02, 0.24982D-02, 0.24971D-02, 0.24959D-02, 0.24946D-02,
     & 0.24932D-02, 0.24918D-02, 0.24904D-02, 0.24892D-02, 0.24879D-02,
     & 0.24868D-02, 0.24857D-02, 0.24847D-02, 0.24838D-02, 0.24830D-02,
     & 0.20566D-02, 0.21376D-02, 0.22048D-02, 0.22622D-02, 0.23096D-02,
     & 0.23483D-02, 0.23799D-02, 0.24054D-02, 0.24260D-02, 0.24424D-02,
     & 0.24555D-02, 0.24659D-02, 0.24741D-02, 0.24805D-02, 0.24854D-02,
     & 0.24892D-02, 0.24921D-02, 0.24943D-02, 0.24959D-02, 0.24971D-02,
     & 0.24980D-02, 0.24985D-02, 0.24989D-02, 0.24992D-02, 0.24993D-02,
     & 0.24993D-02, 0.24993D-02, 0.24993D-02, 0.24992D-02, 0.24991D-02,
     & 0.18146D-02, 0.19227D-02, 0.20142D-02, 0.20916D-02, 0.21564D-02,
     & 0.22110D-02, 0.22580D-02, 0.22973D-02, 0.23300D-02, 0.23573D-02,
     & 0.23799D-02, 0.23986D-02, 0.24142D-02, 0.24271D-02, 0.24378D-02,
     & 0.24467D-02, 0.24541D-02, 0.24603D-02, 0.24654D-02, 0.24697D-02,
     & 0.24733D-02, 0.24763D-02, 0.24788D-02, 0.24809D-02, 0.24827D-02,
     & 0.24842D-02, 0.24855D-02, 0.24866D-02, 0.24876D-02, 0.24884D-02,
     & 0.23599D-02, 0.22672D-02, 0.17590D-02, 0.18610D-02, 0.19482D-02,
     & 0.20225D-02, 0.20858D-02, 0.21395D-02, 0.21849D-02, 0.22242D-02,
     & 0.22580D-02, 0.22867D-02, 0.23112D-02, 0.23319D-02, 0.23496D-02,
     & 0.23646D-02, 0.23774D-02, 0.23883D-02, 0.23977D-02, 0.24057D-02,
     & 0.24126D-02, 0.24185D-02, 0.24236D-02, 0.24280D-02, 0.24319D-02,
     & 0.24352D-02, 0.24380D-02, 0.24406D-02, 0.24427D-02, 0.24446D-02/

      data ((c_grdl(i,j),j=1,30),i=21,30)/
     & 0.20932D-02, 0.22643D-02, 0.23511D-02, 0.23430D-02, 0.22061D-02,
     & 0.17698D-02, 0.18533D-02, 0.19252D-02, 0.19870D-02, 0.20402D-02,
     & 0.20859D-02, 0.21251D-02, 0.21587D-02, 0.21875D-02, 0.22128D-02,
     & 0.22350D-02, 0.22542D-02, 0.22708D-02, 0.22852D-02, 0.22977D-02,
     & 0.23085D-02, 0.23180D-02, 0.23262D-02, 0.23334D-02, 0.23396D-02,
     & 0.23451D-02, 0.23499D-02, 0.23541D-02, 0.23578D-02, 0.23610D-02,
     & 0.21884D-02, 0.22139D-02, 0.21873D-02, 0.21732D-02, 0.22952D-02,
     & 0.23548D-02, 0.23477D-02, 0.22622D-02, 0.20343D-02, 0.17930D-02,
     & 0.18533D-02, 0.19057D-02, 0.19512D-02, 0.19906D-02, 0.20250D-02,
     & 0.20548D-02, 0.20807D-02, 0.21033D-02, 0.21230D-02, 0.21401D-02,
     & 0.21551D-02, 0.21681D-02, 0.21795D-02, 0.21895D-02, 0.21983D-02,
     & 0.22063D-02, 0.22133D-02, 0.22195D-02, 0.22249D-02, 0.22297D-02,
     & 0.12386D-02, 0.17985D-02, 0.21588D-02, 0.21913D-02, 0.21797D-02,
     & 0.21633D-02, 0.21606D-02, 0.22676D-02, 0.23331D-02, 0.23597D-02,
     & 0.23477D-02, 0.22962D-02, 0.21971D-02, 0.20072D-02, 0.17730D-02,
     & 0.18122D-02, 0.18465D-02, 0.18766D-02, 0.19029D-02, 0.19260D-02,
     & 0.19463D-02, 0.19641D-02, 0.19797D-02, 0.19935D-02, 0.20056D-02,
     & 0.20162D-02, 0.20256D-02, 0.20339D-02, 0.20412D-02, 0.20476D-02,
     & 0.82716D-05, 0.40190D-03, 0.95340D-03, 0.14829D-02, 0.19187D-02,
     & 0.21767D-02, 0.22070D-02, 0.22140D-02, 0.22100D-02, 0.20533D-02,
     & 0.21606D-02, 0.22414D-02, 0.22986D-02, 0.23359D-02, 0.23558D-02,
     & 0.23601D-02, 0.23512D-02, 0.23307D-02, 0.23003D-02, 0.22605D-02,
     & 0.22116D-02, 0.21525D-02, 0.20799D-02, 0.19831D-02, 0.17477D-02,
     & 0.17616D-02, 0.17739D-02, 0.17847D-02, 0.17943D-02, 0.18028D-02,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.12116D-03, 0.54261D-03,
     & 0.10084D-02, 0.14425D-02, 0.18110D-02, 0.20769D-02, 0.21939D-02,
     & 0.22069D-02, 0.22042D-02, 0.21836D-02, 0.22099D-02, 0.21530D-02,
     & 0.20888D-02, 0.21491D-02, 0.21981D-02, 0.22374D-02, 0.22686D-02,
     & 0.22931D-02, 0.23121D-02, 0.23269D-02, 0.23381D-02, 0.23463D-02,
     & 0.23521D-02, 0.23560D-02, 0.23585D-02, 0.23599D-02, 0.23604D-02,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.23286D-03,
     & 0.65780D-03, 0.10971D-02, 0.14981D-02, 0.18336D-02, 0.20761D-02,
     & 0.21938D-02, 0.22122D-02, 0.21872D-02, 0.22093D-02, 0.22030D-02,
     & 0.22043D-02, 0.21187D-02, 0.20885D-02, 0.21381D-02, 0.21793D-02,
     & 0.22131D-02, 0.22409D-02, 0.22635D-02, 0.22819D-02, 0.22969D-02,
     & 0.23091D-02, 0.23191D-02, 0.23272D-02, 0.23338D-02, 0.23390D-02,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.26676D-04,
     & 0.34522D-03, 0.75878D-03, 0.11672D-02, 0.15369D-02, 0.18431D-02,
     & 0.20670D-02, 0.21880D-02, 0.22067D-02, 0.21607D-02, 0.22110D-02,
     & 0.21802D-02, 0.22077D-02, 0.22025D-02, 0.21418D-02, 0.20700D-02,
     & 0.21116D-02, 0.21468D-02, 0.21763D-02, 0.22012D-02, 0.22221D-02,
     & 0.22397D-02, 0.22545D-02, 0.22669D-02, 0.22774D-02, 0.22863D-02,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.97572D-04, 0.44784D-03, 0.84266D-03, 0.12206D-02, 0.15594D-02,
     & 0.18400D-02, 0.20491D-02, 0.21740D-02, 0.21898D-02, 0.22014D-02,
     & 0.21895D-02, 0.22140D-02, 0.21854D-02, 0.22070D-02, 0.22075D-02,
     & 0.21817D-02, 0.20863D-02, 0.20707D-02, 0.21010D-02, 0.21269D-02,
     & 0.21491D-02, 0.21681D-02, 0.21844D-02, 0.21984D-02, 0.22105D-02,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.18298D-03, 0.53604D-03, 0.90775D-03, 0.12560D-02,
     & 0.15664D-02, 0.18246D-02, 0.20217D-02, 0.21504D-02, 0.21880D-02,
     & 0.22107D-02, 0.21511D-02, 0.21976D-02, 0.22140D-02, 0.21792D-02,
     & 0.22008D-02, 0.22097D-02, 0.22061D-02, 0.21887D-02, 0.21509D-02,
     & 0.20423D-02, 0.20649D-02, 0.20845D-02, 0.21014D-02, 0.21162D-02,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.16949D-04, 0.26564D-03, 0.60742D-03, 0.95417D-03,
     & 0.12740D-02, 0.15585D-02, 0.17969D-02, 0.19839D-02, 0.21153D-02,
     & 0.21880D-02, 0.21879D-02, 0.22120D-02, 0.21554D-02, 0.21936D-02,
     & 0.22124D-02, 0.22068D-02, 0.21844D-02, 0.21994D-02, 0.22078D-02,
     & 0.22100D-02, 0.22067D-02, 0.21983D-02, 0.21847D-02, 0.21651D-02/

      data ((c_grdl(i,j),j=1,30),i=31,40)/
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.63312D-04, 0.33792D-03, 0.66112D-03,
     & 0.98240D-03, 0.12755D-02, 0.15364D-02, 0.17570D-02, 0.19348D-02,
     & 0.20676D-02, 0.21557D-02, 0.21944D-02, 0.21939D-02, 0.22122D-02,
     & 0.21591D-02, 0.21775D-02, 0.22007D-02, 0.22125D-02, 0.22121D-02,
     & 0.21837D-02, 0.21852D-02, 0.21953D-02, 0.22023D-02, 0.22069D-02,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.11810D-03, 0.39558D-03,
     & 0.69758D-03, 0.99323D-03, 0.12612D-02, 0.15003D-02, 0.17048D-02,
     & 0.18736D-02, 0.20061D-02, 0.21032D-02, 0.21668D-02, 0.21944D-02,
     & 0.21864D-02, 0.22090D-02, 0.22087D-02, 0.21458D-02, 0.21714D-02,
     & 0.21902D-02, 0.22031D-02, 0.22109D-02, 0.22139D-02, 0.22124D-02,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.32897D-05, 0.16902D-03,
     & 0.43732D-03, 0.71688D-03, 0.98757D-03, 0.12320D-02, 0.14509D-02,
     & 0.16407D-02, 0.18000D-02, 0.19301D-02, 0.20321D-02, 0.21079D-02,
     & 0.21603D-02, 0.21901D-02, 0.21826D-02, 0.21915D-02, 0.22074D-02,
     & 0.22121D-02, 0.22037D-02, 0.21587D-02, 0.21595D-02, 0.21732D-02,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.22166D-04,
     & 0.20975D-03, 0.46281D-03, 0.71943D-03, 0.96635D-03, 0.11886D-02,
     & 0.13891D-02, 0.15645D-02, 0.17141D-02, 0.18399D-02, 0.19429D-02,
     & 0.20252D-02, 0.20888D-02, 0.21363D-02, 0.21693D-02, 0.21894D-02,
     & 0.21934D-02, 0.21745D-02, 0.21911D-02, 0.22025D-02, 0.22093D-02,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.46067D-04, 0.23758D-03, 0.47232D-03, 0.70609D-03, 0.93054D-03,
     & 0.11329D-02, 0.13154D-02, 0.14764D-02, 0.16165D-02, 0.17361D-02,
     & 0.18373D-02, 0.19215D-02, 0.19906D-02, 0.20464D-02, 0.20907D-02,
     & 0.21255D-02, 0.21520D-02, 0.21714D-02, 0.21850D-02, 0.21926D-02,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.66707D-04, 0.25174D-03, 0.46668D-03, 0.67845D-03,
     & 0.88123D-03, 0.10653D-02, 0.12302D-02, 0.13779D-02, 0.15076D-02,
     & 0.16202D-02, 0.17171D-02, 0.18000D-02, 0.18704D-02, 0.19296D-02,
     & 0.19792D-02, 0.20205D-02, 0.20546D-02, 0.20828D-02, 0.21059D-02,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.79894D-04, 0.25238D-03, 0.44698D-03,
     & 0.63782D-03, 0.81954D-03, 0.98658D-03, 0.11354D-02, 0.12696D-02,
     & 0.13885D-02, 0.14933D-02, 0.15848D-02, 0.16639D-02, 0.17326D-02,
     & 0.17918D-02, 0.18427D-02, 0.18864D-02, 0.19238D-02, 0.19557D-02,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.70623D-06, 0.84020D-04, 0.24041D-03,
     & 0.41466D-03, 0.58557D-03, 0.74686D-03, 0.89736D-03, 0.10326D-02,
     & 0.11527D-02, 0.12611D-02, 0.13572D-02, 0.14417D-02, 0.15164D-02,
     & 0.15817D-02, 0.16385D-02, 0.16880D-02, 0.17314D-02, 0.17691D-02,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.17825D-05, 0.79147D-04,
     & 0.21723D-03, 0.37119D-03, 0.52293D-03, 0.66548D-03, 0.79925D-03,
     & 0.92124D-03, 0.10303D-02, 0.11271D-02, 0.12137D-02, 0.12915D-02,
     & 0.13600D-02, 0.14203D-02, 0.14738D-02, 0.15209D-02, 0.15623D-02,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.15209D-05,
     & 0.66546D-04, 0.18475D-03, 0.31857D-03, 0.45183D-03, 0.57790D-03,
     & 0.69433D-03, 0.80282D-03, 0.90097D-03, 0.98874D-03, 0.10666D-02,
     & 0.11357D-02, 0.11974D-02, 0.12529D-02, 0.13020D-02, 0.13453D-02/

      data ((c_grdl(i,j),j=1,30),i=41,50)/
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.33293D-06, 0.48504D-04, 0.14534D-03, 0.25894D-03, 0.37417D-03,
     & 0.48430D-03, 0.58657D-03, 0.68015D-03, 0.76668D-03, 0.84516D-03,
     & 0.91538D-03, 0.97800D-03, 0.10335D-02, 0.10828D-02, 0.11264D-02,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.28330D-04, 0.10237D-03, 0.19547D-03,
     & 0.29259D-03, 0.38703D-03, 0.47578D-03, 0.55751D-03, 0.63180D-03,
     & 0.69930D-03, 0.76128D-03, 0.81705D-03, 0.86689D-03, 0.91128D-03,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.10434D-04, 0.60258D-04,
     & 0.13162D-03, 0.21040D-03, 0.28911D-03, 0.36435D-03, 0.43438D-03,
     & 0.49864D-03, 0.55720D-03, 0.60994D-03, 0.65731D-03, 0.70027D-03,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.46510D-06,
     & 0.24751D-04, 0.72876D-04, 0.13208D-03, 0.19454D-03, 0.25582D-03,
     & 0.31415D-03, 0.36832D-03, 0.41805D-03, 0.46327D-03, 0.50404D-03,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.30285D-05, 0.26579D-04, 0.64402D-04, 0.10899D-03,
     & 0.15563D-03, 0.20147D-03, 0.24499D-03, 0.28566D-03, 0.32313D-03,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.17265D-05, 0.16708D-04,
     & 0.41547D-04, 0.71479D-04, 0.10342D-03, 0.13540D-03, 0.16625D-03,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.38338D-05, 0.15388D-04, 0.31512D-04, 0.50005D-04,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.28207D-06,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00/

      DO I=1,50
        DO J=1,30
          UDS_GRD(I,J) = UDS_GRDL(I,J)
          G_GRD(I,J)   = G_GRDL(I,J)
          C_GRD(I,J)   = C_GRDL(I,J)
        ENDDO
      ENDDO

      RETURN
      END
      SUBROUTINE I_LO_Q3G3
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      PARAMETER (NQ2MAX=30,NZMAX=50)
      DIMENSION UDS_GRD(NZMAX,NQ2MAX),C_GRD(NZMAX,NQ2MAX),
     &          G_GRD(NZMAX,NQ2MAX)
      DIMENSION UDS_GRDL(NZMAX,NQ2MAX),C_GRDL(NZMAX,NQ2MAX),
     &          G_GRDL(NZMAX,NQ2MAX)
      COMMON /CPARAM/ UDS_GRD,C_GRD,G_GRD 

      data ((uds_grdl(i,j),j=1,30),i=1,10)/
     & 0.87730D-03, 0.24960D-02, 0.41586D-02, 0.58604D-02, 0.75974D-02,
     & 0.93648D-02, 0.11154D-01, 0.12973D-01, 0.14817D-01, 0.16684D-01,
     & 0.18572D-01, 0.20475D-01, 0.22393D-01, 0.24329D-01, 0.26279D-01,
     & 0.28242D-01, 0.30217D-01, 0.32201D-01, 0.34193D-01, 0.36193D-01,
     & 0.38202D-01, 0.40216D-01, 0.42236D-01, 0.44261D-01, 0.46290D-01,
     & 0.48322D-01, 0.50356D-01, 0.52392D-01, 0.54430D-01, 0.56468D-01,
     & 0.91285D-03, 0.25022D-02, 0.41289D-02, 0.57889D-02, 0.74787D-02,
     & 0.91941D-02, 0.10929D-01, 0.12687D-01, 0.14466D-01, 0.16263D-01,
     & 0.18077D-01, 0.19904D-01, 0.21743D-01, 0.23595D-01, 0.25458D-01,
     & 0.27330D-01, 0.29211D-01, 0.31099D-01, 0.32992D-01, 0.34891D-01,
     & 0.36795D-01, 0.38702D-01, 0.40613D-01, 0.42526D-01, 0.44440D-01,
     & 0.46357D-01, 0.48273D-01, 0.50189D-01, 0.52105D-01, 0.54019D-01,
     & 0.95550D-03, 0.25132D-02, 0.41022D-02, 0.57190D-02, 0.73603D-02,
     & 0.90227D-02, 0.10701D-01, 0.12398D-01, 0.14112D-01, 0.15840D-01,
     & 0.17581D-01, 0.19332D-01, 0.21093D-01, 0.22862D-01, 0.24639D-01,
     & 0.26422D-01, 0.28211D-01, 0.30005D-01, 0.31802D-01, 0.33602D-01,
     & 0.35404D-01, 0.37207D-01, 0.39012D-01, 0.40816D-01, 0.42620D-01,
     & 0.44425D-01, 0.46228D-01, 0.48029D-01, 0.49827D-01, 0.51622D-01,
     & 0.10068D-02, 0.25308D-02, 0.40804D-02, 0.56524D-02, 0.72442D-02,
     & 0.88529D-02, 0.10475D-01, 0.12111D-01, 0.13759D-01, 0.15419D-01,
     & 0.17087D-01, 0.18763D-01, 0.20445D-01, 0.22133D-01, 0.23826D-01,
     & 0.25523D-01, 0.27222D-01, 0.28923D-01, 0.30627D-01, 0.32331D-01,
     & 0.34034D-01, 0.35737D-01, 0.37438D-01, 0.39138D-01, 0.40836D-01,
     & 0.42533D-01, 0.44227D-01, 0.45917D-01, 0.47603D-01, 0.49284D-01,
     & 0.10687D-02, 0.25565D-02, 0.40644D-02, 0.55900D-02, 0.71308D-02,
     & 0.86846D-02, 0.10249D-01, 0.11823D-01, 0.13406D-01, 0.14996D-01,
     & 0.16592D-01, 0.18193D-01, 0.19798D-01, 0.21406D-01, 0.23015D-01,
     & 0.24626D-01, 0.26237D-01, 0.27848D-01, 0.29460D-01, 0.31070D-01,
     & 0.32678D-01, 0.34282D-01, 0.35884D-01, 0.37482D-01, 0.39076D-01,
     & 0.40670D-01, 0.42258D-01, 0.43841D-01, 0.45419D-01, 0.46990D-01,
     & 0.11433D-02, 0.25925D-02, 0.40569D-02, 0.55342D-02, 0.70226D-02,
     & 0.85203D-02, 0.10026D-01, 0.11537D-01, 0.13054D-01, 0.14575D-01,
     & 0.16099D-01, 0.17626D-01, 0.19154D-01, 0.20682D-01, 0.22210D-01,
     & 0.23736D-01, 0.25260D-01, 0.26783D-01, 0.28305D-01, 0.29824D-01,
     & 0.31337D-01, 0.32847D-01, 0.34351D-01, 0.35851D-01, 0.37346D-01,
     & 0.38839D-01, 0.40325D-01, 0.41805D-01, 0.43278D-01, 0.44744D-01,
     & 0.12339D-02, 0.26421D-02, 0.40609D-02, 0.54884D-02, 0.69231D-02,
     & 0.83637D-02, 0.98094D-02, 0.11258D-01, 0.12709D-01, 0.14160D-01,
     & 0.15612D-01, 0.17065D-01, 0.18517D-01, 0.19967D-01, 0.21414D-01,
     & 0.22857D-01, 0.24297D-01, 0.25734D-01, 0.27168D-01, 0.28597D-01,
     & 0.30020D-01, 0.31437D-01, 0.32847D-01, 0.34252D-01, 0.35650D-01,
     & 0.37046D-01, 0.38434D-01, 0.39814D-01, 0.41187D-01, 0.42552D-01,
     & 0.13442D-02, 0.27094D-02, 0.40810D-02, 0.54576D-02, 0.68377D-02,
     & 0.82206D-02, 0.96063D-02, 0.10992D-01, 0.12376D-01, 0.13760D-01,
     & 0.15141D-01, 0.16521D-01, 0.17898D-01, 0.19271D-01, 0.20639D-01,
     & 0.22001D-01, 0.23359D-01, 0.24711D-01, 0.26061D-01, 0.27403D-01,
     & 0.28738D-01, 0.30065D-01, 0.31386D-01, 0.32698D-01, 0.34004D-01,
     & 0.35307D-01, 0.36601D-01, 0.37886D-01, 0.39163D-01, 0.40431D-01,
     & 0.14789D-02, 0.27995D-02, 0.41227D-02, 0.54473D-02, 0.67724D-02,
     & 0.80973D-02, 0.94228D-02, 0.10745D-01, 0.12065D-01, 0.13380D-01,
     & 0.14691D-01, 0.16000D-01, 0.17304D-01, 0.18601D-01, 0.19892D-01,
     & 0.21176D-01, 0.22454D-01, 0.23725D-01, 0.24992D-01, 0.26251D-01,
     & 0.27501D-01, 0.28743D-01, 0.29977D-01, 0.31202D-01, 0.32420D-01,
     & 0.33633D-01, 0.34837D-01, 0.36032D-01, 0.37217D-01, 0.38394D-01,
     & 0.16439D-02, 0.29181D-02, 0.41913D-02, 0.54628D-02, 0.67318D-02,
     & 0.79982D-02, 0.92631D-02, 0.10523D-01, 0.11777D-01, 0.13025D-01,
     & 0.14267D-01, 0.15504D-01, 0.16736D-01, 0.17960D-01, 0.19176D-01,
     & 0.20383D-01, 0.21583D-01, 0.22775D-01, 0.23962D-01, 0.25140D-01,
     & 0.26308D-01, 0.27467D-01, 0.28617D-01, 0.29758D-01, 0.30891D-01,
     & 0.32019D-01, 0.33136D-01, 0.34244D-01, 0.35343D-01, 0.36431D-01/

      data ((uds_grdl(i,j),j=1,30),i=11,20)/
     & 0.18468D-02, 0.30729D-02, 0.42950D-02, 0.55126D-02, 0.67251D-02,
     & 0.79327D-02, 0.91371D-02, 0.10334D-01, 0.11523D-01, 0.12705D-01,
     & 0.13879D-01, 0.15047D-01, 0.16208D-01, 0.17360D-01, 0.18502D-01,
     & 0.19635D-01, 0.20759D-01, 0.21875D-01, 0.22985D-01, 0.24084D-01,
     & 0.25174D-01, 0.26253D-01, 0.27323D-01, 0.28383D-01, 0.29434D-01,
     & 0.30480D-01, 0.31515D-01, 0.32540D-01, 0.33556D-01, 0.34561D-01,
     & 0.20972D-02, 0.32746D-02, 0.44452D-02, 0.56088D-02, 0.67652D-02,
     & 0.79147D-02, 0.90593D-02, 0.10195D-01, 0.11321D-01, 0.12437D-01,
     & 0.13545D-01, 0.14646D-01, 0.15739D-01, 0.16821D-01, 0.17892D-01,
     & 0.18954D-01, 0.20005D-01, 0.21048D-01, 0.22084D-01, 0.23109D-01,
     & 0.24124D-01, 0.25128D-01, 0.26121D-01, 0.27105D-01, 0.28079D-01,
     & 0.29047D-01, 0.30005D-01, 0.30952D-01, 0.31889D-01, 0.32816D-01,
     & 0.24067D-02, 0.35344D-02, 0.46531D-02, 0.57626D-02, 0.68629D-02,
     & 0.79546D-02, 0.90400D-02, 0.10114D-01, 0.11178D-01, 0.12232D-01,
     & 0.13275D-01, 0.14310D-01, 0.15336D-01, 0.16350D-01, 0.17354D-01,
     & 0.18346D-01, 0.19328D-01, 0.20300D-01, 0.21265D-01, 0.22219D-01,
     & 0.23161D-01, 0.24093D-01, 0.25014D-01, 0.25924D-01, 0.26825D-01,
     & 0.27720D-01, 0.28604D-01, 0.29477D-01, 0.30340D-01, 0.31193D-01,
     & 0.27904D-02, 0.38688D-02, 0.49361D-02, 0.59925D-02, 0.70381D-02,
     & 0.80735D-02, 0.91013D-02, 0.10117D-01, 0.11121D-01, 0.12113D-01,
     & 0.13094D-01, 0.14067D-01, 0.15029D-01, 0.15979D-01, 0.16918D-01,
     & 0.17844D-01, 0.18760D-01, 0.19666D-01, 0.20564D-01, 0.21450D-01,
     & 0.22325D-01, 0.23188D-01, 0.24041D-01, 0.24883D-01, 0.25715D-01,
     & 0.26541D-01, 0.27356D-01, 0.28160D-01, 0.28954D-01, 0.29738D-01,
     & 0.32657D-02, 0.42955D-02, 0.53126D-02, 0.63170D-02, 0.73093D-02,
     & 0.82902D-02, 0.92624D-02, 0.10221D-01, 0.11168D-01, 0.12101D-01,
     & 0.13023D-01, 0.13936D-01, 0.14837D-01, 0.15726D-01, 0.16603D-01,
     & 0.17467D-01, 0.18321D-01, 0.19163D-01, 0.19998D-01, 0.20821D-01,
     & 0.21632D-01, 0.22431D-01, 0.23220D-01, 0.23998D-01, 0.24766D-01,
     & 0.25527D-01, 0.26277D-01, 0.27017D-01, 0.27747D-01, 0.28466D-01,
     & 0.38548D-02, 0.48361D-02, 0.58031D-02, 0.67563D-02, 0.76961D-02,
     & 0.86235D-02, 0.95413D-02, 0.10445D-01, 0.11335D-01, 0.12212D-01,
     & 0.13077D-01, 0.13931D-01, 0.14775D-01, 0.15605D-01, 0.16422D-01,
     & 0.17227D-01, 0.18021D-01, 0.18804D-01, 0.19578D-01, 0.20340D-01,
     & 0.21090D-01, 0.21829D-01, 0.22557D-01, 0.23274D-01, 0.23982D-01,
     & 0.24682D-01, 0.25372D-01, 0.26051D-01, 0.26720D-01, 0.27378D-01,
     & 0.45826D-02, 0.55168D-02, 0.64355D-02, 0.73393D-02, 0.82287D-02,
     & 0.91048D-02, 0.99707D-02, 0.10822D-01, 0.11659D-01, 0.12482D-01,
     & 0.13292D-01, 0.14092D-01, 0.14880D-01, 0.15655D-01, 0.16416D-01,
     & 0.17165D-01, 0.17903D-01, 0.18629D-01, 0.19347D-01, 0.20052D-01,
     & 0.20746D-01, 0.21428D-01, 0.22099D-01, 0.22760D-01, 0.23410D-01,
     & 0.24054D-01, 0.24687D-01, 0.25309D-01, 0.25922D-01, 0.26524D-01,
     & 0.54775D-02, 0.63660D-02, 0.72382D-02, 0.80945D-02, 0.89357D-02,
     & 0.97628D-02, 0.10579D-01, 0.11380D-01, 0.12166D-01, 0.12938D-01,
     & 0.13697D-01, 0.14445D-01, 0.15181D-01, 0.15903D-01, 0.16612D-01,
     & 0.17308D-01, 0.17993D-01, 0.18666D-01, 0.19331D-01, 0.19983D-01,
     & 0.20623D-01, 0.21252D-01, 0.21870D-01, 0.22477D-01, 0.23075D-01,
     & 0.23665D-01, 0.24245D-01, 0.24815D-01, 0.25374D-01, 0.25924D-01,
     & 0.65686D-02, 0.74134D-02, 0.82410D-02, 0.90519D-02, 0.98470D-02,
     & 0.10628D-01, 0.11397D-01, 0.12150D-01, 0.12888D-01, 0.13612D-01,
     & 0.14321D-01, 0.15020D-01, 0.15707D-01, 0.16379D-01, 0.17039D-01,
     & 0.17685D-01, 0.18320D-01, 0.18943D-01, 0.19557D-01, 0.20159D-01,
     & 0.20749D-01, 0.21328D-01, 0.21896D-01, 0.22453D-01, 0.23001D-01,
     & 0.23541D-01, 0.24071D-01, 0.24591D-01, 0.25100D-01, 0.25601D-01,
     & 0.78819D-02, 0.86849D-02, 0.94698D-02, 0.10237D-01, 0.10988D-01,
     & 0.11724D-01, 0.12448D-01, 0.13156D-01, 0.13849D-01, 0.14526D-01,
     & 0.15189D-01, 0.15841D-01, 0.16481D-01, 0.17106D-01, 0.17718D-01,
     & 0.18317D-01, 0.18903D-01, 0.19479D-01, 0.20045D-01, 0.20599D-01,
     & 0.21141D-01, 0.21672D-01, 0.22192D-01, 0.22701D-01, 0.23201D-01,
     & 0.23693D-01, 0.24175D-01, 0.24647D-01, 0.25109D-01, 0.25562D-01/

      data ((uds_grdl(i,j),j=1,30),i=21,30)/
     & 0.94326D-02, 0.10196D-01, 0.10939D-01, 0.11665D-01, 0.12374D-01,
     & 0.13067D-01, 0.13748D-01, 0.14412D-01, 0.15060D-01, 0.15692D-01,
     & 0.16310D-01, 0.16917D-01, 0.17510D-01, 0.18090D-01, 0.18655D-01,
     & 0.19208D-01, 0.19748D-01, 0.20276D-01, 0.20796D-01, 0.21303D-01,
     & 0.21798D-01, 0.22282D-01, 0.22754D-01, 0.23217D-01, 0.23669D-01,
     & 0.24114D-01, 0.24549D-01, 0.24974D-01, 0.25389D-01, 0.25795D-01,
     & 0.11212D-01, 0.11936D-01, 0.12639D-01, 0.13323D-01, 0.13990D-01,
     & 0.14640D-01, 0.15277D-01, 0.15897D-01, 0.16500D-01, 0.17087D-01,
     & 0.17660D-01, 0.18220D-01, 0.18767D-01, 0.19299D-01, 0.19818D-01,
     & 0.20323D-01, 0.20815D-01, 0.21296D-01, 0.21767D-01, 0.22225D-01,
     & 0.22672D-01, 0.23107D-01, 0.23531D-01, 0.23945D-01, 0.24348D-01,
     & 0.24745D-01, 0.25131D-01, 0.25507D-01, 0.25874D-01, 0.26231D-01,
     & 0.13164D-01, 0.13850D-01, 0.14514D-01, 0.15157D-01, 0.15782D-01,
     & 0.16389D-01, 0.16982D-01, 0.17557D-01, 0.18114D-01, 0.18655D-01,
     & 0.19180D-01, 0.19692D-01, 0.20191D-01, 0.20674D-01, 0.21143D-01,
     & 0.21598D-01, 0.22039D-01, 0.22469D-01, 0.22889D-01, 0.23297D-01,
     & 0.23692D-01, 0.24075D-01, 0.24447D-01, 0.24809D-01, 0.25160D-01,
     & 0.25504D-01, 0.25838D-01, 0.26162D-01, 0.26476D-01, 0.26782D-01,
     & 0.15159D-01, 0.15807D-01, 0.16431D-01, 0.17033D-01, 0.17613D-01,
     & 0.18175D-01, 0.18721D-01, 0.19247D-01, 0.19755D-01, 0.20245D-01,
     & 0.20717D-01, 0.21176D-01, 0.21621D-01, 0.22049D-01, 0.22462D-01,
     & 0.22861D-01, 0.23246D-01, 0.23619D-01, 0.23981D-01, 0.24330D-01,
     & 0.24666D-01, 0.24991D-01, 0.25304D-01, 0.25607D-01, 0.25899D-01,
     & 0.26183D-01, 0.26457D-01, 0.26722D-01, 0.26977D-01, 0.27223D-01,
     & 0.16943D-01, 0.17550D-01, 0.18129D-01, 0.18683D-01, 0.19213D-01,
     & 0.19721D-01, 0.20211D-01, 0.20680D-01, 0.21127D-01, 0.21555D-01,
     & 0.21965D-01, 0.22359D-01, 0.22737D-01, 0.23098D-01, 0.23443D-01,
     & 0.23773D-01, 0.24089D-01, 0.24391D-01, 0.24682D-01, 0.24959D-01,
     & 0.25224D-01, 0.25477D-01, 0.25719D-01, 0.25950D-01, 0.26171D-01,
     & 0.26383D-01, 0.26585D-01, 0.26778D-01, 0.26961D-01, 0.27137D-01,
     & 0.17298D-01, 0.17894D-01, 0.18461D-01, 0.19002D-01, 0.19518D-01,
     & 0.20012D-01, 0.20486D-01, 0.20938D-01, 0.21368D-01, 0.21779D-01,
     & 0.22171D-01, 0.22547D-01, 0.22906D-01, 0.23249D-01, 0.23575D-01,
     & 0.23886D-01, 0.24182D-01, 0.24465D-01, 0.24735D-01, 0.24993D-01,
     & 0.25238D-01, 0.25472D-01, 0.25694D-01, 0.25906D-01, 0.26107D-01,
     & 0.26299D-01, 0.26481D-01, 0.26654D-01, 0.26818D-01, 0.26974D-01,
     & 0.17599D-01, 0.18184D-01, 0.18738D-01, 0.19266D-01, 0.19767D-01,
     & 0.20245D-01, 0.20703D-01, 0.21138D-01, 0.21552D-01, 0.21945D-01,
     & 0.22319D-01, 0.22675D-01, 0.23015D-01, 0.23338D-01, 0.23645D-01,
     & 0.23936D-01, 0.24213D-01, 0.24476D-01, 0.24726D-01, 0.24963D-01,
     & 0.25188D-01, 0.25402D-01, 0.25604D-01, 0.25796D-01, 0.25977D-01,
     & 0.26149D-01, 0.26311D-01, 0.26464D-01, 0.26609D-01, 0.26746D-01,
     & 0.17845D-01, 0.18417D-01, 0.18959D-01, 0.19472D-01, 0.19959D-01,
     & 0.20421D-01, 0.20862D-01, 0.21279D-01, 0.21675D-01, 0.22049D-01,
     & 0.22404D-01, 0.22741D-01, 0.23061D-01, 0.23364D-01, 0.23651D-01,
     & 0.23922D-01, 0.24178D-01, 0.24420D-01, 0.24650D-01, 0.24866D-01,
     & 0.25071D-01, 0.25264D-01, 0.25446D-01, 0.25617D-01, 0.25778D-01,
     & 0.25930D-01, 0.26072D-01, 0.26206D-01, 0.26331D-01, 0.26448D-01,
     & 0.18033D-01, 0.18593D-01, 0.19121D-01, 0.19619D-01, 0.20090D-01,
     & 0.20535D-01, 0.20958D-01, 0.21357D-01, 0.21734D-01, 0.22089D-01,
     & 0.22424D-01, 0.22742D-01, 0.23041D-01, 0.23323D-01, 0.23589D-01,
     & 0.23839D-01, 0.24074D-01, 0.24296D-01, 0.24504D-01, 0.24699D-01,
     & 0.24883D-01, 0.25055D-01, 0.25216D-01, 0.25367D-01, 0.25508D-01,
     & 0.25639D-01, 0.25761D-01, 0.25874D-01, 0.25980D-01, 0.26078D-01,
     & 0.18163D-01, 0.18709D-01, 0.19223D-01, 0.19705D-01, 0.20160D-01,
     & 0.20587D-01, 0.20991D-01, 0.21371D-01, 0.21728D-01, 0.22063D-01,
     & 0.22378D-01, 0.22674D-01, 0.22952D-01, 0.23212D-01, 0.23456D-01,
     & 0.23685D-01, 0.23899D-01, 0.24099D-01, 0.24285D-01, 0.24459D-01,
     & 0.24622D-01, 0.24772D-01, 0.24913D-01, 0.25043D-01, 0.25163D-01,
     & 0.25273D-01, 0.25375D-01, 0.25468D-01, 0.25554D-01, 0.25632D-01/

      data ((uds_grdl(i,j),j=1,30),i=31,40)/
     & 0.18234D-01, 0.18766D-01, 0.19263D-01, 0.19729D-01, 0.20165D-01,
     & 0.20574D-01, 0.20958D-01, 0.21317D-01, 0.21653D-01, 0.21968D-01,
     & 0.22261D-01, 0.22535D-01, 0.22791D-01, 0.23029D-01, 0.23251D-01,
     & 0.23458D-01, 0.23650D-01, 0.23828D-01, 0.23992D-01, 0.24144D-01,
     & 0.24284D-01, 0.24413D-01, 0.24533D-01, 0.24642D-01, 0.24741D-01,
     & 0.24830D-01, 0.24911D-01, 0.24985D-01, 0.25050D-01, 0.25109D-01,
     & 0.18246D-01, 0.18761D-01, 0.19241D-01, 0.19689D-01, 0.20106D-01,
     & 0.20495D-01, 0.20857D-01, 0.21195D-01, 0.21509D-01, 0.21801D-01,
     & 0.22072D-01, 0.22323D-01, 0.22556D-01, 0.22771D-01, 0.22970D-01,
     & 0.23154D-01, 0.23324D-01, 0.23479D-01, 0.23620D-01, 0.23750D-01,
     & 0.23868D-01, 0.23976D-01, 0.24073D-01, 0.24161D-01, 0.24240D-01,
     & 0.24308D-01, 0.24369D-01, 0.24422D-01, 0.24468D-01, 0.24508D-01,
     & 0.18197D-01, 0.18694D-01, 0.19156D-01, 0.19583D-01, 0.19980D-01,
     & 0.20347D-01, 0.20687D-01, 0.21002D-01, 0.21293D-01, 0.21562D-01,
     & 0.21810D-01, 0.22037D-01, 0.22245D-01, 0.22437D-01, 0.22612D-01,
     & 0.22773D-01, 0.22919D-01, 0.23051D-01, 0.23169D-01, 0.23276D-01,
     & 0.23372D-01, 0.23457D-01, 0.23533D-01, 0.23600D-01, 0.23658D-01,
     & 0.23705D-01, 0.23745D-01, 0.23778D-01, 0.23805D-01, 0.23826D-01,
     & 0.18089D-01, 0.18566D-01, 0.19006D-01, 0.19412D-01, 0.19787D-01,
     & 0.20131D-01, 0.20447D-01, 0.20737D-01, 0.21003D-01, 0.21248D-01,
     & 0.21471D-01, 0.21673D-01, 0.21857D-01, 0.22024D-01, 0.22175D-01,
     & 0.22311D-01, 0.22434D-01, 0.22543D-01, 0.22637D-01, 0.22721D-01,
     & 0.22794D-01, 0.22858D-01, 0.22912D-01, 0.22958D-01, 0.22995D-01,
     & 0.23021D-01, 0.23041D-01, 0.23054D-01, 0.23062D-01, 0.23063D-01,
     & 0.17922D-01, 0.18376D-01, 0.18793D-01, 0.19176D-01, 0.19526D-01,
     & 0.19846D-01, 0.20135D-01, 0.20399D-01, 0.20640D-01, 0.20859D-01,
     & 0.21057D-01, 0.21233D-01, 0.21390D-01, 0.21532D-01, 0.21658D-01,
     & 0.21770D-01, 0.21868D-01, 0.21953D-01, 0.22024D-01, 0.22084D-01,
     & 0.22135D-01, 0.22176D-01, 0.22209D-01, 0.22234D-01, 0.22250D-01,
     & 0.22256D-01, 0.22256D-01, 0.22250D-01, 0.22238D-01, 0.22222D-01,
     & 0.17690D-01, 0.18118D-01, 0.18510D-01, 0.18866D-01, 0.19189D-01,
     & 0.19482D-01, 0.19743D-01, 0.19980D-01, 0.20193D-01, 0.20384D-01,
     & 0.20555D-01, 0.20704D-01, 0.20834D-01, 0.20949D-01, 0.21050D-01,
     & 0.21137D-01, 0.21211D-01, 0.21271D-01, 0.21317D-01, 0.21354D-01,
     & 0.21382D-01, 0.21401D-01, 0.21412D-01, 0.21416D-01, 0.21412D-01,
     & 0.21398D-01, 0.21378D-01, 0.21353D-01, 0.21323D-01, 0.21289D-01,
     & 0.17400D-01, 0.17801D-01, 0.18163D-01, 0.18490D-01, 0.18785D-01,
     & 0.19049D-01, 0.19280D-01, 0.19487D-01, 0.19671D-01, 0.19834D-01,
     & 0.19976D-01, 0.20097D-01, 0.20200D-01, 0.20288D-01, 0.20362D-01,
     & 0.20423D-01, 0.20472D-01, 0.20508D-01, 0.20530D-01, 0.20543D-01,
     & 0.20549D-01, 0.20546D-01, 0.20536D-01, 0.20519D-01, 0.20495D-01,
     & 0.20461D-01, 0.20423D-01, 0.20379D-01, 0.20332D-01, 0.20281D-01,
     & 0.17053D-01, 0.17421D-01, 0.17751D-01, 0.18047D-01, 0.18310D-01,
     & 0.18542D-01, 0.18741D-01, 0.18916D-01, 0.19070D-01, 0.19203D-01,
     & 0.19316D-01, 0.19408D-01, 0.19482D-01, 0.19542D-01, 0.19590D-01,
     & 0.19625D-01, 0.19649D-01, 0.19660D-01, 0.19658D-01, 0.19649D-01,
     & 0.19631D-01, 0.19607D-01, 0.19577D-01, 0.19540D-01, 0.19497D-01,
     & 0.19444D-01, 0.19387D-01, 0.19327D-01, 0.19263D-01, 0.19196D-01,
     & 0.16646D-01, 0.16977D-01, 0.17272D-01, 0.17533D-01, 0.17762D-01,
     & 0.17960D-01, 0.18124D-01, 0.18266D-01, 0.18387D-01, 0.18489D-01,
     & 0.18572D-01, 0.18634D-01, 0.18679D-01, 0.18711D-01, 0.18731D-01,
     & 0.18740D-01, 0.18739D-01, 0.18726D-01, 0.18700D-01, 0.18667D-01,
     & 0.18628D-01, 0.18583D-01, 0.18533D-01, 0.18477D-01, 0.18415D-01,
     & 0.18345D-01, 0.18271D-01, 0.18194D-01, 0.18115D-01, 0.18033D-01,
     & 0.16180D-01, 0.16471D-01, 0.16726D-01, 0.16948D-01, 0.17139D-01,
     & 0.17300D-01, 0.17428D-01, 0.17535D-01, 0.17622D-01, 0.17691D-01,
     & 0.17742D-01, 0.17773D-01, 0.17788D-01, 0.17791D-01, 0.17784D-01,
     & 0.17767D-01, 0.17741D-01, 0.17704D-01, 0.17655D-01, 0.17600D-01,
     & 0.17540D-01, 0.17475D-01, 0.17405D-01, 0.17331D-01, 0.17252D-01,
     & 0.17165D-01, 0.17075D-01, 0.16984D-01, 0.16890D-01, 0.16795D-01/

      data ((uds_grdl(i,j),j=1,30),i=41,50)/
     & 0.15654D-01, 0.15899D-01, 0.16110D-01, 0.16290D-01, 0.16440D-01,
     & 0.16561D-01, 0.16650D-01, 0.16719D-01, 0.16771D-01, 0.16806D-01,
     & 0.16825D-01, 0.16824D-01, 0.16809D-01, 0.16783D-01, 0.16749D-01,
     & 0.16706D-01, 0.16655D-01, 0.16595D-01, 0.16523D-01, 0.16447D-01,
     & 0.16367D-01, 0.16283D-01, 0.16195D-01, 0.16104D-01, 0.16010D-01,
     & 0.15907D-01, 0.15803D-01, 0.15698D-01, 0.15592D-01, 0.15485D-01,
     & 0.15068D-01, 0.15262D-01, 0.15424D-01, 0.15557D-01, 0.15663D-01,
     & 0.15742D-01, 0.15789D-01, 0.15819D-01, 0.15833D-01, 0.15833D-01,
     & 0.15818D-01, 0.15786D-01, 0.15740D-01, 0.15686D-01, 0.15625D-01,
     & 0.15556D-01, 0.15481D-01, 0.15398D-01, 0.15305D-01, 0.15210D-01,
     & 0.15111D-01, 0.15009D-01, 0.14905D-01, 0.14798D-01, 0.14689D-01,
     & 0.14574D-01, 0.14458D-01, 0.14341D-01, 0.14224D-01, 0.14107D-01,
     & 0.14419D-01, 0.14556D-01, 0.14664D-01, 0.14746D-01, 0.14803D-01,
     & 0.14836D-01, 0.14840D-01, 0.14829D-01, 0.14805D-01, 0.14768D-01,
     & 0.14720D-01, 0.14655D-01, 0.14579D-01, 0.14497D-01, 0.14409D-01,
     & 0.14317D-01, 0.14219D-01, 0.14115D-01, 0.14002D-01, 0.13888D-01,
     & 0.13772D-01, 0.13655D-01, 0.13536D-01, 0.13417D-01, 0.13295D-01,
     & 0.13169D-01, 0.13043D-01, 0.12917D-01, 0.12792D-01, 0.12667D-01,
     & 0.13701D-01, 0.13774D-01, 0.13823D-01, 0.13849D-01, 0.13855D-01,
     & 0.13839D-01, 0.13797D-01, 0.13744D-01, 0.13680D-01, 0.13606D-01,
     & 0.13524D-01, 0.13428D-01, 0.13323D-01, 0.13214D-01, 0.13101D-01,
     & 0.12985D-01, 0.12866D-01, 0.12743D-01, 0.12613D-01, 0.12483D-01,
     & 0.12352D-01, 0.12222D-01, 0.12091D-01, 0.11960D-01, 0.11829D-01,
     & 0.11694D-01, 0.11561D-01, 0.11429D-01, 0.11298D-01, 0.11168D-01,
     & 0.12898D-01, 0.12901D-01, 0.12884D-01, 0.12850D-01, 0.12800D-01,
     & 0.12734D-01, 0.12645D-01, 0.12548D-01, 0.12445D-01, 0.12335D-01,
     & 0.12219D-01, 0.12093D-01, 0.11960D-01, 0.11826D-01, 0.11691D-01,
     & 0.11554D-01, 0.11417D-01, 0.11277D-01, 0.11132D-01, 0.10989D-01,
     & 0.10847D-01, 0.10707D-01, 0.10567D-01, 0.10429D-01, 0.10291D-01,
     & 0.10152D-01, 0.10014D-01, 0.98791D-02, 0.97459D-02, 0.96148D-02,
     & 0.11992D-01, 0.11916D-01, 0.11828D-01, 0.11730D-01, 0.11621D-01,
     & 0.11501D-01, 0.11364D-01, 0.11225D-01, 0.11082D-01, 0.10937D-01,
     & 0.10790D-01, 0.10636D-01, 0.10479D-01, 0.10323D-01, 0.10169D-01,
     & 0.10015D-01, 0.98626D-02, 0.97100D-02, 0.95556D-02, 0.94039D-02,
     & 0.92547D-02, 0.91081D-02, 0.89639D-02, 0.88221D-02, 0.86823D-02,
     & 0.85422D-02, 0.84051D-02, 0.82708D-02, 0.81393D-02, 0.80105D-02,
     & 0.10955D-01, 0.10804D-01, 0.10647D-01, 0.10485D-01, 0.10319D-01,
     & 0.10148D-01, 0.99665D-02, 0.97859D-02, 0.96066D-02, 0.94287D-02,
     & 0.92522D-02, 0.90733D-02, 0.88948D-02, 0.87201D-02, 0.85490D-02,
     & 0.83813D-02, 0.82170D-02, 0.80549D-02, 0.78937D-02, 0.77367D-02,
     & 0.75836D-02, 0.74344D-02, 0.72888D-02, 0.71468D-02, 0.70080D-02,
     & 0.68709D-02, 0.67374D-02, 0.66075D-02, 0.64810D-02, 0.63577D-02,
     & 0.98746D-02, 0.95861D-02, 0.93077D-02, 0.90388D-02, 0.87789D-02,
     & 0.85270D-02, 0.82800D-02, 0.80424D-02, 0.78137D-02, 0.75932D-02,
     & 0.73807D-02, 0.71746D-02, 0.69754D-02, 0.67836D-02, 0.65989D-02,
     & 0.64210D-02, 0.62496D-02, 0.60842D-02, 0.59243D-02, 0.57701D-02,
     & 0.56215D-02, 0.54782D-02, 0.53398D-02, 0.52063D-02, 0.50774D-02,
     & 0.49531D-02, 0.48330D-02, 0.47168D-02, 0.46046D-02, 0.44960D-02,
     & 0.66257D-02, 0.63472D-02, 0.60843D-02, 0.58358D-02, 0.56008D-02,
     & 0.53786D-02, 0.51690D-02, 0.49701D-02, 0.47814D-02, 0.46024D-02,
     & 0.44324D-02, 0.42717D-02, 0.41193D-02, 0.39741D-02, 0.38357D-02,
     & 0.37038D-02, 0.35781D-02, 0.34586D-02, 0.33455D-02, 0.32372D-02,
     & 0.31336D-02, 0.30344D-02, 0.29395D-02, 0.28487D-02, 0.27617D-02,
     & 0.26795D-02, 0.26006D-02, 0.25247D-02, 0.24517D-02, 0.23816D-02,
     & 0.41397D-03, 0.39347D-03, 0.37426D-03, 0.35624D-03, 0.33935D-03,
     & 0.32352D-03, 0.30881D-03, 0.29494D-03, 0.28187D-03, 0.26955D-03,
     & 0.25792D-03, 0.24706D-03, 0.23685D-03, 0.22718D-03, 0.21800D-03,
     & 0.20931D-03, 0.20106D-03, 0.19328D-03, 0.18600D-03, 0.17906D-03,
     & 0.17244D-03, 0.16614D-03, 0.16013D-03, 0.15441D-03, 0.14896D-03,
     & 0.14387D-03, 0.13900D-03, 0.13433D-03, 0.12986D-03, 0.12557D-03/

      data ((g_grdl(i,j),j=1,30),i=1,10)/
     & 0.11776D+01, 0.12610D+01, 0.13438D+01, 0.14257D+01, 0.15070D+01,
     & 0.15875D+01, 0.16674D+01, 0.17466D+01, 0.18249D+01, 0.19023D+01,
     & 0.19790D+01, 0.20551D+01, 0.21305D+01, 0.22050D+01, 0.22786D+01,
     & 0.23515D+01, 0.24235D+01, 0.24949D+01, 0.25656D+01, 0.26356D+01,
     & 0.27047D+01, 0.27730D+01, 0.28405D+01, 0.29073D+01, 0.29734D+01,
     & 0.30390D+01, 0.31038D+01, 0.31678D+01, 0.32311D+01, 0.32936D+01,
     & 0.11699D+01, 0.12484D+01, 0.13260D+01, 0.14027D+01, 0.14787D+01,
     & 0.15538D+01, 0.16283D+01, 0.17019D+01, 0.17746D+01, 0.18464D+01,
     & 0.19174D+01, 0.19878D+01, 0.20574D+01, 0.21261D+01, 0.21939D+01,
     & 0.22609D+01, 0.23271D+01, 0.23925D+01, 0.24574D+01, 0.25214D+01,
     & 0.25846D+01, 0.26470D+01, 0.27086D+01, 0.27694D+01, 0.28296D+01,
     & 0.28892D+01, 0.29481D+01, 0.30062D+01, 0.30636D+01, 0.31203D+01,
     & 0.11610D+01, 0.12346D+01, 0.13072D+01, 0.13788D+01, 0.14495D+01,
     & 0.15194D+01, 0.15886D+01, 0.16569D+01, 0.17242D+01, 0.17906D+01,
     & 0.18561D+01, 0.19210D+01, 0.19850D+01, 0.20482D+01, 0.21104D+01,
     & 0.21718D+01, 0.22324D+01, 0.22923D+01, 0.23516D+01, 0.24100D+01,
     & 0.24675D+01, 0.25243D+01, 0.25803D+01, 0.26356D+01, 0.26901D+01,
     & 0.27442D+01, 0.27975D+01, 0.28501D+01, 0.29019D+01, 0.29530D+01,
     & 0.11508D+01, 0.12196D+01, 0.12872D+01, 0.13539D+01, 0.14196D+01,
     & 0.14844D+01, 0.15485D+01, 0.16115D+01, 0.16737D+01, 0.17348D+01,
     & 0.17951D+01, 0.18547D+01, 0.19135D+01, 0.19713D+01, 0.20283D+01,
     & 0.20844D+01, 0.21397D+01, 0.21943D+01, 0.22482D+01, 0.23013D+01,
     & 0.23536D+01, 0.24051D+01, 0.24558D+01, 0.25058D+01, 0.25551D+01,
     & 0.26039D+01, 0.26520D+01, 0.26994D+01, 0.27460D+01, 0.27920D+01,
     & 0.11389D+01, 0.12028D+01, 0.12657D+01, 0.13275D+01, 0.13883D+01,
     & 0.14481D+01, 0.15072D+01, 0.15653D+01, 0.16224D+01, 0.16785D+01,
     & 0.17337D+01, 0.17883D+01, 0.18420D+01, 0.18947D+01, 0.19467D+01,
     & 0.19977D+01, 0.20479D+01, 0.20974D+01, 0.21463D+01, 0.21944D+01,
     & 0.22417D+01, 0.22882D+01, 0.23339D+01, 0.23790D+01, 0.24233D+01,
     & 0.24672D+01, 0.25104D+01, 0.25528D+01, 0.25946D+01, 0.26357D+01,
     & 0.11249D+01, 0.11842D+01, 0.12423D+01, 0.12993D+01, 0.13553D+01,
     & 0.14104D+01, 0.14646D+01, 0.15179D+01, 0.15701D+01, 0.16214D+01,
     & 0.16718D+01, 0.17215D+01, 0.17703D+01, 0.18183D+01, 0.18653D+01,
     & 0.19116D+01, 0.19570D+01, 0.20017D+01, 0.20458D+01, 0.20891D+01,
     & 0.21316D+01, 0.21734D+01, 0.22145D+01, 0.22548D+01, 0.22945D+01,
     & 0.23338D+01, 0.23723D+01, 0.24102D+01, 0.24474D+01, 0.24840D+01,
     & 0.11087D+01, 0.11633D+01, 0.12169D+01, 0.12693D+01, 0.13206D+01,
     & 0.13710D+01, 0.14206D+01, 0.14692D+01, 0.15168D+01, 0.15634D+01,
     & 0.16091D+01, 0.16542D+01, 0.16984D+01, 0.17417D+01, 0.17842D+01,
     & 0.18258D+01, 0.18667D+01, 0.19069D+01, 0.19465D+01, 0.19853D+01,
     & 0.20233D+01, 0.20606D+01, 0.20973D+01, 0.21333D+01, 0.21686D+01,
     & 0.22035D+01, 0.22377D+01, 0.22713D+01, 0.23043D+01, 0.23367D+01,
     & 0.10900D+01, 0.11402D+01, 0.11893D+01, 0.12372D+01, 0.12841D+01,
     & 0.13300D+01, 0.13751D+01, 0.14192D+01, 0.14623D+01, 0.15045D+01,
     & 0.15459D+01, 0.15865D+01, 0.16263D+01, 0.16653D+01, 0.17034D+01,
     & 0.17407D+01, 0.17773D+01, 0.18132D+01, 0.18486D+01, 0.18832D+01,
     & 0.19170D+01, 0.19502D+01, 0.19827D+01, 0.20146D+01, 0.20459D+01,
     & 0.20768D+01, 0.21070D+01, 0.21366D+01, 0.21656D+01, 0.21941D+01,
     & 0.10687D+01, 0.11146D+01, 0.11593D+01, 0.12030D+01, 0.12456D+01,
     & 0.12872D+01, 0.13280D+01, 0.13679D+01, 0.14068D+01, 0.14448D+01,
     & 0.14819D+01, 0.15184D+01, 0.15540D+01, 0.15889D+01, 0.16230D+01,
     & 0.16563D+01, 0.16888D+01, 0.17208D+01, 0.17521D+01, 0.17828D+01,
     & 0.18128D+01, 0.18421D+01, 0.18708D+01, 0.18989D+01, 0.19264D+01,
     & 0.19535D+01, 0.19801D+01, 0.20060D+01, 0.20315D+01, 0.20563D+01,
     & 0.10442D+01, 0.10859D+01, 0.11264D+01, 0.11659D+01, 0.12044D+01,
     & 0.12419D+01, 0.12786D+01, 0.13144D+01, 0.13493D+01, 0.13833D+01,
     & 0.14165D+01, 0.14490D+01, 0.14807D+01, 0.15117D+01, 0.15419D+01,
     & 0.15714D+01, 0.16002D+01, 0.16284D+01, 0.16561D+01, 0.16831D+01,
     & 0.17094D+01, 0.17352D+01, 0.17603D+01, 0.17849D+01, 0.18089D+01,
     & 0.18326D+01, 0.18557D+01, 0.18783D+01, 0.19003D+01, 0.19219D+01/

      data ((g_grdl(i,j),j=1,30),i=11,20)/
     & 0.10161D+01, 0.10538D+01, 0.10903D+01, 0.11258D+01, 0.11604D+01,
     & 0.11940D+01, 0.12268D+01, 0.12588D+01, 0.12898D+01, 0.13201D+01,
     & 0.13495D+01, 0.13783D+01, 0.14064D+01, 0.14337D+01, 0.14604D+01,
     & 0.14863D+01, 0.15116D+01, 0.15363D+01, 0.15605D+01, 0.15841D+01,
     & 0.16071D+01, 0.16295D+01, 0.16514D+01, 0.16727D+01, 0.16936D+01,
     & 0.17140D+01, 0.17340D+01, 0.17535D+01, 0.17725D+01, 0.17910D+01,
     & 0.98456D+00, 0.10184D+01, 0.10511D+01, 0.10829D+01, 0.11137D+01,
     & 0.11437D+01, 0.11729D+01, 0.12012D+01, 0.12288D+01, 0.12555D+01,
     & 0.12815D+01, 0.13068D+01, 0.13315D+01, 0.13555D+01, 0.13789D+01,
     & 0.14016D+01, 0.14237D+01, 0.14452D+01, 0.14663D+01, 0.14867D+01,
     & 0.15067D+01, 0.15261D+01, 0.15449D+01, 0.15633D+01, 0.15813D+01,
     & 0.15989D+01, 0.16160D+01, 0.16327D+01, 0.16489D+01, 0.16647D+01,
     & 0.94894D+00, 0.97913D+00, 0.10083D+01, 0.10366D+01, 0.10639D+01,
     & 0.10904D+01, 0.11162D+01, 0.11412D+01, 0.11655D+01, 0.11889D+01,
     & 0.12117D+01, 0.12339D+01, 0.12555D+01, 0.12764D+01, 0.12967D+01,
     & 0.13164D+01, 0.13355D+01, 0.13541D+01, 0.13723D+01, 0.13899D+01,
     & 0.14071D+01, 0.14237D+01, 0.14399D+01, 0.14556D+01, 0.14709D+01,
     & 0.14859D+01, 0.15005D+01, 0.15146D+01, 0.15283D+01, 0.15417D+01,
     & 0.90968D+00, 0.93653D+00, 0.96243D+00, 0.98743D+00, 0.10116D+01,
     & 0.10350D+01, 0.10577D+01, 0.10796D+01, 0.11008D+01, 0.11213D+01,
     & 0.11412D+01, 0.11605D+01, 0.11793D+01, 0.11974D+01, 0.12150D+01,
     & 0.12320D+01, 0.12485D+01, 0.12645D+01, 0.12801D+01, 0.12952D+01,
     & 0.13098D+01, 0.13240D+01, 0.13378D+01, 0.13511D+01, 0.13641D+01,
     & 0.13767D+01, 0.13890D+01, 0.14009D+01, 0.14125D+01, 0.14237D+01,
     & 0.86681D+00, 0.89060D+00, 0.91349D+00, 0.93554D+00, 0.95679D+00,
     & 0.97732D+00, 0.99721D+00, 0.10164D+01, 0.10349D+01, 0.10528D+01,
     & 0.10700D+01, 0.10868D+01, 0.11030D+01, 0.11186D+01, 0.11337D+01,
     & 0.11483D+01, 0.11625D+01, 0.11762D+01, 0.11895D+01, 0.12023D+01,
     & 0.12148D+01, 0.12268D+01, 0.12384D+01, 0.12497D+01, 0.12606D+01,
     & 0.12712D+01, 0.12815D+01, 0.12914D+01, 0.13011D+01, 0.13104D+01,
     & 0.81958D+00, 0.84061D+00, 0.86080D+00, 0.88021D+00, 0.89887D+00,
     & 0.91685D+00, 0.93424D+00, 0.95097D+00, 0.96708D+00, 0.98259D+00,
     & 0.99753D+00, 0.10120D+01, 0.10260D+01, 0.10394D+01, 0.10524D+01,
     & 0.10649D+01, 0.10770D+01, 0.10886D+01, 0.10999D+01, 0.11108D+01,
     & 0.11213D+01, 0.11315D+01, 0.11413D+01, 0.11507D+01, 0.11598D+01,
     & 0.11687D+01, 0.11772D+01, 0.11855D+01, 0.11934D+01, 0.12011D+01,
     & 0.76943D+00, 0.78807D+00, 0.80592D+00, 0.82304D+00, 0.83946D+00,
     & 0.85525D+00, 0.87049D+00, 0.88511D+00, 0.89915D+00, 0.91263D+00,
     & 0.92560D+00, 0.93811D+00, 0.95017D+00, 0.96175D+00, 0.97289D+00,
     & 0.98360D+00, 0.99390D+00, 0.10038D+01, 0.10134D+01, 0.10226D+01,
     & 0.10315D+01, 0.10400D+01, 0.10483D+01, 0.10562D+01, 0.10638D+01,
     & 0.10711D+01, 0.10782D+01, 0.10850D+01, 0.10915D+01, 0.10978D+01,
     & 0.71713D+00, 0.73378D+00, 0.74969D+00, 0.76490D+00, 0.77947D+00,
     & 0.79343D+00, 0.80688D+00, 0.81975D+00, 0.83207D+00, 0.84388D+00,
     & 0.85520D+00, 0.86609D+00, 0.87656D+00, 0.88659D+00, 0.89620D+00,
     & 0.90542D+00, 0.91425D+00, 0.92275D+00, 0.93092D+00, 0.93876D+00,
     & 0.94627D+00, 0.95347D+00, 0.96037D+00, 0.96698D+00, 0.97332D+00,
     & 0.97943D+00, 0.98528D+00, 0.99089D+00, 0.99625D+00, 0.10014D+01,
     & 0.66425D+00, 0.67935D+00, 0.69374D+00, 0.70747D+00, 0.72057D+00,
     & 0.73310D+00, 0.74514D+00, 0.75663D+00, 0.76759D+00, 0.77807D+00,
     & 0.78808D+00, 0.79769D+00, 0.80690D+00, 0.81568D+00, 0.82408D+00,
     & 0.83209D+00, 0.83976D+00, 0.84709D+00, 0.85413D+00, 0.86084D+00,
     & 0.86725D+00, 0.87337D+00, 0.87920D+00, 0.88477D+00, 0.89009D+00,
     & 0.89519D+00, 0.90004D+00, 0.90467D+00, 0.90907D+00, 0.91326D+00,
     & 0.61295D+00, 0.62696D+00, 0.64028D+00, 0.65295D+00, 0.66500D+00,
     & 0.67650D+00, 0.68750D+00, 0.69797D+00, 0.70794D+00, 0.71742D+00,
     & 0.72645D+00, 0.73509D+00, 0.74333D+00, 0.75116D+00, 0.75862D+00,
     & 0.76571D+00, 0.77245D+00, 0.77888D+00, 0.78502D+00, 0.79085D+00,
     & 0.79639D+00, 0.80164D+00, 0.80663D+00, 0.81136D+00, 0.81584D+00,
     & 0.82012D+00, 0.82416D+00, 0.82798D+00, 0.83160D+00, 0.83501D+00/

      data ((g_grdl(i,j),j=1,30),i=21,30)/
     & 0.56623D+00, 0.57963D+00, 0.59232D+00, 0.60434D+00, 0.61574D+00,
     & 0.62657D+00, 0.63691D+00, 0.64670D+00, 0.65598D+00, 0.66478D+00,
     & 0.67311D+00, 0.68105D+00, 0.68859D+00, 0.69572D+00, 0.70247D+00,
     & 0.70885D+00, 0.71489D+00, 0.72061D+00, 0.72604D+00, 0.73116D+00,
     & 0.73599D+00, 0.74055D+00, 0.74484D+00, 0.74887D+00, 0.75267D+00,
     & 0.75625D+00, 0.75960D+00, 0.76275D+00, 0.76568D+00, 0.76843D+00,
     & 0.52784D+00, 0.54109D+00, 0.55358D+00, 0.56536D+00, 0.57647D+00,
     & 0.58698D+00, 0.59695D+00, 0.60634D+00, 0.61520D+00, 0.62354D+00,
     & 0.63140D+00, 0.63884D+00, 0.64585D+00, 0.65244D+00, 0.65864D+00,
     & 0.66445D+00, 0.66991D+00, 0.67504D+00, 0.67985D+00, 0.68436D+00,
     & 0.68856D+00, 0.69249D+00, 0.69614D+00, 0.69955D+00, 0.70270D+00,
     & 0.70562D+00, 0.70832D+00, 0.71081D+00, 0.71310D+00, 0.71519D+00,
     & 0.50484D+00, 0.51835D+00, 0.53101D+00, 0.54287D+00, 0.55398D+00,
     & 0.56441D+00, 0.57422D+00, 0.58340D+00, 0.59198D+00, 0.60000D+00,
     & 0.60749D+00, 0.61450D+00, 0.62104D+00, 0.62713D+00, 0.63279D+00,
     & 0.63805D+00, 0.64292D+00, 0.64743D+00, 0.65159D+00, 0.65542D+00,
     & 0.65895D+00, 0.66218D+00, 0.66514D+00, 0.66783D+00, 0.67026D+00,
     & 0.67242D+00, 0.67437D+00, 0.67610D+00, 0.67763D+00, 0.67897D+00,
     & 0.50606D+00, 0.52008D+00, 0.53309D+00, 0.54516D+00, 0.55635D+00,
     & 0.56672D+00, 0.57634D+00, 0.58523D+00, 0.59345D+00, 0.60101D+00,
     & 0.60798D+00, 0.61437D+00, 0.62023D+00, 0.62558D+00, 0.63047D+00,
     & 0.63491D+00, 0.63894D+00, 0.64255D+00, 0.64576D+00, 0.64862D+00,
     & 0.65116D+00, 0.65340D+00, 0.65534D+00, 0.65701D+00, 0.65840D+00,
     & 0.65948D+00, 0.66035D+00, 0.66100D+00, 0.66146D+00, 0.66172D+00,
     & 0.54625D+00, 0.56052D+00, 0.57356D+00, 0.58544D+00, 0.59625D+00,
     & 0.60606D+00, 0.61488D+00, 0.62287D+00, 0.63005D+00, 0.63650D+00,
     & 0.64225D+00, 0.64728D+00, 0.65169D+00, 0.65555D+00, 0.65890D+00,
     & 0.66176D+00, 0.66417D+00, 0.66612D+00, 0.66758D+00, 0.66870D+00,
     & 0.66949D+00, 0.66997D+00, 0.67016D+00, 0.67007D+00, 0.66971D+00,
     & 0.66898D+00, 0.66805D+00, 0.66693D+00, 0.66562D+00, 0.66415D+00,
     & 0.56452D+00, 0.57871D+00, 0.59159D+00, 0.60325D+00, 0.61379D+00,
     & 0.62327D+00, 0.63169D+00, 0.63923D+00, 0.64595D+00, 0.65191D+00,
     & 0.65715D+00, 0.66163D+00, 0.66546D+00, 0.66874D+00, 0.67150D+00,
     & 0.67377D+00, 0.67558D+00, 0.67692D+00, 0.67775D+00, 0.67825D+00,
     & 0.67842D+00, 0.67829D+00, 0.67787D+00, 0.67718D+00, 0.67621D+00,
     & 0.67487D+00, 0.67333D+00, 0.67162D+00, 0.66974D+00, 0.66769D+00,
     & 0.58621D+00, 0.60017D+00, 0.61277D+00, 0.62410D+00, 0.63425D+00,
     & 0.64328D+00, 0.65119D+00, 0.65819D+00, 0.66435D+00, 0.66972D+00,
     & 0.67435D+00, 0.67818D+00, 0.68134D+00, 0.68395D+00, 0.68603D+00,
     & 0.68762D+00, 0.68875D+00, 0.68940D+00, 0.68952D+00, 0.68931D+00,
     & 0.68879D+00, 0.68797D+00, 0.68687D+00, 0.68550D+00, 0.68387D+00,
     & 0.68185D+00, 0.67965D+00, 0.67728D+00, 0.67476D+00, 0.67209D+00,
     & 0.61147D+00, 0.62511D+00, 0.63731D+00, 0.64818D+00, 0.65781D+00,
     & 0.66627D+00, 0.67352D+00, 0.67984D+00, 0.68530D+00, 0.68995D+00,
     & 0.69384D+00, 0.69690D+00, 0.69927D+00, 0.70108D+00, 0.70237D+00,
     & 0.70318D+00, 0.70352D+00, 0.70337D+00, 0.70268D+00, 0.70168D+00,
     & 0.70038D+00, 0.69879D+00, 0.69693D+00, 0.69482D+00, 0.69245D+00,
     & 0.68968D+00, 0.68675D+00, 0.68367D+00, 0.68045D+00, 0.67709D+00,
     & 0.64050D+00, 0.65361D+00, 0.66523D+00, 0.67547D+00, 0.68442D+00,
     & 0.69215D+00, 0.69859D+00, 0.70410D+00, 0.70872D+00, 0.71252D+00,
     & 0.71554D+00, 0.71770D+00, 0.71915D+00, 0.72005D+00, 0.72044D+00,
     & 0.72034D+00, 0.71979D+00, 0.71875D+00, 0.71716D+00, 0.71526D+00,
     & 0.71308D+00, 0.71063D+00, 0.70793D+00, 0.70499D+00, 0.70181D+00,
     & 0.69822D+00, 0.69449D+00, 0.69063D+00, 0.68664D+00, 0.68254D+00,
     & 0.67349D+00, 0.68590D+00, 0.69676D+00, 0.70618D+00, 0.71428D+00,
     & 0.72109D+00, 0.72654D+00, 0.73104D+00, 0.73464D+00, 0.73742D+00,
     & 0.73942D+00, 0.74052D+00, 0.74089D+00, 0.74074D+00, 0.74008D+00,
     & 0.73895D+00, 0.73739D+00, 0.73533D+00, 0.73271D+00, 0.72982D+00,
     & 0.72666D+00, 0.72326D+00, 0.71962D+00, 0.71577D+00, 0.71169D+00,
     & 0.70720D+00, 0.70259D+00, 0.69788D+00, 0.69308D+00, 0.68817D+00/

      data ((g_grdl(i,j),j=1,30),i=31,40)/
     & 0.71066D+00, 0.72211D+00, 0.73196D+00, 0.74034D+00, 0.74735D+00,
     & 0.75304D+00, 0.75729D+00, 0.76058D+00, 0.76299D+00, 0.76457D+00,
     & 0.76537D+00, 0.76524D+00, 0.76439D+00, 0.76302D+00, 0.76117D+00,
     & 0.75888D+00, 0.75616D+00, 0.75297D+00, 0.74920D+00, 0.74520D+00,
     & 0.74095D+00, 0.73649D+00, 0.73183D+00, 0.72697D+00, 0.72191D+00,
     & 0.71643D+00, 0.71088D+00, 0.70525D+00, 0.69954D+00, 0.69377D+00,
     & 0.75226D+00, 0.76246D+00, 0.77104D+00, 0.77810D+00, 0.78377D+00,
     & 0.78809D+00, 0.79089D+00, 0.79275D+00, 0.79374D+00, 0.79390D+00,
     & 0.79330D+00, 0.79176D+00, 0.78948D+00, 0.78673D+00, 0.78353D+00,
     & 0.77992D+00, 0.77590D+00, 0.77143D+00, 0.76639D+00, 0.76114D+00,
     & 0.75570D+00, 0.75007D+00, 0.74427D+00, 0.73831D+00, 0.73218D+00,
     & 0.72564D+00, 0.71906D+00, 0.71243D+00, 0.70577D+00, 0.69906D+00,
     & 0.79853D+00, 0.80715D+00, 0.81412D+00, 0.81957D+00, 0.82361D+00,
     & 0.82628D+00, 0.82735D+00, 0.82751D+00, 0.82682D+00, 0.82533D+00,
     & 0.82311D+00, 0.81993D+00, 0.81603D+00, 0.81170D+00, 0.80697D+00,
     & 0.80186D+00, 0.79639D+00, 0.79049D+00, 0.78403D+00, 0.77742D+00,
     & 0.77065D+00, 0.76374D+00, 0.75670D+00, 0.74953D+00, 0.74223D+00,
     & 0.73455D+00, 0.72685D+00, 0.71916D+00, 0.71146D+00, 0.70376D+00,
     & 0.84970D+00, 0.85632D+00, 0.86130D+00, 0.86476D+00, 0.86682D+00,
     & 0.86750D+00, 0.86655D+00, 0.86472D+00, 0.86209D+00, 0.85870D+00,
     & 0.85461D+00, 0.84957D+00, 0.84384D+00, 0.83774D+00, 0.83128D+00,
     & 0.82450D+00, 0.81741D+00, 0.80992D+00, 0.80190D+00, 0.79377D+00,
     & 0.78555D+00, 0.77724D+00, 0.76884D+00, 0.76037D+00, 0.75181D+00,
     & 0.74289D+00, 0.73400D+00, 0.72516D+00, 0.71636D+00, 0.70761D+00,
     & 0.90602D+00, 0.91020D+00, 0.91276D+00, 0.91384D+00, 0.91353D+00,
     & 0.91187D+00, 0.90850D+00, 0.90434D+00, 0.89943D+00, 0.89384D+00,
     & 0.88760D+00, 0.88042D+00, 0.87260D+00, 0.86448D+00, 0.85608D+00,
     & 0.84742D+00, 0.83852D+00, 0.82927D+00, 0.81953D+00, 0.80976D+00,
     & 0.79995D+00, 0.79011D+00, 0.78025D+00, 0.77037D+00, 0.76045D+00,
     & 0.75020D+00, 0.74006D+00, 0.73000D+00, 0.72004D+00, 0.71017D+00,
     & 0.96802D+00, 0.96914D+00, 0.96871D+00, 0.96687D+00, 0.96371D+00,
     & 0.95923D+00, 0.95306D+00, 0.94618D+00, 0.93864D+00, 0.93051D+00,
     & 0.92181D+00, 0.91221D+00, 0.90204D+00, 0.89165D+00, 0.88108D+00,
     & 0.87033D+00, 0.85942D+00, 0.84823D+00, 0.83660D+00, 0.82501D+00,
     & 0.81346D+00, 0.80196D+00, 0.79051D+00, 0.77910D+00, 0.76772D+00,
     & 0.75606D+00, 0.74457D+00, 0.73322D+00, 0.72203D+00, 0.71098D+00,
     & 0.10356D+01, 0.10330D+01, 0.10290D+01, 0.10237D+01, 0.10172D+01,
     & 0.10094D+01, 0.99998D+00, 0.98994D+00, 0.97937D+00, 0.96832D+00,
     & 0.95681D+00, 0.94447D+00, 0.93164D+00, 0.91872D+00, 0.90573D+00,
     & 0.89266D+00, 0.87952D+00, 0.86620D+00, 0.85251D+00, 0.83896D+00,
     & 0.82555D+00, 0.81226D+00, 0.79910D+00, 0.78607D+00, 0.77314D+00,
     & 0.76000D+00, 0.74709D+00, 0.73440D+00, 0.72193D+00, 0.70965D+00,
     & 0.11090D+01, 0.11020D+01, 0.10937D+01, 0.10842D+01, 0.10737D+01,
     & 0.10621D+01, 0.10488D+01, 0.10352D+01, 0.10211D+01, 0.10068D+01,
     & 0.99208D+00, 0.97667D+00, 0.96089D+00, 0.94516D+00, 0.92949D+00,
     & 0.91387D+00, 0.89830D+00, 0.88266D+00, 0.86675D+00, 0.85109D+00,
     & 0.83566D+00, 0.82047D+00, 0.80550D+00, 0.79074D+00, 0.77617D+00,
     & 0.76149D+00, 0.74711D+00, 0.73302D+00, 0.71922D+00, 0.70569D+00,
     & 0.11884D+01, 0.11760D+01, 0.11626D+01, 0.11482D+01, 0.11330D+01,
     & 0.11169D+01, 0.10992D+01, 0.10814D+01, 0.10634D+01, 0.10452D+01,
     & 0.10270D+01, 0.10081D+01, 0.98904D+00, 0.97019D+00, 0.95156D+00,
     & 0.93314D+00, 0.91492D+00, 0.89676D+00, 0.87846D+00, 0.86054D+00,
     & 0.84297D+00, 0.82575D+00, 0.80887D+00, 0.79230D+00, 0.77602D+00,
     & 0.75974D+00, 0.74385D+00, 0.72833D+00, 0.71318D+00, 0.69838D+00,
     & 0.12739D+01, 0.12551D+01, 0.12356D+01, 0.12154D+01, 0.11947D+01,
     & 0.11733D+01, 0.11505D+01, 0.11279D+01, 0.11053D+01, 0.10829D+01,
     & 0.10605D+01, 0.10378D+01, 0.10151D+01, 0.99283D+00, 0.97095D+00,
     & 0.94947D+00, 0.92836D+00, 0.90749D+00, 0.88665D+00, 0.86632D+00,
     & 0.84650D+00, 0.82715D+00, 0.80827D+00, 0.78982D+00, 0.77178D+00,
     & 0.75387D+00, 0.73645D+00, 0.71950D+00, 0.70299D+00, 0.68692D+00/

      data ((g_grdl(i,j),j=1,30),i=41,50)/
     & 0.13655D+01, 0.13389D+01, 0.13120D+01, 0.12850D+01, 0.12578D+01,
     & 0.12302D+01, 0.12017D+01, 0.11736D+01, 0.11460D+01, 0.11187D+01,
     & 0.10918D+01, 0.10649D+01, 0.10382D+01, 0.10121D+01, 0.98672D+00,
     & 0.96194D+00, 0.93775D+00, 0.91398D+00, 0.89046D+00, 0.86762D+00,
     & 0.84543D+00, 0.82388D+00, 0.80292D+00, 0.78254D+00, 0.76270D+00,
     & 0.74317D+00, 0.72422D+00, 0.70584D+00, 0.68800D+00, 0.67069D+00,
     & 0.14626D+01, 0.14268D+01, 0.13913D+01, 0.13562D+01, 0.13214D+01,
     & 0.12868D+01, 0.12516D+01, 0.12172D+01, 0.11837D+01, 0.11510D+01,
     & 0.11191D+01, 0.10874D+01, 0.10563D+01, 0.10262D+01, 0.99695D+00,
     & 0.96861D+00, 0.94111D+00, 0.91429D+00, 0.88798D+00, 0.86254D+00,
     & 0.83794D+00, 0.81413D+00, 0.79109D+00, 0.76878D+00, 0.74715D+00,
     & 0.72605D+00, 0.70564D+00, 0.68590D+00, 0.66681D+00, 0.64833D+00,
     & 0.15644D+01, 0.15175D+01, 0.14717D+01, 0.14270D+01, 0.13834D+01,
     & 0.13406D+01, 0.12978D+01, 0.12564D+01, 0.12163D+01, 0.11776D+01,
     & 0.11400D+01, 0.11032D+01, 0.10674D+01, 0.10329D+01, 0.99963D+00,
     & 0.96754D+00, 0.93658D+00, 0.90661D+00, 0.87747D+00, 0.84941D+00,
     & 0.82238D+00, 0.79634D+00, 0.77125D+00, 0.74705D+00, 0.72372D+00,
     & 0.70115D+00, 0.67940D+00, 0.65843D+00, 0.63821D+00, 0.61871D+00,
     & 0.16689D+01, 0.16088D+01, 0.15508D+01, 0.14948D+01, 0.14408D+01,
     & 0.13885D+01, 0.13370D+01, 0.12877D+01, 0.12403D+01, 0.11948D+01,
     & 0.11511D+01, 0.11087D+01, 0.10679D+01, 0.10288D+01, 0.99130D+00,
     & 0.95534D+00, 0.92084D+00, 0.88771D+00, 0.85583D+00, 0.82525D+00,
     & 0.79593D+00, 0.76780D+00, 0.74081D+00, 0.71491D+00, 0.69005D+00,
     & 0.66627D+00, 0.64342D+00, 0.62146D+00, 0.60035D+00, 0.58007D+00,
     & 0.17714D+01, 0.16953D+01, 0.16227D+01, 0.15535D+01, 0.14874D+01,
     & 0.14242D+01, 0.13631D+01, 0.13049D+01, 0.12496D+01, 0.11968D+01,
     & 0.11465D+01, 0.10984D+01, 0.10525D+01, 0.10088D+01, 0.96706D+00,
     & 0.92732D+00, 0.88942D+00, 0.85333D+00, 0.81899D+00, 0.78619D+00,
     & 0.75488D+00, 0.72498D+00, 0.69642D+00, 0.66915D+00, 0.64313D+00,
     & 0.61853D+00, 0.59496D+00, 0.57240D+00, 0.55080D+00, 0.53011D+00,
     & 0.18667D+01, 0.17712D+01, 0.16811D+01, 0.15962D+01, 0.15161D+01,
     & 0.14404D+01, 0.13691D+01, 0.13016D+01, 0.12379D+01, 0.11776D+01,
     & 0.11206D+01, 0.10669D+01, 0.10162D+01, 0.96810D+00, 0.92254D+00,
     & 0.87936D+00, 0.83844D+00, 0.79979D+00, 0.76347D+00, 0.72894D+00,
     & 0.69611D+00, 0.66492D+00, 0.63527D+00, 0.60711D+00, 0.58041D+00,
     & 0.55550D+00, 0.53173D+00, 0.50906D+00, 0.48744D+00, 0.46683D+00,
     & 0.19466D+01, 0.18319D+01, 0.17242D+01, 0.16230D+01, 0.15281D+01,
     & 0.14389D+01, 0.13550D+01, 0.12762D+01, 0.12023D+01, 0.11329D+01,
     & 0.10679D+01, 0.10074D+01, 0.95080D+00, 0.89757D+00, 0.84752D+00,
     & 0.80050D+00, 0.75633D+00, 0.71514D+00, 0.67715D+00, 0.64127D+00,
     & 0.60739D+00, 0.57542D+00, 0.54528D+00, 0.51687D+00, 0.49018D+00,
     & 0.46590D+00, 0.44285D+00, 0.42099D+00, 0.40026D+00, 0.38061D+00,
     & 0.20464D+01, 0.18775D+01, 0.17230D+01, 0.15820D+01, 0.14534D+01,
     & 0.13371D+01, 0.12357D+01, 0.11421D+01, 0.10559D+01, 0.97671D+00,
     & 0.90402D+00, 0.83958D+00, 0.78148D+00, 0.72747D+00, 0.67734D+00,
     & 0.63087D+00, 0.58787D+00, 0.54877D+00, 0.51420D+00, 0.48182D+00,
     & 0.45151D+00, 0.42317D+00, 0.39671D+00, 0.37204D+00, 0.34917D+00,
     & 0.32922D+00, 0.31039D+00, 0.29263D+00, 0.27591D+00, 0.26017D+00,
     & 0.15202D+01, 0.13687D+01, 0.12319D+01, 0.11087D+01, 0.99810D+00,
     & 0.90022D+00, 0.81872D+00, 0.74434D+00, 0.67663D+00, 0.61517D+00,
     & 0.55960D+00, 0.51206D+00, 0.47038D+00, 0.43197D+00, 0.39664D+00,
     & 0.36423D+00, 0.33458D+00, 0.30818D+00, 0.28570D+00, 0.26478D+00,
     & 0.24534D+00, 0.22730D+00, 0.21060D+00, 0.19517D+00, 0.18105D+00,
     & 0.16922D+00, 0.15812D+00, 0.14770D+00, 0.13795D+00, 0.12884D+00,
     & 0.10036D+00, 0.89501D-01, 0.79740D-01, 0.70996D-01, 0.63198D-01,
     & 0.56356D-01, 0.50773D-01, 0.45701D-01, 0.41109D-01, 0.36966D-01,
     & 0.33244D-01, 0.30117D-01, 0.27414D-01, 0.24935D-01, 0.22666D-01,
     & 0.20596D-01, 0.18714D-01, 0.17058D-01, 0.15680D-01, 0.14403D-01,
     & 0.13221D-01, 0.12129D-01, 0.11124D-01, 0.10202D-01, 0.93642D-02,
     & 0.86822D-02, 0.80444D-02, 0.74487D-02, 0.68934D-02, 0.63767D-02/

      data ((c_grdl(i,j),j=1,30),i=1,10)/
     & 0.48881D-02, 0.48920D-02, 0.48953D-02, 0.48983D-02, 0.49009D-02,
     & 0.49032D-02, 0.49052D-02, 0.49070D-02, 0.49085D-02, 0.49099D-02,
     & 0.49111D-02, 0.49122D-02, 0.49132D-02, 0.49140D-02, 0.49147D-02,
     & 0.49154D-02, 0.49160D-02, 0.49165D-02, 0.49169D-02, 0.49174D-02,
     & 0.49177D-02, 0.49180D-02, 0.49183D-02, 0.49186D-02, 0.49188D-02,
     & 0.49190D-02, 0.49191D-02, 0.49193D-02, 0.49194D-02, 0.49195D-02,
     & 0.48775D-02, 0.48821D-02, 0.48862D-02, 0.48897D-02, 0.48929D-02,
     & 0.48956D-02, 0.48981D-02, 0.49002D-02, 0.49021D-02, 0.49037D-02,
     & 0.49052D-02, 0.49065D-02, 0.49076D-02, 0.49086D-02, 0.49095D-02,
     & 0.49103D-02, 0.49110D-02, 0.49116D-02, 0.49121D-02, 0.49126D-02,
     & 0.49131D-02, 0.49134D-02, 0.49138D-02, 0.49141D-02, 0.49143D-02,
     & 0.49146D-02, 0.49148D-02, 0.49149D-02, 0.49151D-02, 0.49152D-02,
     & 0.48646D-02, 0.48702D-02, 0.48751D-02, 0.48795D-02, 0.48832D-02,
     & 0.48866D-02, 0.48895D-02, 0.48920D-02, 0.48943D-02, 0.48963D-02,
     & 0.48981D-02, 0.48996D-02, 0.49010D-02, 0.49022D-02, 0.49032D-02,
     & 0.49042D-02, 0.49050D-02, 0.49058D-02, 0.49064D-02, 0.49070D-02,
     & 0.49075D-02, 0.49079D-02, 0.49083D-02, 0.49087D-02, 0.49090D-02,
     & 0.49093D-02, 0.49095D-02, 0.49097D-02, 0.49099D-02, 0.49101D-02,
     & 0.48490D-02, 0.48558D-02, 0.48618D-02, 0.48670D-02, 0.48716D-02,
     & 0.48756D-02, 0.48791D-02, 0.48822D-02, 0.48850D-02, 0.48874D-02,
     & 0.48895D-02, 0.48913D-02, 0.48930D-02, 0.48944D-02, 0.48957D-02,
     & 0.48969D-02, 0.48979D-02, 0.48987D-02, 0.48995D-02, 0.49002D-02,
     & 0.49008D-02, 0.49014D-02, 0.49018D-02, 0.49023D-02, 0.49026D-02,
     & 0.49030D-02, 0.49033D-02, 0.49035D-02, 0.49038D-02, 0.49040D-02,
     & 0.48300D-02, 0.48383D-02, 0.48456D-02, 0.48519D-02, 0.48575D-02,
     & 0.48623D-02, 0.48666D-02, 0.48704D-02, 0.48737D-02, 0.48766D-02,
     & 0.48791D-02, 0.48814D-02, 0.48834D-02, 0.48851D-02, 0.48867D-02,
     & 0.48880D-02, 0.48892D-02, 0.48903D-02, 0.48912D-02, 0.48921D-02,
     & 0.48928D-02, 0.48935D-02, 0.48940D-02, 0.48945D-02, 0.48950D-02,
     & 0.48954D-02, 0.48957D-02, 0.48961D-02, 0.48963D-02, 0.48966D-02,
     & 0.48067D-02, 0.48169D-02, 0.48258D-02, 0.48335D-02, 0.48403D-02,
     & 0.48462D-02, 0.48514D-02, 0.48560D-02, 0.48600D-02, 0.48635D-02,
     & 0.48666D-02, 0.48693D-02, 0.48717D-02, 0.48739D-02, 0.48757D-02,
     & 0.48774D-02, 0.48788D-02, 0.48801D-02, 0.48813D-02, 0.48823D-02,
     & 0.48832D-02, 0.48839D-02, 0.48846D-02, 0.48852D-02, 0.48858D-02,
     & 0.48863D-02, 0.48867D-02, 0.48871D-02, 0.48874D-02, 0.48877D-02,
     & 0.49459D-02, 0.49141D-02, 0.48631D-02, 0.48111D-02, 0.48194D-02,
     & 0.48266D-02, 0.48330D-02, 0.48385D-02, 0.48434D-02, 0.48477D-02,
     & 0.48514D-02, 0.48548D-02, 0.48577D-02, 0.48602D-02, 0.48625D-02,
     & 0.48645D-02, 0.48663D-02, 0.48678D-02, 0.48692D-02, 0.48704D-02,
     & 0.48715D-02, 0.48724D-02, 0.48733D-02, 0.48740D-02, 0.48747D-02,
     & 0.48753D-02, 0.48758D-02, 0.48762D-02, 0.48766D-02, 0.48770D-02,
     & 0.49938D-02, 0.49773D-02, 0.49577D-02, 0.49341D-02, 0.49037D-02,
     & 0.48547D-02, 0.48104D-02, 0.48172D-02, 0.48231D-02, 0.48284D-02,
     & 0.48330D-02, 0.48370D-02, 0.48406D-02, 0.48437D-02, 0.48464D-02,
     & 0.48489D-02, 0.48510D-02, 0.48529D-02, 0.48546D-02, 0.48561D-02,
     & 0.48574D-02, 0.48585D-02, 0.48595D-02, 0.48604D-02, 0.48612D-02,
     & 0.48619D-02, 0.48625D-02, 0.48631D-02, 0.48636D-02, 0.48640D-02,
     & 0.50205D-02, 0.50115D-02, 0.50004D-02, 0.49874D-02, 0.49725D-02,
     & 0.49555D-02, 0.49361D-02, 0.49132D-02, 0.48834D-02, 0.48256D-02,
     & 0.48104D-02, 0.48153D-02, 0.48197D-02, 0.48235D-02, 0.48269D-02,
     & 0.48298D-02, 0.48324D-02, 0.48347D-02, 0.48368D-02, 0.48386D-02,
     & 0.48402D-02, 0.48416D-02, 0.48428D-02, 0.48439D-02, 0.48449D-02,
     & 0.48457D-02, 0.48465D-02, 0.48472D-02, 0.48478D-02, 0.48483D-02,
     & 0.50313D-02, 0.50287D-02, 0.50239D-02, 0.50171D-02, 0.50088D-02,
     & 0.49992D-02, 0.49885D-02, 0.49768D-02, 0.49642D-02, 0.49507D-02,
     & 0.49361D-02, 0.49202D-02, 0.49024D-02, 0.48812D-02, 0.48520D-02,
     & 0.48065D-02, 0.48097D-02, 0.48125D-02, 0.48150D-02, 0.48172D-02,
     & 0.48192D-02, 0.48209D-02, 0.48224D-02, 0.48238D-02, 0.48249D-02,
     & 0.48260D-02, 0.48269D-02, 0.48277D-02, 0.48285D-02, 0.48291D-02/

      data ((c_grdl(i,j),j=1,30),i=11,20)/
     & 0.52877D-02, 0.50303D-02, 0.50315D-02, 0.50305D-02, 0.50277D-02,
     & 0.50233D-02, 0.50177D-02, 0.50113D-02, 0.50041D-02, 0.49965D-02,
     & 0.49885D-02, 0.49803D-02, 0.49719D-02, 0.49634D-02, 0.49549D-02,
     & 0.49464D-02, 0.49380D-02, 0.49295D-02, 0.49212D-02, 0.49128D-02,
     & 0.49045D-02, 0.48962D-02, 0.48878D-02, 0.48793D-02, 0.48705D-02,
     & 0.48612D-02, 0.48512D-02, 0.48393D-02, 0.48222D-02, 0.48056D-02,
     & 0.53595D-02, 0.53329D-02, 0.53038D-02, 0.50283D-02, 0.50308D-02,
     & 0.50315D-02, 0.50307D-02, 0.50286D-02, 0.50256D-02, 0.50219D-02,
     & 0.50177D-02, 0.50132D-02, 0.50085D-02, 0.50037D-02, 0.49988D-02,
     & 0.49941D-02, 0.49895D-02, 0.49850D-02, 0.49807D-02, 0.49767D-02,
     & 0.49728D-02, 0.49692D-02, 0.49659D-02, 0.49627D-02, 0.49598D-02,
     & 0.49572D-02, 0.49547D-02, 0.49525D-02, 0.49504D-02, 0.49486D-02,
     & 0.54025D-02, 0.53887D-02, 0.53705D-02, 0.53490D-02, 0.53255D-02,
     & 0.53008D-02, 0.52754D-02, 0.50303D-02, 0.50313D-02, 0.50314D-02,
     & 0.50307D-02, 0.50293D-02, 0.50275D-02, 0.50254D-02, 0.50231D-02,
     & 0.50207D-02, 0.50182D-02, 0.50158D-02, 0.50135D-02, 0.50112D-02,
     & 0.50090D-02, 0.50070D-02, 0.50051D-02, 0.50033D-02, 0.50016D-02,
     & 0.50001D-02, 0.49987D-02, 0.49975D-02, 0.49963D-02, 0.49953D-02,
     & 0.54083D-02, 0.54111D-02, 0.54069D-02, 0.53975D-02, 0.53843D-02,
     & 0.53684D-02, 0.53508D-02, 0.53322D-02, 0.53132D-02, 0.52941D-02,
     & 0.52754D-02, 0.50298D-02, 0.50309D-02, 0.50314D-02, 0.50315D-02,
     & 0.50313D-02, 0.50308D-02, 0.50302D-02, 0.50294D-02, 0.50286D-02,
     & 0.50277D-02, 0.50269D-02, 0.50260D-02, 0.50252D-02, 0.50245D-02,
     & 0.50237D-02, 0.50230D-02, 0.50224D-02, 0.50218D-02, 0.50213D-02,
     & 0.53655D-02, 0.53899D-02, 0.54041D-02, 0.54104D-02, 0.54106D-02,
     & 0.54062D-02, 0.53985D-02, 0.53883D-02, 0.53766D-02, 0.53639D-02,
     & 0.53508D-02, 0.53376D-02, 0.53246D-02, 0.53119D-02, 0.52999D-02,
     & 0.52884D-02, 0.52777D-02, 0.50289D-02, 0.50297D-02, 0.50304D-02,
     & 0.50308D-02, 0.50310D-02, 0.50312D-02, 0.50314D-02, 0.50315D-02,
     & 0.50315D-02, 0.50315D-02, 0.50315D-02, 0.50314D-02, 0.50314D-02,
     & 0.52605D-02, 0.53124D-02, 0.53503D-02, 0.53768D-02, 0.53945D-02,
     & 0.54050D-02, 0.54101D-02, 0.54111D-02, 0.54089D-02, 0.54044D-02,
     & 0.53984D-02, 0.53914D-02, 0.53838D-02, 0.53758D-02, 0.53678D-02,
     & 0.53600D-02, 0.53524D-02, 0.53452D-02, 0.53384D-02, 0.53320D-02,
     & 0.53260D-02, 0.53206D-02, 0.53156D-02, 0.53110D-02, 0.53068D-02,
     & 0.53030D-02, 0.52996D-02, 0.52965D-02, 0.52937D-02, 0.52912D-02,
     & 0.50741D-02, 0.51622D-02, 0.52309D-02, 0.52837D-02, 0.53236D-02,
     & 0.53534D-02, 0.53750D-02, 0.53903D-02, 0.54006D-02, 0.54069D-02,
     & 0.54101D-02, 0.54111D-02, 0.54105D-02, 0.54086D-02, 0.54060D-02,
     & 0.54028D-02, 0.53992D-02, 0.53955D-02, 0.53918D-02, 0.53882D-02,
     & 0.53847D-02, 0.53813D-02, 0.53782D-02, 0.53752D-02, 0.53725D-02,
     & 0.53700D-02, 0.53677D-02, 0.53656D-02, 0.53637D-02, 0.53620D-02,
     & 0.47997D-02, 0.49262D-02, 0.50262D-02, 0.51127D-02, 0.51820D-02,
     & 0.52369D-02, 0.52800D-02, 0.53136D-02, 0.53398D-02, 0.53598D-02,
     & 0.53751D-02, 0.53865D-02, 0.53950D-02, 0.54011D-02, 0.54053D-02,
     & 0.54081D-02, 0.54099D-02, 0.54108D-02, 0.54111D-02, 0.54110D-02,
     & 0.54106D-02, 0.54100D-02, 0.54093D-02, 0.54085D-02, 0.54076D-02,
     & 0.54067D-02, 0.54059D-02, 0.54051D-02, 0.54043D-02, 0.54036D-02,
     & 0.43896D-02, 0.45786D-02, 0.47314D-02, 0.48552D-02, 0.49546D-02,
     & 0.50357D-02, 0.51064D-02, 0.51642D-02, 0.52112D-02, 0.52492D-02,
     & 0.52800D-02, 0.53048D-02, 0.53249D-02, 0.53412D-02, 0.53543D-02,
     & 0.53649D-02, 0.53734D-02, 0.53804D-02, 0.53860D-02, 0.53905D-02,
     & 0.53942D-02, 0.53971D-02, 0.53995D-02, 0.54015D-02, 0.54030D-02,
     & 0.54043D-02, 0.54054D-02, 0.54062D-02, 0.54069D-02, 0.54075D-02,
     & 0.51956D-02, 0.50371D-02, 0.47096D-02, 0.44717D-02, 0.46218D-02,
     & 0.47449D-02, 0.48461D-02, 0.49291D-02, 0.49966D-02, 0.50557D-02,
     & 0.51064D-02, 0.51489D-02, 0.51843D-02, 0.52138D-02, 0.52386D-02,
     & 0.52593D-02, 0.52766D-02, 0.52912D-02, 0.53035D-02, 0.53139D-02,
     & 0.53228D-02, 0.53304D-02, 0.53368D-02, 0.53423D-02, 0.53470D-02,
     & 0.53511D-02, 0.53546D-02, 0.53576D-02, 0.53602D-02, 0.53625D-02/

      data ((c_grdl(i,j),j=1,30),i=21,30)/
     & 0.48264D-02, 0.50826D-02, 0.51937D-02, 0.51605D-02, 0.49439D-02,
     & 0.47258D-02, 0.44583D-02, 0.45829D-02, 0.46866D-02, 0.47734D-02,
     & 0.48462D-02, 0.49071D-02, 0.49579D-02, 0.50005D-02, 0.50384D-02,
     & 0.50721D-02, 0.51008D-02, 0.51254D-02, 0.51466D-02, 0.51648D-02,
     & 0.51805D-02, 0.51941D-02, 0.52058D-02, 0.52159D-02, 0.52247D-02,
     & 0.52324D-02, 0.52390D-02, 0.52449D-02, 0.52499D-02, 0.52544D-02,
     & 0.49620D-02, 0.50023D-02, 0.49581D-02, 0.49505D-02, 0.51243D-02,
     & 0.51973D-02, 0.51690D-02, 0.50292D-02, 0.46897D-02, 0.47595D-02,
     & 0.44584D-02, 0.45496D-02, 0.46269D-02, 0.46926D-02, 0.47488D-02,
     & 0.47968D-02, 0.48381D-02, 0.48735D-02, 0.49038D-02, 0.49300D-02,
     & 0.49525D-02, 0.49720D-02, 0.49888D-02, 0.50034D-02, 0.50162D-02,
     & 0.50285D-02, 0.50392D-02, 0.50486D-02, 0.50569D-02, 0.50641D-02,
     & 0.32276D-02, 0.43192D-02, 0.49142D-02, 0.49664D-02, 0.49502D-02,
     & 0.49223D-02, 0.49313D-02, 0.50871D-02, 0.51730D-02, 0.51999D-02,
     & 0.51690D-02, 0.50825D-02, 0.49303D-02, 0.46500D-02, 0.47304D-02,
     & 0.43853D-02, 0.44463D-02, 0.44991D-02, 0.45447D-02, 0.45843D-02,
     & 0.46187D-02, 0.46485D-02, 0.46745D-02, 0.46972D-02, 0.47172D-02,
     & 0.47346D-02, 0.47499D-02, 0.47633D-02, 0.47751D-02, 0.47854D-02,
     & 0.27337D-04, 0.12108D-02, 0.26021D-02, 0.37257D-02, 0.45296D-02,
     & 0.49415D-02, 0.49896D-02, 0.50022D-02, 0.49934D-02, 0.47625D-02,
     & 0.49314D-02, 0.50507D-02, 0.51287D-02, 0.51765D-02, 0.51980D-02,
     & 0.51965D-02, 0.51757D-02, 0.51390D-02, 0.50891D-02, 0.50266D-02,
     & 0.49520D-02, 0.48637D-02, 0.47565D-02, 0.46146D-02, 0.46924D-02,
     & 0.47135D-02, 0.47317D-02, 0.47476D-02, 0.43531D-02, 0.43684D-02,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.38758D-03, 0.15905D-02,
     & 0.27275D-02, 0.36454D-02, 0.43415D-02, 0.47891D-02, 0.49685D-02,
     & 0.49896D-02, 0.49886D-02, 0.49560D-02, 0.49931D-02, 0.49070D-02,
     & 0.48195D-02, 0.49138D-02, 0.49879D-02, 0.50451D-02, 0.50886D-02,
     & 0.51215D-02, 0.51459D-02, 0.51652D-02, 0.51791D-02, 0.51886D-02,
     & 0.51947D-02, 0.51982D-02, 0.51998D-02, 0.51998D-02, 0.51988D-02,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.72651D-03,
     & 0.18859D-02, 0.29241D-02, 0.37558D-02, 0.43816D-02, 0.47878D-02,
     & 0.49681D-02, 0.49976D-02, 0.49632D-02, 0.49947D-02, 0.49839D-02,
     & 0.49840D-02, 0.48567D-02, 0.48190D-02, 0.48968D-02, 0.49597D-02,
     & 0.50100D-02, 0.50499D-02, 0.50816D-02, 0.51066D-02, 0.51266D-02,
     & 0.51422D-02, 0.51550D-02, 0.51656D-02, 0.51738D-02, 0.51802D-02,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.87480D-04,
     & 0.10520D-02, 0.21379D-02, 0.30748D-02, 0.38315D-02, 0.43984D-02,
     & 0.47735D-02, 0.49590D-02, 0.49896D-02, 0.49227D-02, 0.49984D-02,
     & 0.49510D-02, 0.49905D-02, 0.49812D-02, 0.48906D-02, 0.47894D-02,
     & 0.48555D-02, 0.49101D-02, 0.49553D-02, 0.49924D-02, 0.50230D-02,
     & 0.50483D-02, 0.50691D-02, 0.50862D-02, 0.51004D-02, 0.51124D-02,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.31385D-03, 0.13370D-02, 0.23418D-02, 0.31893D-02, 0.38751D-02,
     & 0.43929D-02, 0.47452D-02, 0.49372D-02, 0.49641D-02, 0.49813D-02,
     & 0.49667D-02, 0.50023D-02, 0.49586D-02, 0.49895D-02, 0.49890D-02,
     & 0.49497D-02, 0.48092D-02, 0.47907D-02, 0.48388D-02, 0.48793D-02,
     & 0.49137D-02, 0.49428D-02, 0.49674D-02, 0.49883D-02, 0.50061D-02,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.57729D-03, 0.15732D-02, 0.24960D-02, 0.32643D-02,
     & 0.38886D-02, 0.43656D-02, 0.47013D-02, 0.49018D-02, 0.49600D-02,
     & 0.49955D-02, 0.49077D-02, 0.49788D-02, 0.50022D-02, 0.49495D-02,
     & 0.49806D-02, 0.49932D-02, 0.49868D-02, 0.49602D-02, 0.49039D-02,
     & 0.47447D-02, 0.47813D-02, 0.48126D-02, 0.48395D-02, 0.48626D-02,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.55786D-04, 0.82288D-03, 0.17584D-02, 0.26039D-02,
     & 0.33022D-02, 0.38735D-02, 0.43163D-02, 0.46393D-02, 0.48484D-02,
     & 0.49589D-02, 0.49613D-02, 0.49973D-02, 0.49144D-02, 0.49728D-02,
     & 0.50004D-02, 0.49909D-02, 0.49571D-02, 0.49787D-02, 0.49906D-02,
     & 0.49933D-02, 0.49878D-02, 0.49748D-02, 0.49542D-02, 0.49249D-02/

      data ((c_grdl(i,j),j=1,30),i=31,40)/
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.20539D-03, 0.10313D-02, 0.18943D-02,
     & 0.26685D-02, 0.33053D-02, 0.38305D-02, 0.42444D-02, 0.45569D-02,
     & 0.47745D-02, 0.49097D-02, 0.49691D-02, 0.49704D-02, 0.49976D-02,
     & 0.49190D-02, 0.49485D-02, 0.49834D-02, 0.50005D-02, 0.49991D-02,
     & 0.49562D-02, 0.49584D-02, 0.49728D-02, 0.49829D-02, 0.49894D-02,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.37806D-03, 0.11932D-02,
     & 0.19858D-02, 0.26932D-02, 0.32754D-02, 0.37601D-02, 0.41489D-02,
     & 0.44518D-02, 0.46758D-02, 0.48297D-02, 0.49261D-02, 0.49691D-02,
     & 0.49589D-02, 0.49931D-02, 0.49921D-02, 0.48995D-02, 0.49392D-02,
     & 0.49678D-02, 0.49870D-02, 0.49982D-02, 0.50023D-02, 0.49996D-02,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.10908D-04, 0.53489D-03,
     & 0.13083D-02, 0.20340D-02, 0.26803D-02, 0.32135D-02, 0.36623D-02,
     & 0.40297D-02, 0.43219D-02, 0.45490D-02, 0.47180D-02, 0.48370D-02,
     & 0.49165D-02, 0.49623D-02, 0.49520D-02, 0.49667D-02, 0.49907D-02,
     & 0.49975D-02, 0.49847D-02, 0.49184D-02, 0.49208D-02, 0.49419D-02,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.72807D-04,
     & 0.65777D-03, 0.13777D-02, 0.20403D-02, 0.26319D-02, 0.31209D-02,
     & 0.35386D-02, 0.38849D-02, 0.41659D-02, 0.43927D-02, 0.45707D-02,
     & 0.47069D-02, 0.48076D-02, 0.48805D-02, 0.49299D-02, 0.49612D-02,
     & 0.49678D-02, 0.49406D-02, 0.49661D-02, 0.49833D-02, 0.49934D-02,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.15015D-03, 0.74047D-03, 0.14034D-02, 0.20070D-02, 0.25492D-02,
     & 0.30016D-02, 0.33883D-02, 0.37130D-02, 0.39842D-02, 0.42063D-02,
     & 0.43882D-02, 0.45344D-02, 0.46504D-02, 0.47410D-02, 0.48106D-02,
     & 0.48640D-02, 0.49041D-02, 0.49332D-02, 0.49543D-02, 0.49662D-02,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.21621D-03, 0.78218D-03, 0.13882D-02, 0.19379D-02,
     & 0.24336D-02, 0.28543D-02, 0.32098D-02, 0.35159D-02, 0.37743D-02,
     & 0.39913D-02, 0.41714D-02, 0.43219D-02, 0.44461D-02, 0.45481D-02,
     & 0.46315D-02, 0.46992D-02, 0.47540D-02, 0.47982D-02, 0.48338D-02,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.25809D-03, 0.78408D-03, 0.13347D-02,
     & 0.18357D-02, 0.22862D-02, 0.26781D-02, 0.30071D-02, 0.32929D-02,
     & 0.35374D-02, 0.37462D-02, 0.39238D-02, 0.40731D-02, 0.41999D-02,
     & 0.43072D-02, 0.43977D-02, 0.44739D-02, 0.45382D-02, 0.45923D-02,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.23486D-05, 0.27114D-03, 0.74882D-03,
     & 0.12461D-02, 0.17023D-02, 0.21085D-02, 0.24716D-02, 0.27818D-02,
     & 0.30439D-02, 0.32751D-02, 0.34739D-02, 0.36438D-02, 0.37917D-02,
     & 0.39179D-02, 0.40256D-02, 0.41177D-02, 0.41976D-02, 0.42662D-02,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.59191D-05, 0.25572D-03,
     & 0.68010D-03, 0.11251D-02, 0.15386D-02, 0.19053D-02, 0.22370D-02,
     & 0.25275D-02, 0.27767D-02, 0.29891D-02, 0.31748D-02, 0.33386D-02,
     & 0.34796D-02, 0.36010D-02, 0.37078D-02, 0.38005D-02, 0.38807D-02,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.50521D-05,
     & 0.21569D-03, 0.58262D-03, 0.97608D-03, 0.13479D-02, 0.16825D-02,
     & 0.19777D-02, 0.22457D-02, 0.24801D-02, 0.26830D-02, 0.28571D-02,
     & 0.30076D-02, 0.31398D-02, 0.32578D-02, 0.33605D-02, 0.34497D-02/

      data ((c_grdl(i,j),j=1,30),i=41,50)/
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.11080D-05, 0.15799D-03, 0.46241D-03, 0.80331D-03, 0.11335D-02,
     & 0.14356D-02, 0.17049D-02, 0.19422D-02, 0.21573D-02, 0.23478D-02,
     & 0.25139D-02, 0.26585D-02, 0.27838D-02, 0.28927D-02, 0.29877D-02,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.92848D-04, 0.32890D-03, 0.61498D-03,
     & 0.90134D-03, 0.11694D-02, 0.14128D-02, 0.16295D-02, 0.18205D-02,
     & 0.19900D-02, 0.21441D-02, 0.22802D-02, 0.23997D-02, 0.25043D-02,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.34445D-04, 0.19564D-03,
     & 0.42003D-03, 0.65973D-03, 0.89124D-03, 0.11060D-02, 0.13003D-02,
     & 0.14740D-02, 0.16287D-02, 0.17649D-02, 0.18847D-02, 0.19925D-02,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.15475D-05,
     & 0.81221D-04, 0.23583D-03, 0.42148D-03, 0.61215D-03, 0.79416D-03,
     & 0.96341D-03, 0.11171D-02, 0.12554D-02, 0.13790D-02, 0.14883D-02,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.10044D-04, 0.87164D-04, 0.20886D-03, 0.34963D-03,
     & 0.49398D-03, 0.63299D-03, 0.76234D-03, 0.88124D-03, 0.98915D-03,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.57337D-05, 0.54998D-04,
     & 0.13560D-03, 0.23139D-03, 0.33218D-03, 0.43175D-03, 0.52646D-03,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.12707D-04, 0.50681D-04, 0.10317D-03, 0.16280D-03,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.93890D-06,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00,
     & 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00, 0.00000D+00/

      DO I=1,50
        DO J=1,30
          UDS_GRD(I,J) = UDS_GRDL(I,J)
          G_GRD(I,J)   = G_GRDL(I,J)
          C_GRD(I,J)   = C_GRDL(I,J)
        ENDDO
      ENDDO

      RETURN
      END


